@@ -104,9 +104,15 @@
 /* Define this if you have ip_mreqn in netinet/in.h */
 #undef HAVE_IP_MREQN
 
+/* Define this if you have a dxr3 mpeg encoder card */
+#undef HAVE_DXR3
+
 /* Define this if you have libfame mpeg encoder installed (fame.sf.net) */
 #undef HAVE_LIBFAME
 
+/* Define this if you have libfame 0.8.10 or above */
+#undef HAVE_NEW_LIBFAME
+
 /* Define this if you have librte mpeg encoder installed (zapping.sf.net) */
 #undef HAVE_LIBRTE
 
