@@ -26,7 +26,7 @@
  * (c) 2001 James Courtier-Dutton <James@superbug.demon.co.uk>
  *
  * 
- * $Id: audio_alsa_out.c,v 1.52 2002/04/27 22:11:12 matt2000 Exp $
+ * $Id: audio_alsa_out.c,v 1.75 2002/07/31 06:29:08 pmhahn Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -59,17 +59,11 @@
 #include "compat.h"
 #include "audio_out.h"
 
-#ifndef AFMT_S16_NE
-# if defined(sparc) || defined(__sparc__) || defined(PPC)
-/* Big endian machines */
-#  define AFMT_S16_NE AFMT_S16_BE
-# else
-#  define AFMT_S16_NE AFMT_S16_LE
-# endif
-#endif
-
+#define ALSA_LOG
 #define AO_OUT_ALSA_IFACE_VERSION 4
 
+#define BUFFER_TIME               100*1000
+#define PERIOD_TIME               10*1000
 #define GAP_TOLERANCE             5000
 
 #define MIXER_MASK_LEFT           (1 << 0)
@@ -106,7 +100,6 @@ typedef struct alsa_driver_s {
     long               right_vol;
     int                mute;
   } mixer;
-
 } alsa_driver_t;
 
 static snd_output_t *jcd_out;
@@ -127,6 +120,9 @@ static void *ao_alsa_handle_event_thread(void *data) {
   pthread_exit(NULL);
 }
 
+/*
+ * Get and convert volume to percent value
+ */
 static int ao_alsa_get_percent_from_volume(long val, long min, long max)
 {
   int range = max - min;
@@ -139,6 +135,9 @@ static int ao_alsa_get_percent_from_volume(long val, long min, long max)
   return tmp;
 }
 
+/*
+ * Convert percent value to volume and set
+ */
 static long ao_alsa_get_volume_from_percent(int val, long min, long max)
 {
   int range = max - min;
@@ -164,16 +163,8 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   snd_pcm_hw_params_t  *params;
   snd_pcm_sw_params_t  *swparams;
   snd_pcm_sframes_t     buffer_size;
-  snd_pcm_sframes_t     period_size,tmp;
-  /*
-  snd_aes_iec958_t      spdif;
-  snd_ctl_elem_value_t *ctl;
-  snd_ctl_t            *ctl_handle;
-  snd_pcm_info_t       *info;
-  char                  ctl_name[12];
-  int                   ctl_card;
-  */
-  int                   err, step;
+  snd_pcm_sframes_t     period_size;
+  int                   err, dir;
  // int                 open_mode=1; //NONBLOCK
   int                   open_mode=0; //BLOCK
 
@@ -187,7 +178,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_default_device",
                                          "default",
-                                         "device used for mono output",
+                                         _("device used for mono output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -197,7 +188,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_front_device",
                                          "default",
-                                         "device used for stereo output",
+                                         _("device used for stereo output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -207,7 +198,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_surround40_device",
                                          "surround40",
-                                         "device used for 4-channel output",
+                                         _("device used for 4-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -217,7 +208,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_surround50_device",
                                          "surround51",
-                                         "device used for 5-channel output",
+                                         _("device used for 5-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -227,7 +218,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_surround51_device",
                                          "surround51",
-                                         "device used for 5.1-channel output",
+                                         _("device used for 5.1-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -238,7 +229,7 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     pcm_device = config->register_string(config,
                                          "audio.alsa_a52_device",
                                          "iec958:AES0=0x6,AES1=0x82,AES2=0x0,AES3=0x2",
-                                         "device used for 5.1-channel output",
+                                         _("device used for 5.1-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -248,8 +239,10 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     return 0;
   } 
 
+#ifdef ALSA_LOG
   printf("audio_alsa_out: Audio Device name = %s\n",pcm_device);
   printf("audio_alsa_out: Number of channels = %d\n",this->num_channels);
+#endif
 
   if (this->audio_fd != NULL) {
     xlerror ("Already open...WHY!");
@@ -260,68 +253,15 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
   this->input_sample_rate      = rate;
   this->bits_per_sample        = bits;
   this->bytes_in_buffer        = 0;
-  /* FIXME: Can use an ALSA function here */
-  this->bytes_per_frame=(this->bits_per_sample*this->num_channels)/8;
   /*
    * open audio device
    */
-
   err=snd_pcm_open(&this->audio_fd, pcm_device, direction, open_mode);      
   if(err <0 ) {                                                           
     printf ("audio_alsa_out: snd_pcm_open() failed: %s\n", snd_strerror(err));               
     printf ("audio_alsa_out: >>> check if another program don't already use PCM <<<\n");     
     return 0;
   }
-/*
- * This is all not needed in the new ALSA API
- * Beginning of old alsa section.
- * 
-  if ((mode & AO_CAP_MODE_A52) || (mode & AO_CAP_MODE_AC5)) {
-    snd_pcm_info_alloca(&info);
-
-    if ((err = snd_pcm_info(this->audio_fd, info)) < 0) {
-      fprintf(stderr, "info: %s\n", snd_strerror(err));
-      goto __close;
-    }
-    printf ("audio_alsa_out: device: %d, subdevice: %d\n", 
-	    snd_pcm_info_get_device(info),
-	    snd_pcm_info_get_subdevice(info));
-
-    spdif.status[0] = IEC958_AES0_NONAUDIO |
-                      IEC958_AES0_CON_EMPHASIS_NONE;
-    spdif.status[1] = IEC958_AES1_CON_ORIGINAL |
-                      IEC958_AES1_CON_PCM_CODER;
-    spdif.status[2] = 0;
-    spdif.status[3] = IEC958_AES3_CON_FS_48000;
-
-    snd_ctl_elem_value_alloca(&ctl);
-    snd_ctl_elem_value_set_interface(ctl, SND_CTL_ELEM_IFACE_PCM);
-    snd_ctl_elem_value_set_device(ctl,snd_pcm_info_get_device(info));
-    snd_ctl_elem_value_set_subdevice(ctl, snd_pcm_info_get_subdevice(info));
-    snd_ctl_elem_value_set_name(ctl, SND_CTL_NAME_IEC958("",PLAYBACK,PCM_STREAM));
-    snd_ctl_elem_value_set_iec958(ctl, &spdif);
-    ctl_card = snd_pcm_info_get_card(info);
-    if (ctl_card < 0) {
-      printf ("audio_alsa_out: unable to setup the IEC958 (S/PDIF) interface - PCM has no assigned card");
-      goto __close;
-    }
-    sprintf(ctl_name, "hw:%d", ctl_card);
-    printf("hw:%d\n", ctl_card);
-    if ((err = snd_ctl_open(&ctl_handle, ctl_name, 0)) < 0) {
-      printf ("audio_alsa_out: unable to open the control interface '%s':%s", 
-	      ctl_name, snd_strerror(err));
-      goto __close;
-    }
-    if ((err = snd_ctl_elem_write(ctl_handle, ctl)) < 0) {
-      printf ("audio_alsa_out: unable to update the IEC958 control: %s", 
-	      snd_strerror(err));
-      goto __close;
-    }
-    snd_ctl_close(ctl_handle);
-  }
- *
- * End of old alsa section.
- */
   /* We wanted non blocking open but now put it back to normal */
   snd_pcm_nonblock(this->audio_fd, 0);
   /*
@@ -339,65 +279,105 @@ static int ao_alsa_open(ao_driver_t *this_gen, uint32_t bits, uint32_t rate, int
     printf ("audio_alsa_out: access type not available\n");
     goto __close;
   }
-  err = snd_pcm_hw_params_set_format(this->audio_fd, params, bits == 16 ? SND_PCM_FORMAT_S16_LE : SND_PCM_FORMAT_U8);
+  /* set the sample format ([SU]{8,16{LE,BE}})*/
+  err = snd_pcm_hw_params_set_format(this->audio_fd, params, bits == 16 ?
+#ifdef WORDS_BIGENDIAN
+		  SND_PCM_FORMAT_S16_BE
+#else
+		  SND_PCM_FORMAT_S16_LE
+#endif
+		  : SND_PCM_FORMAT_U8);
   if (err < 0) {
     printf ("audio_alsa_out: sample format non available\n");
     goto __close;
   }
+  /* set the count of channels */
   err = snd_pcm_hw_params_set_channels(this->audio_fd, params, this->num_channels);
   if (err < 0) {
     printf ("audio_alsa_out: channels count non available\n");
     goto __close;
   }
-  err = snd_pcm_hw_params_set_rate_near(this->audio_fd, params, rate, 0);
-    if (err < 0) {
-      printf ("audio_alsa_out: rate not available\n");
-      goto __close;
-    }
-  buffer_size = snd_pcm_hw_params_set_buffer_size_near(this->audio_fd, params,
-                                                       500000);
-  if (buffer_size < 0) {
+  /* set the stream rate [Hz] */
+  dir=0;
+  err = snd_pcm_hw_params_set_rate_near(this->audio_fd, params, rate, &dir);
+  if (err < 0) {
+    printf ("audio_alsa_out: rate not available\n");
+    goto __close;
+  }
+  this->output_sample_rate = (uint32_t)err;
+  if (this->input_sample_rate != this->output_sample_rate) {
+    printf ("audio_alsa_out: audio rate : %d requested, %d provided by device/sec\n",
+	    this->input_sample_rate, this->output_sample_rate);
+  }
+  /* set the ring-buffer time [us] (large enough for x us|y samples ...) */
+  dir=0;
+  err = snd_pcm_hw_params_set_buffer_time_near(this->audio_fd, params, BUFFER_TIME, &dir);
+  if (err < 0) {
     printf ("audio_alsa_out: buffer time not available\n");
     goto __close;
   }
-  step = 2;
-  period_size = 128;
-  do {
-    period_size *= 2;
-    tmp = snd_pcm_hw_params_set_period_size_near(this->audio_fd, params,
-                                                 period_size, 0);
-    printf("audio_alsa_out:open:period_size=%ld tmp=%ld\n",period_size,tmp);
-    
-    if (period_size < 0) {
-      printf ("audio_alsa_out: period size not available");
-      goto __close;
-    }
-  } while (period_size <= (buffer_size/2) && (period_size != tmp));
-  if (buffer_size == period_size) {
-    printf ("audio_alsa_out: buffer time and period time match, could not use\n");
+  buffer_size = snd_pcm_hw_params_get_buffer_size(params);
+  /* set the period time [us] (interrupt every x us|y samples ...) */
+  dir=0;
+  err = snd_pcm_hw_params_set_period_time_near(this->audio_fd, params, PERIOD_TIME, &dir);
+  if (err < 0) {
+    printf ("audio_alsa_out: period time not available");
     goto __close;
   }
+  period_size = snd_pcm_hw_params_get_period_size(params, NULL);
+  if (2*period_size >= buffer_size) {
+    printf ("audio_alsa_out: buffer to small, could not use\n");
+    goto __close;
+  }
+  /* Check for pause/resume support */
   this->has_pause_resume = ( snd_pcm_hw_params_can_pause (params)
 			    && snd_pcm_hw_params_can_resume (params) );
-  if ((err = snd_pcm_hw_params(this->audio_fd, params)) < 0) {
+  /* write the parameters to device */
+  err = snd_pcm_hw_params(this->audio_fd, params);
+  if (err < 0) {
     printf ("audio_alsa_out: pcm hw_params failed: %s\n", snd_strerror(err));
     goto __close;
   }
-  this->output_sample_rate = this->input_sample_rate;
   this->sample_rate_factor = (double) this->output_sample_rate / (double) this->input_sample_rate;
+  this->bytes_per_frame = snd_pcm_frames_to_bytes (this->audio_fd, 1);
   /*
    * audio buffer size handling
    */
   /* Copy current parameters into swparams */
-  snd_pcm_sw_params_current(this->audio_fd, swparams);
-  tmp=snd_pcm_sw_params_set_xfer_align(this->audio_fd, swparams, period_size);
-  tmp=snd_pcm_sw_params_set_avail_min(this->audio_fd, swparams, 1);
-  tmp=snd_pcm_sw_params_set_start_threshold(this->audio_fd, swparams, period_size);
+  err = snd_pcm_sw_params_current(this->audio_fd, swparams);
+  if (err < 0) {
+    printf ("audio_alsa_out: Unable to determine current swparams: %s\n", snd_strerror(err));
+    goto __close;
+  }
+  /* align all transfers to 1 sample */
+  err = snd_pcm_sw_params_set_xfer_align(this->audio_fd, swparams, 1);
+  if (err < 0) {
+    printf ("audio_alsa_out: Unable to set transfer alignment: %s\n", snd_strerror(err));
+    goto __close;
+  }
+  /* allow the transfer when at least period_size samples can be processed */
+  err = snd_pcm_sw_params_set_avail_min(this->audio_fd, swparams, period_size);
+  if (err < 0) {
+    printf ("audio_alsa_out: Unable to set available min: %s\n", snd_strerror(err));
+    goto __close;
+  }
+  /* start the transfer when the buffer contains at least period_size samples */
+  err = snd_pcm_sw_params_set_start_threshold(this->audio_fd, swparams, period_size);
+  if (err < 0) {
+    printf ("audio_alsa_out: Unable to set start threshold: %s\n", snd_strerror(err));
+    goto __close;
+  }
 
   /* Install swparams into current parameters */
-  snd_pcm_sw_params(this->audio_fd, swparams);
+  err = snd_pcm_sw_params(this->audio_fd, swparams);
+  if (err < 0) {
+    printf ("audio_alsa_out: Unable to set swparams: %s\n", snd_strerror(err));
+    goto __close;
+  }
+#ifdef ALSA_LOG
   snd_pcm_dump_setup(this->audio_fd, jcd_out); 
   snd_pcm_sw_params_dump(swparams, jcd_out);
+#endif
   return this->output_sample_rate;
 __close:
   snd_pcm_close (this->audio_fd);
@@ -405,23 +385,35 @@ __close:
   return 0;
 }
 
+/*
+ * Return the number of audio channels
+ */
 static int ao_alsa_num_channels(ao_driver_t *this_gen)
 {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   return this->num_channels;
 }
 
+/*
+ * Return the number of bytes per frame
+ */
 static int ao_alsa_bytes_per_frame(ao_driver_t *this_gen)
 {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   return this->bytes_per_frame;
 }
 
+/*
+ * Return gap tolerance (in pts)
+ */
 static int ao_alsa_get_gap_tolerance (ao_driver_t *this_gen)
 {
   return GAP_TOLERANCE;
 }
 
+/*
+ * Return the delay. is frames measured by looking at pending samples
+ */
 static int ao_alsa_delay (ao_driver_t *this_gen) 
 {
   snd_pcm_status_t  *pcm_stat;
@@ -437,6 +429,9 @@ static int ao_alsa_delay (ao_driver_t *this_gen)
   return delay;
 }
 
+/*
+ * Handle over/under-run
+ */
 static void xrun(alsa_driver_t *this) 
 {
   snd_pcm_status_t *status;
@@ -461,6 +456,9 @@ static void xrun(alsa_driver_t *this)
   }
 }
 
+/*
+ * Write audio data to output buffer (blocking)
+ */
 static int ao_alsa_write(ao_driver_t *this_gen,int16_t *data, uint32_t count)
 {
   snd_pcm_sframes_t result;
@@ -479,14 +477,15 @@ static int ao_alsa_write(ao_driver_t *this_gen,int16_t *data, uint32_t count)
     }
     if (result > 0) {
       number_of_frames -= result;
-      /* FIXME: maybe not *2 as int16 */
       buffer += result * this->bytes_per_frame;
     }
   }
-  /* FIXME: What should this really be? */
-  return 1;
+  return 1; /* audio samples were processed ok */
 }
 
+/*
+ * This is called when the decoder no longer uses the audio
+ */
 static void ao_alsa_close(ao_driver_t *this_gen)
 {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
@@ -495,25 +494,22 @@ static void ao_alsa_close(ao_driver_t *this_gen)
   this->has_pause_resume = 0; /* This is set at open time */
 }
 
+/*
+ * Find out what output modes + capatilities are supported
+ */
 static uint32_t ao_alsa_get_capabilities (ao_driver_t *this_gen) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   return this->capabilities;
 }
 
+/*
+ * Shut down audio output driver plugin and free all resources allocated
+ */
 static void ao_alsa_exit(ao_driver_t *this_gen)
 {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   void          *p;
 
-  config_values_t *config = this->config;
-
-  config->update_num (config, "audio.alsa_mixer_volume", 
-		   (((ao_alsa_get_percent_from_volume(this->mixer.left_vol, 
-						      this->mixer.min, this->mixer.max)) + 
-		     (ao_alsa_get_percent_from_volume(this->mixer.right_vol, 
-						      this->mixer.min, this->mixer.max))) /2));
-  config->save(config);
-
   pthread_mutex_destroy(&this->mixer.mutex);
   /*
    * Destroy the mixer thread and cleanup the mixer, so that
@@ -533,6 +529,9 @@ static void ao_alsa_exit(ao_driver_t *this_gen)
   free (this);
 }
 
+/*
+ * Get a property of audio driver
+ */
 static int ao_alsa_get_property (ao_driver_t *this_gen, int property) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   int err;
@@ -575,7 +574,7 @@ static int ao_alsa_get_property (ao_driver_t *this_gen, int property) {
 }
 
 /*
- *
+ * Set a property of audio driver
  */
 static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
@@ -645,45 +644,74 @@ static int ao_alsa_set_property (ao_driver_t *this_gen, int property, int value)
   return ~value;
 }
 
-
+/*
+ * Misc control operations
+ */
 static int ao_alsa_ctrl(ao_driver_t *this_gen, int cmd, ...) {
   alsa_driver_t *this = (alsa_driver_t *) this_gen;
   int result;
-#if 0
-  /* Alsa 0.9.x pause and result is not stable enough at the moment.
-   * FIXME: Change these to snd_pcm_drop and restart.
+
+  /* Alsa 0.9.x pause and resume is not stable enough at the moment.
+   * Use snd_pcm_drop and restart instead.
    */
   switch (cmd) {
 
   case AO_CTRL_PLAY_PAUSE:
-    if ((this->has_pause_resume) && (this->audio_fd > 0)) {
-      if ((result=snd_pcm_pause(this->audio_fd, 1)) < 0) {
-        printf("Pause call failed err=%d\n",result);
+    if (this->audio_fd > 0) {
+      if (this->has_pause_resume) {
+        if ((result=snd_pcm_pause(this->audio_fd, 1)) < 0) {
+          printf("audio_alsa_out: Pause call failed err=%d\n", result);
+          this->has_pause_resume = 0;
+          ao_alsa_ctrl(this_gen, AO_CTRL_PLAY_PAUSE);
+        }
+      } else {
+        if ((result=snd_pcm_reset(this->audio_fd)) < 0) {
+          printf("audio_alsa_out: Reset call failed err=%d\n",result);
+        }
+        if ((result=snd_pcm_drain(this->audio_fd)) < 0) {
+          printf("audio_alsa_out: Drain call failed err=%d\n",result);
+        }
+        if ((result=snd_pcm_prepare(this->audio_fd)) < 0) {
+          printf("audio_alsa_out: Prepare call failed err=%d\n",result);
+        }
       }
     }
     break;
 
   case AO_CTRL_PLAY_RESUME:
-    if ((this->has_pause_resume) && (this->audio_fd > 0) ) {
-      if ((result=snd_pcm_pause(this->audio_fd, 0)) < 0) {
-        printf("Resume call failed err=%d\n",result);
+    if (this->audio_fd > 0) {
+      if (this->has_pause_resume) {
+        if ((result=snd_pcm_pause(this->audio_fd, 0)) < 0) {
+          printf("audio_alsa_out: Resume call failed err=%d\n",result);
+          this->has_pause_resume = 0;
+          ao_alsa_ctrl(this_gen, AO_CTRL_PLAY_RESUME);
+        }
       }
     }
     break;
 
   case AO_CTRL_FLUSH_BUFFERS:
     if (this->audio_fd > 0) {
-      snd_pcm_drop(this->audio_fd);
-      snd_pcm_prepare(this->audio_fd);
+      printf("audio_alsa_out: flush buffer\n");
+      if ((result=snd_pcm_reset(this->audio_fd)) < 0) {
+        printf("audio_alsa_out: Reset call failed err=%d\n",result);
+      }
+      if ((result=snd_pcm_drain(this->audio_fd)) < 0) {
+        printf("audio_alsa_out: Drain call failed err=%d\n",result);
+      }
+      if ((result=snd_pcm_prepare(this->audio_fd)) < 0) {
+        printf("audio_alsa_out: Prepare call failed err=%d\n",result);
+      }
     }
     break;
   }
-#endif
 
   return 0;
 }
 
-
+/*
+ * Initialize mixer
+ */
 static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
   alsa_driver_t        *this = (alsa_driver_t *) this_gen;
   config_values_t      *config = this->config;
@@ -703,7 +731,7 @@ static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
   pcm_device = config->register_string(config,
 				       "audio.alsa_default_device",
 				       "default",
-				       "device used for mono output",
+				       _("device used for mono output"),
 				       NULL,
 				       NULL,
 				       NULL);
@@ -837,7 +865,7 @@ static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
   this->mixer.name = config->register_string(config,
                                              "audio.alsa_mixer_name",
                                              "PCM",
-                                             "alsa mixer device",
+                                             _("alsa mixer device"),
                                              NULL,
                                              NULL,
                                              NULL);
@@ -872,6 +900,9 @@ static void ao_alsa_mixer_init(ao_driver_t *this_gen) {
 
 }
 
+/*
+ * Initialize plugin
+ */
 ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
   alsa_driver_t *this;
@@ -885,42 +916,42 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   pcm_device = config->register_string(config,
                                          "audio.alsa_default_device",
                                          "default",
-                                         "device used for mono output",
+                                         _("device used for mono output"),
                                          NULL,
                                          NULL,
                                          NULL);
   pcm_device = config->register_string(config,
                                          "audio.alsa_front_device",
                                          "default",
-                                         "device used for stereo output",
+                                         _("device used for stereo output"),
                                          NULL,
                                          NULL,
                                          NULL);
   pcm_device = config->register_string(config,
                                          "audio.alsa_surround40_device",
                                          "surround40",
-                                         "device used for 4-channel output",
+                                         _("device used for 4-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
   pcm_device = config->register_string(config,
                                          "audio.alsa_surround50_device",
                                          "surround51",
-                                         "device used for 5-channel output",
+                                         _("device used for 5-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
   pcm_device = config->register_string(config,
                                          "audio.alsa_surround51_device",
                                          "surround51",
-                                         "device used for 5.1-channel output",
+                                         _("device used for 5.1-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
   pcm_device = config->register_string(config,
                                          "audio.alsa_a52_device",
                                          "iec958:AES0=0x6,AES1=0x82,AES2=0x0,AES3=0x2",
-                                         "device used for 5.1-channel output",
+                                         _("device used for 5.1-channel output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -929,7 +960,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   pcm_device = config->register_string(config,
                                          "audio.alsa_default_device",
                                          "default",
-                                         "device used for mono output",
+                                         _("device used for mono output"),
                                          NULL,
                                          NULL,
                                          NULL);
@@ -940,7 +971,6 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   /*
    * open that device
    */
-  
   err=snd_pcm_open(&this->audio_fd, pcm_device, SND_PCM_STREAM_PLAYBACK, 0);
   if(err <0 ) {
     xlerror("snd_pcm_open() failed: %d", err); 
@@ -962,8 +992,14 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
     printf ("audio_alsa_out: access type not available");
     return NULL;
   }
+
   this->capabilities = 0;
+
   printf ("audio_alsa_out : supported modes are ");
+  if (!(snd_pcm_hw_params_test_format(this->audio_fd, params, SND_PCM_FORMAT_U8))) {
+    this->capabilities |= AO_CAP_8BITS;
+    printf ("8bit ");
+  }
   if (!(snd_pcm_hw_params_test_channels(this->audio_fd, params, 1))) {
     this->capabilities |= AO_CAP_MODE_MONO;
     printf ("mono ");
@@ -976,7 +1012,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
         config->register_bool (config,
                                "audio.four_channel",
                                0,
-                               "used to inform xine about what the sound card can do",
+                               _("used to inform xine about what the sound card can do"),
                                NULL,
                                NULL,
                                NULL) ) {
@@ -989,7 +1025,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
         config->register_bool (config,
                                "audio.five_channel",
                                0,
-                               "used to inform xine about what the sound card can do",
+                               _("used to inform xine about what the sound card can do"),
                                NULL,
                                NULL,
                                NULL) ) {
@@ -1002,7 +1038,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
         config->register_bool (config,
                                "audio.five_lfe_channel",
                                0,
-                               "used to inform xine about what the sound card can do",
+                               _("used to inform xine about what the sound card can do"),
                                NULL,
                                NULL,
                                NULL) ) {
@@ -1020,7 +1056,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   if (config->register_bool (config,
                                "audio.a52_pass_through",
                                0,
-                               "used to inform xine about what the sound card can do",
+                               _("used to inform xine about what the sound card can do"),
                                NULL,
                                NULL,
                                NULL) ) {
@@ -1039,14 +1075,14 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   this->mixer.name = config->register_string(config,
                                              "audio.alsa_mixer_name",
                                              "PCM",
-                                             "alsa mixer device",
+                                             _("alsa mixer device"),
                                              NULL,
                                              NULL,
                                              NULL);
 
   pthread_mutex_init(&this->mixer.mutex, NULL);
   ao_alsa_mixer_init(&this->ao_driver);
-   
+
   this->ao_driver.get_capabilities    = ao_alsa_get_capabilities;
   this->ao_driver.get_property        = ao_alsa_get_property;
   this->ao_driver.set_property        = ao_alsa_set_property;
@@ -1066,11 +1102,13 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 static ao_info_t ao_info_alsa9 = {
   AO_OUT_ALSA_IFACE_VERSION,
   "alsa09",
-  "xine audio output plugin using alsa-compliant audio devices/drivers",
+  NULL,
   11
 };
 
 ao_info_t *get_audio_out_plugin_info() {
+
+  ao_info_alsa9.description = _("xine audio output plugin using alsa-compliant audio devices/drivers");
   return &ao_info_alsa9;
 }
 
