@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_oss_out.c,v 1.62 2002/04/28 16:11:12 guenter Exp $
+ * $Id: audio_oss_out.c,v 1.71 2002/07/05 20:54:37 miguelfreitas Exp $
  *
  * 20-8-2001 First implementation of Audio sync and Audio driver separation.
  * Copyright (C) 2001 James Courtier-Dutton James@superbug.demon.co.uk
@@ -47,14 +47,18 @@
 #include <math.h>
 #include <unistd.h>
 #if defined(__OpenBSD__)
-#include <soundcard.h>
-#elif defined(__FreeBSD__)
-#include <machine/soundcard.h>
+# include <soundcard.h>
+#elif defined (__FreeBSD__)
+#  if __FreeBSD__ < 4
+#   include <machine/soundcard.h>
+#  else
+#   include <sys/soundcard.h>
+#  endif
 #else
-#if defined(__linux__)
-#include <linux/config.h> /* Check for DEVFS */
-#endif
-#include <sys/soundcard.h>
+# if defined(__linux__)
+#  include <linux/config.h> /* Check for DEVFS */
+# endif
+# include <sys/soundcard.h>
 #endif
 #include <sys/ioctl.h>
 #include <inttypes.h>
@@ -399,6 +403,7 @@ static int ao_oss_write(ao_driver_t *this_gen,
 
   oss_driver_t *this = (oss_driver_t *) this_gen;
 
+  //printf ("audio_oss_out: ao_oss_write()\n");
   if (this->sync_method == OSS_SYNC_SOFTSYNC) {
     int            simulated_bytes_in_buffer, frames ;
     struct timeval tv;
@@ -442,10 +447,6 @@ static uint32_t ao_oss_get_capabilities (ao_driver_t *this_gen) {
 static void ao_oss_exit(ao_driver_t *this_gen) {
 
   oss_driver_t    *this   = (oss_driver_t *) this_gen;
-  config_values_t *config = this->config;
-
-
-  config->update_num (config, "audio.mixer_volume", this->mixer.volume);
 
   if (this->audio_fd != -1)
     close(this->audio_fd);
@@ -582,17 +583,28 @@ static int ao_oss_set_property (ao_driver_t *this_gen, int property, int value)
 }
 
 static int ao_oss_ctrl(ao_driver_t *this_gen, int cmd, ...) {
-  /*oss_driver_t *this = (oss_driver_t *) this_gen;*/
+  oss_driver_t *this = (oss_driver_t *) this_gen;
 
   switch (cmd) {
 
   case AO_CTRL_PLAY_PAUSE:
+    printf ("audio_oss_out: AO_CTRL_PLAY_PAUSE\n");
+    if (this->sync_method != OSS_SYNC_SOFTSYNC)
+      ioctl(this->audio_fd, SNDCTL_DSP_RESET, NULL);
+    /*  Uncomment the following lines if RESET causes problems
+     *  ao_oss_close(this_gen);
+     *  ao_oss_open(this_gen, this->bits_per_sample, this->input_sample_rate, this->mode);
+     */
     break;
 
   case AO_CTRL_PLAY_RESUME:
+    printf ("audio_oss_out: AO_CTRL_PLAY_RESUME\n");
     break;
 
   case AO_CTRL_FLUSH_BUFFERS:
+    printf ("audio_oss_out: AO_CTRL_FLUSH_BUFFERS\n");
+    if (this->sync_method != OSS_SYNC_SOFTSYNC)
+      ioctl(this->audio_fd, SNDCTL_DSP_RESET, NULL);
     break;
   }
 
@@ -612,7 +624,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   int              rate ;
   int              devnum;
   int              audio_fd;
-  int              num_channels, status, arg;
+  int              num_channels, bits, status, arg;
   static char     *sync_methods[] = {"auto", "getodelay", "getoptr", "softsync", "probebuffer", NULL};
   
   this = (oss_driver_t *) malloc (sizeof (oss_driver_t));
@@ -625,7 +637,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
   best_rate = 0;
   devnum = config->register_num (config, "audio.oss_device_num", -1,
-				 "/dev/dsp# device to use for oss output, -1 => auto_detect",
+				 _("/dev/dsp# device to use for oss output, -1 => auto_detect"),
 				 NULL, NULL, NULL);
 
   if (devnum >= 0) {
@@ -693,32 +705,24 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
   this->sync_method = config->register_enum (config, "audio.oss_sync_method", OSS_SYNC_AUTO_DETECT,
 					     sync_methods, 
-					     "A/V sync method to use by OSS, depends on driver/hardware",
+					     _("A/V sync method to use by OSS, depends on driver/hardware"),
 					     NULL, NULL, NULL);
 
   if (this->sync_method == OSS_SYNC_AUTO_DETECT) {
 
-    ioctl (audio_fd, SNDCTL_DSP_GETCAPS, &caps);
-    
-    if ((caps & DSP_CAP_REALTIME) > 0) {
+    count_info info;
 
-      count_info info;
+    /*
+     * check if SNDCTL_DSP_GETODELAY works. if so, using it is preferred.
+     */
 
-      /*
-       * check if SNDCTL_DSP_GETODELAY works. if so, using it is preferred.
-       */
-
-      if (ioctl(audio_fd, SNDCTL_DSP_GETODELAY, &info) != -1) {
-	printf("audio_oss_out: using SNDCTL_DSP_GETODELAY\n");
-	this->sync_method = OSS_SYNC_GETODELAY;
-      } else if (ioctl(audio_fd, SNDCTL_DSP_GETOPTR, &info) != -1) {
-	printf("audio_oss_out: using SNDCTL_DSP_GETOPTR\n");
-	this->sync_method = OSS_SYNC_GETOPTR;
-      } else 
-	this->sync_method = OSS_SYNC_SOFTSYNC;
+    if (ioctl(audio_fd, SNDCTL_DSP_GETODELAY, &info) != -1) {
+      printf("audio_oss_out: using SNDCTL_DSP_GETODELAY\n");
+      this->sync_method = OSS_SYNC_GETODELAY;
+    } else if (ioctl(audio_fd, SNDCTL_DSP_GETOPTR, &info) != -1) {
+      printf("audio_oss_out: using SNDCTL_DSP_GETOPTR\n");
+      this->sync_method = OSS_SYNC_GETOPTR;
     } else {
-      printf ("audio_oss_out: realtime check: *FAILED*\n");
-      
       this->sync_method = OSS_SYNC_SOFTSYNC;
     }
   }
@@ -768,12 +772,16 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 
   this->latency = config->register_range (config, "audio.oss_latency", 0,
 					  -3000, 3000, 
-					  "Adjust a/v sync for OSS softsync",
-					  "Use this to manually adjust a/v sync if you're using softsync",
+					  _("Adjust a/v sync for OSS softsync"),
+					  _("Use this to manually adjust a/v sync if you're using softsync"),
 					  NULL, NULL);
   
   this->capabilities = 0;
-
+  
+  bits = 8;
+  if( ioctl(audio_fd, SNDCTL_DSP_SAMPLESIZE,&bits) != -1 )
+    this->capabilities |= AO_CAP_8BITS;
+    
   printf ("audio_oss_out : supported modes are ");
   num_channels = 1; 
   status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels); 
@@ -791,7 +799,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels); 
   if ( (status != -1) && (num_channels==4) ) {
     if (config->register_bool (config, "audio.four_channel", 0,
-			       "Enable 4.0 channel analog surround output",
+			       _("Enable 4.0 channel analog surround output"),
 			       NULL, NULL, NULL)) {
       this->capabilities |= AO_CAP_MODE_4CHANNEL;
       printf ("4-channel ");
@@ -802,7 +810,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels); 
   if ( (status != -1) && (num_channels==5) ) {
     if (config->register_bool (config, "audio.five_channel", 0,
-			       "Enable 5.0 channel analog surround output",
+			       _("Enable 5.0 channel analog surround output"),
 			       NULL, NULL, NULL)) {
       this->capabilities |= AO_CAP_MODE_5CHANNEL;
       printf ("5-channel ");
@@ -813,7 +821,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   status = ioctl(audio_fd, SNDCTL_DSP_CHANNELS, &num_channels); 
   if ( (status != -1) && (num_channels==6) ) {
     if (config->register_bool (config, "audio.five_lfe_channel", 0,
-			       "Enable 5.1 channel analog surround output",
+			       _("Enable 5.1 channel analog surround output"),
 			       NULL, NULL, NULL)) {
       this->capabilities |= AO_CAP_MODE_5_1CHANNEL;
       printf ("5.1-channel ");
@@ -824,7 +832,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
   ioctl(audio_fd,SNDCTL_DSP_GETFMTS,&caps);
   if (caps & AFMT_AC3) {
     if (config->register_bool (config, "audio.a52_pass_through", 0,
-			       "Enable A52 / AC5 digital audio output via spdif",
+			       _("Enable A52 / AC5 digital audio output via spdif"),
 			       NULL, NULL, NULL)) {
       this->capabilities |= AO_CAP_MODE_A52;
       this->capabilities |= AO_CAP_MODE_AC5;
@@ -840,7 +848,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
    */
 
   this->mixer.name = config->register_string(config, "audio.mixer_name", "/dev/mixer",
-					     "oss mixer device", NULL, NULL, NULL);
+					     _("oss mixer device"), NULL, NULL, NULL);
   {
     int mixer_fd;
     int audio_devs;
@@ -879,18 +887,7 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
     
     this->mixer.mute = 0;
     this->mixer.volume = ao_oss_get_property (&this->ao_driver, this->mixer.prop);
-    if (config->register_bool (config, "audio.remember_volume", 0,
-			       "restore volume level at startup", 
-			       "if this not set, xine will not touch any mixer settings at startup",
-			       NULL, NULL)) {
-      this->mixer.volume = config->register_range (config,
-						   "audio.oss_mixer_volume",
-						   50, 0, 100, "Audio volume",
-						   NULL, NULL, NULL);
-      (void) ao_oss_set_property(&this->ao_driver,
-				 this->mixer.prop, this->mixer.volume);
-      
-    }
+
   }
   close (audio_fd);
 
@@ -918,11 +915,11 @@ ao_driver_t *init_audio_out_plugin (config_values_t *config) {
 static ao_info_t ao_info_oss = {
   AO_OUT_OSS_IFACE_VERSION,
   "oss",
-  "xine audio output plugin using oss-compliant audio devices/drivers",
+  NULL,
   10
 };
 
 ao_info_t *get_audio_out_plugin_info() {
+  ao_info_oss.description = _("xine audio output plugin using oss-compliant audio devices/drivers");
   return &ao_info_oss;
 }
-
