@@ -1,27 +1,27 @@
 /*
  * Copyright (C) 2000-2002 the xine project
- * 
+ *
  * This file is part of xine, a free video player.
- * 
+ *
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * xine is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.35 2002/04/19 22:57:07 miguelfreitas Exp $
+ * $Id: demux_asf.c,v 1.50 2002/07/14 22:27:25 miguelfreitas Exp $
  *
  * demultiplexer for asf streams
  *
- * based on ffmpeg's 
+ * based on ffmpeg's
  * ASF compatible encoder and decoder.
  * Copyright (c) 2000, 2001 Gerard Lantau.
  */
@@ -41,12 +41,6 @@
 #include "demux.h"
 #include "xineutils.h"
 
-#define	WINE_TYPEDEFS_ONLY
-#include "libw32dll/wine/avifmt.h"
-#include "libw32dll/wine/windef.h"
-#include "libw32dll/wine/vfw.h"
-#include "libw32dll/wine/mmreg.h"
-
 /*
 #define LOG
 */
@@ -60,7 +54,7 @@
 
 #define DEFRAG_BUFSIZE    65536
 
-#define VALID_ENDS    "asf,wmv"
+#define VALID_ENDS    "asf,wmv,wma"
 
 typedef struct {
   int               num;
@@ -70,11 +64,11 @@ typedef struct {
   int               timestamp;
   int               ts_per_kbyte;
   int               defrag;
-    
+
   uint32_t          buf_type;
   int               stream_id;
   fifo_buffer_t    *fifo;
-  
+
   uint8_t          *buffer;
 } asf_stream_t;
 
@@ -95,16 +89,16 @@ typedef struct demux_asf_s {
   int               seqno;
   int               packet_size;
   int               packet_flags;
-  
+
   asf_stream_t      streams[MAX_NUM_STREAMS];
   int               num_streams;
   int               num_audio_streams;
   int               num_video_streams;
 
-  uint16_t          wavex[128];
+  uint16_t          wavex[1024];
   int               wavex_size;
 
-  uint16_t          bih[128];
+  uint16_t          bih[1024];
   int               bih_size;
 
   char              title[512];
@@ -121,14 +115,15 @@ typedef struct demux_asf_s {
 
   int64_t           last_video_pts;
   int32_t           frame_duration;
-  
+
   /* only for reading */
   int               packet_padsize;
   int               nb_frames;
   int               segtype;
   int               frame;
-  
+
   pthread_t         thread;
+  int                  thread_running;
   pthread_mutex_t   mutex;
 
   int               status;
@@ -136,7 +131,7 @@ typedef struct demux_asf_s {
   int               send_end_buffers;
 
   int               send_discontinuity;
-  
+
   /* byte reordering from audio streams */
   int               reorder_h;
   int               reorder_w;
@@ -173,7 +168,7 @@ static const GUID audio_conceal_none = {
   0x49f1a440, 0x4ece, 0x11d0, { 0xa3, 0xac, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 },
 };
 
-static const GUID audio_conceal_interleave = { 
+static const GUID audio_conceal_interleave = {
   0xbfc3cd50, 0x618f, 0x11cf, {0x8b, 0xb2, 0x00, 0xaa, 0x00, 0xb4, 0xe2, 0x20} };
 
 static const GUID video_stream = {
@@ -211,7 +206,7 @@ static const GUID head1_guid = {
 static const GUID head2_guid = {
   0xabd3d211, 0xa9ba, 0x11cf, { 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 },
 };
-    
+
 /* I am not a number !!! This GUID is the one found on the PC used to
    generate the stream */
 static const GUID my_guid = {
@@ -226,7 +221,7 @@ static uint8_t get_byte (demux_asf_t *this) {
   i = this->input->read (this->input, &buf, 1);
 
   /* printf ("%02x ", buf); */
-  
+
   if (i != 1) {
 #ifdef LOG
     printf ("demux_asf: end of data\n");
@@ -294,7 +289,7 @@ static uint64_t get_le64 (demux_asf_t *this) {
 }
 
 static void get_guid (demux_asf_t *this, GUID *g) {
-  int i; 
+  int i;
 
   g->v1 = get_le32(this);
   g->v2 = get_le16(this);
@@ -323,7 +318,7 @@ static void get_str16_nolen(demux_asf_t *this, int len,
 static void asf_send_audio_header (demux_asf_t *this, int stream_id) {
 
   buf_element_t *buf;
-  WAVEFORMATEX  *wavex = (WAVEFORMATEX *) this->wavex ;
+  xine_waveformatex  *wavex = (xine_waveformatex *) this->wavex ;
 
   if (!this->audio_fifo)
     return;
@@ -333,6 +328,7 @@ static void asf_send_audio_header (demux_asf_t *this, int stream_id) {
     
   if ( !this->streams[this->num_streams].buf_type ) {
     printf ("demux_asf: unknown audio type 0x%x\n", wavex->wFormatTag);
+    xine_report_codec( this->xine, XINE_CODEC_AUDIO, wavex->wFormatTag, 0, 0);
     this->streams[this->num_streams].buf_type     = BUF_CONTROL_NOP;
   } else
     xine_log (this->xine, XINE_LOG_FORMAT,
@@ -353,7 +349,6 @@ static void asf_send_audio_header (demux_asf_t *this, int stream_id) {
     this->streams[this->num_streams].defrag = 0;
   
   buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-  buf->content = buf->mem;
   memcpy (buf->content, this->wavex, this->wavex_size);
 
 #ifdef LOG
@@ -379,15 +374,17 @@ static unsigned long str2ulong(unsigned char *str) {
 static void asf_send_video_header (demux_asf_t *this, int stream_id) {
 
   buf_element_t    *buf;
-  BITMAPINFOHEADER *bih = (BITMAPINFOHEADER *) this->bih;
+  xine_bmiheader   *bih = (xine_bmiheader *) this->bih;
 
   this->streams[this->num_streams].buf_type = 
-    fourcc_to_buf_video((void*)&bih->biCompression);
+    fourcc_to_buf_video(bih->biCompression);
     
   if( !this->streams[this->num_streams].buf_type ) {
     printf ("demux_asf: unknown video format %.4s\n",
 	    (char*)&bih->biCompression);
     
+    xine_report_codec( this->xine, XINE_CODEC_VIDEO, bih->biCompression, 0, 0);
+
     this->status = DEMUX_FINISHED;
     return;
   }
@@ -403,7 +400,6 @@ static void asf_send_video_header (demux_asf_t *this, int stream_id) {
 	    buf_video_name(this->streams[this->num_streams].buf_type));
 
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->content = buf->mem;
   buf->decoder_flags   = BUF_FLAG_HEADER;
   buf->decoder_info[1] = 3000; /* FIXME ? */
   memcpy (buf->content, &this->bih, this->bih_size);
@@ -494,6 +490,7 @@ static int asf_read_header (demux_asf_t *this) {
         uint8_t buffer[6];
 
 	this->input->read (this->input, (uint8_t *) this->wavex, total_size);
+	xine_waveformatex_le2me( (xine_waveformatex *) this->wavex );
 	
         /*
 	printf ("total size: %d bytes\n", total_size);
@@ -515,23 +512,31 @@ static int asf_read_header (demux_asf_t *this) {
         }
 
 	this->wavex_size = total_size; /* 18 + this->wavex[8]; */
-     
+
 	asf_send_audio_header (this, stream_id);
 #ifdef LOG
-        printf ("found a_stream id=%d \n", stream_id);
+        printf ("demux_asf: found a_stream id=%d \n", stream_id);
 #endif
       } else {
 
+	int i;
+
 	get_le32(this); /* width */
 	get_le32(this); /* height */
 	get_byte(this);
-	this->bih_size = get_le16(this); /* size */
+	
+	i = get_le16(this); /* size */
+	if( i > 0 && i < sizeof(this->bih) ) {
+	  this->bih_size = i;
+	  this->input->read (this->input, (uint8_t *) this->bih, this->bih_size);
+	  xine_bmiheader_le2me( (xine_bmiheader *) this->bih );
 
-	this->input->read (this->input, (uint8_t *) this->bih, this->bih_size);
-
-	asf_send_video_header (this, stream_id);
+	  asf_send_video_header (this, stream_id);
+	}
+	else
+	  printf ("demux_asf: invalid bih_size received (%d), v_stream ignored.\n", i );
 #ifdef LOG
-	printf ("found v_stream id=%d \n", stream_id);
+	printf ("demux_asf: found v_stream id=%d \n", stream_id);
 #endif
       }
       pos2 = this->input->get_current_pos (this->input);
@@ -670,25 +675,12 @@ static void hexdump (unsigned char *data, int len, xine_t *xine) {
 
 static void asf_send_discontinuity (demux_asf_t *this, int64_t pts) {
 
-  buf_element_t  *buf;
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type          = BUF_CONTROL_NEWPTS;
-  buf->disc_off      = pts;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type          = BUF_CONTROL_NEWPTS;
-    buf->disc_off      = pts;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
-
+  xine_demux_control_newpts(this->xine, pts, 0);
 }
 
 
 static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream, 
-				      int frag_offset, int seq, int timestamp, 
+				      int frag_offset, int seq, int timestamp,
 				      int frag_len, int payload_size) {
 
   buf_element_t *buf;
@@ -714,7 +706,7 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
       }
     }
   }
-  
+
   
   while( frag_len ) {
     if ( frag_len < stream->fifo->buffer_pool_buf_size )
@@ -723,7 +715,6 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
       bufsize = stream->fifo->buffer_pool_buf_size;
   
     buf = stream->fifo->buffer_pool_alloc (stream->fifo);
-    buf->content = buf->mem;
     this->input->read (this->input, buf->content, bufsize);
 
     if (stream->fifo == this->video_fifo) {
@@ -780,8 +771,7 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
 
       stream->frag_offset = 0;
 
-    } else 
-      buf->decoder_flags   = 0;
+    }
 
     if( !this->keyframe_found )
       buf->decoder_flags   |= BUF_FLAG_PREVIEW;
@@ -800,7 +790,7 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
     printf("asf_send_buffer seq=%d frag_offset=%d frag_len=%d\n",
     seq, frag_offset, frag_len );
   */
-  
+
   if (stream->frag_offset == 0) {
     /* new packet */
     stream->seq = seq;
@@ -828,7 +818,6 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
 	      bufsize = stream->fifo->buffer_pool_buf_size;
 
 	    buf = stream->fifo->buffer_pool_alloc (stream->fifo);
-	    buf->content = buf->mem;
 	    xine_fast_memcpy (buf->content, p, bufsize);
 
 	    if (stream->fifo == this->video_fifo) {
@@ -851,15 +840,13 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
 
 	    buf->type       = stream->buf_type;
 	    buf->size       = bufsize;
-          
+
 	    stream->frag_offset -= bufsize;
 	    p+=bufsize;
-          
+
 	    /* test if whole packet read */
 	    if ( !stream->frag_offset )
-	      buf->decoder_flags   = BUF_FLAG_FRAME_END;
-	    else 
-	      buf->decoder_flags = 0;
+	      buf->decoder_flags   |= BUF_FLAG_FRAME_END;
 	  
 	    if( !this->keyframe_found )
 	      buf->decoder_flags   |= BUF_FLAG_PREVIEW;
@@ -879,7 +866,7 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
       }
     }
   }
-      
+
   
   if( frag_offset ) {
     if( timestamp )
@@ -1047,7 +1034,7 @@ static void asf_read_packet(demux_asf_t *this) {
           asf_send_buffer_defrag (this, stream, 0, seq, timestamp, 
 				  object_length, object_length);
         else
-          asf_send_buffer_nodefrag (this, stream, 0, seq, timestamp, 
+          asf_send_buffer_nodefrag (this, stream, 0, seq, timestamp,
 				    object_length, object_length);
       }
       else {
@@ -1120,26 +1107,35 @@ static void asf_read_packet(demux_asf_t *this) {
  */
 
 static void *demux_asf_loop (void *this_gen) {
-  buf_element_t *buf;
-
+  
   demux_asf_t *this = (demux_asf_t *) this_gen;
 
   printf ("demux_asf: demux loop starting...\n"); 
 
-  this->send_end_buffers = 1;
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
+
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      asf_read_packet (this);
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->video_fifo->size(this->video_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    asf_read_packet (this);
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
 
   /*
     printf ("demux_asf: demux loop finished (status: %d)\n",
@@ -1149,20 +1145,10 @@ static void *demux_asf_loop (void *this_gen) {
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-    
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
-
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
-  
+
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
 
   pthread_exit(NULL);
@@ -1174,19 +1160,18 @@ static void demux_asf_close (demux_plugin_t *this_gen) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
   free (this);
-  
+
 }
 
 static void demux_asf_stop (demux_plugin_t *this_gen) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
-  buf_element_t *buf;
   int i;
   void *p;
-  
+
   pthread_mutex_lock( &this->mutex );
 
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_asf: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -1198,20 +1183,10 @@ static void demux_asf_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; 
-  this->video_fifo->put (this->video_fifo, buf);
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
-  
   for (i=0; i<this->num_streams; i++) {
     if( this->streams[i].buffer ) {
       free( this->streams[i].buffer );
@@ -1223,42 +1198,31 @@ static void demux_asf_stop (demux_plugin_t *this_gen) {
 static int demux_asf_get_status (demux_plugin_t *this_gen) {
   demux_asf_t *this = (demux_asf_t *) this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
-static void demux_asf_start (demux_plugin_t *this_gen,
-			     fifo_buffer_t *video_fifo, 
+static int demux_asf_start (demux_plugin_t *this_gen,
+			     fifo_buffer_t *video_fifo,
 			     fifo_buffer_t *audio_fifo,
 			     off_t start_pos, int start_time) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
-  buf_element_t *buf;
   int err;
-  int starting;
-  
+  int status;
+
   pthread_mutex_lock( &this->mutex );
 
-  starting = (this->status != DEMUX_OK);
   this->status = DEMUX_OK;
-  
-  if( starting ) {
+
+  if( !this->thread_running ) {
     this->audio_fifo   = audio_fifo;
     this->video_fifo   = video_fifo;
-    
-    /* 
+
+    /*
      * send start buffer
      */
+    xine_demux_control_start(this->xine);
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
-  
     /*
      * initialize asf engine
      */
@@ -1269,14 +1233,14 @@ static void demux_asf_start (demux_plugin_t *this_gen,
     this->packet_size              = 0;
     this->seqno                    = 0;
     this->frame_duration           = 3000;
-  
+
     if (this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE)
       this->input->seek (this->input, 0, SEEK_SET);
-                                       
+
     if (!asf_read_header (this)) {
-    
+
       this->status = DEMUX_FINISHED;
-    } else { 
+    } else {
       this->header_size = this->input->get_current_pos (this->input);
 
       xine_log (this->xine, XINE_LOG_FORMAT,
@@ -1290,16 +1254,16 @@ static void demux_asf_start (demux_plugin_t *this_gen,
     }
   }
   else {
-    xine_flush_engine(this->xine);
+    xine_demux_flush_engine(this->xine);
   }
-  
+
   if( this->status == DEMUX_OK ) {
     /*
      * seek to start position
      */
     this->send_discontinuity       = 2;
     this->last_video_pts           = 0;
-    this->keyframe_found           = 0;
+    this->keyframe_found           = (this->num_video_streams==0);
 
     if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
 
@@ -1316,29 +1280,39 @@ static void demux_asf_start (demux_plugin_t *this_gen,
      * now start demuxing
      */
 
-    if( starting ) {
+    if( !this->thread_running ) {
+      this->send_end_buffers = 1;
+      this->thread_running = 1;
       if ((err = pthread_create (&this->thread,
 			       NULL, demux_asf_loop, this)) != 0) {
         printf ("demux_asf: can't create new thread (%s)\n",
 	      strerror(err));
-        exit (1);
+        abort();
       }
     }
   }
-  
+
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
-  
+  return status;
+
+/*
   if( !starting && this->status != DEMUX_OK ) {
     void *p;
     pthread_join (this->thread, &p);
   }
+*/
+
 }
 
-static void demux_asf_seek (demux_plugin_t *this_gen,
+static int demux_asf_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_asf_t *this = (demux_asf_t *) this_gen;
 
-	demux_asf_start (this_gen, this->video_fifo, this->audio_fifo, 
+	return demux_asf_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -1383,10 +1357,10 @@ static int demux_asf_open(demux_plugin_t *this_gen,
     
     if(!ending)
       return DEMUX_CANNOT_HANDLE;
-    
+
     xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							    "mrl.ends_asf", VALID_ENDS,
-							    "valid mrls ending for asf demuxer",
+							    _("valid mrls ending for asf demuxer"),
 							    NULL, NULL, NULL)));
     while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
       
@@ -1424,10 +1398,10 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_asf_t     *this;
 
-  if (iface != 7) {
-    printf ("demux_asf: plugin doesn't support plugin API version %d.\n"
-	    "           this means there's a version mismatch between xine and this "
-	    "           demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_asf: plugin doesn't support plugin API version %d.\n"
+	      "           this means there's a version mismatch between xine and this "
+	      "           demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
     return NULL;
   }
@@ -1438,8 +1412,8 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   (void*) this->config->register_string(this->config,
 					"mrl.ends_asf", VALID_ENDS,
-					"valid mrls ending for asf demuxer",
-					NULL, NULL, NULL);    
+					_("valid mrls ending for asf demuxer"),
+					NULL, NULL, NULL);
   
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
   this->demux_plugin.open              = demux_asf_open;
