@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_cda.c,v 1.12 2002/04/23 13:30:42 esnel Exp $
+ * $Id: demux_cda.c,v 1.21 2002/07/05 17:31:59 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -52,6 +52,7 @@ typedef struct {
   input_plugin_t      *input;
 
   pthread_t            thread;
+  int                  thread_running;
   pthread_mutex_t      mutex;
   
   off_t                start;
@@ -89,42 +90,40 @@ static int demux_cda_next (demux_cda_t *this) {
  */
 static void *demux_cda_loop (void *this_gen) {
   demux_cda_t    *this = (demux_cda_t *) this_gen;
-  buf_element_t  *buf;
 
-  this->send_end_buffers = 1;
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    xine_usec_sleep(100000);
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      xine_usec_sleep(100000);
+      if (!demux_cda_next(this))
+        this->status = DEMUX_FINISHED;
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->audio_fifo->size(this->audio_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
 
-    if (!demux_cda_next(this))
-      this->status = DEMUX_FINISHED;
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
-  
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
   
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
 
   pthread_exit(NULL);
@@ -135,12 +134,11 @@ static void *demux_cda_loop (void *this_gen) {
  */
 static void demux_cda_stop (demux_plugin_t *this_gen) {
   demux_cda_t    *this = (demux_cda_t *) this_gen;
-  buf_element_t  *buf;
   void           *p;
   
   pthread_mutex_lock( &this->mutex );
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_cda: stop...ignored\n");
     return;
   }
@@ -154,20 +152,9 @@ static void demux_cda_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_flush_engine(this->xine);
 
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 /*
@@ -176,19 +163,19 @@ static void demux_cda_stop (demux_plugin_t *this_gen) {
 static int demux_cda_get_status (demux_plugin_t *this_gen) {
   demux_cda_t *this = (demux_cda_t *) this_gen;
   
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
 /*
  *
  */
-static void demux_cda_start (demux_plugin_t *this_gen,
+static int demux_cda_start (demux_plugin_t *this_gen,
 			     fifo_buffer_t *video_fifo, 
 			     fifo_buffer_t *audio_fifo,
 			     off_t start_pos, int start_time) {
   demux_cda_t    *this = (demux_cda_t *) this_gen;
-  buf_element_t  *buf;
   int             err;
+  int status;
 
   pthread_mutex_lock( &this->mutex );
 
@@ -196,19 +183,11 @@ static void demux_cda_start (demux_plugin_t *this_gen,
   
   this->blocksize  = this->input->get_blocksize(this->input);
 
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     this->video_fifo = video_fifo;
     this->audio_fifo = audio_fifo;
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_start(this->xine);
   }
   
   /*
@@ -216,24 +195,32 @@ static void demux_cda_start (demux_plugin_t *this_gen,
    */
   this->input->seek(this->input, this->start, SEEK_SET);
 
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     
     this->status = DEMUX_OK;
+    this->send_end_buffers = 1;
+    this->thread_running = 1;
     if ((err = pthread_create (&this->thread,
 			       NULL, demux_cda_loop, this)) != 0) {
       printf ("demux_cda: can't create new thread (%s)\n", strerror(err));
-      exit(1);
+      abort();
     }      
   }
+
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
 
-static void demux_cda_seek (demux_plugin_t *this_gen,
+static int demux_cda_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_cda_t *this = (demux_cda_t *) this_gen;
 
-	demux_cda_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_cda_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -306,10 +293,10 @@ static int demux_cda_get_stream_length (demux_plugin_t *this_gen) {
 demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   demux_cda_t *this;
   
-  if (iface != 7) {
-    printf ("demux_cda: plugin doesn't support plugin API version %d.\n"
-	    "           this means there's a version mismatch between xine and this "
-	    "           demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_cda: plugin doesn't support plugin API version %d.\n"
+	      "           this means there's a version mismatch between xine and this "
+	      "           demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
     return NULL;
   }
