@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_elem.c,v 1.39 2002/04/23 13:30:42 esnel Exp $
+ * $Id: demux_elem.c,v 1.49 2002/07/14 22:27:25 miguelfreitas Exp $
  *
  * demultiplexer for elementary mpeg streams
  * 
@@ -47,26 +47,27 @@
 
 typedef struct {  
 
-  demux_plugin_t   demux_plugin;
+  demux_plugin_t      demux_plugin;
 
-  xine_t          *xine;
+  xine_t              *xine;
 
-  config_values_t *config;
+  config_values_t     *config;
 
-  fifo_buffer_t   *video_fifo;
-  fifo_buffer_t   *audio_fifo;
+  fifo_buffer_t       *video_fifo;
+  fifo_buffer_t       *audio_fifo;
 
-  input_plugin_t  *input;
+  input_plugin_t      *input;
 
-  pthread_t        thread;
-  pthread_mutex_t  mutex;
+  pthread_t            thread;
+  int                  thread_running;
+  pthread_mutex_t      mutex;
 
-  int              blocksize;
-  int              status;
+  int                  blocksize;
+  int                  status;
   
-  int              send_end_buffers;
+  int                  send_end_buffers;
 
-  uint8_t          scratch[4096];
+  uint8_t              scratch[4096];
 } demux_mpeg_elem_t ;
 
 
@@ -86,8 +87,6 @@ static int demux_mpeg_elem_next (demux_mpeg_elem_t *this, int preview_mode) {
 
   if (preview_mode)
     buf->decoder_flags = BUF_FLAG_PREVIEW;
-  else
-    buf->decoder_flags = 0;
 
   buf->pts             = 0;
   /*buf->scr             = 0;*/
@@ -103,40 +102,42 @@ static int demux_mpeg_elem_next (demux_mpeg_elem_t *this, int preview_mode) {
  *
  */
 static void *demux_mpeg_elem_loop (void *this_gen) {
-  buf_element_t *buf = NULL;
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
 
-  this->send_end_buffers = 1;
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    if (!demux_mpeg_elem_next(this, 0))
-      this->status = DEMUX_FINISHED;
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      if (!demux_mpeg_elem_next(this, 0))
+        this->status = DEMUX_FINISHED;
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->video_fifo->size(this->video_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
 
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-    
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
+
+  printf ("demux_elem: demux loop finished.\n");
+
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
 
   pthread_exit(NULL);
@@ -148,31 +149,25 @@ static void *demux_mpeg_elem_loop (void *this_gen) {
 static void demux_mpeg_elem_stop (demux_plugin_t *this_gen) {
 
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
-  buf_element_t     *buf = NULL;
   void *p;
   
   pthread_mutex_lock( &this->mutex );
 
+  if (!this->thread_running) {
+    printf ("demux_elem: stop...ignored\n");
+    pthread_mutex_unlock( &this->mutex );
+    return;
+  }
+
   this->send_end_buffers = 0;
   this->status = DEMUX_FINISHED;
 
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; 
-
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; 
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 /*
@@ -181,28 +176,26 @@ static void demux_mpeg_elem_stop (demux_plugin_t *this_gen) {
 static int demux_mpeg_elem_get_status (demux_plugin_t *this_gen) {
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
 /*
  *
  */
-static void demux_mpeg_elem_start (demux_plugin_t *this_gen,
+static int demux_mpeg_elem_start (demux_plugin_t *this_gen,
 				   fifo_buffer_t *video_fifo, 
 				   fifo_buffer_t *audio_fifo,
 				   off_t start_pos, int start_time) {
 
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
-  buf_element_t *buf;
   int err;
-  int starting;
+  int status;
 
   pthread_mutex_lock( &this->mutex );
   
-  starting = (this->status != DEMUX_OK);
   this->status = DEMUX_OK;
 
-  if( starting ) {
+  if( !this->thread_running ) {
     this->video_fifo  = video_fifo;
     this->audio_fifo  = audio_fifo;
     
@@ -210,15 +203,7 @@ static void demux_mpeg_elem_start (demux_plugin_t *this_gen,
     if (!this->blocksize)
       this->blocksize = 2048;
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_start(this->xine);
 
     if((this->input->get_capabilities(this->input) & INPUT_CAP_PREVIEW) != 0) {
       int num_buffers = NUM_PREVIEW_BUFFERS;
@@ -231,9 +216,8 @@ static void demux_mpeg_elem_start (demux_plugin_t *this_gen,
         num_buffers--;
       }
     }
-  }
-  else {
-    xine_flush_engine(this->xine);
+  } else {
+    xine_demux_flush_engine(this->xine);
   }
   
   if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
@@ -248,22 +232,29 @@ static void demux_mpeg_elem_start (demux_plugin_t *this_gen,
    */
   this->status = DEMUX_OK;
 
-  if( starting ) {
+  if( !this->thread_running ) {
+    this->send_end_buffers = 1;
+    this->thread_running = 1;
     if ((err = pthread_create (&this->thread,
 			     NULL, demux_mpeg_elem_loop, this)) != 0) {
       printf ("demux_elem: can't create new thread (%s)\n",
 	      strerror(err));
-      exit (1);
+      abort();
     }
   }
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
-static void demux_mpeg_elem_seek (demux_plugin_t *this_gen,
+static int demux_mpeg_elem_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_mpeg_elem_t *this = (demux_mpeg_elem_t *) this_gen;
 
-	demux_mpeg_elem_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_mpeg_elem_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -323,7 +314,7 @@ static int demux_mpeg_elem_open(demux_plugin_t *this_gen,
     if(suffix) {
       xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							      "mrl.ends_elem", VALID_ENDS,
-							      "valid mrls ending for elementary demuxer",
+							      _("valid mrls ending for elementary demuxer"),
 							      NULL, NULL, NULL)));
       while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
 	
@@ -373,21 +364,21 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_mpeg_elem_t *this;
 
-  if (iface != 7) {
-    printf ("demux_elem: plugin doesn't support plugin API version %d.\n"
-	    "            this means there's a version mismatch between xine and this "
-	    "            demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_elem: plugin doesn't support plugin API version %d.\n"
+	      "            this means there's a version mismatch between xine and this "
+	      "            demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
     return NULL;
   }
   
-  this         = malloc (sizeof (demux_mpeg_elem_t));
+  this         = xine_xmalloc (sizeof (demux_mpeg_elem_t));
   this->config = xine->config;
   this->xine   = xine;
 
   (void*) this->config->register_string(this->config,
 					"mrl.ends_elem", VALID_ENDS,
-					"valid mrls ending for elementary demuxer",
+					_("valid mrls ending for elementary demuxer"),
 					NULL, NULL, NULL);    
 
   this->demux_plugin.interface_version = DEMUX_MPEG_ELEM_IFACE_VERSION;
