@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg_block.c,v 1.95 2002/04/27 16:33:24 miguelfreitas Exp $
+ * $Id: demux_mpeg_block.c,v 1.109 2002/07/19 03:03:36 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  *
@@ -48,6 +48,11 @@
 #define NUM_PREVIEW_BUFFERS   250
 #define DISC_TRESHOLD       90000
 
+#define WRAP_THRESHOLD     120000 
+#define PTS_AUDIO 0
+#define PTS_VIDEO 1
+
+
 /* redefine abs as macro to handle 64-bit diffs.
    i guess llabs may not be available everywhere */
 #define abs(x) ( (x<0) ? (-x) : (x) )
@@ -65,6 +70,7 @@ typedef struct demux_mpeg_block_s {
   input_plugin_t       *input;
 
   pthread_t             thread;
+  int                   thread_running;
   pthread_mutex_t       mutex;
 
   int                   status;
@@ -79,12 +85,70 @@ typedef struct demux_mpeg_block_s {
 
   uint8_t              *scratch, *scratch_base;
 
-  int64_t               last_scr;
-  int                   ignore_scr_discont;
   int64_t               nav_last_end_pts;
+  int64_t               nav_last_start_pts;
+  int64_t               last_pts[2];
+  int                   send_newpts;
+  int                   buf_flag_seek;
+
 } demux_mpeg_block_t ;
 
 
+/* OK, i think demux_mpeg_block discontinuity handling demands some
+   explanation:
+   
+   - The preferred discontinuity handling/detection for DVD is based on
+   NAV packets information. Most of the time it will provide us very
+   accurate and reliable information.
+   
+   - Has been shown that sometimes a DVD discontinuity may happen before
+   a new NAV packet arrives (seeking?). To avoid sending wrong PTS to
+   decoders we _used_ to check for SCR discontinuities. Unfortunately
+   some VCDs have very broken SCR values causing false triggering.
+   
+   - To fix the above problem (also note that VCDs don't have NAV
+   packets) we fallback to the same PTS-based wrap detection as used
+   in demux_mpeg. The only trick is to not send discontinuity information
+   if NAV packets have already done the job.
+   
+   [Miguel 02-05-2002]
+*/
+
+static void check_newpts( demux_mpeg_block_t *this, int64_t pts, int video )
+{
+  int64_t diff;
+  
+  diff = pts - this->last_pts[video];
+
+  if( pts && (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
+
+    /* check if pts is outside nav pts range. any stream without nav must enter here. */
+    if( pts > this->nav_last_end_pts || pts < this->nav_last_start_pts )
+    {
+#ifdef LOG
+      printf("demux_mpeg_block: discontinuity detected by pts wrap\n");
+#endif
+      if (this->buf_flag_seek) {
+        xine_demux_control_newpts(this->xine, pts, BUF_FLAG_SEEK);
+        this->buf_flag_seek = 0;
+      } else {
+        xine_demux_control_newpts(this->xine, pts, 0);
+      }
+      this->send_newpts = 0;
+    } else {
+#ifdef LOG
+      printf("demux_mpeg_block: no wrap detected\n" );
+#endif
+    }
+    
+    this->last_pts[1-video] = 0;
+  }
+  
+  if( pts )
+    this->last_pts[video] = pts;
+}
+
+
 static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_mode) {
 
   buf_element_t *buf = NULL;
@@ -94,7 +158,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
   int64_t        pts;
   uint32_t       packet_len;
   uint32_t       stream_id;
-  int64_t        scr = this->last_scr;
+  int64_t        scr = 0;
 
 #ifdef LOG
   printf ("demux_mpeg_block: read_block\n");
@@ -179,10 +243,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 #ifdef LOG
     printf ("demux_mpeg_block: type %08x != BUF_DEMUX_BLOCK\n", buf->type);
 #endif
-    /*
-    if (buf->type == BUF_CONTROL_NOP) 
-      this->send_disc = 1;
-    */
+
     return;
   }
 
@@ -191,7 +252,7 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->decoder_flags = BUF_FLAG_PREVIEW;
   else
     buf->decoder_flags = 0;
-
+    
   buf->input_pos = this->input->get_current_pos (this->input);
   buf->input_length = this->input->get_length (this->input);
 
@@ -249,8 +310,6 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       printf ("demux_mpeg_block: SCR=%lld\n", scr);
 #endif
 
-      /* buf->scr = scr; */
-
       /* mux_rate */
 
       if (!this->rate) {
@@ -324,70 +383,22 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     if (this->nav_last_end_pts != start_pts && !preview_mode) {
 
 #ifdef LOG
-      printf ("demux_mpeg_block: informing metronom about new start pts\n");
+      printf("demux_mpeg_block: discontinuity detected by nav packet\n" );
 #endif
-
-      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->type = BUF_CONTROL_NEWPTS;
-      buf->disc_off = start_pts;
-      this->video_fifo->put (this->video_fifo, buf);
-
-      if (this->audio_fifo) {
-	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-	buf->type = BUF_CONTROL_NEWPTS;
-	buf->disc_off = start_pts;
-	this->audio_fifo->put (this->audio_fifo, buf);
+      if (this->buf_flag_seek) {
+        xine_demux_control_newpts(this->xine, start_pts, BUF_FLAG_SEEK);
+        this->buf_flag_seek = 0;
+      } else {
+        xine_demux_control_newpts(this->xine, start_pts, 0);
       }
-
     }
-    
     this->nav_last_end_pts = end_pts;
-    this->ignore_scr_discont = 1;
-
-    /*
-    for (i=0; i<120; i++)
-      printf ("%02x ", p[i]);
-    printf ("\n");
-    */
+    this->nav_last_start_pts = start_pts;
+    this->send_newpts = 0;
 
     return ;
   }
 
-  /* discontinuity ? */
-  if (scr && !preview_mode)  {  
-    int64_t scr_diff = scr - this->last_scr;
-
-#ifdef LOG
-    printf ("demux_mpeg_block: scr %d last_scr %d diff %d\n",
-	    scr, this->last_scr, scr_diff);
-#endif
-
-    if ( (abs(scr_diff) > DISC_TRESHOLD) &&
-        !this->ignore_scr_discont) {
-      
-      buf_element_t *buf;
-
-#ifdef LOG
-      printf ("demux_mpeg_block: DISCONTINUITY/NEWPTS!\n");
-#endif
-
-      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->type = BUF_CONTROL_DISCONTINUITY;
-      buf->disc_off = scr_diff;
-      this->video_fifo->put (this->video_fifo, buf);
-
-      if (this->audio_fifo) {
-	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-	buf->type = BUF_CONTROL_DISCONTINUITY;
-	buf->disc_off = scr_diff;
-	this->audio_fifo->put (this->audio_fifo, buf);
-      }
-    }
-    this->ignore_scr_discont = 0;
-    this->last_scr = scr;
-  }
-
-
   if (bMpeg1) {
 
     if (stream_id == 0xBF) {
@@ -496,6 +507,9 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       buf->size      = packet_len-1;
       buf->type      = BUF_SPU_PACKAGE + spu_id;
       buf->pts       = pts;
+      if( !preview_mode )
+        check_newpts( this, pts, PTS_VIDEO );
+      
       buf->input_pos = this->input->get_current_pos(this->input);
       buf->input_length = this->input->get_length (this->input);
       
@@ -518,6 +532,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
         buf->type      = BUF_AUDIO_A52 + track;
       }
       buf->pts       = pts;
+      if( !preview_mode )
+        check_newpts( this, pts, PTS_AUDIO );
 
       buf->input_pos = this->input->get_current_pos(this->input);
       buf->input_length = this->input->get_length (this->input);
@@ -569,16 +585,19 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       }
       dynamic_range = p[6];
 
-      buf->decoder_info[1] = sample_rate;
-      buf->decoder_info[2] = bits_per_sample;
-      buf->decoder_info[3] = num_channels;
-
+      /* send lpcm config byte */
+      buf->decoder_flags |= BUF_FLAG_SPECIAL;
+      buf->decoder_info[1] = BUF_SPECIAL_LPCM_CONFIG;
+      buf->decoder_info[2] = p[5];
+      
       pcm_offset = 7;
 
       buf->content   = p+pcm_offset;
       buf->size      = packet_len-pcm_offset;
       buf->type      = BUF_AUDIO_LPCM_BE + track;
       buf->pts       = pts;
+      if( !preview_mode )
+        check_newpts( this, pts, PTS_AUDIO );
 
       buf->input_pos = this->input->get_current_pos(this->input);
       buf->input_length = this->input->get_length (this->input);
@@ -597,6 +616,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->size      = packet_len;
     buf->type      = BUF_VIDEO_MPEG;
     buf->pts       = pts;
+    if( !preview_mode )
+      check_newpts( this, pts, PTS_VIDEO );
 
     buf->input_pos = this->input->get_current_pos(this->input);
     buf->input_length = this->input->get_length (this->input);
@@ -614,6 +635,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->size      = packet_len;
     buf->type      = BUF_AUDIO_MPEG + track;
     buf->pts       = pts;
+    if( !preview_mode )
+        check_newpts( this, pts, PTS_AUDIO );
 
     buf->input_pos = this->input->get_current_pos(this->input);
     buf->input_length = this->input->get_length (this->input);
@@ -634,25 +657,33 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 
 static void *demux_mpeg_block_loop (void *this_gen) {
 
-  buf_element_t *buf = NULL;
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
 
   /* printf ("demux_mpeg_block: demux loop starting...\n"); */
+  pthread_mutex_lock( &this->mutex );
 
-  this->send_end_buffers = 1;
+  /* do-while needed to seek after demux finished */
+  do {
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    demux_mpeg_block_parse_pack(this, 0);
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      demux_mpeg_block_parse_pack(this, 0);
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->video_fifo->size(this->video_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
 
   /*
   printf ("demux_mpeg_block: demux loop finished (status: %d)\n",
@@ -662,20 +693,10 @@ static void *demux_mpeg_block_loop (void *this_gen) {
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM;
-    this->video_fifo->put (this->video_fifo, buf);
-    
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
-
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
 
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
   
   pthread_exit(NULL);
@@ -830,12 +851,11 @@ static void demux_mpeg_block_close (demux_plugin_t *this_gen) {
 static void demux_mpeg_block_stop (demux_plugin_t *this_gen) {
 
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
-  buf_element_t *buf;
   void *p;
 
   pthread_mutex_lock( &this->mutex );
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_mpeg_block: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -847,29 +867,41 @@ static void demux_mpeg_block_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
   
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER;
-
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; 
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 static int demux_mpeg_block_get_status (demux_plugin_t *this_gen) {
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
-static void demux_mpeg_block_start (demux_plugin_t *this_gen,
+static int demux_mpeg_detect_blocksize(demux_mpeg_block_t *this, 
+				       input_plugin_t *input)
+{
+  input->seek(input, 2048, SEEK_SET);
+  if (!input->read(input, this->scratch, 4))
+    return 0;
+
+  if (this->scratch[0] || this->scratch[1]
+      || (this->scratch[2] != 0x01) || (this->scratch[3] != 0xba)) {
+
+    input->seek(input, 2324, SEEK_SET);
+    if (!input->read(input, this->scratch, 4))
+      return 0;
+    if (this->scratch[0] || this->scratch[1] 
+        || (this->scratch[2] != 0x01) || (this->scratch[3] != 0xba)) 
+      return 0;
+     
+    return 2324;
+  } else
+    return 2048;
+}
+
+
+static int demux_mpeg_block_start (demux_plugin_t *this_gen,
 				    fifo_buffer_t *video_fifo,
 				    fifo_buffer_t *audio_fifo,
 				    off_t start_pos, int start_time) {
@@ -877,112 +909,124 @@ static void demux_mpeg_block_start (demux_plugin_t *this_gen,
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
   buf_element_t *buf;
   int err;
+  int status;
 
   pthread_mutex_lock( &this->mutex );
 
   this->video_fifo  = video_fifo;
   this->audio_fifo  = audio_fifo;
 
-  if( this->status != DEMUX_OK ) {
-    /* 
-     * send start buffer
+  if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
+    if (!this->blocksize)
+      this->blocksize = demux_mpeg_detect_blocksize( this, this->input );
+  }
+  
+  if( this->blocksize ) {
+  
+    if( !this->thread_running ) {
+      /* 
+       * send start buffer
+       */
+
+      xine_demux_control_start(this->xine);
+
+      if (!this->rate)
+        this->rate = demux_mpeg_block_estimate_rate (this);
+
+      if((this->input->get_capabilities(this->input) & INPUT_CAP_PREVIEW) != 0) {
+
+        int num_buffers = NUM_PREVIEW_BUFFERS;
+
+        this->input->seek (this->input, 0, SEEK_SET);
+
+        this->status = DEMUX_OK ;
+        while ( (num_buffers>0) && (this->status == DEMUX_OK) ) {
+   
+          demux_mpeg_block_parse_pack(this, 1);
+          num_buffers --;
+        }
+      }
+      this->status = DEMUX_FINISHED;
+    }
+  
+    if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
+    
+      if (start_pos) {
+        start_pos /= (off_t) this->blocksize;
+        start_pos *= (off_t) this->blocksize;
+  
+        this->input->seek (this->input, start_pos, SEEK_SET);
+      } else if (start_time) {
+        start_pos = start_time * this->rate * 50;
+        start_pos /= (off_t) this->blocksize;
+        start_pos *= (off_t) this->blocksize;
+
+        this->input->seek (this->input, start_pos, SEEK_SET);
+      } else
+        this->input->seek (this->input, 0, SEEK_SET);
+    }
+
+    /*
+     * query CLUT from the input plugin
      */
 
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
 
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
+    if ((this->input->get_capabilities(this->input) & INPUT_CAP_CLUT) &&
+        ((this->input->get_optional_data(this->input, buf->mem, INPUT_OPTIONAL_DATA_CLUT)
+         == INPUT_OPTIONAL_SUCCESS))) {
+      buf->type = BUF_SPU_CLUT;
+     
+      this->video_fifo->put(this->video_fifo, buf);
+    } else {
+      buf->free_buffer(buf);
     }
 
-    if (!this->rate)
-      this->rate = demux_mpeg_block_estimate_rate (this);
-
-    this->last_scr         = 0;
-    this->nav_last_end_pts = 0;
-    this->ignore_scr_discont = 0;
-
-    if((this->input->get_capabilities(this->input) & INPUT_CAP_PREVIEW) != 0) {
-
-      int num_buffers = NUM_PREVIEW_BUFFERS;
-
-      this->input->seek (this->input, 0, SEEK_SET);
-
-      this->status = DEMUX_OK ;
-      while ( (num_buffers>0) && (this->status == DEMUX_OK) ) {
-
-        demux_mpeg_block_parse_pack(this, 1);
-        num_buffers --;
-      }
-    }
-    this->status = DEMUX_FINISHED;
-  }
-  
-  if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
-    if (start_pos) {
-      start_pos /= (off_t) this->blocksize;
-      start_pos *= (off_t) this->blocksize;
-
-      this->input->seek (this->input, start_pos, SEEK_SET);
-    } else if (start_time) {
-      start_pos = start_time * this->rate * 50;
-      start_pos /= (off_t) this->blocksize;
-      start_pos *= (off_t) this->blocksize;
-
-      this->input->seek (this->input, start_pos, SEEK_SET);
-    } else
-      this->input->seek (this->input, 0, SEEK_SET);
-  }
-
-  /*
-   * query CLUT from the input plugin
-   */
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-
-  if ((this->input->get_capabilities(this->input) & INPUT_CAP_CLUT) &&
-      ((this->input->get_optional_data(this->input, buf->mem, INPUT_OPTIONAL_DATA_CLUT)
-	== INPUT_OPTIONAL_SUCCESS))) {
-    buf->type = BUF_SPU_CLUT;
-    buf->content = buf->mem;
+    /*
+     * now start demuxing
+     */
+    this->send_newpts = 1;
+    if( !this->thread_running ) {
     
-    this->video_fifo->put(this->video_fifo, buf);
-  } else {
-    buf->free_buffer(buf);
-  }
+      this->buf_flag_seek = 0;
+      this->nav_last_end_pts = this->nav_last_start_pts = 0;
+      this->status   = DEMUX_OK ;
+      this->last_pts[0]   = 0;
+      this->last_pts[1]   = 0;
 
-  /*
-   * now start demuxing
-   */
-
-  if( this->status != DEMUX_OK )
-  {
-    this->status   = DEMUX_OK ;
-    this->last_scr = 0;
-    this->nav_last_end_pts = 0;
-    this->ignore_scr_discont = 0;
-
-    if ((err = pthread_create (&this->thread,
+      this->send_end_buffers = 1;
+      this->thread_running = 1;
+      if ((err = pthread_create (&this->thread,
 			     NULL, demux_mpeg_block_loop, this)) != 0) {
-      printf ("demux_mpeg_block: can't create new thread (%s)\n",
+        printf ("demux_mpeg_block: can't create new thread (%s)\n",
 	      strerror(err));
-      exit (1);
+        abort();
+      }
+    } else {
+      this->buf_flag_seek = 1;
+      this->nav_last_end_pts = this->nav_last_start_pts = 0;
+      xine_demux_flush_engine(this->xine);
     }
   }
   else {
-    xine_flush_engine(this->xine);
+    xine_log (this->xine, XINE_LOG_MSG,
+	_("demux_mpeg_block: unknown block size. try using demux_mpeg.\n"));
+    this->status   = DEMUX_FINISHED ;
   }
+
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
-static void demux_mpeg_block_seek (demux_plugin_t *this_gen,
+static int demux_mpeg_block_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_mpeg_block_t *this = (demux_mpeg_block_t *) this_gen;
 
-	demux_mpeg_block_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_mpeg_block_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -1020,27 +1064,11 @@ static int demux_mpeg_block_open(demux_plugin_t *this_gen,
 
       this->blocksize = input->get_blocksize(input);
 
-      if (!this->blocksize) {
+      if (!this->blocksize)
+        this->blocksize = demux_mpeg_detect_blocksize( this, input );
 
-	/* detect blocksize */
-	input->seek(input, 2048, SEEK_SET);
-	if (!input->read(input, this->scratch, 4))
-	  return DEMUX_CANNOT_HANDLE;
-
-	if (this->scratch[0] || this->scratch[1]
-	    || (this->scratch[2] != 0x01) || (this->scratch[3] != 0xba)) {
-
-	  input->seek(input, 2324, SEEK_SET);
-	  if (!input->read(input, this->scratch, 4))
-	    return DEMUX_CANNOT_HANDLE;
-	  if (this->scratch[0] || this->scratch[1] 
-	      || (this->scratch[2] != 0x01) || (this->scratch[3] != 0xba)) 
-	    return DEMUX_CANNOT_HANDLE;
-	  this->blocksize = 2324;
-	  
-	} else
-	  this->blocksize = 2048;
-      }
+      if (!this->blocksize)
+        return DEMUX_CANNOT_HANDLE;
 
       input->seek(input, 0, SEEK_SET);
       if (input->read(input, this->scratch, this->blocksize)) {
@@ -1070,7 +1098,7 @@ static int demux_mpeg_block_open(demux_plugin_t *this_gen,
 
     xine_strdupa(valid_mrls, (this->config->register_string(this->config,
 							    "mrl.mrls_mpeg_block", VALID_MRLS,
-							    "valid mrls for mpeg block demuxer",
+							    _("valid mrls for mpeg block demuxer"),
 							    NULL, NULL, NULL)));
     
     MRL = input->get_mrl (input);
@@ -1111,14 +1139,14 @@ static int demux_mpeg_block_open(demux_plugin_t *this_gen,
     
     xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							    "mrl.ends_mpeg_block", VALID_ENDS,
-							    "valid mrls ending for mpeg block demuxer",
+							    _("valid mrls ending for mpeg block demuxer"),
 							    NULL, NULL, NULL)));
     while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
       
       while(*m == ' ' || *m == '\t') m++;
 
       if(!strcasecmp((ending + 1), m)) {
-	this->blocksize = 2048;
+	this->blocksize = 0;
 	demux_mpeg_block_accept_input (this, input);	  
 	return DEMUX_CAN_HANDLE;
       }
@@ -1160,12 +1188,12 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_mpeg_block_t *this;
 
-  if (iface != 7) {
-    printf ("demux_mpeg_block: plugin doesn't support plugin API version %d.\n"
-	    "                  this means there's a version mismatch between xine and this "
-	    "                  demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_mpeg_block: plugin doesn't support plugin API version %d.\n"
+	      "                  this means there's a version mismatch between xine and this "
+	      "                  demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
-  return NULL;
+    return NULL;
   }
 
   this         = xine_xmalloc (sizeof (demux_mpeg_block_t));
@@ -1174,11 +1202,11 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   /* Calling register_string() configure valid mrls in configfile */
   (void*) this->config->register_string(this->config, "mrl.mrls_mpeg_block", VALID_MRLS,
-					"valid mrls for mpeg block demuxer",
+					_("valid mrls for mpeg block demuxer"),
 					NULL, NULL, NULL);
   (void*) this->config->register_string(this->config,
 					"mrl.ends_mpeg_block", VALID_ENDS,
-					"valid mrls ending for mpeg block demuxer",
+					_("valid mrls ending for mpeg block demuxer"),
 					NULL, NULL, NULL);    
 
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
