@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpgaudio.c,v 1.39 2002/04/23 13:30:42 esnel Exp $
+ * $Id: demux_mpgaudio.c,v 1.51 2002/07/05 17:32:00 mroi Exp $
  *
  * demultiplexer for mpeg audio (i.e. mp3) streams
  *
@@ -42,6 +42,8 @@
 #define DEMUX_MPGAUDIO_IFACE_VERSION 3
 
 #define VALID_ENDS                   "mp3,mp2,mpa,mpega"
+#define WRAP_THRESHOLD       120000
+
 
 typedef struct {
 
@@ -57,6 +59,7 @@ typedef struct {
   input_plugin_t      *input;
 
   pthread_t            thread;
+  int                  thread_running;
   pthread_mutex_t      mutex;
 
   int                  status;
@@ -64,9 +67,14 @@ typedef struct {
   int                  send_end_buffers;
 
   int                  stream_length;
+  long                 bitrate;
+  int64_t              last_pts;
+  int                  send_newpts;
+
 } demux_mpgaudio_t ;
 
 
+/* bitrate table tabsel_123[mpeg version][layer][bitrate index] */
 int tabsel_123[2][3][16] = {
    { {0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,},
      {0,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,},
@@ -77,6 +85,7 @@ int tabsel_123[2][3][16] = {
      {0,8,16,24,32,40,48,56,64,80,96,112,128,144,160,} }
 };
 
+/* sampling rate frequency table */
 long freqs[9] = { 44100, 48000, 32000,
                   22050, 24000, 16000 ,
                   11025 , 12000 , 8000 };
@@ -108,27 +117,42 @@ static void mpg123_decode_header(demux_mpgaudio_t *this,unsigned long newhead)
   int lay, sampling_frequency, bitrate_index, padding;
   long framesize = 1;
   static int bs[4] = {0, 384, 1152, 1152};
-  double tpf, bitrate;
+  double tpf;
+  char * ver;
 
+  /*
+   * lsf==0 && mpeg25==0 : MPEG Version 1 (ISO/IEC 11172-3)
+   * lsf==1 && mpeg25==0 : MPEG Version 2 (ISO/IEC 13818-3)
+   * lsf==1 && mpeg25==1 : MPEG Version 2.5 (later extension of MPEG2)
+   */
   if( newhead & (1<<20) ) {
     lsf = (newhead & (1<<19)) ? 0x0 : 0x1;
+    if (lsf) {
+      ver = "2";
+    } else {
+      ver = "1";
+    }
     mpeg25 = 0;
-  }
-  else {
+  } else {
     lsf = 1;
     mpeg25 = 1;
+    ver = "2.5";
   }
 
+  /* Layer I, II, III */
   lay = 4-((newhead>>17)&3);
 
+/*
   if(mpeg25) {
     sampling_frequency = 6 + ((newhead>>10)&0x3);
   }
   else {
     sampling_frequency = ((newhead>>10)&0x3) + (lsf*3);
   }
+*/
 
   bitrate_index = ((newhead>>12)&0xf);
+/*
   padding   = ((newhead>>9)&0x1);
 
   switch(lay)
@@ -153,17 +177,39 @@ static void mpg123_decode_header(demux_mpgaudio_t *this,unsigned long newhead)
   tpf = (double) bs[lay];
   tpf /= freqs[sampling_frequency] << lsf;
 
-  bitrate = (double) framesize / tpf;
+  this->bitrate = (double) framesize / tpf;
+*/
+  this->bitrate = tabsel_123[lsf][lay - 1][bitrate_index];
   xine_log (this->xine, XINE_LOG_FORMAT, 
-	    _("mpgaudio: bitrate = %.2fkbps\n"), bitrate/1024.0*8.0 );
-  this->stream_length = (int)(this->input->get_length(this->input) / bitrate);
+	    _("demux_mpgaudio: MPEG %s  Layer %d  %ldkbps\n"), ver, lay, this->bitrate );
+  this->stream_length = (int)(this->input->get_length(this->input) / (this->bitrate * 1000 / 8));
+}
+
+static void check_newpts( demux_mpgaudio_t *this, int64_t pts )
+{
+  int64_t diff;
+
+  diff = pts - this->last_pts;
+
+  if( pts &&
+      (this->send_newpts || (this->last_pts && abs(diff)>WRAP_THRESHOLD) ) ) {
+
+    xine_demux_control_newpts(this->xine, pts, 0);
+    this->send_newpts = 0;
+  }
+
+  if( pts )
+    this->last_pts = pts;
 }
 
 static int demux_mpgaudio_next (demux_mpgaudio_t *this) {
 
   buf_element_t *buf = NULL;
   uint32_t head;
-  
+  off_t buffer_pos;
+  uint64_t pts;
+
+  buffer_pos = this->input->get_current_pos(this->input);
   if(this->audio_fifo)
     buf = this->input->read_block(this->input, 
 				  this->audio_fifo, 2048);
@@ -189,11 +235,15 @@ static int demux_mpgaudio_next (demux_mpgaudio_t *this) {
     }
   }
 
-  buf->pts             = 0;
+  pts = (90000 * buffer_pos) / (this->bitrate * 1000 / 8);
+  check_newpts(this, pts);
+
+  /*buf->pts             = 0;*/
   /*buf->scr             = 0;*/
   buf->input_pos       = this->input->get_current_pos(this->input);
   buf->input_time      = buf->input_pos * this->stream_length /
                          this->input->get_length(this->input);
+  buf->pts             = pts;
   buf->type            = BUF_AUDIO_MPEG;
   buf->decoder_info[0] = 1;
 
@@ -204,41 +254,41 @@ static int demux_mpgaudio_next (demux_mpgaudio_t *this) {
 }
 
 static void *demux_mpgaudio_loop (void *this_gen) {
-  buf_element_t *buf;
   demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
 
-  this->send_end_buffers = 1;
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    if (!demux_mpgaudio_next(this))
-      this->status = DEMUX_FINISHED;
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      if (!demux_mpgaudio_next(this))
+        this->status = DEMUX_FINISHED;
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->audio_fifo->size(this->audio_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
   
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM;
-    this->video_fifo->put (this->video_fifo, buf);
-    
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
+  printf ("demux_mpgaudio: demux loop finished.\n");
 
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
   pthread_exit(NULL);
 
@@ -248,12 +298,11 @@ static void *demux_mpgaudio_loop (void *this_gen) {
 static void demux_mpgaudio_stop (demux_plugin_t *this_gen) {
 
   demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
-  buf_element_t *buf;
   void *p;
 
   pthread_mutex_lock( &this->mutex );
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_mpgaudio_block: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -265,25 +314,15 @@ static void demux_mpgaudio_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER;
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 static int demux_mpgaudio_get_status (demux_plugin_t *this_gen) {
   demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
 static uint32_t demux_mpgaudio_read_head(input_plugin_t *input)
@@ -310,28 +349,35 @@ static uint32_t demux_mpgaudio_read_head(input_plugin_t *input)
   return head;
 }
 
-static void demux_mpgaudio_start (demux_plugin_t *this_gen,
+static int demux_mpgaudio_start (demux_plugin_t *this_gen,
 				  fifo_buffer_t *video_fifo, 
 				  fifo_buffer_t *audio_fifo,
 				  off_t start_pos, int start_time) {
 
   demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
-  buf_element_t *buf;
   int err;
+  int status;
   
   pthread_mutex_lock( &this->mutex );
 
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     this->video_fifo  = video_fifo;
     this->audio_fifo  = audio_fifo;
   
     this->stream_length = 0;
+    this->last_pts      = 0;
+
+    if( !audio_fifo ) {
+      xine_log (this->xine, XINE_LOG_FORMAT, _("demux_mpgaudio: no audio driver!\n") );
+      pthread_mutex_unlock( &this->mutex );
+      return DEMUX_FINISHED;
+    }
   }
   
   if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
     uint32_t head;
 
-    if( this->status != DEMUX_OK ) {
+    if( !this->thread_running ) {
       head = demux_mpgaudio_read_head(this->input);
 
       if (mpg123_head_check(head))
@@ -342,44 +388,45 @@ static void demux_mpgaudio_start (demux_plugin_t *this_gen,
          start_pos = start_time * this->input->get_length(this->input) /
                      this->stream_length;
 
-
     this->input->seek (this->input, start_pos, SEEK_SET);
-  }
-  
-  if( this->status != DEMUX_OK ) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
 
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+  }
+
+  this->status = DEMUX_OK;
+  this->send_newpts = 1;
+
+  if( !this->thread_running ) {
+    xine_demux_control_start(this->xine);
   
     /*
      * now start demuxing
      */
 
-    this->status = DEMUX_OK;
+    this->send_end_buffers = 1;
+    this->thread_running = 1;
     if ((err = pthread_create (&this->thread,
 			     NULL, demux_mpgaudio_loop, this)) != 0) {
       printf ("demux_mpgaudio: can't create new thread (%s)\n",
 	      strerror(err));
-      exit (1);
+      abort();
     }
   }
   else {
-    xine_flush_engine(this->xine);
+    xine_demux_flush_engine(this->xine);
   }
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
-static void demux_mpgaudio_seek (demux_plugin_t *this_gen,
+static int demux_mpgaudio_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_mpgaudio_t *this = (demux_mpgaudio_t *) this_gen;
 
-	demux_mpgaudio_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_mpgaudio_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -419,7 +466,7 @@ static int demux_mpgaudio_open(demux_plugin_t *this_gen,
     
     xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							    "mrl.ends_mgaudio", VALID_ENDS,
-							    "valid mrls ending for mpeg audio demuxer",
+							    _("valid mrls ending for mpeg audio demuxer"),
 							    NULL, NULL, NULL)));
     while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
       
@@ -474,21 +521,21 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_mpgaudio_t *this;
 
-  if (iface != 7) {
-    printf ("demux_mpeg: plugin doesn't support plugin API version %d.\n"
-	    "            this means there's a version mismatch between xine and this "
-	    "            demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_mpeg: plugin doesn't support plugin API version %d.\n"
+	      "            this means there's a version mismatch between xine and this "
+	      "            demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
     return NULL;
   }
   
-  this         = malloc (sizeof (demux_mpgaudio_t));
+  this         = xine_xmalloc (sizeof (demux_mpgaudio_t));
   this->config = xine->config;
   this->xine   = xine;
 
   (void*) this->config->register_string(this->config,
 					"mrl.ends_mgaudio", VALID_ENDS,
-					"valid mrls ending for mpeg audio demuxer",
+					_("valid mrls ending for mpeg audio demuxer"),
 					NULL, NULL, NULL);
 
   this->demux_plugin.interface_version = DEMUX_MPGAUDIO_IFACE_VERSION;
