@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_ogg.c,v 1.22 2002/04/14 03:31:44 guenter Exp $
+ * $Id: demux_ogg.c,v 1.31 2002/07/14 22:27:25 miguelfreitas Exp $
  *
  * demultiplexer for ogg streams
  *
@@ -36,11 +36,6 @@
 
 #include <ogg/ogg.h>
 
-#define	WINE_TYPEDEFS_ONLY
-#include "libw32dll/wine/avifmt.h"
-#include "libw32dll/wine/windef.h"
-#include "libw32dll/wine/vfw.h"
-
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "demux.h"
@@ -101,6 +96,7 @@ typedef struct demux_ogg_s {
   input_plugin_t       *input;
 
   pthread_t             thread;
+  int                   thread_running;
   pthread_mutex_t       mutex;
 
   int                   status;
@@ -121,6 +117,7 @@ typedef struct demux_ogg_s {
 
 } demux_ogg_t ;
 
+
 static void hex_dump (uint8_t *p, int length) {
 
   int i;
@@ -226,7 +223,7 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 
 	  dsogg_header_t   *oggh;
 	  buf_element_t    *buf;
-	  BITMAPINFOHEADER  bih;
+	  xine_bmiheader    bih;
 
 #ifdef LOG
 	  printf ("demux_ogg: direct show filter created stream detected, hexdump:\n");
@@ -235,7 +232,7 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 
 	  oggh = (dsogg_header_t *) &op.packet[1];
 
-	  this->buf_types[stream_num] = fourcc_to_buf_video (oggh->subtype);
+	  this->buf_types[stream_num] = fourcc_to_buf_video (*(uint32_t *)oggh->subtype);
 
 #ifdef LOG
 	  printf ("demux_ogg: subtype          %.4s\n", oggh->subtype);
@@ -249,7 +246,7 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 	  printf ("demux_ogg: buf_type         %08x\n",this->buf_types[stream_num]);  
 #endif
 
-	  bih.biSize=sizeof(BITMAPINFOHEADER);
+	  bih.biSize=sizeof(xine_bmiheader);
 	  bih.biWidth = oggh->hubba.video.width;
 	  bih.biHeight= oggh->hubba.video.height;
 	  bih.biPlanes= 0;
@@ -262,12 +259,11 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 	  bih.biClrImportant=0;
 
 	  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-	  buf->content = buf->mem;
 	  buf->decoder_flags = BUF_FLAG_HEADER;
 	  this->frame_duration = oggh->time_unit * 9 / 1000;
 	  buf->decoder_info[1] = this->frame_duration;
-	  memcpy (buf->content, &bih, sizeof (BITMAPINFOHEADER));
-	  buf->size = sizeof (BITMAPINFOHEADER);	  
+	  memcpy (buf->content, &bih, sizeof (xine_bmiheader));
+	  buf->size = sizeof (xine_bmiheader);	  
 	  buf->type = this->buf_types[stream_num];
 	  this->video_fifo->put (this->video_fifo, buf);
 
@@ -291,8 +287,6 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 	
 	buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
 	
-	buf->content = buf->mem;
-	
 	{
 	  int op_size = sizeof(op);
 	  ogg_packet *og_ghost;
@@ -310,8 +304,6 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 	buf->pts    = 0; /* FIXME */
 	buf->size   = op.bytes;
 	
-	buf->decoder_flags = 0;
-
 	buf->input_pos  = this->input->get_current_pos (this->input);
 	buf->input_time = 0;
 	
@@ -332,11 +324,8 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 
 	  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
 	
-	  buf->content = buf->mem;
-
 	  if ( (todo-done)>(buf->max_size-1)) {
 	    buf->size  = buf->max_size-1;
-	    buf->decoder_flags = 0;
 	  } else {
 	    buf->size = todo-done;
 	    buf->decoder_flags = BUF_FLAG_FRAME_END;
@@ -374,27 +363,34 @@ static void demux_ogg_send_package (demux_ogg_t *this) {
 static void *demux_ogg_loop (void *this_gen) {
   
   demux_ogg_t *this = (demux_ogg_t *) this_gen;
-  buf_element_t *buf;
-
 #ifdef LOG
   printf ("demux_ogg: demux loop starting...\n"); 
 #endif
 
-  this->send_end_buffers = 1;
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    demux_ogg_send_package (this);
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
 
+      demux_ogg_send_package (this);
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->audio_fifo->size(this->audio_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
+    
 #ifdef LOG
   printf ("demux_ogg: demux loop finished (status: %d)\n",
 	  this->status);
@@ -403,20 +399,10 @@ static void *demux_ogg_loop (void *this_gen) {
   this->status = DEMUX_FINISHED;
 
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-    
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
-
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
 
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
   pthread_exit(NULL);
 
@@ -433,14 +419,13 @@ static void demux_ogg_close (demux_plugin_t *this_gen) {
 static void demux_ogg_stop (demux_plugin_t *this_gen) {
 
   demux_ogg_t *this = (demux_ogg_t *) this_gen;
-  buf_element_t *buf;
   void *p;
 
   pthread_mutex_lock( &this->mutex );
 
   printf ("demux_ogg: demux_ogg_stop\n");
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_ogg: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -452,39 +437,29 @@ static void demux_ogg_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 static int demux_ogg_get_status (demux_plugin_t *this_gen) {
   demux_ogg_t *this = (demux_ogg_t *) this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
-static void demux_ogg_start (demux_plugin_t *this_gen,
+static int demux_ogg_start (demux_plugin_t *this_gen,
 			     fifo_buffer_t *video_fifo, 
 			     fifo_buffer_t *audio_fifo,
 			     off_t start_pos, int start_time) {
 
   demux_ogg_t *this = (demux_ogg_t *) this_gen;
-  buf_element_t *buf;
   int err, i;
 
   pthread_mutex_lock( &this->mutex );
+  err = 1;
 
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     this->video_fifo  = video_fifo;
     this->audio_fifo  = audio_fifo;
 
@@ -492,20 +467,11 @@ static void demux_ogg_start (demux_plugin_t *this_gen,
      * send start buffer
      */
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_start(this->xine);
 
     /*
      * initialize ogg engine
      */
-
     ogg_sync_init(&this->oy);
 
     this->num_streams = 0;
@@ -521,49 +487,59 @@ static void demux_ogg_start (demux_plugin_t *this_gen,
   /*
    * seek to start position
    */
-
   if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
 
     off_t cur_pos = this->input->get_current_pos (this->input);
 
-    /*
-      if ( (!start_pos) && (start_time))
-      start_pos = start_time * this->rate;
-    */
-
+/*
+      if ( (!start_pos) && (start_time)) {
+        start_pos = start_time * this->rate;
+      }
+*/
+/*
     if (start_pos<cur_pos)
       start_pos = cur_pos;
+*/
 
     this->input->seek (this->input, start_pos, SEEK_SET);
+
+    /* send a new pts */
+    if( this->thread_running ) {
+      xine_demux_control_newpts(this->xine, start_pos / 90, 0);
+    }
   }
 
-  
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     /*
      * now start demuxing
      */
 
     this->status = DEMUX_OK;
-
+    this->send_end_buffers = 1;
+    this->thread_running = 1;
     if ((err = pthread_create (&this->thread,
 			       NULL, demux_ogg_loop, this)) != 0) {
       printf ("demux_ogg: can't create new thread (%s)\n",
 	      strerror(err));
-      exit (1);
+      abort();
     }
-  } else {
-    xine_flush_engine(this->xine);
+  }
+  else {
+    xine_demux_flush_engine(this->xine);
+    err = 0;
   }
   
   pthread_mutex_unlock( &this->mutex );
+
+  return err ? DEMUX_FINISHED : DEMUX_OK;
 }
 
-static void demux_ogg_seek (demux_plugin_t *this_gen,
+static int demux_ogg_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_ogg_t *this = (demux_ogg_t *) this_gen;
 
-	demux_ogg_start (this_gen, this->video_fifo, this->audio_fifo,
-			 start_pos, start_time);
+  return demux_ogg_start (this_gen, this->video_fifo, this->audio_fifo,
+			  start_pos, start_time);
 }
 
 static int demux_ogg_open(demux_plugin_t *this_gen,
@@ -624,7 +600,7 @@ static int demux_ogg_open(demux_plugin_t *this_gen,
     
     xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							    "mrl.ends_ogg", VALID_ENDS,
-							    "valid mrls ending for ogg demuxer",
+							    _("valid mrls ending for ogg demuxer"),
 							    NULL, NULL, NULL)));
     while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
       
@@ -661,7 +637,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_ogg_t     *this;
 
-  if (iface != 7) {
+  if (iface != 10) {
     printf( _("demux_ogg: plugin doesn't support plugin API version %d.\n"
 	      "           this means there's a version mismatch between xine and this "
 	      "           demuxer plugin.\nInstalling current demux plugins should help.\n"),
@@ -675,7 +651,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   (void*) this->config->register_string(this->config,
 					"mrl.ends_ogg", VALID_ENDS,
-					"valid mrls ending for ogg demuxer",
+					_("valid mrls ending for ogg demuxer"),
 					NULL, NULL, NULL);
 
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
