@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000, 2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_pes.c,v 1.25 2002/04/23 13:30:42 esnel Exp $
+ * $Id: demux_pes.c,v 1.34 2002/07/14 22:27:25 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 2 PES (Packetized Elementary Streams)
  * reads streams of variable blocksizes
@@ -60,6 +60,7 @@ typedef struct demux_pes_s {
   input_plugin_t      *input;
 
   pthread_t            thread;
+  int                  thread_running;
   pthread_mutex_t      mutex;
 
   unsigned char        dummy_space[100000];
@@ -105,7 +106,7 @@ static uint32_t read_bytes (demux_pes_t *this, int n) {
     break;
   default:
     printf ("demux_pes: how how - something wrong in wonderland demux:read_bytes (%d)\n", n);
-    exit (1);
+    abort();
   }
 
   return res;
@@ -115,19 +116,7 @@ static void check_newpts( demux_pes_t *this, int64_t pts )
 {
   if( this->send_newpts && !this->preview_mode && pts ) {
     
-    buf_element_t *buf;
-  
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type = BUF_CONTROL_NEWPTS;
-    buf->disc_off = pts;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if (this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type = BUF_CONTROL_NEWPTS;
-      buf->disc_off = pts;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_newpts(this->xine, pts, 0);
     this->send_newpts = 0;
   }
 }
@@ -192,8 +181,6 @@ static void parse_mpeg2_packet (demux_pes_t *this, int nID) {
     buf->pts       = pts;
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
-    else
-      buf->decoder_flags = 0;
 
     buf->input_pos = this->input->get_current_pos (this->input);
     
@@ -239,8 +226,6 @@ static void parse_mpeg2_packet (demux_pes_t *this, int nID) {
     buf->pts       = pts;
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
-    else
-      buf->decoder_flags = 0;
     
     buf->input_pos = this->input->get_current_pos(this->input);
 
@@ -287,8 +272,6 @@ static void parse_mpeg2_packet (demux_pes_t *this, int nID) {
     buf->pts  = pts;
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
-    else
-      buf->decoder_flags = 0;
     
     buf->input_pos = this->input->get_current_pos(this->input);
 
@@ -330,42 +313,41 @@ static void demux_pes_resync (demux_pes_t *this, uint32_t buf) {
 static void *demux_pes_loop (void *this_gen) {
 
   demux_pes_t *this = (demux_pes_t *) this_gen;
-  buf_element_t *buf;
   uint32_t w=0;
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    w = parse_pack (this);
+  /* do-while needed to seek after demux finished */
+  do {
 
-    if (w != 0x000001)
-      demux_pes_resync (this, w);
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+      w = parse_pack (this);
+
+      if (w != 0x000001)
+        demux_pes_resync (this, w);
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->video_fifo->size(this->video_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+
+  } while( this->status == DEMUX_OK );
   
   if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
   }
 
   printf ("demux_pes: demux loop finished (status: %d, buf:%x)\n",
 	  this->status, w);
 
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
   pthread_exit(NULL);
 
@@ -375,12 +357,11 @@ static void *demux_pes_loop (void *this_gen) {
 static void demux_pes_stop (demux_plugin_t *this_gen) {
 
   demux_pes_t *this = (demux_pes_t *) this_gen;
-  buf_element_t *buf;
   void *p;
 
   pthread_mutex_lock( &this->mutex );
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_pes: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -392,39 +373,29 @@ static void demux_pes_stop (demux_plugin_t *this_gen) {
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
-
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_flush_engine(this->xine);
 
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 static int demux_pes_get_status (demux_plugin_t *this_gen) {
   demux_pes_t *this = (demux_pes_t *) this_gen;
-  return this->status;
+
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
-static void demux_pes_start (demux_plugin_t *this_gen,
+static int demux_pes_start (demux_plugin_t *this_gen,
 			     fifo_buffer_t *video_fifo,
 			     fifo_buffer_t *audio_fifo,
 			     off_t start_pos, int start_time) {
 
   demux_pes_t *this = (demux_pes_t *) this_gen;
-  buf_element_t *buf;
   int err;
+  int status;
 
   pthread_mutex_lock( &this->mutex );
 
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     this->video_fifo  = video_fifo;
     this->audio_fifo  = audio_fifo;
 
@@ -432,15 +403,7 @@ static void demux_pes_start (demux_plugin_t *this_gen,
      * send start buffer
      */
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_start(this->xine);
 
     if ((this->input->get_capabilities (this->input) & INPUT_CAP_PREVIEW) != 0 ) {
 
@@ -479,30 +442,36 @@ static void demux_pes_start (demux_plugin_t *this_gen,
 
   this->send_newpts = 1;
     
-  if( this->status != DEMUX_OK ) {
+  if( !this->thread_running ) {
     this->preview_mode = 0;
     this->send_end_buffers = 1;
     this->status = DEMUX_OK ;
+    this->thread_running = 1;
 
     if ((err = pthread_create (&this->thread,
 			     NULL, demux_pes_loop, this)) != 0) {
       printf ("demux_pes: can't create new thread (%s)\n",
 	      strerror(err));
-      exit (1);
+      abort();
     }
   }
   else {
-    xine_flush_engine(this->xine);
+    xine_demux_flush_engine(this->xine);
   }
-  pthread_mutex_unlock( &this->mutex );
 
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
+  pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
-static void demux_pes_seek (demux_plugin_t *this_gen,
+static int demux_pes_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_pes_t *this = (demux_pes_t *) this_gen;
 
-	demux_pes_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_pes_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
@@ -554,7 +523,7 @@ static int demux_pes_open(demux_plugin_t *this_gen,
     
     xine_strdupa(valid_mrls, (this->config->register_string(this->config,
 							    "mrl.mrls_pes", VALID_MRLS,
-							    "valid mrls for pes demuxer",
+							    _("valid mrls for pes demuxer"),
 							    NULL, NULL, NULL)));
     
     media = strstr(MRL, "://");
@@ -584,7 +553,7 @@ static int demux_pes_open(demux_plugin_t *this_gen,
     
     xine_strdupa(valid_ends, (this->config->register_string(this->config,
 							    "mrl.ends_pes", VALID_ENDS,
-							    "valid mrls ending for pes demuxer",
+							    _("valid mrls ending for pes demuxer"),
 							    NULL, NULL, NULL)));
     while((m = xine_strsep(&valid_ends, ",")) != NULL) { 
       
@@ -629,10 +598,10 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
 
   demux_pes_t     *this;
 
-  if (iface != 7) {
-    printf ("demux_pes: plugin doesn't support plugin API version %d.\n"
-	    "           this means there's a version mismatch between xine and this "
-	    "           demuxer plugin.\nInstalling current demux plugins should help.\n",
+  if (iface != 10) {
+    printf (_("demux_pes: plugin doesn't support plugin API version %d.\n"
+	      "           this means there's a version mismatch between xine and this "
+	      "           demuxer plugin.\nInstalling current demux plugins should help.\n"),
 	    iface);
     return NULL;
   }
@@ -642,11 +611,11 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->xine   = xine;
 
   (void*) this->config->register_string(this->config, "mrl.mrls_pes", VALID_MRLS,
-					"valid mrls for pes demuxer",
+					_("valid mrls for pes demuxer"),
 					NULL, NULL, NULL);
   (void*) this->config->register_string(this->config,
 					"mrl.ends_pes", VALID_ENDS,
-					"valid mrls ending for pes demuxer",
+					_("valid mrls ending for pes demuxer"),
 					NULL, NULL, NULL);    
   
   this->demux_plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
