@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_ts.c,v 1.42 2002/04/23 13:30:42 esnel Exp $
+ * $Id: demux_ts.c,v 1.52 2002/07/05 17:32:01 mroi Exp $
  *
  * Demultiplexer for MPEG2 Transport Streams.
  *
@@ -34,26 +34,42 @@
  *
  * Date        Author
  * ----        ------
- * 07-Jan-2002 Andr Draszik <andid@gmx.net>
- *                              - added support for single-section PMTs
- *                                spanning multiple TS packets.
- * 10-Sep-2001 James Courtier-Dutton <jcdutton>
- *                              Re-wrote sync code so that it now does not loose any data.
- * 27-Aug-2001 Hubert Matthews  Reviewed by: n/a
- *                              Added in synchronisation code.
  *
- *  1-Aug-2001 James Courtier-Dutton <jcdutton>
- *                              Reviewed by: n/a
- *                              TS Streams with zero PES lenght should now work.
+ * 30-May-2002 Mauro Borghi
+ *                  - dynamic allocation leaks fixes
+ *
+ * 27-May-2002 Giovanni Baronetti and Mauro Borghi <mauro.borghi@tilab.com>
+ *                  - fill buffers before putting them in fifos 
+ *                  - force PMT reparsing when PMT PID changes
+ *                  - accept non seekable input plugins -- FIX?
+ *                  - accept dvb as input plugin
+ *                  - optimised read operations
+ *                  - modified resync code
+ *
+ * 16-May-2002 Thibaut Mattern <tmattern@noos.fr>
+ *                  - fix demux loop
+ *
+ * 07-Jan-2002 Andr Draszik <andid@gmx.net>
+ *                  - added support for single-section PMTs
+ *                    spanning multiple TS packets
+ *
+ * 10-Sep-2001 James Courtier-Dutton <jcdutton>
+ *                  - re-wrote sync code so that it now does not loose any data
+ *
+ * 27-Aug-2001 Hubert Matthews  Reviewed by: n/a
+ *                  - added in synchronisation code
+ *
+ *  1-Aug-2001 James Courtier-Dutton <jcdutton>  Reviewed by: n/a
+ *                  - TS Streams with zero PES lenght should now work
  *
  * 30-Jul-2001 shaheedhaque     Reviewed by: n/a
- *                              PATs and PMTs seem to work.
+ *                  - PATs and PMTs seem to work
  *
  * 29-Jul-2001 shaheedhaque     Reviewed by: n/a
- *                              Compiles!
-
+ *                  - Compiles!
  *
- * TODO: do without memcpys, seeking (if possible), preview buffers
+ *
+ * TODO: do without memcpys, preview buffers
  */
 
 #ifdef HAVE_CONFIG_H
@@ -71,7 +87,7 @@
 #include "xineutils.h"
 #include "demux.h"
 
-#define VALID_MRLS   "fifo,stdin"
+#define VALID_MRLS   "fifo,stdin,dvb,tcp"
 #define VALID_ENDS   "m2t,ts,trp"
 
 #ifdef __GNUC__
@@ -97,19 +113,25 @@
 /*
 #define TS_LOG
 #define TS_PMT_LOG
+#define TS_READ_STATS // activates read statistics generation
 */
 
 /*
- * The maximum number of PIDs we are prepared to handle in a single program is the
- * number that fits in a single-packet PMT.
+ *  The maximum number of PIDs we are prepared to handle in a single program
+ *  is the number that fits in a single-packet PMT.
  */
 #define PKT_SIZE 188
 #define BODY_SIZE (188 - 4)
 #define MAX_PIDS ((BODY_SIZE - 1 - 13) / 4)
 #define MAX_PMTS ((BODY_SIZE - 1 - 13) / 4)
 #define SYNC_BYTE   0x47
-#define MIN_SYNCS   5
-#define BUF_SIZE    ((MIN_SYNCS+1) * PKT_SIZE)
+
+#define MIN_SYNCS 3
+#define NPKT_PER_READ 100
+
+#define BUF_SIZE (NPKT_PER_READ * PKT_SIZE)
+
+#define MAX_PES_BUF_SIZE 2048
 
 #define NULL_PID 0x1fff
 #define INVALID_PID ((unsigned int)(-1))
@@ -134,13 +156,13 @@ typedef struct {
   uint8_t         *content;
   uint32_t         size;
   uint32_t         type;
-  uint32_t         PTS;
+  int64_t          pts;
   buf_element_t   *buf;
-  int              pes_buf_next;
-  int              pes_len;
-  int              pes_len_zero;
   unsigned int     counter;
-  int              broken_pes;
+  uint8_t          descriptor_tag;
+  int64_t          packet_count;
+  int              corrupted_pes;
+  uint32_t         buffered_bytes;
 
 } demux_ts_media;
 
@@ -151,7 +173,7 @@ typedef struct {
   demux_plugin_t   plugin;
 
   xine_t          *xine;
-  
+
   config_values_t *config;
 
   fifo_buffer_t   *audio_fifo;
@@ -160,6 +182,7 @@ typedef struct {
   input_plugin_t  *input;
 
   pthread_t        thread;
+  int              thread_running;
   pthread_mutex_t  mutex;
 
   int              status;
@@ -182,14 +205,23 @@ typedef struct {
   int64_t          PCR;
   int64_t          last_PCR;
   unsigned int     pid;
+  unsigned int     pid_count;
   unsigned int     videoPid;
   unsigned int     audioPid;
   unsigned int     videoMedia;
   unsigned int     audioMedia;
- 
+
   int              send_end_buffers;
   int              ignore_scr_discont;
   int              send_newpts;
+
+  unsigned int scrambled_pids[MAX_PIDS];
+  unsigned int scrambled_npids;
+
+#ifdef TS_READ_STATS
+  uint32_t rstat[NPKT_PER_READ + 1];
+#endif
+
 } demux_ts;
 
 static void demux_ts_build_crc32_table(demux_ts *this) {
@@ -205,7 +237,8 @@ static void demux_ts_build_crc32_table(demux_ts *this) {
   }
 }
 
-static uint32_t demux_ts_compute_crc32(demux_ts *this, uint8_t *data, uint32_t length, uint32_t crc32) {
+static uint32_t demux_ts_compute_crc32(demux_ts *this, uint8_t *data, 
+				       uint32_t length, uint32_t crc32) {
   uint32_t i;
 
   for(i = 0; i < length; i++) {
@@ -217,20 +250,8 @@ static uint32_t demux_ts_compute_crc32(demux_ts *this, uint8_t *data, uint32_t l
 static void check_newpts( demux_ts *this, int64_t pts )
 {
   if( this->send_newpts && pts ) {
-    
-    buf_element_t *buf;
-  
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type = BUF_CONTROL_NEWPTS;
-    buf->disc_off = pts;
-    this->video_fifo->put (this->video_fifo, buf);
 
-    if (this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type = BUF_CONTROL_NEWPTS;
-      buf->disc_off = pts;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_newpts(this->xine, pts, 0);
     this->send_newpts = 0;
     this->ignore_scr_discont = 1;
   }
@@ -331,7 +352,7 @@ static void demux_ts_parse_pat (demux_ts *this, unsigned char *original_pkt,
   if (crc32 != calc_crc32) {
     LOG_MSG (this->xine, _("demux_ts: demux error! PAT with invalid CRC32: "
                            "packet_crc32: %.8x calc_crc32: %.8x\n"),
-                         crc32,calc_crc32); 
+                         crc32,calc_crc32);
     return;
   }
 
@@ -363,9 +384,20 @@ static void demux_ts_parse_pat (demux_ts *this, unsigned char *original_pkt,
       program_count++;
     }
     this->program_number[program_count] = program_number;
+
+    /* force PMT reparsing when pmt_pid changes */
+    if (this->pmt_pid[program_count] != pmt_pid) {
+      this->pmt_pid[program_count] = pmt_pid;
+      this->audioPid = INVALID_PID;
+      this->videoPid = INVALID_PID;
+    }
+
     this->pmt_pid[program_count] = pmt_pid;
-    this->pmt[program_count]                     = NULL;
-    this->pmt_write_ptr[program_count]           = NULL;
+    if (this->pmt[program_count] != NULL) {
+      free(this->pmt[program_count]);
+      this->pmt[program_count] = NULL;
+      this->pmt_write_ptr[program_count] = NULL;
+    }
 #ifdef TS_LOG
     if (this->program_number[program_count] != INVALID_PROGRAM)
       printf ("demux_ts: PAT acquired count=%d programNumber=0x%04x "
@@ -377,16 +409,18 @@ static void demux_ts_parse_pat (demux_ts *this, unsigned char *original_pkt,
   }
 }
 
-static int demux_ts_parse_pes_header (demux_ts_media *m, 
+static int demux_ts_parse_pes_header (demux_ts_media *m,
 				      uint8_t *buf, int packet_len,
                                       xine_t *xine) {
 
   unsigned char *p;
   uint32_t       header_len;
-  uint32_t       PTS;
+  int64_t        pts;
   uint32_t       stream_id;
+  int            pkt_len;
 
   p = buf;
+  pkt_len = packet_len;
 
   /* we should have a PES packet here */
 
@@ -410,16 +444,15 @@ static int demux_ts_parse_pes_header (demux_ts_media *m,
           stream_id, packet_len, packet_len);
 #endif
 
-  if (p[7] & 0x80) { /* PTS avail */
-
-    PTS  = (p[ 9] & 0x0E) << 29 ;
-    PTS |=  p[10]         << 22 ;
-    PTS |= (p[11] & 0xFE) << 14 ;
-    PTS |=  p[12]         <<  7 ;
-    PTS |= (p[13] & 0xFE) >>  1 ;
+  if (p[7] & 0x80) { /* pts avail */
 
+    pts  = (int64_t)(p[ 9] & 0x0E) << 29 ;
+    pts |=  p[10]         << 22 ;
+    pts |= (p[11] & 0xFE) << 14 ;
+    pts |=  p[12]         <<  7 ;
+    pts |= (p[13] & 0xFE) >>  1 ;
   } else
-    PTS = 0;
+    pts = 0;
 
   /* code works but not used in xine
   if (p[7] & 0x40) {
@@ -434,7 +467,7 @@ static int demux_ts_parse_pes_header (demux_ts_media *m,
     DTS = 0;
   */
 
-  m->PTS       = PTS;
+  m->pts       = pts;
 //  buf->input_pos = this->input->get_current_pos(this->input);
   /* FIXME: not working correctly */
 //  buf->input_time = buf->input_pos / (this->rate * 50);
@@ -447,11 +480,23 @@ static int demux_ts_parse_pes_header (demux_ts_media *m,
   if (stream_id == 0xbd) {
 
     int track, spu_id;
-
+      
+#ifdef LOG
+    printf ("demux_ts: audio buf = %02X %02X %02X %02X %02X %02X %02X %02X\n",
+	  p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7]);
+#endif
     track = p[0] & 0x0F; /* hack : ac3 track */
+    // we check the descriptor tag first because some stations
+    // do not include any of the ac3 header info in their audio tracks
+    // these "raw" streams may begin with a byte that looks like a stream type.
+    if((m->descriptor_tag == 0x81) ||    /* ac3 - raw */ 
+       (p[0] == 0x0B && p[1] == 0x77)) { /* ac3 - syncword */
+      m->content   = p;
+      m->size = packet_len;
+      m->type = BUF_AUDIO_A52;
+      return 1;
 
-    if ((p[0] & 0xE0) == 0x20) {
-
+    } else if ((p[0] & 0xE0) == 0x20) {
       spu_id = (p[0] & 0x1f);
 
       m->content   = p+1;
@@ -509,88 +554,78 @@ static int demux_ts_parse_pes_header (demux_ts_media *m,
   return 0 ;
 }
 
-/*
- * buffer arriving pes data
- * Input is 188 bytes of Transport stream
- * Build a PES packet. PES packets can get as big as 65536
- * If PES packet length was empty(zero) work it out based on seeing the next PUS.
- * Once we have a complete PES packet, give PES packet a valid length field.
- * then queue it. The queuing routine might have to cut it up to make bits < 4096. FIXME: implement cut up.
- * Currently if PES packets are >4096, corruption occurs.
- */
 
+/*
+ *  buffer arriving pes data
+ */
 static void demux_ts_buffer_pes(demux_ts *this, unsigned char *ts,
                                 unsigned int mediaIndex,
                                 unsigned int pus,
                                 unsigned int cc,
                                 unsigned int len) {
 
-  buf_element_t *buf;
-
   demux_ts_media *m = &this->media[mediaIndex];
+
   if (!m->fifo) {
-
+#ifdef TS_LOG
     LOG_MSG(this->xine, _("fifo unavailable (%d)\n"), mediaIndex);
-
+#endif
     return; /* To avoid segfault if video out or audio out plugin not loaded */
-
   }
 
-  /*
-   * By checking the CC here, we avoid the need to check for the no-payload
-   * case (i.e. adaptation field only) when it does not get bumped.
-   */
+  /* By checking the CC here, we avoid the need to check for the no-payload
+     case (i.e. adaptation field only) when it does not get bumped. */
   if (m->counter != INVALID_CC) {
     if ((m->counter & 0x0f) != cc) {
-      LOG_MSG (this->xine, _("demux_ts: dropped input packet cc: %d "
-                             "expected: %d\n"),
-                           cc, m->counter);
+      LOG_MSG (this->xine, _("demux_ts: unexpected cc %d (expected %d)\n"),
+	       cc, m->counter);
     }
   }
-
   m->counter = cc;
   m->counter++;
 
-  if (pus) {
+  if (pus) { /* new PES packet */
 
-    /* new PES packet */
-
-    if (ts[0] || ts[1] || ts[2] != 1) {
-      LOG_MSG_STDERR (this->xine, _("demux_ts: PUSI set but no PES header "
-                                    "(corrupt stream?)\n"));
-      return;
+    if (m->buffered_bytes) {
+      m->buf->content = m->buf->mem;
+      m->buf->size = m->buffered_bytes;
+      m->buf->type = m->type;
+      m->buf->pts = m->pts;
+      m->buf->decoder_info[0] = 1;
+      m->buf->input_pos = this->input->get_current_pos(this->input);
+      m->fifo->put(m->fifo, m->buf);
+      m->buffered_bytes = 0;
+      m->buf = NULL; /* forget about buf -- not our responsibility anymore */
     }
 
-
-    if (!demux_ts_parse_pes_header (m, ts, len, this->xine)) {
-      m->broken_pes = 1;
-      LOG_MSG (this->xine, _("demux_ts: broken pes encountered\n"));
-    } else {
-      check_newpts( this, m->PTS );
-      
-      m->broken_pes = 0;
-      buf = m->fifo->buffer_pool_alloc(m->fifo);
-      memcpy (buf->mem, ts+len-m->size, m->size); /* FIXME: reconstruct parser to do without memcpys */
-      buf->content         = buf->mem;
-      buf->size            = m->size;
-      buf->type            = m->type;
-      buf->pts             = m->PTS;
-      /* buf->scr             = this->PCR; */
-      buf->decoder_info[0] = 1;
-      m->fifo->put (m->fifo, buf);
+    if (!demux_ts_parse_pes_header(m, ts, len, this->xine)) {
+      m->corrupted_pes = 1;
+      LOG_MSG (this->xine, _("demux_ts: corrupted pes encountered\n"));
     }
+    else {
+      check_newpts(this, m->pts);
+      m->corrupted_pes = 0;
+      m->buf = m->fifo->buffer_pool_alloc(m->fifo);
+      memcpy(m->buf->mem, ts+len-m->size, m->size);
+      m->buffered_bytes = m->size;
+    }
+  }
 
-  } else if (!m->broken_pes) {
-    buf = m->fifo->buffer_pool_alloc(m->fifo);
-    memcpy (buf->mem, ts, len); /* FIXME: reconstruct parser to do without memcpys */
-    buf->content         = buf->mem;
-    buf->size            = len;
-    buf->type            = m->type;
-    buf->pts             = 0;
-    /* buf->scr             = 0; */
-    buf->input_pos       = this->input->get_current_pos(this->input);
-    buf->decoder_info[0] = 1;
-    m->fifo->put (m->fifo, buf);
+  else if (!m->corrupted_pes) { /* no pus -- PES packet continuation */
+
+    if ((m->buffered_bytes + len) > MAX_PES_BUF_SIZE) {
+      m->buf->content = m->buf->mem;
+      m->buf->size = m->buffered_bytes;
+      m->buf->type = m->type;
+      m->buf->pts = m->pts;
+      m->buf->decoder_info[0] = 1;
+      m->buf->input_pos = this->input->get_current_pos(this->input);
+      m->fifo->put(m->fifo, m->buf);
+      m->buffered_bytes = 0;
+      m->buf = m->fifo->buffer_pool_alloc(m->fifo);
+    }
+    memcpy(m->buf->mem + m->buffered_bytes, ts, len);
+    m->buffered_bytes += len;
   }
 }
 
@@ -600,18 +635,20 @@ static void demux_ts_buffer_pes(demux_ts *this, unsigned char *ts,
 static void demux_ts_pes_new(demux_ts *this,
                              unsigned int mediaIndex,
                              unsigned int pid,
-                             fifo_buffer_t *fifo) {
+                             fifo_buffer_t *fifo,
+			     uint8_t descriptor) {
 
   demux_ts_media *m = &this->media[mediaIndex];
 
   /* new PID seen - initialise stuff */
   m->pid = pid;
   m->fifo = fifo;
-  m->buf = 0;
-  m->pes_buf_next = 0;
-  m->pes_len = 0;
+  if (m->buf != NULL) m->buf->free_buffer(m->buf);
+  m->buf = NULL;
   m->counter = INVALID_CC;
-  m->broken_pes = 1;
+  m->descriptor_tag = descriptor;
+  m->corrupted_pes = 1;
+  m->buffered_bytes = 0;
 }
 
 /*
@@ -677,9 +714,8 @@ static void demux_ts_parse_pmt (demux_ts      *this,
    * to copy the complete section into one chunk.
    */
   if (pusi) {
-    free (this->pmt[program_count]);
-    this->pmt[program_count] =
-      (uint8_t *) calloc (1024, sizeof (unsigned char));
+    if (this->pmt[program_count] != NULL) free(this->pmt[program_count]);
+    this->pmt[program_count] = (uint8_t *) calloc(1024, sizeof(unsigned char));
     this->pmt_write_ptr[program_count] = this->pmt[program_count];
 
     table_id                  =  pkt[5] ;
@@ -820,10 +856,11 @@ static void demux_ts_parse_pmt (demux_ts      *this,
 #ifdef TS_PMT_LOG
         printf ("demux_ts: PMT video pid %.4x\n", pid);
 #endif
-        demux_ts_pes_new(this, mediaIndex, pid, this->video_fifo);
+        demux_ts_pes_new(this, mediaIndex, pid, this->video_fifo,stream[0]);
+	this->videoMedia = mediaIndex;
+	this->videoPid = pid;
       }
-      this->videoPid = pid;
-      this->videoMedia = mediaIndex;
+
       break;
     case ISO_11172_AUDIO:
     case ISO_13818_AUDIO:
@@ -831,7 +868,7 @@ static void demux_ts_parse_pmt (demux_ts      *this,
 #ifdef TS_PMT_LOG
         printf ("demux_ts: PMT audio pid %.4x\n", pid);
 #endif
-        demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo);
+        demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo,stream[0]);
         this->audioPid = pid;
         this->audioMedia = mediaIndex;
       }
@@ -851,7 +888,7 @@ static void demux_ts_parse_pmt (demux_ts      *this,
 #ifdef TS_PMT_LOG
           printf ("demux_ts: PMT AC3 audio pid %.4x\n", pid);
 #endif
-          demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo);
+          demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo,stream[0]);
           this->audioPid = pid;
           this->audioMedia = mediaIndex;
           break;
@@ -867,10 +904,7 @@ static void demux_ts_parse_pmt (demux_ts      *this,
       printf ("\n");
 #endif
       if (this->audioPid == INVALID_PID) {
-#ifdef TS_PMT_LOG
-        printf ("demux_ts: PMT AC3 audio pid %.4x\n", pid);
-#endif
-        demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo);
+	demux_ts_pes_new(this, mediaIndex, pid, this->audio_fifo, stream[0]);
         this->audioPid = pid;
         this->audioMedia = mediaIndex;
       }
@@ -908,53 +942,143 @@ static void demux_ts_parse_pmt (demux_ts      *this,
   }
 }
 
-void correct_for_sync(demux_ts *this, uint8_t *buf) {
-  int32_t n, read_length;
-  if((buf[0] == SYNC_BYTE) && (buf[PKT_SIZE] == SYNC_BYTE) &&
-     (buf[PKT_SIZE*2] == SYNC_BYTE) && (buf[PKT_SIZE*3] == SYNC_BYTE)) {
-        return;
+
+/*
+ *
+ */
+static int sync_correct(demux_ts *this, uint8_t *buf, int32_t npkt_read) {
+
+  int p = 0;
+  int n = 0;
+  int i = 0;
+  int sync_ok = 0;
+  int read_length;
+
+  fprintf(stderr, "demux_ts: about to resync!\n");
+
+  for (p=0; p < npkt_read; p++) {
+    for(n=0; n < PKT_SIZE; n++) {
+      sync_ok = 1;
+      for (i=0; i < MIN(MIN_SYNCS, npkt_read - p); i++) {
+	if (buf[n + ((i+p) * PKT_SIZE)] != SYNC_BYTE) {
+	  sync_ok = 0;
+	  break;
+	}
+      }
+      if (sync_ok) break;
+    }
+    if (sync_ok) break;
   }
-  for(n=1;n<PKT_SIZE;n++) {
-    if((buf[n] == SYNC_BYTE) && (buf[n+PKT_SIZE] == SYNC_BYTE) &&
-     (buf[n+(PKT_SIZE*2)] == SYNC_BYTE) && (buf[n+(PKT_SIZE*3)] == SYNC_BYTE)) {
-      /* Found sync, fill in */
-     memmove(&buf[0],&buf[n],((PKT_SIZE*MIN_SYNCS)-n));
-     read_length = this->input->read(this->input, &buf[(PKT_SIZE*MIN_SYNCS)-n], n);
-     return;
+
+  if (sync_ok) {
+    /* Found sync, fill in */
+    memmove(&buf[0], &buf[n + p * PKT_SIZE],
+	    ((PKT_SIZE * (npkt_read - p)) - n));
+    read_length = this->input->read(this->input,
+				    &buf[(PKT_SIZE * (npkt_read - p)) - n],
+				    n + p * PKT_SIZE);
+    /* FIXME: when read_length is not as required... we now stop demuxing */
+    if (read_length != (n + p * PKT_SIZE)) {
+      fprintf(stderr, "demux_ts sync_correct: sync found, but read failed\n");
+      return 0;
     }
   }
-  LOG_MSG(this->xine, _("RE-Sync failed\n")); /* Sync up here */
-  return;
+  else {
+    fprintf(stderr, "demux_ts sync_correct: sync not found! Stop demuxing\n");
+    return 0;
+  }
+  fprintf(stderr, "demux_ts: resync successful!\n");
+  return 1;
+}
 
+/*
+ *
+ */
+static int sync_detect(demux_ts *this, uint8_t *buf, int32_t npkt_read) {
+
+  int i, sync_ok;
+
+  sync_ok = 1;
+
+  for (i=0; i < MIN(MIN_SYNCS, npkt_read); i++) {
+    if (buf[i * PKT_SIZE] != SYNC_BYTE) {
+      sync_ok = 0;
+      break;
+    }
+  }
+  if (!sync_ok) return sync_correct(this, buf, npkt_read);
+  return sync_ok;
 }
 
 
-/* Main synchronisation routine.
+/*
+ *  Main synchronisation routine.
  */
-
 static unsigned char * demux_synchronise(demux_ts * this) {
-  static int32_t packet_number=MIN_SYNCS;
-  static uint8_t buf[BUF_SIZE]; /* This should change to a malloc. */
-  uint8_t       *return_pointer = NULL;
-  int32_t n, read_length;
 
-  if (packet_number == MIN_SYNCS) {
-    for(n=0;n<MIN_SYNCS;n++) {
-      read_length = this->input->read(this->input, &buf[n*PKT_SIZE], PKT_SIZE);
-      if(read_length != PKT_SIZE) {
-        this->status = DEMUX_FINISHED;
-        return NULL;
+  static int32_t packet_number = 0;
+  // NEW: var to keep track of number of last read packets
+  static int32_t npkt_read = 0;
+  static int32_t read_zero = 0;
+
+  static uint8_t buf[BUF_SIZE]; /* This should change to a malloc. */
+  uint8_t *return_pointer = NULL;
+  int32_t read_length;
+  if (packet_number >= npkt_read) {
+
+    /* NEW: handle read returning less packets than NPKT_PER_READ... */
+    do {
+      read_length = this->input->read(this->input, buf,
+				      PKT_SIZE * NPKT_PER_READ);
+      if (read_length % PKT_SIZE) {
+	fprintf(stderr,
+		"demux_ts: read returned %d bytes (not a multiple of %d!)\n",
+		read_length, PKT_SIZE);
+	this->status = DEMUX_FINISHED;
+	return NULL;
       }
+      npkt_read = read_length / PKT_SIZE;
+
+#ifdef TS_READ_STATS
+      this->rstat[npkt_read]++;
+#endif
+      // what if npkt_read < 5 ? --> ok in sync_detect
+
+      // NEW: stop demuxing if read returns 0 a few times... (200)
+
+      if (npkt_read == 0) {
+	// fprintf(stderr, "demux_ts: read 0 packets! (%d)\n", read_zero);
+	read_zero++;
+      }
+      else read_zero = 0;
+
+      if (read_zero > 200) {
+	fprintf(stderr, "demux_ts: read 0 packets too many times!\n");
+	this->status = DEMUX_FINISHED;
+	return NULL;
+      }
+
+    } while (! read_length);
+
+    packet_number = 0;
+
+    if (!sync_detect(this, &buf[0], npkt_read)) {
+      fprintf(stderr, "demux_ts: sync error.\n");
+      this->status = DEMUX_FINISHED;
+      return NULL;
     }
-    packet_number=0;
-    correct_for_sync(this,&buf[0]);
+
   }
-  return_pointer=&buf[PKT_SIZE*packet_number];
+  return_pointer = &buf[PKT_SIZE * packet_number];
   packet_number++;
   return return_pointer;
 }
 
-static uint32_t demux_ts_adaptation_field_parse( uint8_t *data, uint32_t adaptation_field_length) {
+
+static uint32_t 
+demux_ts_adaptation_field_parse(uint8_t *data, 
+				uint32_t adaptation_field_length) {
+
   uint32_t    discontinuity_indicator=0;
   uint32_t    random_access_indicator=0;
   uint32_t    elementary_stream_priority_indicator=0;
@@ -1052,6 +1176,7 @@ static void demux_ts_parse_packet (demux_ts *this) {
   unsigned int   data_offset;
   unsigned int   data_len;
   uint32_t       program_count;
+  int i;
 
   /* get next synchronised packet, or NULL */
   originalPkt = demux_synchronise(this);
@@ -1062,7 +1187,8 @@ static void demux_ts_parse_packet (demux_ts *this) {
   transport_error_indicator      = (originalPkt[1]  >> 7) & 0x01;
   payload_unit_start_indicator   = (originalPkt[1] >> 6) & 0x01;
   transport_priority             = (originalPkt[1] >> 5) & 0x01;
-  pid                            = ((originalPkt[1] << 8) | originalPkt[2]) & 0x1fff;
+  pid                            = ((originalPkt[1] << 8) | 
+				    originalPkt[2]) & 0x1fff;
   transport_scrambling_control   = (originalPkt[3] >> 6)  & 0x03;
   adaptation_field_control       = (originalPkt[3] >> 4) & 0x03;
   continuity_counter             = originalPkt[3] & 0x0f;
@@ -1079,6 +1205,20 @@ static void demux_ts_parse_packet (demux_ts *this) {
     LOG_MSG_STDERR(this->xine, _("demux error! transport error\n"));
     return;
   }
+  if (transport_scrambling_control) {
+    if (this->videoPid == pid) {
+      fprintf(stderr, "demux_ts: selected videoPid is scrambled. Stopping.\n");
+      this->status = DEMUX_FINISHED;
+    }
+    for (i=0; i < this->scrambled_npids; i++) {
+      if (this->scrambled_pids[i] == pid) return;
+    }
+    this->scrambled_pids[this->scrambled_npids] = pid;
+    this->scrambled_npids++;
+    // LOG_MSG_STDERR(this->xine, _("demux_ts: PID %d is scrambled!\n"), pid);
+    fprintf(stderr, "demux_ts: PID %d is scrambled!\n", pid);
+    return;
+  }
 
   data_offset = 4;
   if (adaptation_field_control & 0x1) {
@@ -1124,8 +1264,9 @@ static void demux_ts_parse_packet (demux_ts *this) {
 
     if (data_len > PKT_SIZE) {
 
-      LOG_MSG (this->xine, _("demux_ts: demux error! invalid payload size %d\n"),
-                           data_len);
+      LOG_MSG (this->xine, 
+	       _("demux_ts: demux error! invalid payload size %d\n"),
+	       data_len);
 
     } else {
 
@@ -1182,53 +1323,74 @@ static void demux_ts_parse_packet (demux_ts *this) {
   }
 }
 
+
 /*
  * Sit in a loop eating data.
  */
 static void *demux_ts_loop(void *gen_this) {
 
   demux_ts *this = (demux_ts *)gen_this;
-  buf_element_t *buf;
+  int i;
+
+  pthread_mutex_lock( &this->mutex );
+  /* do-while needed to seek after demux finished */
+  do {
+
+    /* main demuxer loop */
+    while(this->status == DEMUX_OK) {
+
+      demux_ts_parse_packet(this);
+
+      /* someone may want to interrupt us */
+      pthread_mutex_unlock( &this->mutex );
+      pthread_mutex_lock( &this->mutex );
+    }
+
+    /* wait before sending end buffers: user might want to do a new seek */
+    while(this->send_end_buffers && this->video_fifo->size(this->video_fifo) &&
+          this->status != DEMUX_OK){
+      pthread_mutex_unlock( &this->mutex );
+      xine_usec_sleep(100000);
+      pthread_mutex_lock( &this->mutex );
+    }
+     
+  } while( this->status == DEMUX_OK );
 
-  while(1) {
-    
-    pthread_mutex_lock( &this->mutex );
-    
-    if( this->status != DEMUX_OK)
-      break;
-    
-    demux_ts_parse_packet(this);
-    
-    pthread_mutex_unlock( &this->mutex );
-  
-  }
   
 #ifdef TS_LOG
   printf ("demux_ts: demux loop finished (status: %d)\n", this->status);
 #endif
-  
-  this->status = DEMUX_FINISHED;
-  if (this->send_end_buffers) {
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type            = BUF_CONTROL_END;
-      buf->decoder_flags   = BUF_FLAG_END_STREAM; /* stream finished */
-      this->audio_fifo->put (this->audio_fifo, buf);
+ 
+  for (i = 0; i < MAX_PIDS; i++) {
+    if (this->media[i].buf != NULL) {
+      this->media[i].buf->free_buffer(this->media[i].buf);
+      this->media[i].buf = NULL;
     }
   }
+ 
+  this->status = DEMUX_FINISHED;
+  if (this->send_end_buffers) {
+    xine_demux_control_end(this->xine, BUF_FLAG_END_STREAM);
+  }
 
+  this->thread_running = 0;
   pthread_mutex_unlock( &this->mutex );
   pthread_exit(NULL);
   return NULL;
 }
 
-static void demux_ts_close(demux_plugin_t *gen_this) {
-  free (gen_this);
+static void demux_ts_close(demux_plugin_t *this_gen) {
+  int i;
+  demux_ts *this = (demux_ts *)this_gen;
+
+  for (i=0; i < MAX_PMTS; i++) {
+    if (this->pmt[i] != NULL) free(this->pmt[i]);
+  }
+  for (i=0; i < MAX_PIDS; i++) {
+    if (this->media[i].buf != NULL) 
+      this->media[i].buf->free_buffer(this->media[i].buf);
+  }
+  free(this_gen);
 }
 
 static char *demux_ts_get_id(void) {
@@ -1243,7 +1405,7 @@ static int demux_ts_get_status(demux_plugin_t *this_gen) {
 
   demux_ts *this = (demux_ts *)this_gen;
 
-  return this->status;
+  return (this->thread_running?DEMUX_OK:DEMUX_FINISHED);
 }
 
 static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
@@ -1259,12 +1421,20 @@ static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
   case STAGE_BY_CONTENT: {
     uint8_t buf[4096];
 
+    // Mauro -- Say DEMUX_CAN_HANDLE with dvb input plugin, even if
+    // non-seekable and non-previewable.
+    if (strncasecmp(input->get_identifier(input), "dvb", 4) == 0) {
+      this->input = input;
+      return DEMUX_CAN_HANDLE;
+    }
+
+    
     if((input->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0) {
       input->seek(input, 0, SEEK_SET);
-
-      if(input->get_blocksize(input))
-       return DEMUX_CANNOT_HANDLE;
-
+      /* why is this a restriction ???      
+       if(input->get_blocksize(input))
+ 	return DEMUX_CANNOT_HANDLE;
+      */
       if(input->read(input, buf, 6)) {
 
        if(buf[0] == 0x47)
@@ -1279,6 +1449,7 @@ static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
 
       if(buf[0] == 0x47) {
 	this->input = input;
+        this->blockSize = PKT_SIZE;
 	return DEMUX_CAN_HANDLE;
       }
     }
@@ -1288,11 +1459,12 @@ static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
   break;
   case STAGE_BY_EXTENSION:
 
-    xine_strdupa(valid_mrls, (this->config->register_string(this->config,
-                                                            "mrl.mrls_ts", VALID_MRLS,
-                                                            "valid mrls for ts demuxer",
-                                                            NULL, NULL, NULL)));
-
+    xine_strdupa(valid_mrls, 
+		 (this->config->register_string(this->config,
+						"mrl.mrls_ts", VALID_MRLS,
+						_("valid mrls for ts demuxer"),
+						NULL, NULL, NULL)));
+    
     mrl = input->get_mrl(input);
     media = strstr(mrl, "://");
 
@@ -1322,10 +1494,11 @@ static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
       LOG_MSG(this->xine, "demux_ts_open: ending %s of %s\n", ending, mrl);
 #endif
 
-      xine_strdupa(valid_ends, (this->config->register_string(this->config,
-                                                              "mrl.ends_ts", VALID_ENDS,
-                                                              "valid mrls ending for ts demuxer",
-                                                              NULL, NULL, NULL)));
+      xine_strdupa(valid_ends, 
+		   (this->config->register_string(this->config,
+						  "mrl.ends_ts", VALID_ENDS,
+						  _("valid mrls ending for ts demuxer"),
+						  NULL, NULL, NULL)));
       while((m = xine_strsep(&valid_ends, ",")) != NULL) {
 
         while(*m == ' ' || *m == '\t') m++;
@@ -1347,18 +1520,21 @@ static int demux_ts_open(demux_plugin_t *this_gen, input_plugin_t *input,
   return DEMUX_CAN_HANDLE;
 }
 
-static void demux_ts_start(demux_plugin_t *this_gen,
+static int demux_ts_start(demux_plugin_t *this_gen,
                            fifo_buffer_t *video_fifo,
                            fifo_buffer_t *audio_fifo,
                            off_t start_pos, int start_time) {
 
   demux_ts *this = (demux_ts *)this_gen;
-  buf_element_t *buf;
   int err;
+  int status;
 
   pthread_mutex_lock( &this->mutex );
 
-  if( this->status != DEMUX_OK ) {
+  this->videoPid = INVALID_PID;
+  this->audioPid = INVALID_PID;
+
+  if( !this->thread_running ) {
     this->video_fifo  = video_fifo;
     this->audio_fifo  = audio_fifo;
 
@@ -1366,20 +1542,12 @@ static void demux_ts_start(demux_plugin_t *this_gen,
      * send start buffer
      */
 
-    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-    buf->type    = BUF_CONTROL_START;
-    this->video_fifo->put (this->video_fifo, buf);
-
-    if(this->audio_fifo) {
-      buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-      buf->type    = BUF_CONTROL_START;
-      this->audio_fifo->put (this->audio_fifo, buf);
-    }
+    xine_demux_control_start(this->xine);
   }
   
-  if ((this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE) != 0 ) {
+  if (this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) {
 
-    if ( (!start_pos) && (start_time))
+    if ((!start_pos) && (start_time))
       start_pos = start_time * this->rate * 50;
 
     this->input->seek (this->input, start_pos, SEEK_SET);
@@ -1390,42 +1558,76 @@ static void demux_ts_start(demux_plugin_t *this_gen,
   
   demux_ts_build_crc32_table(this);
   
-  if( this->status != DEMUX_OK ) {
+  this->status = DEMUX_OK ;
+  if( !this->thread_running ) {
     /*
      * Now start demuxing.
      */
-    this->status = DEMUX_OK ;
     this->send_end_buffers = 1;
     this->last_PCR = 0;
+    this->thread_running = 1;
+    this->scrambled_npids = 0;
   
     if ((err = pthread_create(&this->thread, NULL, demux_ts_loop, this)) != 0) {
-      LOG_MSG_STDERR(this->xine, _("demux_ts: can't create new thread (%s)\n"), strerror(err));
-      exit (1);
+      LOG_MSG_STDERR(this->xine, 
+		     _("demux_ts: can't create new thread (%s)\n"), 
+		     strerror(err));
+      abort();
     }
   }
   else {
-    xine_flush_engine(this->xine);
+    xine_demux_flush_engine(this->xine);
   }
+
+  /* this->status is saved because we can be interrupted between
+   * pthread_mutex_unlock and return
+   */
+  status = this->status;
   pthread_mutex_unlock( &this->mutex );
+  return status;
 }
 
-static void demux_ts_seek (demux_plugin_t *this_gen,
+static int demux_ts_seek (demux_plugin_t *this_gen,
 			     off_t start_pos, int start_time) {
   demux_ts *this = (demux_ts *)this_gen;
 
-	demux_ts_start (this_gen, this->video_fifo, this->audio_fifo,
+	return demux_ts_start (this_gen, this->video_fifo, this->audio_fifo,
 			 start_pos, start_time);
 }
 
 static void demux_ts_stop(demux_plugin_t *this_gen)
 {
   demux_ts *this = (demux_ts *)this_gen;
-  buf_element_t *buf;
   void *p;
 
+#ifdef TS_READ_STATS
+  int i, prev_zero;
+  uint32_t pkt_total;
+
+  pkt_total = 0;
+  for (i=0; i<=NPKT_PER_READ; i++) {
+    pkt_total += this->rstat[i];
+  }
+  prev_zero = 1;
+  for (i=0; i<=NPKT_PER_READ; i++) {
+    if (this->rstat[i]) {
+      fprintf(stderr, "%3d pkts --> %6.3f%% [ %d ]\n", i,
+	      (this->rstat[i] * 100.0 / pkt_total), this->rstat[i]);
+      prev_zero = 0;
+    }
+    else if (prev_zero) {
+
+    }
+    else {
+      fprintf(stderr, "...\n");
+      prev_zero = 1;
+    }
+  }
+#endif
+
   pthread_mutex_lock( &this->mutex );
   
-  if (this->status != DEMUX_OK) {
+  if (!this->thread_running) {
     printf ("demux_ts: stop...ignored\n");
     pthread_mutex_unlock( &this->mutex );
     return;
@@ -1437,19 +1639,9 @@ static void demux_ts_stop(demux_plugin_t *this_gen)
   pthread_mutex_unlock( &this->mutex );
   pthread_join (this->thread, &p);
 
-  xine_flush_engine(this->xine);
+  xine_demux_flush_engine(this->xine);
 
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_END;
-  buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if (this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_END;
-    buf->decoder_flags   = BUF_FLAG_END_USER; /* user finished */
-    this->audio_fifo->put (this->audio_fifo, buf);
-  }
+  xine_demux_control_end(this->xine, BUF_FLAG_END_USER);
 }
 
 static int demux_ts_get_stream_length (demux_plugin_t *this_gen) {
@@ -1465,11 +1657,12 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   demux_ts        *this;
   int              i;
 
-  if (iface != 7) {
+  if (iface != 10) {
     LOG_MSG (xine,
              _("demux_ts: plugin doesn't support plugin API version %d.\n"
-               "          this means there's a version mismatch between xine and this "
-               "          demuxer plugin.\nInstalling current demux plugins should help.\n"),
+               "          This means there's a version mismatch between xine "
+	       "and this demuxer plugin.\n"
+               "          Installing current demux plugins should help.\n"),
              iface);
     return NULL;
   }
@@ -1481,12 +1674,13 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->config = xine->config;
   this->xine   = xine;
 
-  (void*) this->config->register_string(this->config, "mrl.mrls_ts", VALID_MRLS,
-                                        "valid mrls for ts demuxer",
+  (void*) this->config->register_string(this->config, "mrl.mrls_ts", 
+					VALID_MRLS,
+                                        _("valid mrls for ts demuxer"),
                                         NULL, NULL, NULL);
   (void*) this->config->register_string(this->config,
                                         "mrl.ends_ts", VALID_ENDS,
-                                        "valid mrls ending for ts demuxer",
+                                        _("valid mrls ending for ts demuxer"),
                                         NULL, NULL, NULL);
 
   this->plugin.interface_version = DEMUXER_PLUGIN_IFACE_VERSION;
@@ -1503,8 +1697,10 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   /*
    * Initialise our specialised data.
    */
-  for (i = 0; i < MAX_PIDS; i++)
+  for (i = 0; i < MAX_PIDS; i++) {
     this->media[i].pid = INVALID_PID;
+    this->media[i].buf = NULL;
+  }
 
   for (i = 0; i < MAX_PMTS; i++) {
     this->program_number[i]          = INVALID_PROGRAM;
@@ -1516,6 +1712,7 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->programNumber = INVALID_PROGRAM;
   this->pcrPid = INVALID_PID;
   this->PCR    = 0;
+  this->scrambled_npids = 0;
   this->videoPid = INVALID_PID;
   this->audioPid = INVALID_PID;
 
@@ -1524,8 +1721,11 @@ demux_plugin_t *init_demuxer_plugin(int iface, xine_t *xine) {
   this->status = DEMUX_FINISHED;
   pthread_mutex_init( &this->mutex, NULL );
 
+#ifdef TS_READ_STATS
+  for (i=0; i<=NPKT_PER_READ; i++) {
+    this->rstat[i] = 0;
+  }
+#endif
+
   return (demux_plugin_t *)this;
 }
-
-
-
