@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.47 2002/04/24 08:12:13 f1rmb Exp $
+ * $Id: input_dvd.c,v 1.52 2002/07/05 17:32:01 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -128,13 +128,13 @@ typedef struct {
 static void device_change_cb(void *data, cfg_entry_t *cfg) {
   dvd_input_plugin_t *this = (dvd_input_plugin_t *) data;
   
-  this->device = strdup(cfg->str_value);
+  this->device = cfg->str_value;
 }
 
 static void rawdevice_change_cb(void *data, cfg_entry_t *cfg) {
   dvd_input_plugin_t *this = (dvd_input_plugin_t *) data;
   
-  this->raw_device = strdup(cfg->str_value);
+  this->raw_device = cfg->str_value;
 }
 
 static int openDrive (dvd_input_plugin_t *this) {
@@ -343,8 +343,9 @@ static int openDVDFile (dvd_input_plugin_t *this,
 
   if( encrypted ) {
     LOG_MSG(this->xine,
-	    _("\ninput_dvd: Sorry, xine doesn't play encrypted DVDs. The legal status of CSS\n"
-	      "           decryption is unclear and we will not provide such code.\n\n"));
+	    _("\ninput_dvd: Sorry, this plugin doesn't play encrypted DVDs. The legal status\n"
+	      "           of CSS decryption is unclear and we can't provide such code.\n"
+	      "           Please check http://dvd.sf.net for more information.\n"));
     return 0;
   }
 
@@ -808,7 +809,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   config_values_t    *config;
   int i;
 
-  if (iface != 6) {
+  if (iface != 8) {
     LOG_MSG(xine,
 	    _("dvd input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -853,10 +854,10 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   */
   
   this->device = config->register_string(config, "input.dvd_device", DVD,
-					 "path to your local dvd device file",
+					 _("path to your local dvd device file"),
 					 NULL, device_change_cb, (void *)this);
   this->raw_device = config->register_string(config, "input.dvd_raw_device", RDVD,
-					     "path to a raw device set up for dvd access",
+					     _("path to a raw device set up for dvd access"),
 					     NULL, rawdevice_change_cb, (void*)this);
 #ifdef __sun
   check_solaris_vold_device(this);
