@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.47 2002/04/24 08:12:13 f1rmb Exp $
+ * $Id: input_file.c,v 1.52 2002/07/05 17:32:02 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -282,8 +282,8 @@ static int file_plugin_open (input_plugin_t *this_gen, char *mrl) {
 
   this->mrl = strdup(mrl);
 
-  if (!strncasecmp (this->mrl, "file:",5))
-    filename = &this->mrl[5];
+  if (!strncasecmp (this->mrl, "file://", 7))
+    filename = &this->mrl[7];
   else
     filename = this->mrl;
 
@@ -805,9 +805,10 @@ static int file_plugin_get_optional_data (input_plugin_t *this_gen,
   
   file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
 
+#ifdef LOG
   LOG_MSG(this->xine, _("input_file: get optional data, type %08x, sub %p\n"),
 	  data_type, this->sub);
-
+#endif
 
   switch(data_type) {
   case INPUT_OPTIONAL_DATA_TEXTSPU0:
@@ -854,7 +855,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   file_input_plugin_t *this;
   config_values_t     *config;
 
-  if (iface != 6) {
+  if (iface != 8) {
     LOG_MSG(xine,
 	    _("file input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -904,12 +905,12 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
       strcpy(current_dir, ".");
 
     this->origin_path = config->register_string(this->config, "input.file_origin_path",
-						current_dir, "origin path to grab file mrls",
+						current_dir, _("origin path to grab file mrls"),
 						NULL, origin_change_cb, (void *) this);
   }
   
   this->show_hidden_files = this->config->register_bool(this->config, "input.file_hidden_files", 
-							1, "hidden files displaying.",
+							1, _("hidden files displaying."),
 							NULL, hidden_bool_cb, (void *) this);
   
   return (input_plugin_t *) this;
