@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_vcd.c,v 1.41 2002/04/24 08:12:13 f1rmb Exp $
+ * $Id: input_vcd.c,v 1.46 2002/07/05 17:32:02 mroi Exp $
  *
  */
 
@@ -145,7 +145,7 @@ typedef struct {
 static void device_change_cb(void *data, cfg_entry_t *cfg) {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) data;
   
-  this->device = strdup(cfg->str_value);
+  this->device = cfg->str_value;
 }
 
 #if defined (__linux__) || defined(__sun)
@@ -1068,7 +1068,7 @@ static char **vcd_plugin_get_autoplay_list (input_plugin_t *this_gen,
   this->fd = open (this->device, O_RDONLY);
 
   if (this->fd == -1) {
-    LOG_MSG_STDERR(this->xine, _("unable to open %s: %s."), this->device, strerror(errno));
+    LOG_MSG_STDERR(this->xine, _("unable to open %s: %s.\n"), this->device, strerror(errno));
     return NULL;
   }
 
@@ -1141,7 +1141,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   config_values_t    *config;
   int                 i;
 
-  if (iface != 6) {
+  if (iface != 8) {
     LOG_MSG(xine, 
 	    _("vcd input plugin doesn't support plugin API version %d.\n"
 	      "PLUGIN DISABLED.\n"
@@ -1181,7 +1181,7 @@ input_plugin_t *init_input_plugin (int iface, xine_t *xine) {
   this->input_plugin.is_branch_possible= NULL;
   
   this->device = config->register_string(config, "input.vcd_device", CDROM,
-					 "path to your local vcd device file",
+					 _("path to your local vcd device file"),
 					 NULL, device_change_cb, (void *)this);
 
   this->mrls = (mrl_t **) xine_xmalloc(sizeof(mrl_t*));
