@@ -22,6 +22,7 @@
 #include "config.h"
 
 #include <inttypes.h>
+#include <stdio.h> /* For printf */
 #include <stdlib.h>
 #include <string.h>
 
@@ -40,16 +41,21 @@ static int q_4_pointer;
 
 static uint8_t halfrate[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3};
 
-sample_t * a52_init (uint32_t mm_accel)
+sample_t * a52_init (uint32_t mm_accel, sample_t **samples_base)
 {
     sample_t * samples;
     int i;
 
     imdct_init (mm_accel);
 
-    samples = xine_xmalloc_aligned (16, 256 * 12 * sizeof (sample_t));
-    if (samples == NULL)
+    samples = xine_xmalloc_aligned (16, 256 * 12 * sizeof (sample_t),
+				    (void**) samples_base);
+    if (samples == NULL) {
+
+      printf ("liba52: samples malloc failed!\n");
+
 	return NULL;
+    }
 
     for (i = 0; i < 256 * 12; i++)
 	samples[i] = 0;
