@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.25 2002/04/23 15:45:21 esnel Exp $
+ * $Id: xine_decoder.c,v 1.28 2002/07/05 17:32:02 mroi Exp $
  *
  * stuff needed to turn liba52 into a xine decoder plugin
  */
@@ -546,11 +546,11 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
   a52dec_decoder_t *this ;
   config_values_t *cfg;
 
-  if (iface_version != 7) {
-    printf( "liba52: plugin doesn't support plugin API version %d.\n"
-	    "liba52: this means there's a version mismatch between xine and this "
-	    "liba52: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 9) {
+    printf(_("liba52: plugin doesn't support plugin API version %d.\n"
+	     "liba52: this means there's a version mismatch between xine and this "
+	     "liba52: decoder plugin.\nInstalling current plugins should help.\n"),
+	     iface_version);
     return NULL;
   }
 
@@ -571,13 +571,13 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   this->a52_level = (float) cfg->register_range (cfg, "codec.a52_level", 100,
 						 0, 200,
-						 "a/52 volume control",
+						 _("a/52 volume control"),
 						 NULL, NULL, NULL) / 100.0;
   this->disable_dynrng = !cfg->register_bool (cfg, "codec.a52_dynrng", 0,
-					      "enable a/52 dynamic range compensation",
+					      _("enable a/52 dynamic range compensation"),
 					      NULL, NULL, NULL);
   this->enable_surround_downmix = cfg->register_bool (cfg, "codec.a52_surround_downmix", 0,
-						      "enable audio downmixing to 2.0 surround stereo",
+						      _("enable audio downmixing to 2.0 surround stereo"),
 						      NULL, NULL, NULL);
 
   return (audio_decoder_t *) this;
