@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.31 2002/04/28 10:12:49 esnel Exp $
+ * $Id: xine_decoder.c,v 1.40 2002/07/15 21:42:33 esnel Exp $
  *
  * xine decoder plugin using divx4
  *
@@ -60,7 +60,9 @@
 #if CATCH_SIGSEGV
 #include <signal.h>
 
-/* #define LOG */
+/*
+#define LOG 
+*/
 
 /* to be able to restore the old handler */
 void (*old_handler)(int);
@@ -73,25 +75,10 @@ void catch_sigsegv(int sig)
          "divx4: divx4_forceversion:1\n"
          "divx4: see xine-ui/doc/README.divx4 for details.\n"
          "divx4: fatal error; exiting.\n");
-  exit(1);
+  abort();
 }
 #endif
 
-/* now this is ripped of wine's vfw.h */
-typedef struct {
-    long        biSize;
-    long        biWidth;
-    long        biHeight;
-    short       biPlanes;
-    short       biBitCount;
-    long        biCompression;
-    long        biSizeImage;
-    long        biXPelsPerMeter;
-    long        biYPelsPerMeter;
-    long        biClrUsed;
-    long        biClrImportant;
-} BITMAPINFOHEADER;
-
 typedef struct divx4_decoder_s {
   video_decoder_t   video_decoder;
 
@@ -99,14 +86,13 @@ typedef struct divx4_decoder_s {
   int               video_step;
   int               decoder_ok;
 
-  BITMAPINFOHEADER  bih;
-  long		    biWidth;
-  long		    biHeight;
+  xine_bmiheader    bih;
   unsigned char     *buf;
   int               size;
   int               bufsize;
   
   decoreFunc        decore; /* ptr to decore function in libdivxdecore */
+
   /* version as reported by decore with GET_OPT_VERSION command */
   int		    version;
   /* whether to decode MSMPEG4_V3 format
@@ -126,11 +112,6 @@ typedef struct divx4_decoder_s {
 
 #define VIDEOBUFSIZE 128*1024
 
-static unsigned long str2ulong(void *data) {
-
-  unsigned char *str = data;
-  return ( str[0] | (str[1]<<8) | (str[2]<<16) | (str[3]<<24) );
-}
 
 static char* decore_retval(int ret)
 {
@@ -203,31 +184,30 @@ static int divx4_check_version(divx4_decoder_t *this)
 static void divx4_set_pp(divx4_decoder_t *this) {
   DEC_SET setpp;   /* for setting postproc level */
   int ret;
+
+#ifdef LOG
+  printf ("divx4: this->decoder_ok=%d\n", this->decoder_ok);
+#endif
+
+  if (!this->decoder_ok)
+    return;
     
   /* multiply postproc level by 10 for internal consumption */
-  printf("divx4: Setting post processing level to %d (see ~/.xinerc)\n"
+  printf("divx4: Setting post processing level to %d (see ~/.xine/options)\n"
          "divx4: Valid range 0-6, reduce if you get frame drop\n", 
          this->postproc); 
-  setpp.postproc_level=this->postproc*10;
-  setpp.deblock_hor_luma = 0;
-  setpp.deblock_ver_luma = 0;
-  setpp.deblock_hor_chr = 0;
-  setpp.deblock_ver_chr = 0;
-  setpp.dering_luma = 0;
-  setpp.dering_chr = 0;
-  setpp.pp_semaphore = 0;
 
+  setpp.postproc_level   = this->postproc*10; 
   setpp.deblock_hor_luma = 0;
   setpp.deblock_ver_luma = 0;
-  setpp.deblock_hor_chr = 0;
-  setpp.deblock_ver_chr = 0;
-  setpp.dering_luma = 0;
-  setpp.dering_chr = 0;
-  setpp.pp_semaphore = 0;
+  setpp.deblock_hor_chr  = 0;
+  setpp.deblock_ver_chr  = 0;
+  setpp.dering_luma      = 0;
+  setpp.dering_chr       = 0;
+  setpp.pp_semaphore     = 0;
 
   ret = this->decore((unsigned long)this, DEC_OPT_SETPP, &setpp, 0);
-  if (ret != DEC_OK)
-  {
+  if (ret != DEC_OK) {
     printf("divx4: decore DEC_OPT_SETPP command returned %s.\n", decore_retval(ret));
     /* perhaps not fatal, so we'll continue */
   }
@@ -235,21 +215,24 @@ static void divx4_set_pp(divx4_decoder_t *this) {
 
 
 /* helper function to initialize decore */
-static int divx4_init_decoder(divx4_decoder_t *this, buf_element_t *buf)
-{
+static int divx4_init_decoder(divx4_decoder_t *this, buf_element_t *buf) {
+
   DEC_PARAM param; /* for init                   */
   int ret, codec_type;
 
-  memcpy ( &this->bih, buf->content, sizeof (BITMAPINFOHEADER));
-  this->biWidth = str2ulong(&this->bih.biWidth);
-  this->biHeight = str2ulong(&this->bih.biHeight);
+#ifdef LOG
+  printf ("divx4: init_decoder\n");
+#endif
+
+  memcpy ( &this->bih, buf->content, sizeof (xine_bmiheader));
   this->video_step = buf->decoder_info[1];
 
   codec_type = buf->type & 0xFFFF0000;
 
   /* do we need divx 3.11 compatibility mode? */
   switch (buf->type & 0xFFFF0000) {
-  case BUF_VIDEO_MSMPEG4_V12:
+  case BUF_VIDEO_MSMPEG4_V1:
+  case BUF_VIDEO_MSMPEG4_V2:
   case BUF_VIDEO_MSMPEG4_V3:
     if (this->version >= 20020303) { 
       param.codec_version=311;
@@ -273,8 +256,8 @@ static int divx4_init_decoder(divx4_decoder_t *this, buf_element_t *buf)
   }
 
   /* setup decoder; inspired by avifile's plugin */
-  param.x_dim=this->biWidth;
-  param.y_dim=this->biHeight;
+  param.x_dim=this->bih.biWidth;
+  param.y_dim=this->bih.biHeight;
   param.time_incr = 15; /* no idea what this does */
 
   param.build_number=0;
@@ -289,6 +272,8 @@ static int divx4_init_decoder(divx4_decoder_t *this, buf_element_t *buf)
     return 0;
   }
 
+  this->decoder_ok = 1;
+
   divx4_set_pp( this );
   
   return 1;
@@ -306,15 +291,15 @@ static inline void divx4_copy_frame(divx4_decoder_t *this, vo_frame_t *img,
   /* copy y data; use shortcut if stride_y equals width */
   src_offset = 0;
   dst_offset = 0;
-  if (pict.stride_y == img->width) {
-    xine_fast_memcpy(img->base[0]+dst_offset, pict.y, this->biWidth*this->biHeight);
-    dst_offset += this->biWidth * this->biHeight;
+  if (pict.stride_y == img->pitches[0]) {
+    xine_fast_memcpy(img->base[0]+dst_offset, pict.y, img->pitches[0]*this->bih.biHeight);
+    dst_offset += img->pitches[0] * this->bih.biHeight;
   }
   else { /* copy line by line */
-    for (i=0; i<this->biHeight; i++) {
-      xine_fast_memcpy(img->base[0]+dst_offset, pict.y+src_offset, this->biWidth);
+    for (i=0; i<this->bih.biHeight; i++) {
+      xine_fast_memcpy(img->base[0]+dst_offset, pict.y+src_offset, this->bih.biWidth);
       src_offset += pict.stride_y;
-      dst_offset += this->biWidth;
+      dst_offset += img->pitches[0];
     }
   }
 
@@ -325,17 +310,19 @@ static inline void divx4_copy_frame(divx4_decoder_t *this, vo_frame_t *img,
  /* copy u and v data */
   src_offset = 0;
   dst_offset = 0;
-  if (pict.stride_uv == img->width>>1) {
-    xine_fast_memcpy(img->base[1]+dst_offset, pict.u, (this->biWidth*this->biHeight)/4);
-    xine_fast_memcpy(img->base[2]+dst_offset, pict.v, (this->biWidth*this->biHeight)/4);
-    dst_offset += (this->biWidth*this->biHeight)/4;
+  if (pict.stride_uv == img->pitches[1] && pict.stride_uv == img->pitches[2]) {
+    xine_fast_memcpy(img->base[1]+dst_offset, pict.u, (img->pitches[1]*this->bih.biHeight)/4);
+    xine_fast_memcpy(img->base[2]+dst_offset, pict.v, (img->pitches[2]*this->bih.biHeight)/4);
+    dst_offset += (this->bih.biWidth*this->bih.biHeight)/4;
   }
   else {
-    for (i=0; i<this->biHeight>>1; i++) {
-      xine_fast_memcpy(img->base[1]+dst_offset, pict.u+src_offset, this->biWidth/2);
-      xine_fast_memcpy(img->base[2]+dst_offset, pict.v+src_offset, this->biWidth/2);
+    int dst_offset_v = 0;
+    for (i=0; i<this->bih.biHeight>>1; i++) {
+      xine_fast_memcpy(img->base[1]+dst_offset, pict.u+src_offset, this->bih.biWidth/2);
+      xine_fast_memcpy(img->base[2]+dst_offset_v, pict.v+src_offset, this->bih.biWidth/2);
       src_offset += pict.stride_uv;
-      dst_offset += this->biWidth/2;
+      dst_offset += img->pitches[1];
+      dst_offset_v += img->pitches[2];
     }
   } 
      
@@ -343,18 +330,18 @@ static inline void divx4_copy_frame(divx4_decoder_t *this, vo_frame_t *img,
      with slices of 16 lines. Too bad we can't set the y,u and v
      stride values (because then we wouldn't need the first copy) */
   if (img->copy && img->bad_frame == 0) {
-    int height = this->biHeight;
-    int stride = this->biWidth;
-    uint8_t* src[3];
-	  
+    int height = img->height;
+    uint8_t *src[3];
+
     src[0] = img->base[0];
     src[1] = img->base[1];
     src[2] = img->base[2];
+
     while ((height -= 16) >= 0) {
       img->copy(img, src);
-      src[0] += 16 * stride;
-      src[1] +=  4 * stride;
-      src[2] +=  4 * stride;
+      src[0] += 16 * img->pitches[0];
+      src[1] +=  8 * img->pitches[1];
+      src[2] +=  8 * img->pitches[2];
     }
   }
 }
@@ -363,9 +350,13 @@ static int divx4_can_handle (video_decoder_t *this_gen, int buf_type) {
   divx4_decoder_t *this = (divx4_decoder_t *) this_gen;
   buf_type &= 0xFFFF0000;
 
+#ifdef LOG
+  printf ("divx4: can_handle\n");
+#endif
+
   /* divx4 currently does not support MSMPEG4 v1/v2 */
   return ( (buf_type == BUF_VIDEO_MSMPEG4_V3 && this->can_handle_311) ||
-           /* buf_type == BUF_VIDEO_MSMPEG4_V12 || */
+           /* buf_type == BUF_VIDEO_MSMPEG4_V2 || */
            (buf_type == BUF_VIDEO_MPEG4) ||
 	   (buf_type == BUF_VIDEO_DIVX5));
 }
@@ -375,6 +366,10 @@ static void divx4_init (video_decoder_t *this_gen, vo_instance_t *video_out) {
 
   divx4_decoder_t *this = (divx4_decoder_t *) this_gen;
 
+#ifdef LOG
+  printf ("divx4: divx4_init\n");
+#endif
+
   this->video_out  = video_out;
   this->decoder_ok = 0;
   this->buf = NULL;
@@ -406,7 +401,7 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
     divx4_get_version(this);	
     this->decoder_ok = ( divx4_check_version(this) &&
-                         divx4_init_decoder(this, buf) );
+                         divx4_init_decoder (this, buf) );
     if (this->decoder_ok) {
       this->video_out->open (this->video_out);
     
@@ -444,8 +439,8 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   if (buf->decoder_flags & BUF_FLAG_FRAME_END)  { /* need to decode a frame */
     /* allocate image (taken from ffmpeg plugin) */
-    img = this->video_out->get_frame (this->video_out, this->biWidth,
-				      this->biHeight, XINE_ASPECT_RATIO_DONT_TOUCH, 
+    img = this->video_out->get_frame (this->video_out, this->bih.biWidth,
+				      this->bih.biHeight, XINE_ASPECT_RATIO_DONT_TOUCH, 
 				      IMGFMT_YV12, 
 				      VO_BOTH_FIELDS);
 
@@ -459,7 +454,7 @@ static void divx4_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     frame.bmp=&pict; /* decore will set ptrs to internal y,u&v buffers */
     frame.length=this->size;
     frame.render_flag=1;
-    frame.stride=this->biWidth;
+    frame.stride=this->bih.biWidth;
 
     if(this->use_311_compat)
       ret = this->decore((unsigned long)this, DEC_OPT_FRAME_311, &frame, 0);
@@ -490,6 +485,8 @@ static void divx4_close (video_decoder_t *this_gen)  {
 
   divx4_decoder_t *this = (divx4_decoder_t *) this_gen;
 
+  printf ("divx4: close\n");
+
   if (this->decoder_ok) {
     /* FIXME: this segfaults here */
     /* Note: we NEED this; after 0.9.4, xine closes and reopens
@@ -505,9 +502,13 @@ static void divx4_close (video_decoder_t *this_gen)  {
   this->buf = NULL;
 }
 
-static void divx4_update_postproc(void *this_gen, cfg_entry_t *entry)
-{
+static void divx4_update_postproc(void *this_gen, cfg_entry_t *entry) {
+
   divx4_decoder_t *this = (divx4_decoder_t *) this_gen;
+
+#ifdef LOG
+  printf ("divx4: update_postproc this=0x%08x, decoder_ok = %d\n", this, this->decoder_ok);
+#endif
   
   if( this->postproc != entry->num_value ) {
     this->postproc = entry->num_value;
@@ -517,17 +518,29 @@ static void divx4_update_postproc(void *this_gen, cfg_entry_t *entry)
 
 
 static char *divx4_get_id(void) {
+#ifdef LOG
+  printf ("divx4: get_id\n");
+#endif
   return "divx4 video decoder";
 }
 
 static void divx4_flush(video_decoder_t *this_gen) {
+#ifdef LOG
+  printf ("divx4: flush\n");
+#endif
 }
 
 static void divx4_reset(video_decoder_t *this_gen) {
   /* seems to handle seeking quite nicelly without any code here */
+#ifdef LOG
+  printf ("divx4: reset\n");
+#endif
 }
 
 static void divx4_dispose(video_decoder_t *this_gen) {
+#ifdef LOG
+  printf ("divx4: dispose\n");
+#endif
   free (this_gen);
 }
 
@@ -541,11 +554,11 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   decoreFunc libdecore_func = 0;
   config_values_t *cfg;
 
-  if (iface_version != 7) {
-    printf( "divx4: plugin doesn't support plugin API version %d.\n"
-	    "divx4: this means there's a version mismatch between xine and this "
-	    "divx4: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 10) {
+    printf(_("divx4: plugin doesn't support plugin API version %d.\n"
+	     "divx4: this means there's a version mismatch between xine and this "
+	     "divx4: decoder plugin.\nInstalling current plugins should help.\n"),
+	     iface_version);
     
     return NULL;
   }
@@ -555,7 +568,7 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
      if it fails, print a message and return 0 so that xine ignores
      us from then on. */
   libdecore_name = cfg->register_string (cfg, "codec.divx4_libdivxdecore", "libdivxdecore.so",
-					 "Relative path to libdivxdecore.so to open",
+					 _("Relative path to libdivxdecore.so to open"),
 					 NULL, NULL, NULL);  
 
   libdecore_handle = dlopen(libdecore_name, RTLD_LAZY);
@@ -568,6 +581,8 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
 
   this = (divx4_decoder_t *) malloc (sizeof (divx4_decoder_t));
 
+  this->decoder_ok = 0;
+
   this->video_decoder.interface_version   = iface_version;
   this->video_decoder.can_handle          = divx4_can_handle;
   this->video_decoder.init                = divx4_init;
@@ -578,20 +593,20 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   this->video_decoder.reset               = divx4_reset;
   this->video_decoder.dispose             = divx4_dispose;
   this->video_decoder.priority            = cfg->register_num (cfg, "codec.divx4_priority", 4,
-							       "priority of the divx4 plugin (>5 => enable)",
+							       _("priority of the divx4 plugin (>5 => enable)"),
 							       NULL, NULL, NULL); 
   this->decore = libdecore_func;
   this->postproc 			  = cfg->register_range (cfg, "codec.divx4_postproc", 3,
 								 0, 6,
-								 "the postprocessing level, 0 = none and fast, 6 = all and slow",
+								 _("the postprocessing level, 0 = none and fast, 6 = all and slow"),
 								 NULL, divx4_update_postproc, this);
   this->can_handle_311			  = cfg->register_bool (cfg, "codec.divx4_msmpeg4v3", 1,
-								"use divx4 plugin for msmpeg4v3 streams",
+								_("use divx4 plugin for msmpeg4v3 streams"),
 								NULL, NULL, NULL);
   this->size				  = 0;
   /* allow override of version checking by user */
   this->version 			  = cfg->register_num(cfg, "codec.divx4_forceversion", 0,
-							      "Divx version to check for (set to 0 (default) if unsure)",
+							      _("Divx version to check for (set to 0 (default) if unsure)"),
 							      NULL, NULL, NULL);
 
   /* if the version set in the config file, we can check right now. 
@@ -608,6 +623,10 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   if (this->postproc > 10) this->postproc=10;
   if (this->postproc < 0) this->postproc=0;
 
+#ifdef LOG
+  printf ("divx4: this=0x%08x, decoder_ok = %d\n", this, this->decoder_ok);
+#endif
+
   return (video_decoder_t *) this;
 }
 
