@@ -1,8 +1,6 @@
 #ifndef COMMON_H
 #define COMMON_H
 
-#undef DEBUG
-
 #define FFMPEG_VERSION_INT 0x000406
 #define FFMPEG_VERSION     "0.4.6"
 
@@ -12,24 +10,36 @@
 
 //#define ALT_BITSTREAM_WRITER
 //#define ALIGNED_BITSTREAM_WRITER
-//#define ALT_BITSTREAM_READER
-//#define ALIGNED_BITSTREAM
-#define FAST_GET_FIRST_VLC
-//#define DUMP_STREAM // only works with the ALT_BITSTREAM_READER
+
+#define ALT_BITSTREAM_READER
+//#define LIBMPEG2_BITSTREAM_READER
+//#define A32_BITSTREAM_READER
 
 #ifdef HAVE_AV_CONFIG_H
 /* only include the following when compiling package */
-#include "../config.h"
+#include "config.h"
 
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#include <math.h>
 
 #ifndef ENODATA
 #define ENODATA  61
 #endif
 
+#endif /* HAVE_AV_CONFIG_H */
+
+/* Suppress restrict if it was not defined in config.h.  */
+#ifndef restrict
+#define restrict
+#endif
+
+#if defined(__GNUC__) && (__GNUC__ > 3 || __GNUC__ == 3 && __GNUC_MINOR__ > 0)
+#define always_inline __attribute__((always_inline)) inline
+#else
+#define always_inline inline
 #endif
 
 #ifdef CONFIG_WIN32
@@ -51,6 +61,8 @@ typedef UINT16 uint16_t;
 typedef INT16 int16_t;
 typedef UINT32 uint32_t;
 typedef INT32 int32_t;
+typedef UINT64 uint64_t;
+typedef INT64 int64_t;
 
 #ifndef __MINGW32__
 #define INT64_C(c)     (c ## i64)
@@ -58,14 +70,6 @@ typedef INT32 int32_t;
 
 #define inline __inline
 
-/*
-  Disable warning messages:
-    warning C4244: '=' : conversion from 'double' to 'float', possible loss of data
-    warning C4305: 'argument' : truncation from 'const double' to 'float'
-*/
-#pragma warning( disable : 4244 )
-#pragma warning( disable : 4305 )
-
 #else
 #define INT64_C(c)     (c ## LL)
 #define UINT64_C(c)    (c ## ULL)
@@ -78,22 +82,9 @@ typedef INT32 int32_t;
 #define DEBUG
 #endif
 
-// code from bits/byteswap.h (C) 1997, 1998 Free Software Foundation, Inc.
-#define bswap_32(x) \
-     ((((x) & 0xff000000) >> 24) | (((x) & 0x00ff0000) >>  8) | \
-      (((x) & 0x0000ff00) <<  8) | (((x) & 0x000000ff) << 24))
-#define be2me_32(x) bswap_32(x)
-
 #define snprintf _snprintf
 
-#ifndef __MINGW32__
-/* no config.h with VC */
-#define CONFIG_ENCODERS 1
-#define CONFIG_DECODERS 1
-#define CONFIG_AC3      1
-#endif
-
-#else
+#else /* CONFIG_WIN32 */
 
 /* unix */
 
@@ -112,8 +103,6 @@ typedef signed char INT8;
 typedef signed int INT32;
 typedef signed long long INT64;
 
-#include "xine-engine/bswap.h"
-
 #ifdef HAVE_AV_CONFIG_H
 
 #ifdef __FreeBSD__
@@ -133,9 +122,18 @@ typedef signed long long INT64;
 
 #endif /* !CONFIG_WIN32 */
 
+#ifdef HAVE_AV_CONFIG_H
+
+#include "bswap.h"
+
+#if defined(__MINGW32__) || defined(__CYGWIN__) || \
+    defined(__OS2__) || defined (__OpenBSD__)
+#define MANGLE(a) "_" #a
+#else
+#define MANGLE(a) #a
+#endif
 
 /* debug stuff */
-#ifdef HAVE_AV_CONFIG_H
 
 #ifndef DEBUG
 #define NDEBUG
@@ -149,19 +147,50 @@ inline void dprintf(const char* fmt,...) {}
 
 #else
 
+#if __GNUC__
 #ifdef DEBUG
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95) || !defined(__GNUC__)
-#define dprintf(...)	     printf(__VA_ARGS__)
-#else
 #define dprintf(fmt,args...) printf(fmt, ## args)
-#endif
 #else
 #define dprintf(fmt,args...)
 #endif
+#else
+#ifdef DEBUG
+#define dprintf(...) printf(__VA_ARGS__)
+#else
+#define dprintf(...)
+#endif
+#endif
 
 #endif /* !CONFIG_WIN32 */
 
-#endif /* HAVE_AV_CONFIG_H */
+#define av_abort()      do { fprintf(stderr, "Abort at %s:%d\n", __FILE__, __LINE__); abort(); } while (0)
+
+/* assume b>0 */
+#define ROUNDED_DIV(a,b) (((a)>0 ? (a) + ((b)>>1) : (a) - ((b)>>1))/(b))
+#define ABS(a) ((a) >= 0 ? (a) : (-(a)))
+#define MAX(a,b) ((a) > (b) ? (a) : (b))
+#define MIN(a,b) ((a) > (b) ? (b) : (a))
+
+#ifdef ARCH_X86
+// avoid +32 for shift optimization (gcc should do that ...)
+static inline  int32_t NEG_SSR32( int32_t a, int8_t s){
+    asm ("sarl %1, %0\n\t"
+         : "+r" (a)
+         : "ic" ((uint8_t)(-s))
+    );
+    return a;
+}
+static inline uint32_t NEG_USR32(uint32_t a, int8_t s){
+    asm ("shrl %1, %0\n\t"
+         : "+r" (a)
+         : "ic" ((uint8_t)(-s))
+    );
+    return a;
+}
+#else
+#define NEG_SSR32(a,s) ((( int32_t)(a))>>(32-(s)))
+#define NEG_USR32(a,s) (((uint32_t)(a))>>(32-(s)))
+#endif
 
 /* bit output */
 
@@ -189,6 +218,7 @@ void init_put_bits(PutBitContext *s,
 INT64 get_bit_count(PutBitContext *s); /* XXX: change function name */
 void align_put_bits(PutBitContext *s);
 void flush_put_bits(PutBitContext *s);
+void put_string(PutBitContext * pbc, char *s);
 
 /* jpeg specific put_bits */
 void jflush_put_bits(PutBitContext *s);
@@ -196,26 +226,38 @@ void jflush_put_bits(PutBitContext *s);
 /* bit input */
 
 typedef struct GetBitContext {
+    UINT8 *buffer, *buffer_end;
 #ifdef ALT_BITSTREAM_READER
     int index;
-    UINT8 *buffer;
-#else
-    UINT32 bit_buf;
-    int bit_cnt;
-    UINT8 *buf, *buf_ptr, *buf_end;
+#elif defined LIBMPEG2_BITSTREAM_READER
+    UINT8 *buffer_ptr;
+    UINT32 cache;
+    int bit_count;
+#elif defined A32_BITSTREAM_READER
+    UINT32 *buffer_ptr;
+    UINT32 cache0;
+    UINT32 cache1;
+    int bit_count;
 #endif
     int size;
 } GetBitContext;
 
 static inline int get_bits_count(GetBitContext *s);
 
+#define VLC_TYPE INT16
+
 typedef struct VLC {
     int bits;
-    INT16 *table_codes;
-    INT8 *table_bits;
+    VLC_TYPE (*table)[2]; // code, bits
     int table_size, table_allocated;
 } VLC;
 
+typedef struct RL_VLC_ELEM {
+    int16_t level;
+    int8_t len;
+    uint8_t run;
+} RL_VLC_ELEM;
+
 /* used to avoid missaligned exceptions on some archs (alpha, ...) */
 #ifdef ARCH_X86
 #define unaligned32(a) (*(UINT32*)(a))
@@ -250,7 +292,7 @@ static inline void put_bits(PutBitContext *s, int n, unsigned int value)
 #endif
     //    printf("put_bits=%d %x\n", n, value);
     assert(n == 32 || value < (1U << n));
-
+    
     bit_buf = s->bit_buf;
     bit_left = s->bit_left;
 
@@ -430,7 +472,6 @@ static inline void jput_bits(PutBitContext *s, int n, int value)
  }
 #endif
 
-
 static inline uint8_t* pbBufPtr(PutBitContext *s)
 {
 #ifdef ALT_BITSTREAM_WRITER
@@ -440,65 +481,234 @@ static inline uint8_t* pbBufPtr(PutBitContext *s)
 #endif
 }
 
-void init_get_bits(GetBitContext *s, 
-                   UINT8 *buffer, int buffer_size);
+/* Bitstream reader API docs:
+name
+    abritary name which is used as prefix for the internal variables
+
+gb
+    getbitcontext
+
+OPEN_READER(name, gb)
+    loads gb into local variables
+
+CLOSE_READER(name, gb)
+    stores local vars in gb
+
+UPDATE_CACHE(name, gb)
+    refills the internal cache from the bitstream
+    after this call at least MIN_CACHE_BITS will be available,
+
+GET_CACHE(name, gb)
+    will output the contents of the internal cache, next bit is MSB of 32 or 64 bit (FIXME 64bit)
+
+SHOW_UBITS(name, gb, num)
+    will return the nest num bits
+
+SHOW_SBITS(name, gb, num)
+    will return the nest num bits and do sign extension
+
+SKIP_BITS(name, gb, num)
+    will skip over the next num bits
+    note, this is equinvalent to SKIP_CACHE; SKIP_COUNTER
+
+SKIP_CACHE(name, gb, num)
+    will remove the next num bits from the cache (note SKIP_COUNTER MUST be called before UPDATE_CACHE / CLOSE_READER)
+
+SKIP_COUNTER(name, gb, num)
+    will increment the internal bit counter (see SKIP_CACHE & SKIP_BITS)
+
+LAST_SKIP_CACHE(name, gb, num)
+    will remove the next num bits from the cache if it is needed for UPDATE_CACHE otherwise it will do nothing
+
+LAST_SKIP_BITS(name, gb, num)
+    is equinvalent to SKIP_LAST_CACHE; SKIP_COUNTER
+
+for examples see get_bits, show_bits, skip_bits, get_vlc
+*/
+
+#ifdef ALT_BITSTREAM_READER
+#   define MIN_CACHE_BITS 25
+
+#   define OPEN_READER(name, gb)\
+        int name##_index= (gb)->index;\
+        int name##_cache= 0;\
+
+#   define CLOSE_READER(name, gb)\
+        (gb)->index= name##_index;\
+
+#   define UPDATE_CACHE(name, gb)\
+        name##_cache= be2me_32( unaligned32( ((uint8_t *)(gb)->buffer)+(name##_index>>3) ) ) << (name##_index&0x07);\
+
+#   define SKIP_CACHE(name, gb, num)\
+        name##_cache <<= (num);\
+
+// FIXME name?
+#   define SKIP_COUNTER(name, gb, num)\
+        name##_index += (num);\
+
+#   define SKIP_BITS(name, gb, num)\
+        {\
+            SKIP_CACHE(name, gb, num)\
+            SKIP_COUNTER(name, gb, num)\
+        }\
+
+#   define LAST_SKIP_BITS(name, gb, num) SKIP_COUNTER(name, gb, num)
+#   define LAST_SKIP_CACHE(name, gb, num) ;
+
+#   define SHOW_UBITS(name, gb, num)\
+        NEG_USR32(name##_cache, num)
+
+#   define SHOW_SBITS(name, gb, num)\
+        NEG_SSR32(name##_cache, num)
+
+#   define GET_CACHE(name, gb)\
+        ((uint32_t)name##_cache)
+
+static inline int get_bits_count(GetBitContext *s){
+    return s->index;
+}
+#elif defined LIBMPEG2_BITSTREAM_READER
+//libmpeg2 like reader
+
+#   define MIN_CACHE_BITS 16
+
+#   define OPEN_READER(name, gb)\
+        int name##_bit_count=(gb)->bit_count;\
+        int name##_cache= (gb)->cache;\
+        uint8_t * name##_buffer_ptr=(gb)->buffer_ptr;\
+
+#   define CLOSE_READER(name, gb)\
+        (gb)->bit_count= name##_bit_count;\
+        (gb)->cache= name##_cache;\
+        (gb)->buffer_ptr= name##_buffer_ptr;\
+
+#   define UPDATE_CACHE(name, gb)\
+    if(name##_bit_count > 0){\
+        name##_cache+= ((name##_buffer_ptr[0]<<8) + name##_buffer_ptr[1]) << name##_bit_count;\
+        name##_buffer_ptr+=2;\
+        name##_bit_count-= 16;\
+    }\
+
+#   define SKIP_CACHE(name, gb, num)\
+        name##_cache <<= (num);\
+
+#   define SKIP_COUNTER(name, gb, num)\
+        name##_bit_count += (num);\
+
+#   define SKIP_BITS(name, gb, num)\
+        {\
+            SKIP_CACHE(name, gb, num)\
+            SKIP_COUNTER(name, gb, num)\
+        }\
+
+#   define LAST_SKIP_BITS(name, gb, num) SKIP_BITS(name, gb, num)
+#   define LAST_SKIP_CACHE(name, gb, num) SKIP_CACHE(name, gb, num)
+
+#   define SHOW_UBITS(name, gb, num)\
+        NEG_USR32(name##_cache, num)
+
+#   define SHOW_SBITS(name, gb, num)\
+        NEG_SSR32(name##_cache, num)
+
+#   define GET_CACHE(name, gb)\
+        ((uint32_t)name##_cache)
+
+static inline int get_bits_count(GetBitContext *s){
+    return (s->buffer_ptr - s->buffer)*8 - 16 + s->bit_count;
+}
+
+#elif defined A32_BITSTREAM_READER
+
+#   define MIN_CACHE_BITS 32
+
+#   define OPEN_READER(name, gb)\
+        int name##_bit_count=(gb)->bit_count;\
+        uint32_t name##_cache0= (gb)->cache0;\
+        uint32_t name##_cache1= (gb)->cache1;\
+        uint32_t * name##_buffer_ptr=(gb)->buffer_ptr;\
+
+#   define CLOSE_READER(name, gb)\
+        (gb)->bit_count= name##_bit_count;\
+        (gb)->cache0= name##_cache0;\
+        (gb)->cache1= name##_cache1;\
+        (gb)->buffer_ptr= name##_buffer_ptr;\
+
+#   define UPDATE_CACHE(name, gb)\
+    if(name##_bit_count > 0){\
+        const uint32_t next= be2me_32( *name##_buffer_ptr );\
+        name##_cache0 |= NEG_USR32(next,name##_bit_count);\
+        name##_cache1 |= next<<name##_bit_count;\
+        name##_buffer_ptr++;\
+        name##_bit_count-= 32;\
+    }\
+
+#ifdef ARCH_X86
+#   define SKIP_CACHE(name, gb, num)\
+        asm(\
+            "shldl %2, %1, %0		\n\t"\
+            "shll %2, %1		\n\t"\
+            : "+r" (name##_cache0), "+r" (name##_cache1)\
+            : "Ic" ((uint8_t)num)\
+           );
+#else
+#   define SKIP_CACHE(name, gb, num)\
+        name##_cache0 <<= (num);\
+        name##_cache0 |= NEG_USR32(name##_cache1,num);\
+        name##_cache1 <<= (num);
+#endif
+
+#   define SKIP_COUNTER(name, gb, num)\
+        name##_bit_count += (num);\
+
+#   define SKIP_BITS(name, gb, num)\
+        {\
+            SKIP_CACHE(name, gb, num)\
+            SKIP_COUNTER(name, gb, num)\
+        }\
+
+#   define LAST_SKIP_BITS(name, gb, num) SKIP_BITS(name, gb, num)
+#   define LAST_SKIP_CACHE(name, gb, num) SKIP_CACHE(name, gb, num)
+
+#   define SHOW_UBITS(name, gb, num)\
+        NEG_USR32(name##_cache0, num)
+
+#   define SHOW_SBITS(name, gb, num)\
+        NEG_SSR32(name##_cache0, num)
+
+#   define GET_CACHE(name, gb)\
+        (name##_cache0)
+
+static inline int get_bits_count(GetBitContext *s){
+    return ((uint8_t*)s->buffer_ptr - s->buffer)*8 - 32 + s->bit_count;
+}
 
-#ifndef ALT_BITSTREAM_READER
-unsigned int get_bits_long(GetBitContext *s, int n);
-unsigned int show_bits_long(GetBitContext *s, int n);
 #endif
 
 static inline unsigned int get_bits(GetBitContext *s, int n){
-#ifdef ALT_BITSTREAM_READER
-#ifdef ALIGNED_BITSTREAM
-    int index= s->index;
-    uint32_t result1= be2me_32( ((uint32_t *)s->buffer)[index>>5] );
-    uint32_t result2= be2me_32( ((uint32_t *)s->buffer)[(index>>5) + 1] );
-#ifdef ARCH_X86
-    asm ("shldl %%cl, %2, %0\n\t"
-         : "=r" (result1)
-	 : "0" (result1), "r" (result2), "c" (index));
-#else
-    result1<<= (index&0x1F);
-    result2= (result2>>1) >> (31-(index&0x1F));
-    result1|= result2;
-#endif
-    result1>>= 32 - n;
-    index+= n;
-    s->index= index;
-    
-    return result1;
-#else //ALIGNED_BITSTREAM
-    int index= s->index;
-    uint32_t result= be2me_32( unaligned32( ((uint8_t *)s->buffer)+(index>>3) ) );
+    register int tmp;
+    OPEN_READER(re, s)
+    UPDATE_CACHE(re, s)
+    tmp= SHOW_UBITS(re, s, n);
+    LAST_SKIP_BITS(re, s, n)
+    CLOSE_READER(re, s)
+    return tmp;
+}
 
-    result<<= (index&0x07);
-    result>>= 32 - n;
-    index+= n;
-    s->index= index;
-#ifdef DUMP_STREAM
-    while(n){
-        printf("%d", (result>>(n-1))&1);
-        n--;
-    }
-    printf(" ");
-#endif
-    
-    return result;
-#endif //!ALIGNED_BITSTREAM
-#else //ALT_BITSTREAM_READER
-    if(s->bit_cnt>=n){
-        /* most common case here */
-        unsigned int val = s->bit_buf >> (32 - n);
-        s->bit_buf <<= n;
-	s->bit_cnt -= n;
-#ifdef STATS
-	st_bit_counts[st_current_index] += n;
-#endif
-	return val;
-    }
-    return get_bits_long(s,n);
-#endif //!ALT_BITSTREAM_READER
+static inline unsigned int show_bits(GetBitContext *s, int n){
+    register int tmp;
+    OPEN_READER(re, s)
+    UPDATE_CACHE(re, s)
+    tmp= SHOW_UBITS(re, s, n);
+//    CLOSE_READER(re, s)
+    return tmp;
+}
+
+static inline void skip_bits(GetBitContext *s, int n){
+ //Note gcc seems to optimize this to s->index+=n for the ALT_READER :))
+    OPEN_READER(re, s)
+    UPDATE_CACHE(re, s)
+    LAST_SKIP_BITS(re, s, n)
+    CLOSE_READER(re, s)
 }
 
 static inline unsigned int get_bits1(GetBitContext *s){
@@ -509,158 +719,23 @@ static inline unsigned int get_bits1(GetBitContext *s){
     result>>= 8 - 1;
     index++;
     s->index= index;
-#ifdef DUMP_STREAM
-    printf("%d ", result);
-#endif
-    
+
     return result;
 #else
-    if(s->bit_cnt>0){
-        /* most common case here */
-        unsigned int val = s->bit_buf >> 31;
-        s->bit_buf <<= 1;
-	s->bit_cnt--;
-#ifdef STATS
-	st_bit_counts[st_current_index]++;
-#endif
-	return val;
-    }
-    return get_bits_long(s,1);
+    return get_bits(s, 1);
 #endif
 }
 
-/* This function is identical to get_bits(), the only */
-/* diference is that it doesn't touch the buffer      */
-/* it is usefull to see the buffer.                   */
-static inline unsigned int show_bits(GetBitContext *s, int n)
-{
-#ifdef ALT_BITSTREAM_READER
-#ifdef ALIGNED_BITSTREAM
-    int index= s->index;
-    uint32_t result1= be2me_32( ((uint32_t *)s->buffer)[index>>5] );
-    uint32_t result2= be2me_32( ((uint32_t *)s->buffer)[(index>>5) + 1] );
-#ifdef ARCH_X86
-    asm ("shldl %%cl, %2, %0\n\t"
-         : "=r" (result1)
-	 : "0" (result1), "r" (result2), "c" (index));
-#else
-    result1<<= (index&0x1F);
-    result2= (result2>>1) >> (31-(index&0x1F));
-    result1|= result2;
-#endif
-    result1>>= 32 - n;
-    
-    return result1;
-#else //ALIGNED_BITSTREAM
-    int index= s->index;
-    uint32_t result= be2me_32( unaligned32( ((uint8_t *)s->buffer)+(index>>3) ) );
-
-    result<<= (index&0x07);
-    result>>= 32 - n;
-    
-    return result;
-#endif //!ALIGNED_BITSTREAM
-#else //ALT_BITSTREAM_READER
-    if(s->bit_cnt>=n) {
-        /* most common case here */
-        unsigned int val = s->bit_buf >> (32 - n);
-        return val;
-    }
-    return show_bits_long(s,n);
-#endif //!ALT_BITSTREAM_READER
-}
-
-static inline int show_aligned_bits(GetBitContext *s, int offset, int n)
-{
-#ifdef ALT_BITSTREAM_READER
-#ifdef ALIGNED_BITSTREAM
-    int index= (s->index + offset + 7)&(~7);
-    uint32_t result1= be2me_32( ((uint32_t *)s->buffer)[index>>5] );
-    uint32_t result2= be2me_32( ((uint32_t *)s->buffer)[(index>>5) + 1] );
-#ifdef ARCH_X86
-    asm ("shldl %%cl, %2, %0\n\t"
-         : "=r" (result1)
-	 : "0" (result1), "r" (result2), "c" (index));
-#else
-    result1<<= (index&0x1F);
-    result2= (result2>>1) >> (31-(index&0x1F));
-    result1|= result2;
-#endif
-    result1>>= 32 - n;
-    
-    return result1;
-#else //ALIGNED_BITSTREAM
-    int index= (s->index + offset + 7)>>3;
-    uint32_t result= be2me_32( unaligned32( ((uint8_t *)s->buffer)+index ) );
-
-    result>>= 32 - n;
-    
-    return result;
-#endif //!ALIGNED_BITSTREAM
-#else //ALT_BITSTREAM_READER
-    int index= (get_bits_count(s) + offset + 7)>>3;
-    uint32_t result= be2me_32( unaligned32( ((uint8_t *)s->buf)+index ) );
-
-    result>>= 32 - n;
-//printf(" %X %X %d \n", (int)(((uint8_t *)s->buf)+index ), (int)s->buf_ptr, s->bit_cnt);    
-    return result;
-#endif //!ALT_BITSTREAM_READER
-}
-
-static inline void skip_bits(GetBitContext *s, int n){
-#ifdef ALT_BITSTREAM_READER
-    s->index+= n;
-#ifdef DUMP_STREAM
-    {
-        int result;
-        s->index-= n;
-        result= get_bits(s, n);
-    }
-#endif
-
-#else
-    if(s->bit_cnt>=n){
-        /* most common case here */
-        s->bit_buf <<= n;
-	s->bit_cnt -= n;
-#ifdef STATS
-	st_bit_counts[st_current_index] += n;
-#endif
-    } else {
-	get_bits_long(s,n);
-    }
-#endif
+static inline unsigned int show_bits1(GetBitContext *s){
+    return show_bits(s, 1);
 }
 
 static inline void skip_bits1(GetBitContext *s){
-#ifdef ALT_BITSTREAM_READER
-    s->index++;
-#ifdef DUMP_STREAM
-    s->index--;
-    printf("%d ", get_bits1(s));
-#endif
-#else
-    if(s->bit_cnt>0){
-        /* most common case here */
-        s->bit_buf <<= 1;
-	s->bit_cnt--;
-#ifdef STATS
-	st_bit_counts[st_current_index]++;
-#endif
-    } else {
-	get_bits_long(s,1);
-    }
-#endif
+    skip_bits(s, 1);
 }
 
-static inline int get_bits_count(GetBitContext *s)
-{
-#ifdef ALT_BITSTREAM_READER
-    return s->index;
-#else
-    return (s->buf_ptr - s->buf) * 8 - s->bit_cnt;
-#endif
-}
+void init_get_bits(GetBitContext *s,
+                   UINT8 *buffer, int buffer_size);
 
 int check_marker(GetBitContext *s, char *msg);
 void align_get_bits(GetBitContext *s);
@@ -669,150 +744,85 @@ int init_vlc(VLC *vlc, int nb_bits, int nb_codes,
              const void *codes, int codes_wrap, int codes_size);
 void free_vlc(VLC *vlc);
 
-#ifdef ALT_BITSTREAM_READER
-#ifdef ALIGNED_BITSTREAM
-#ifdef ARCH_X86
-#define SHOW_BITS(s, val, n) \
-    val= be2me_32( ((uint32_t *)(s)->buffer)[bit_cnt>>5] );\
-    {uint32_t result2= be2me_32( ((uint32_t *)(s)->buffer)[(bit_cnt>>5) + 1] );\
-    asm ("shldl %%cl, %2, %0\n\t"\
-         : "=r" (val)\
-         : "0" (val), "r" (result2), "c" (bit_cnt));\
-    ((uint32_t)val)>>= 32 - n;}
-#else //ARCH_X86
-#define SHOW_BITS(s, val, n) \
-    val= be2me_32( ((uint32_t *)(s)->buffer)[bit_cnt>>5] );\
-    {uint32_t result2= be2me_32( ((uint32_t *)(s)->buffer)[(bit_cnt>>5) + 1] );\
-    val<<= (bit_cnt&0x1F);\
-    result2= (result2>>1) >> (31-(bit_cnt&0x1F));\
-    val|= result2;\
-    ((uint32_t)val)>>= 32 - n;}
-#endif //!ARCH_X86
-#else //ALIGNED_BITSTREAM
-#define SHOW_BITS(s, val, n) \
-    val= be2me_32( unaligned32( ((uint8_t *)(s)->buffer)+(bit_cnt>>3) ) );\
-    val<<= (bit_cnt&0x07);\
-    ((uint32_t)val)>>= 32 - n;
-#endif // !ALIGNED_BITSTREAM
-#define FLUSH_BITS(n) bit_cnt+=n; 
-#define SAVE_BITS(s) bit_cnt= (s)->index;
-#define RESTORE_BITS(s) (s)->index= bit_cnt;
-#else
-
-/* macro to go faster */
-/* n must be <= 24 */
-/* XXX: optimize buffer end test */
-#define SHOW_BITS(s, val, n)\
+#define GET_VLC(code, name, gb, table, bits, max_depth)\
 {\
-    if (bit_cnt < n && buf_ptr < (s)->buf_end) {\
-        bit_buf |= *buf_ptr++ << (24 - bit_cnt);\
-        bit_cnt += 8;\
-        if (bit_cnt < n && buf_ptr < (s)->buf_end) {\
-            bit_buf |= *buf_ptr++ << (24 - bit_cnt);\
-            bit_cnt += 8;\
-            if (bit_cnt < n && buf_ptr < (s)->buf_end) {\
-                bit_buf |= *buf_ptr++ << (24 - bit_cnt);\
-                bit_cnt += 8;\
-            }\
+    int n, index, nb_bits;\
+\
+    index= SHOW_UBITS(name, gb, bits);\
+    code = table[index][0];\
+    n    = table[index][1];\
+\
+    if(max_depth > 1 && n < 0){\
+        LAST_SKIP_BITS(name, gb, bits)\
+        UPDATE_CACHE(name, gb)\
+\
+        nb_bits = -n;\
+\
+        index= SHOW_UBITS(name, gb, nb_bits) + code;\
+        code = table[index][0];\
+        n    = table[index][1];\
+        if(max_depth > 2 && n < 0){\
+            LAST_SKIP_BITS(name, gb, nb_bits)\
+            UPDATE_CACHE(name, gb)\
+\
+            nb_bits = -n;\
+\
+            index= SHOW_UBITS(name, gb, nb_bits) + code;\
+            code = table[index][0];\
+            n    = table[index][1];\
         }\
     }\
-    val = bit_buf >> (32 - n);\
+    SKIP_BITS(name, gb, n)\
 }
 
-/* SHOW_BITS with n1 >= n must be been done before */
-#define FLUSH_BITS(n)\
+#define GET_RL_VLC(level, run, name, gb, table, bits, max_depth)\
 {\
-    bit_buf <<= n;\
-    bit_cnt -= n;\
+    int n, index, nb_bits;\
+\
+    index= SHOW_UBITS(name, gb, bits);\
+    level = table[index].level;\
+    n     = table[index].len;\
+\
+    if(max_depth > 1 && n < 0){\
+        LAST_SKIP_BITS(name, gb, bits)\
+        UPDATE_CACHE(name, gb)\
+\
+        nb_bits = -n;\
+\
+        index= SHOW_UBITS(name, gb, nb_bits) + level;\
+        level = table[index].level;\
+        n     = table[index].len;\
+    }\
+    run= table[index].run;\
+    SKIP_BITS(name, gb, n)\
 }
 
-#define SAVE_BITS(s) \
-{\
-    bit_cnt = (s)->bit_cnt;\
-    bit_buf = (s)->bit_buf;\
-    buf_ptr = (s)->buf_ptr;\
-}
-
-#define RESTORE_BITS(s) \
-{\
-    (s)->buf_ptr = buf_ptr;\
-    (s)->bit_buf = bit_buf;\
-    (s)->bit_cnt = bit_cnt;\
-}
-#endif // !ALT_BITSTREAM_READER
-
+// deprecated, dont use get_vlc for new code, use get_vlc2 instead or use GET_VLC directly
 static inline int get_vlc(GetBitContext *s, VLC *vlc)
 {
-    int code, n, nb_bits, index;
-    INT16 *table_codes;
-    INT8 *table_bits;
-    int bit_cnt;
-#ifndef ALT_BITSTREAM_READER
-    UINT32 bit_buf;
-    UINT8 *buf_ptr;
-#endif
+    int code;
+    VLC_TYPE (*table)[2]= vlc->table;
+    
+    OPEN_READER(re, s)
+    UPDATE_CACHE(re, s)
 
-    SAVE_BITS(s);
-    nb_bits = vlc->bits;
-    table_codes = vlc->table_codes;
-    table_bits = vlc->table_bits;
+    GET_VLC(code, re, s, table, vlc->bits, 3)    
 
-#ifdef FAST_GET_FIRST_VLC
-    SHOW_BITS(s, index, nb_bits);
-    code = table_codes[index];
-    n = table_bits[index];
-    if (n > 0) {
-        /* most common case (90%)*/
-        FLUSH_BITS(n);
-#ifdef DUMP_STREAM
-        {
-            int n= bit_cnt - s->index;
-            skip_bits(s, n);
-            RESTORE_BITS(s);
-        }
-#endif
-        RESTORE_BITS(s);
-        return code;
-    } else if (n == 0) {
-        return -1;
-    } else {
-        FLUSH_BITS(nb_bits);
-        nb_bits = -n;
-        table_codes = vlc->table_codes + code;
-        table_bits = vlc->table_bits + code;
-    }
-#endif
-    for(;;) {
-        SHOW_BITS(s, index, nb_bits);
-        code = table_codes[index];
-        n = table_bits[index];
-        if (n > 0) {
-            /* most common case */
-            FLUSH_BITS(n);
-#ifdef STATS
-            st_bit_counts[st_current_index] += n;
-#endif
-            break;
-        } else if (n == 0) {
-            return -1;
-        } else {
-            FLUSH_BITS(nb_bits);
-#ifdef STATS
-            st_bit_counts[st_current_index] += nb_bits;
-#endif
-            nb_bits = -n;
-            table_codes = vlc->table_codes + code;
-            table_bits = vlc->table_bits + code;
-        }
-    }
-#ifdef DUMP_STREAM
-    {
-        int n= bit_cnt - s->index;
-        skip_bits(s, n);
-        RESTORE_BITS(s);
-    }
-#endif
-    RESTORE_BITS(s);
+    CLOSE_READER(re, s)
+    return code;
+}
+
+static always_inline int get_vlc2(GetBitContext *s, VLC_TYPE (*table)[2],
+                                  int bits, int max_depth)
+{
+    int code;
+    
+    OPEN_READER(re, s)
+    UPDATE_CACHE(re, s)
+
+    GET_VLC(code, re, s, table, bits, max_depth)
+
+    CLOSE_READER(re, s)
     return code;
 }
 
@@ -888,7 +898,57 @@ static inline int mid_pred(int a, int b, int c)
     return a + b + c - vmin - vmax;
 }
 
-/* memory */
-void *av_mallocz(int size);
+static inline int clip(int a, int amin, int amax)
+{
+    if (a < amin)
+        return amin;
+    else if (a > amax)
+        return amax;
+    else
+        return a;
+}
 
+/* math */
+int ff_gcd(int a, int b);
+
+static inline int ff_sqrt(int a)
+{
+    int ret=0;
+    int s;
+    int ret_sq=0;
+
+    for(s=15; s>=0; s--){
+        int b= ret_sq + (1<<(s*2)) + (ret<<s)*2;
+        if(b<=a){
+            ret_sq=b;
+            ret+= 1<<s;
+        }
+    }
+    return ret;
+}
+#define RUNTIME_CPUDETECT
+
+#if __CPU__ >= 686 && !defined(RUNTIME_CPUDETECT)
+#define COPY3_IF_LT(x,y,a,b,c,d)\
+asm volatile (\
+    "cmpl %0, %3	\n\t"\
+    "cmovl %3, %0	\n\t"\
+    "cmovl %4, %1	\n\t"\
+    "cmovl %5, %2	\n\t"\
+    : "+r" (x), "+r" (a), "+r" (c)\
+    : "r" (y), "r" (b), "r" (d)\
+);
+#else
+#define COPY3_IF_LT(x,y,a,b,c,d)\
+if((y)<(x)){\
+     (x)=(y);\
+     (a)=(b);\
+     (c)=(d);\
+}
 #endif
+
+#define CLAMP_TO_8BIT(d) ((d > 0xff) ? 0xff : (d < 0) ? 0 : d)
+
+#endif /* HAVE_AV_CONFIG_H */
+
+#endif /* COMMON_H */
