@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.35 2002/04/27 17:03:30 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.50 2002/07/30 20:13:09 miguelfreitas Exp $
  *
  * xine decoder plugin using ffmpeg
  *
@@ -46,26 +46,6 @@
 #define LOG
 */
 
-/* now this is ripped of wine's vfw.h */
-typedef struct {
-    long        biSize;
-    long        biWidth;
-    long        biHeight;
-    short       biPlanes;
-    short       biBitCount;
-    long        biCompression;
-    long        biSizeImage;
-    long        biXPelsPerMeter;
-    long        biYPelsPerMeter;
-    long        biClrUsed;
-    long        biClrImportant;
-} BITMAPINFOHEADER;
-#ifndef mmioFOURCC
-#define mmioFOURCC( ch0, ch1, ch2, ch3 )                                         \
-        ( (long)(unsigned char)(ch0) | ( (long)(unsigned char)(ch1) << 8 ) |     \
-        ( (long)(unsigned char)(ch2) << 16 ) | ( (long)(unsigned char)(ch3) << 24 ) )
-#endif
-
 typedef struct ff_decoder_s {
   video_decoder_t   video_decoder;
 
@@ -73,13 +53,12 @@ typedef struct ff_decoder_s {
   int               video_step;
   int               decoder_ok;
 
-  BITMAPINFOHEADER  bih;
-  long		    biWidth;
-  long		    biHeight;
+  xine_bmiheader    bih;
   unsigned char     *buf;
   int               bufsize;
   int               size;
   int               skipframes;
+  int               illegal_vlc;
 
   AVPicture         av_picture;
   AVCodecContext    context;
@@ -87,32 +66,13 @@ typedef struct ff_decoder_s {
 
 #define VIDEOBUFSIZE 128*1024
 
-/*
-#define IMGFMT_YUY2  mmioFOURCC('Y','U','Y','2')
-#define IMGFMT_YV12  mmioFOURCC('Y','V','1','2')
-*/
-
-static unsigned long str2ulong(void *data)
-{
-  unsigned char *str = data;
-  return ( str[0] | (str[1]<<8) | (str[2]<<16) | (str[3]<<24) );
-}
-
-/*
-static unsigned short str2ushort(void *data)
-{
-  unsigned char *str = data;
-  return ( str[0] | (str[1]<<8) );
-}
-*/
-
 static int ff_can_handle (video_decoder_t *this_gen, int buf_type) {
   buf_type &= 0xFFFF0000;
 
-  /* ffmpeg currently does not support MSMPEG4 v1/v2 */
-  /* there's some problem with I263 too */
   return ( buf_type == BUF_VIDEO_MSMPEG4_V3 ||
-           /* buf_type == BUF_VIDEO_MSMPEG4_V12 || */
+           buf_type == BUF_VIDEO_MSMPEG4_V2 ||
+           buf_type == BUF_VIDEO_MSMPEG4_V1 ||
+           buf_type == BUF_VIDEO_WMV7 ||
            buf_type == BUF_VIDEO_MPEG4 ||
            buf_type == BUF_VIDEO_XVID  ||
            buf_type == BUF_VIDEO_DIVX5 ||
@@ -120,6 +80,8 @@ static int ff_can_handle (video_decoder_t *this_gen, int buf_type) {
 	   /* buf_type == BUF_VIDEO_I263 || */
 	   buf_type == BUF_VIDEO_H263 ||
 	   buf_type == BUF_VIDEO_RV10 ||
+	   /* PIX_FMT_YUV410P must be supported to enable svq1 */
+	   /* buf_type == BUF_VIDEO_SORENSON_V1 || */
 	   buf_type == BUF_VIDEO_JPEG);
 }
 
@@ -151,31 +113,26 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
     /* init package containing bih */
 
-    memcpy ( &this->bih, buf->content, sizeof (BITMAPINFOHEADER));
-    this->biWidth = str2ulong(&this->bih.biWidth);
-    this->biHeight = str2ulong(&this->bih.biHeight);
+    memcpy ( &this->bih, buf->content, sizeof (xine_bmiheader));
     this->video_step = buf->decoder_info[1];
 
     /* init codec */
 
     codec_type = buf->type & 0xFFFF0000;
 
-    /*
-    if (str2ulong(&this->bih.biCompression) == mmioFOURCC('D', 'I', 'V', 'X')) {
-      printf ("ffmpeg: mpeg4 (opendivx) format detected\n");
-
-      codec = avcodec_find_decoder (CODEC_ID_OPENDIVX);
-    } else {
-      printf ("ffmpeg: ms mpeg4 format detected\n");
-      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4);
-    }
-    */
-
-    switch (buf->type & 0xFFFF0000) {
-    case BUF_VIDEO_MSMPEG4_V12:
+    switch (codec_type) {
+    case BUF_VIDEO_MSMPEG4_V1:
+      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V1);
+      break;
+    case BUF_VIDEO_MSMPEG4_V2:
+      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V2);
+      break;
     case BUF_VIDEO_MSMPEG4_V3:
       codec = avcodec_find_decoder (CODEC_ID_MSMPEG4);
       break;
+    case BUF_VIDEO_WMV7:
+      codec = avcodec_find_decoder (CODEC_ID_WMV1);
+      break;
     case BUF_VIDEO_MPEG4 :
     case BUF_VIDEO_XVID :
     case BUF_VIDEO_DIVX5 :
@@ -194,6 +151,9 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     case BUF_VIDEO_RV10:
       codec = avcodec_find_decoder (CODEC_ID_RV10);
       break;
+    case BUF_VIDEO_SORENSON_V1:
+      codec = avcodec_find_decoder (CODEC_ID_SVQ1);
+      break;
     default:
       printf ("ffmpeg: unknown video format (buftype: 0x%08X)\n",
 	      buf->type & 0xFFFF0000);
@@ -203,11 +163,16 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       printf ("ffmpeg: couldn't find decoder\n");
       return;
     }
-
+ 
+    /* force (width % 8 == 0), otherwise there will be 
+     * display problems with Xv.
+     */
+    this->bih.biWidth = (this->bih.biWidth + 7) & (~7);
+  
     memset(&this->context, 0, sizeof(this->context));
-    this->context.width = this->biWidth;
-    this->context.height = this->biHeight;
-
+    this->context.width = this->bih.biWidth;
+    this->context.height = this->bih.biHeight;
+    
     if (avcodec_open (&this->context, codec) < 0) {
       printf ("ffmpeg: couldn't open decoder\n");
       return;
@@ -216,12 +181,20 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
     this->decoder_ok = 1;
     this->video_out->open (this->video_out);
 
+    /* needed to play streams generated by MS ISO MPEG4 codec. 
+       Michael Niedermayer explained:
+       M$ "ISO MPEG4" uses illegal vlc code combinations, a ISO MPEG4 compliant
+       decoder which support error resilience should handle them like errors.
+    */
+    if( this->illegal_vlc )
+      this->context.error_resilience=-1;
     
     if( this->buf )
       free( this->buf );
     
     this->buf = malloc( VIDEOBUFSIZE );
     this->bufsize = VIDEOBUFSIZE;
+    this->size = 0;
     
     this->skipframes = 0;
     
@@ -247,9 +220,10 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       int         got_picture, len, y;
       uint8_t    *dy, *du, *dv, *sy, *su, *sv;
 
-      /* decoder video frame */
+      /* decode video frame */
 
-      /* this->bih.biSizeImage = this->size; */
+      /* skip decoding b frames if too late */
+      this->context.hurry_up = (this->skipframes > 2) ? 1:0;
 
       len = avcodec_decode_video (&this->context, &this->av_picture,
 				  &got_picture, this->buf,
@@ -276,8 +250,8 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
       img = this->video_out->get_frame (this->video_out,
 					/* this->av_picture.linesize[0],  */
-					this->biWidth,
-					this->biHeight,
+					this->bih.biWidth,
+					this->bih.biHeight,
 					ratio, 
 					IMGFMT_YV12,
 					VO_BOTH_FIELDS);
@@ -298,21 +272,21 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	su = this->av_picture.data[1];
 	sv = this->av_picture.data[2];
 
-	for (y=0; y<this->biHeight; y++) {
+	for (y=0; y<this->bih.biHeight; y++) {
 	  
-	  xine_fast_memcpy (dy, sy, this->biWidth);
+	  xine_fast_memcpy (dy, sy, this->bih.biWidth);
 	  
-	  dy += this->biWidth;
+	  dy += img->pitches[0];
 	  
 	  sy += this->av_picture.linesize[0];
 	}
 	
-	for (y=0; y<(this->biHeight/2); y++) {
+	for (y=0; y<(this->bih.biHeight/2); y++) {
 
 	  if (this->context.pix_fmt != PIX_FMT_YUV444P) {
 	  
-	    xine_fast_memcpy (du, su, this->biWidth/2);
-	    xine_fast_memcpy (dv, sv, this->biWidth/2);
+	    xine_fast_memcpy (du, su, this->bih.biWidth/2);
+	    xine_fast_memcpy (dv, sv, this->bih.biWidth/2);
 
 	  } else {
 
@@ -323,13 +297,13 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	    /* subsample */
 
 	    src = su; dst = du;
-	    for (x=0; x<(this->biWidth/2); x++) {
+	    for (x=0; x<(this->bih.biWidth/2); x++) {
 	      *dst = *src;
 	      dst++;
 	      src += 2;
 	    }
 	    src = sv; dst = dv;
-	    for (x=0; x<(this->biWidth/2); x++) {
+	    for (x=0; x<(this->bih.biWidth/2); x++) {
 	      *dst = *src;
 	      dst++;
 	      src += 2;
@@ -337,8 +311,8 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
 	  }
 	  
-	  du += this->biWidth/2;
-	  dv += this->biWidth/2;
+	  du += img->pitches[1];
+	  dv += img->pitches[2];
 	  
 	  if (this->context.pix_fmt != PIX_FMT_YUV420P) {
 	    su += 2*this->av_picture.linesize[1];
@@ -350,19 +324,18 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	}
 	
 	if (img->copy) {
-	  
-	  int height = abs(this->biHeight);
-	  int stride = this->biWidth;
-	  uint8_t* src[3];
-	  
+	  int height = img->height;
+	  uint8_t *src[3];
+
 	  src[0] = img->base[0];
 	  src[1] = img->base[1];
 	  src[2] = img->base[2];
+
 	  while ((height -= 16) >= 0) {
 	    img->copy(img, src);
-	    src[0] += 16 * stride;
-	    src[1] +=  4 * stride;
-	    src[2] +=  4 * stride;
+	    src[0] += 16 * img->pitches[0];
+	    src[1] +=  8 * img->pitches[1];
+	    src[2] +=  8 * img->pitches[2];
 	  }
 	}
       }
@@ -382,7 +355,10 @@ static void ff_flush (video_decoder_t *this_gen) {
 }
 
 static void ff_reset (video_decoder_t *this_gen) {
-  /* seems to handle seeking quite nicelly without any code here */
+  ff_decoder_t *this = (ff_decoder_t *) this_gen;
+
+  /* invalidate buffer on seek */
+  this->size = 0;
 }
 
 static void ff_close (video_decoder_t *this_gen) {
@@ -405,6 +381,28 @@ static char *ff_get_id(void) {
   return "ffmpeg video decoder";
 }
 
+void avcodec_register_all(void)
+{
+    static int inited = 0;
+    
+    if (inited != 0)
+	return;
+    inited = 1;
+
+    /* decoders */
+    register_avcodec(&h263_decoder);
+    register_avcodec(&mpeg4_decoder);
+    register_avcodec(&msmpeg4v1_decoder);
+    register_avcodec(&msmpeg4v2_decoder);
+    register_avcodec(&msmpeg4v3_decoder);
+    register_avcodec(&wmv1_decoder);
+    register_avcodec(&wmv2_decoder);
+    register_avcodec(&mpeg_decoder);
+    register_avcodec(&h263i_decoder);
+    register_avcodec(&rv10_decoder);
+    register_avcodec(&svq1_decoder);
+    register_avcodec(&mjpeg_decoder);
+}
 
 static void init_routine(void) {
   avcodec_init();
@@ -420,11 +418,11 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   ff_decoder_t *this ;
   static pthread_once_t once_control = PTHREAD_ONCE_INIT;
 
-  if (iface_version != 7) {
-    printf( "ffmpeg: plugin doesn't support plugin API version %d.\n"
-	    "ffmpeg: this means there's a version mismatch between xine and this "
-	    "ffmpeg: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 10) {
+    printf(_("ffmpeg: plugin doesn't support plugin API version %d.\n"
+	     "ffmpeg: this means there's a version mismatch between xine and this "
+	     "ffmpeg: decoder plugin.\nInstalling current plugins should help.\n"),
+	     iface_version);
     
     return NULL;
   }
@@ -443,6 +441,9 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   this->video_decoder.dispose             = ff_dispose;
   this->size				  = 0;
 
+  this->illegal_vlc = xine->config->register_bool (xine->config, "codec.ffmpeg_illegal_vlc", 1,
+                      _("allow illegal vlc codes in mpeg4 streams"), NULL, NULL, NULL);
+
   pthread_once( &once_control, init_routine );
 
   return (video_decoder_t *) this;
