@@ -17,12 +17,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.23 2002/04/23 15:45:24 esnel Exp $
+ * $Id: xine_decoder.c,v 1.30 2002/07/19 03:03:37 miguelfreitas Exp $
  * 
  * 31-8-2001 Added LPCM rate sensing.
  *   (c) 2001 James Courtier-Dutton James@superbug.demon.co.uk
  *
- * stuff needed to turn libac3 into a xine decoder plugin
  */
 #ifndef __sun
 #define _XOPEN_SOURCE 500
@@ -81,54 +80,106 @@ void lpcm_init (audio_decoder_t *this_gen, ao_instance_t *audio_out) {
   this->cpu_be        = ( htons(1) == 1 );
 }
 
-
 void lpcm_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   lpcm_decoder_t *this = (lpcm_decoder_t *) this_gen;
   int16_t        *sample_buffer=(int16_t *)buf->content;
   int             stream_be;
   audio_buffer_t *audio_buffer;
+  int             format_changed = 0;
+  
+  /* Drop preview data */
+  if (buf->decoder_flags & BUF_FLAG_PREVIEW)
+    return;
 
-  if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
+  /* get config byte from mpeg2 stream */
+  if ( (buf->decoder_flags & BUF_FLAG_SPECIAL) &&
+        buf->decoder_info[1] == BUF_SPECIAL_LPCM_CONFIG ) {
+    int bits_per_sample = 16;
+    int sample_rate;
+    int num_channels;
+      
+    num_channels = (buf->decoder_info[2] & 0x7) + 1;
+    sample_rate = buf->decoder_info[2] & 0x10 ? 96000 : 48000;
+    switch ((buf->decoder_info[2]>>6) & 3) {
+      case 0: bits_per_sample = 16; break;
+      case 1: bits_per_sample = 20; break;
+      case 2: bits_per_sample = 24; break;
+    }
+    
+    if( this->bits_per_sample != bits_per_sample ||
+        this->number_of_channels != num_channels ||
+        this->rate != sample_rate ||
+        !this->output_open ) {
+      this->bits_per_sample = bits_per_sample;
+      this->number_of_channels = num_channels;
+      this->rate = sample_rate;
+      format_changed++;
+    } 
+  }
+  
+  if( buf->decoder_flags & BUF_FLAG_HEADER ) {
     this->rate=buf->decoder_info[1];
     this->bits_per_sample=buf->decoder_info[2] ; 
     this->number_of_channels=buf->decoder_info[3] ; 
+    format_changed++;
+  }
+  
+  /*
+   * (re-)open output device
+   */
+  if ( format_changed ) {
+    if (this->output_open)
+        this->audio_out->close (this->audio_out);
+
     this->ao_cap_mode=(this->number_of_channels == 2) ? AO_CAP_MODE_STEREO : AO_CAP_MODE_MONO; 
-    return;
+   
+    this->output_open = this->audio_out->open (this->audio_out,
+                                               (this->bits_per_sample>16)?16:this->bits_per_sample,
+                                               this->rate,
+                                               this->ao_cap_mode) ;
   }
 
-  if (!this->output_open) {      
-    /*
-     * with dvdnav we do not get a preview buffer with audio format
-     * information (buf->decoder_flags & BUF_FLAG_PREVIEW).
-     * grab the audio format from the first audio data buffer, in case
-     * the audio format is not yet known.
-     */
-    if (this->rate == 0 && this->bits_per_sample == 0) {
-      this->rate=buf->decoder_info[1];
-      this->bits_per_sample=buf->decoder_info[2] ; 
-      this->number_of_channels=buf->decoder_info[3] ; 
-      this->ao_cap_mode=(this->number_of_channels == 2) ? AO_CAP_MODE_STEREO : AO_CAP_MODE_MONO; 
-    }
-    printf ("liblpcm: opening audio output (%d Hz sampling rate, mode=%d)\n",
-	    this->rate, this->ao_cap_mode);
-    this->output_open = this->audio_out->open (this->audio_out, this->bits_per_sample, 
-						this->rate,
-						this->ao_cap_mode) ;
-  }
-  if (!this->output_open) 
+  if (!this->output_open || (buf->decoder_flags & BUF_FLAG_HEADER) ) 
     return;
 
   audio_buffer = this->audio_out->get_buffer (this->audio_out);
-
+  
   /* Swap LPCM samples into native byte order, if necessary */
-
+  buf->type &= 0xffff0000;
   stream_be = ( buf->type == BUF_AUDIO_LPCM_BE );
-  if (stream_be != this->cpu_be)
-    swab (sample_buffer, audio_buffer->mem, buf->size);
-  else
+  
+  if( this->bits_per_sample == 16 ){
+    if (stream_be != this->cpu_be)
+      swab (sample_buffer, audio_buffer->mem, buf->size);
+    else
+      memcpy (audio_buffer->mem, sample_buffer, buf->size);
+  }
+  else if( this->bits_per_sample == 20 ) {
+    uint8_t *s = (uint8_t *)sample_buffer;
+    uint8_t *d = (uint8_t *)audio_buffer->mem;
+    int n = buf->size;
+    
+    if (stream_be != this->cpu_be) {
+      while( n >= 0 ) {
+        swab( s, d, 8 );
+        s += 10;
+        d += 8;
+        n -= 10; 
+      }
+    } else {
+      while( n >= 0 ) {
+        memcpy( d, s, 8 );
+        s += 10;
+        d += 8;
+        n -= 10; 
+      }
+    }
+  }
+  else {
     memcpy (audio_buffer->mem, sample_buffer, buf->size);
-
+  }
+  
   audio_buffer->vpts       = buf->pts;
   audio_buffer->num_frames = (((buf->size*8)/this->number_of_channels)/this->bits_per_sample);
 
@@ -157,11 +208,11 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   lpcm_decoder_t *this ;
 
-  if (iface_version != 7) {
-    printf( "liblpcm: plugin doesn't support plugin API version %d.\n"
-	    "liblpcm: this means there's a version mismatch between xine and this "
-	    "liblpcm: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 9) {
+    printf(_("liblpcm: plugin doesn't support plugin API version %d.\n"
+	     "liblpcm: this means there's a version mismatch between xine and this "
+	     "liblpcm: decoder plugin.\nInstalling current plugins should help.\n"),
+	     iface_version);
     return NULL;
   }
 
