@@ -1,7 +1,7 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2002 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.19 2002/04/23 15:45:24 esnel Exp $
+ * $Id: xine_decoder.c,v 1.23 2002/07/28 21:10:40 heikos Exp $
  *
  * stuff needed to turn libmad into a xine decoder plugin
  */
@@ -279,11 +279,11 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   mad_decoder_t *this ;
 
-  if (iface_version != 7) {
-    printf( "libmad: plugin doesn't support plugin API version %d.\n"
-	    "libmad: this means there's a version mismatch between xine and this "
-	    "libmad: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 9) {
+    printf(_("libmad: plugin doesn't support plugin API version %d.\n"
+	     "libmad: this means there's a version mismatch between xine and this "
+	     "libmad: decoder plugin.\nInstalling current plugins should help.\n"),
+	     iface_version);
 
     return NULL;
   }
