@@ -175,7 +175,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     picture = mpeg2dec->picture;
     is_frame_done = mpeg2dec->in_slice && ((!code) || (code >= 0xb0));
 
-    if (is_frame_done) {
+    if (is_frame_done && picture->current_frame != NULL) {
 	mpeg2dec->in_slice = 0;
 
 	if (((picture->picture_structure == FRAME_PICTURE) ||
@@ -207,7 +207,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0x00:	/* picture_start_code */
 	if (mpeg2_header_picture (picture, buffer)) {
 	    fprintf (stderr, "bad picture header\n");
-	    exit (1);
+	    abort();
 	}
 
 	mpeg2dec->is_frame_needed=0;
@@ -287,8 +287,11 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0xb3:	/* sequence_header_code */
 	if (mpeg2_header_sequence (picture, buffer)) {
 	    fprintf (stderr, "bad sequence header\n");
-	    /* exit (1); */
-	} else if (mpeg2dec->is_sequence_needed 
+	    /* abort(); */
+	    break;
+	}
+	if (mpeg2dec->force_aspect) picture->aspect_ratio_information = mpeg2dec->force_aspect;
+	if (mpeg2dec->is_sequence_needed 
 	    || (picture->frame_width != picture->coded_picture_width)
 	    || (picture->frame_height != picture->coded_picture_height)) {
             xine_frame_change_event_t notify_event;
@@ -305,6 +308,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    notify_event.event.type = XINE_EVENT_FRAME_CHANGE;
 	    notify_event.width = picture->coded_picture_width;
 	    notify_event.height = picture->coded_picture_height;
+	    notify_event.aspect = picture->aspect_ratio_information;
 	    xine_send_event(mpeg2dec->xine, &notify_event.event);
 
 	    if (picture->forward_reference_frame) 
@@ -327,7 +331,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0xb5:	/* extension_start_code */
 	if (mpeg2_header_extension (picture, buffer)) {
 	    fprintf (stderr, "bad extension\n");
-	    exit (1);
+	    abort();
 	}
 	break;
 
@@ -338,7 +342,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     case 0xb8:	/* group of pictures start code */
 	if (mpeg2_header_group_of_pictures (picture, buffer)) {
 	  printf ("libmpeg2: bad group of pictures\n");
-	  exit (1);
+	  abort();
 	}
     default:
 	if (code >= 0xb9)
@@ -401,7 +405,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    }
 	}
 
-	if (!(mpeg2dec->drop_frame)) {
+	if (!mpeg2dec->drop_frame && picture->current_frame != NULL) {
 	  mpeg2_slice (picture, code, buffer);
 	  if( picture->v_offset > picture->limit_y ) 
 	    picture->current_frame->bad_frame = 0;
@@ -490,32 +494,38 @@ void mpeg2_reset (mpeg2dec_t * mpeg2dec) {
   
   mpeg2dec->pts = 0;
   
-  if( !picture->mpeg1 )
+  if( !picture->mpeg1 ) 
     mpeg2dec->is_sequence_needed = 1;
+  else {
+    /* to free reference frames one also needs to fix slice.c to 
+     * abort when they are NULL. unfortunately it seems to break
+     * DVD menus.
+     *
+     * ...so let's do this for mpeg-1 only :)
+     */
+    if ( picture->current_frame && 
+	 picture->current_frame != picture->backward_reference_frame &&
+	 picture->current_frame != picture->forward_reference_frame )
+      picture->current_frame->free (picture->current_frame);
+    picture->current_frame = NULL;
+    
+    if (picture->forward_reference_frame)
+      picture->forward_reference_frame->free (picture->forward_reference_frame);
+    picture->forward_reference_frame = NULL;
+    
+    if (picture->backward_reference_frame)
+      picture->backward_reference_frame->free (picture->backward_reference_frame);
+    picture->backward_reference_frame = NULL;
+  }
   
   mpeg2dec->in_slice = 0;
 
-  /* to free reference frames one also needs to fix slice.c to 
-   * abort when they are NULL. unfortunately it seems to break
-   * DVD menus.
-   */
-  /*
-  if ( picture->current_frame && 
-       picture->current_frame != picture->backward_reference_frame &&
-       picture->current_frame != picture->forward_reference_frame )
-    picture->current_frame->free (picture->current_frame);
-  picture->current_frame = NULL;
-  
-  if (picture->forward_reference_frame)
-    picture->forward_reference_frame->free (picture->forward_reference_frame);
-  picture->forward_reference_frame = NULL;
-  
-  if (picture->backward_reference_frame)
-    picture->backward_reference_frame->free (picture->backward_reference_frame);
-  picture->backward_reference_frame = NULL;
-  */
 }
 
+/* flush must never allocate any frame (get_frame/duplicate_frame).
+ * it is called from inside video_out loop and frame allocation
+ * may cause some (rare) deadlocks.
+ */
 void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
 
   picture_t *picture = mpeg2dec->picture;
@@ -525,8 +535,6 @@ void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
   
   if (picture->current_frame && !picture->current_frame->drawn &&
       !picture->current_frame->bad_frame) {
-
-    vo_frame_t *img;
     
     printf ("libmpeg2: blasting out current frame %d on flush\n",
 	    picture->current_frame->id);
@@ -534,16 +542,8 @@ void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
     picture->current_frame->drawn = 1;
     get_frame_duration(mpeg2dec, picture->current_frame);
     
-    /* output a copy instead of the frame used by decoder */
-    img = picture->current_frame->instance->duplicate_frame(picture->current_frame->instance, 
-								       picture->current_frame);
-    img->pts = 0;
-    img->bad_frame = 0;
-    img->drawn = 1; 
-    
-    img->draw(img);
-    img->free(img);
-
+    picture->current_frame->pts = 0;
+    picture->current_frame->draw(picture->current_frame);
   }
 
 }
@@ -631,6 +631,7 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 	printf ("libmpeg2: bad sequence header\n");
 	continue;
       }
+      if (mpeg2dec->force_aspect) picture->aspect_ratio_information = mpeg2dec->force_aspect;
 	  
       if (mpeg2dec->is_sequence_needed) {
         xine_frame_change_event_t notify_event;
@@ -649,6 +650,7 @@ void mpeg2_find_sequence_header (mpeg2dec_t * mpeg2dec,
 	notify_event.event.type = XINE_EVENT_FRAME_CHANGE;
 	notify_event.width = picture->coded_picture_width;
 	notify_event.height = picture->coded_picture_height;
+	notify_event.aspect = picture->aspect_ratio_information;
 	xine_send_event(mpeg2dec->xine, &notify_event.event);
       }
     } else if (code == 0xb5) {	/* extension_start_code */
