@@ -35,7 +35,7 @@
  * along with this program; see the file COPYING.  If not, write to
  * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: spu.c,v 1.39 2002/04/24 20:26:07 jcdutton Exp $
+ * $Id: spu.c,v 1.43 2002/07/20 16:28:14 mroi Exp $
  *
  */
 
@@ -70,7 +70,8 @@
 void spudec_reassembly (spudec_seq_t *seq, uint8_t *pkt_data, u_int pkt_len);
 void spudec_process( spudec_decoder_t *this, uint32_t stream_id);
 void spudec_decode_nav( spudec_decoder_t *this, buf_element_t *buf);
-void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode, vo_overlay_t * overlay );
+void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
+                                vo_overlay_t * overlay, vo_overlay_t * base );
 static void spudec_do_commands (spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
 static void spudec_draw_picture (spudec_state_t *state, spudec_seq_t* seq, vo_overlay_t *ovl);
 static void spudec_discover_clut (spudec_state_t *state, vo_overlay_t *ovl);
@@ -323,6 +324,14 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
 #endif
       pthread_mutex_lock(&this->nav_pci_lock);
       if (this->pci.hli.hl_gi.hli_s_ptm == this->spudec_stream_state[stream_id].pts) {
+        if (this->state.visible == EVENT_HIDE_MENU) {
+          /* menus are hidden via nav packet decoding, not here */
+	  /* FIXME: James is not sure about this solution and may want to look this over.
+	   *        I'm commiting it, because I haven't found a disc it breaks, but it fixes
+	   *        some instead.   Michael Roitzsch */
+          pthread_mutex_unlock(&this->nav_pci_lock);
+          continue;
+        }
         if ( this->pci.hli.hl_gi.fosl_btnn > 0) {
           spu_button_t spu_button;
           xine_spu_event_t spu_event;
@@ -335,7 +344,8 @@ void spudec_process (spudec_decoder_t *this, uint32_t stream_id) {
 #ifdef LOG_BUTTON
         fprintf(stderr, "libspudec:Full Overlay\n");
 #endif
-        spudec_copy_nav_to_overlay(&this->pci, this->state.clut, this->buttonN, 0, &this->overlay );
+        spudec_copy_nav_to_overlay(&this->pci, this->state.clut, this->buttonN, 0, &this->overlay,
+                                   &this->overlay);
       } else {
       /* Subtitle and not a menu button */
         int i;
@@ -576,8 +586,7 @@ static void spudec_do_commands(spudec_state_t *state, spudec_seq_t* seq, vo_over
     default:
       printf("libspudec: unknown seqence command (%02x)\n", buf[0]);
       /* FIXME: SPU should be dropped, and buffers resynced */
-      assert(0);
-      buf++;
+      buf = next_seq;
       break;
     }
   }
@@ -824,7 +833,8 @@ static void spudec_print_overlay( vo_overlay_t *ovl ) {
 	  ovl->clip_trans[0], ovl->clip_trans[1], ovl->clip_trans[2], ovl->clip_trans[3]);
   return;
 } 
-void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode, vo_overlay_t * overlay ) {
+void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button, int32_t mode,
+                                vo_overlay_t * overlay, vo_overlay_t * base ) {
   btni_t *button_ptr;
   int i;
 
@@ -843,10 +853,13 @@ void spudec_copy_nav_to_overlay(pci_t* nav_pci, uint32_t* clut, int32_t button,
   }
   /* FIXME:Only the first grouping of buttons are used at the moment */
   button_ptr = &nav_pci->hli.btnit[button-1];
-  overlay->clip_left = button_ptr->x_start;
-  overlay->clip_top  = button_ptr->y_start;
-  overlay->clip_right = button_ptr->x_end;
-  overlay->clip_bottom = button_ptr->y_end;
+  /* button areas in the nav packet are in screen coordinates,
+   * overlay clipping areas are in overlay coordinates;
+   * therefore we must subtract the display coordinates of the underlying overlay */
+  overlay->clip_left   = (button_ptr->x_start > base->x) ? (button_ptr->x_start - base->x) : 0;
+  overlay->clip_top    = (button_ptr->y_start > base->y) ? (button_ptr->y_start - base->y) : 0;
+  overlay->clip_right  = (button_ptr->x_end   > base->x) ? (button_ptr->x_end   - base->x) : 0;
+  overlay->clip_bottom = (button_ptr->y_end   > base->y) ? (button_ptr->y_end   - base->y) : 0;
   if(button_ptr->btn_coln != 0) {
 #ifdef LOG_BUTTON
     fprintf(stderr, "libspudec: normal button clut\n");
