@@ -24,7 +24,9 @@
 #ifndef HAVE_SPU_API_H
 #define HAVE_SPU_API_H
 
- /*
+#define SPU_DECODER_IFACE_VERSION 9
+
+/*
  * generic xine spu decoder plugin interface
  *
  * for a dynamic plugin make sure you provide this function call:
@@ -54,8 +56,6 @@ struct spu_decoder_s {
 
   int priority;
 
-  metronom_t *metronom;
-
 };
 
 typedef struct spu_button_s spu_button_t;
