@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2000-2001 the xine project
  * 
- * This file is part of xine, a unix video player.
+ * This file is part of xine, a free video player.
  * 
  * xine is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.21 2002/04/23 15:45:25 esnel Exp $
+ * $Id: xine_decoder.c,v 1.38 2002/07/05 17:32:03 mroi Exp $
  *
  * code based on mplayer module:
  *
@@ -36,11 +36,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <ctype.h>
-#ifdef __FreeBSD__
-# include <sys/iconv.h>
-#else
-# include <iconv.h>
-#endif
+#include <iconv.h>
 
 #include "buffer.h"
 #include "events.h"
@@ -70,6 +66,14 @@ typedef struct {
 } subtitle_t;
 
 
+typedef enum {
+  SUBTITLE_SIZE_SMALL = 0,
+  SUBTITLE_SIZE_NORMAL,
+  SUBTITLE_SIZE_LARGE,
+
+  SUBTITLE_SIZE_NUM        /* number of values in enum */
+} subtitle_size;
+
 typedef struct sputext_decoder_s {
   spu_decoder_t      spu_decoder;
 
@@ -83,6 +87,7 @@ typedef struct sputext_decoder_s {
   float              mpsub_position;  
 
   int                width;          /* frame width                */
+  int                height;         /* frame height               */
   int                font_size;
   int                line_height;
   int                uses_time;  
@@ -99,6 +104,9 @@ typedef struct sputext_decoder_s {
   char              *src_encoding;
   char              *dst_encoding;
 
+  subtitle_size      subtitle_size;
+  int64_t            last_subtitle_end; /* no new subtitle before this vpts */
+  int                time_offset;       /* offset in 1/100sec to add to vpts */
 } sputext_decoder_t;
 
 #define FORMAT_MICRODVD  0
@@ -695,7 +703,9 @@ static subtitle_t *sub_read_file (sputext_decoder_t *this) {
       ++this->errs; 
     else {
       int i;
-
+      if (this->num > 0 && first[this->num-1].end == -1) {
+	first[this->num-1].end = sub->start;
+      }
       for(i=0; i<first[this->num].lines; i++)
       { char *tmp;
 	char *in_buff, *out_buff;
@@ -772,43 +782,55 @@ static void spudec_init (spu_decoder_t *this_gen, vo_instance_t *vo_out) {
 
 }
 
+static void update_font_size (sputext_decoder_t *this) {
+  static int sizes[SUBTITLE_SIZE_NUM][4] = {
+    { 16, 16, 16, 20 }, /* SUBTITLE_SIZE_SMALL  */
+    { 16, 16, 20, 24 }, /* SUBTITLE_SIZE_NORMAL */
+    { 16, 20, 24, 32 }, /* SUBTITLE_SIZE_LARGE  */
+  };
+
+  int *vec = sizes[this->subtitle_size];
+  int y;
+
+  if( this->width >= 512 )
+    this->font_size = vec[3];
+  else if( this->width >= 384 )
+    this->font_size = vec[2];
+  else if( this->width >= 320 )
+    this->font_size = vec[1];
+  else
+    this->font_size = vec[0];
+  
+  this->line_height = this->font_size + 10;
+
+  y = this->height - (SUB_MAX_TEXT * this->line_height) - 5;
+  
+  if( this->osd )
+    this->renderer->free_object (this->osd);
+
+  if(this->renderer) {
+    this->osd = this->renderer->new_object (this->renderer, 
+					    this->width,
+					    SUB_MAX_TEXT * this->line_height);
+    
+    this->renderer->set_font (this->osd, this->font, this->font_size);
+    this->renderer->set_position (this->osd, 0, y);
+  }
+}
+
 static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
   sputext_decoder_t *this = (sputext_decoder_t *) this_gen;
   int64_t current_time;
   
   if (buf->decoder_flags & BUF_FLAG_HEADER) {
+    this->width  = buf->decoder_info[1];
+    this->height = buf->decoder_info[2];
 
-    int y;
-
-    this->width = buf->decoder_info[1];
-
-    /* trying to make better sized fonts... 
-       another idea would be changing font size just
-       if we detect the text line wont fit.
-    */
-    this->font_size = 16;
-    if( this->width >= 320 ) 
-      this->font_size = 20;
-    if( this->width >= 384 )
-      this->font_size = 24;
-    if( this->width >= 512 )
-      this->font_size = 32;
-    this->line_height = this->font_size + 10;
-    
     this->renderer = this->xine->osd_renderer;
-    this->osd = this->renderer->new_object (this->renderer, 
-					    this->width,
-					    SUB_MAX_TEXT * this->line_height);
+    this->osd = NULL;
 
-     
-    this->renderer->set_font (this->osd, this->font, this->font_size);
-
-    y = buf->decoder_info[2] - (SUB_MAX_TEXT * this->line_height) - 5;
-
-    this->renderer->set_position (this->osd, 0, y); 
-
-    /*  this->renderer->render_text (this->osd, 0, 0, "sputext decoder", OSD_TEXT1); */
+    update_font_size (this);
     
     current_time = this->xine->metronom->get_current_time (this->xine->metronom);
     this->renderer->show (this->osd, current_time);
@@ -825,9 +847,7 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
 
   } else {
 
-    int64_t     pts, pts_end;
-    int64_t     pts_factor;
-    int         frame_num;
+    int64_t     sub_start, sub_end;
     subtitle_t *subtitle;
 
     /* don't want to see subtitle */
@@ -835,70 +855,73 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
       return;
 
     subtitle = NULL;
-      
-    pts       = buf->pts;
-    pts_end   = pts;
-    frame_num = buf->decoder_info[1];
     
     /* 
      * find out which subtitle to display 
      */
 
     if (!this->uses_time) {
+      int         frame_num;
+      int64_t     pts_factor;
+      long        frame_offset;
 
-      while ( (this->cur < this->num) 
-	      && (this->subtitles[this->cur].start < frame_num) )
-	this->cur++;
-
-      if (this->cur >= this->num)
-	return;
-
-      subtitle = &this->subtitles[this->cur];
-
-      if (subtitle->start > frame_num)
-	return;
+      frame_num = buf->decoder_info[1];
 
       /* FIXME FIXME FIXME 
       pts_factor = this->xine->metronom->get_video_rate (this->xine->metronom);
       */
       pts_factor = 3000;
 
-      pts += this->xine->metronom->vpts_offset;
+      frame_offset = this->time_offset * 900 / pts_factor;
 
-      pts_end = pts + (subtitle->end - subtitle->start) * pts_factor;
+      while ( (this->cur < this->num) 
+	      && (this->subtitles[this->cur].start + frame_offset < frame_num) )
+	this->cur++;
+
+      if (this->cur >= this->num)
+	return;
+
+      subtitle = &this->subtitles[this->cur];
+
+      if (subtitle->start + frame_offset > frame_num)
+	return;
+
+      sub_start = this->xine->metronom->got_spu_packet(this->xine->metronom, buf->pts);
+      sub_end = sub_start + (subtitle->end - subtitle->start) * pts_factor;
 
     } else {
-
       uint32_t start_tenth;
 
-      start_tenth = pts/900;
+      start_tenth = buf->pts/900;
 
 #ifdef LOG
       printf ("sputext: searching for spu for %d\n", start_tenth);
 #endif
 
       while ( (this->cur < this->num) 
-	      && (this->subtitles[this->cur].start < start_tenth) )
+	      && (this->subtitles[this->cur].start + this->time_offset < start_tenth) )
 	this->cur++;
 
       if (this->cur >= this->num)
 	return;
 
 #ifdef LOG
-      printf ("sputext: found >%s<, start %d, end %d\n", this->subtitles[this->cur].text[0],
-	      this->subtitles[this->cur].start, this->subtitles[this->cur].end);
+      printf ("sputext: found >%s<, start %ld, end %ld\n", this->subtitles[this->cur].text[0],
+	      this->subtitles[this->cur].start + this->time_offset, this->subtitles[this->cur].end);
 #endif
 
       subtitle = &this->subtitles[this->cur];
 
-      if (subtitle->start > (start_tenth+20))
+      if (subtitle->start + this->time_offset > (start_tenth+20))
 	return;
 
-      pts += this->xine->metronom->vpts_offset;
-
-      pts_end = pts + (subtitle->end - subtitle->start)*900;
+      sub_start = this->xine->metronom->got_spu_packet(this->xine->metronom, (subtitle->start+this->time_offset)*900);
+      sub_end = sub_start + (subtitle->end - subtitle->start)*900;
     }
 
+    if( !sub_start )
+      return;
+
     if (subtitle) {
       int line, y;
       int font_size;
@@ -929,14 +952,19 @@ static void spudec_decode_data (spu_decoder_t *this_gen, buf_element_t *buf) {
        
       if( font_size != this->font_size )
         this->renderer->set_font (this->osd, this->font, this->font_size);
+
+      if( this->last_subtitle_end && sub_start < this->last_subtitle_end ) {
+	sub_start = this->last_subtitle_end;
+      }
+      this->last_subtitle_end = sub_end;
         
       this->renderer->set_text_palette (this->osd, -1, OSD_TEXT1);
-      this->renderer->show (this->osd, pts );
-      this->renderer->hide (this->osd, pts_end);
+      this->renderer->show (this->osd, sub_start);
+      this->renderer->hide (this->osd, sub_end);
 
 #ifdef LOG
       printf ("sputext: scheduling subtitle >%s< at %lld until %lld, current time is %lld\n",
-	      subtitle->text[0], pts, pts_end, 
+	      subtitle->text[0], sub_start, sub_end, 
 	      this->xine->metronom->get_current_time (this->xine->metronom));
 #endif
 
@@ -997,6 +1025,24 @@ static void update_osd_dst_encoding(void *this_gen, cfg_entry_t *entry)
   printf("libsputext: spu_dst_encoding = %s\n", this->dst_encoding );
 }
 
+static void update_subtitle_size(void *this_gen, cfg_entry_t *entry)
+{
+  sputext_decoder_t *this = (sputext_decoder_t *)this_gen;
+
+  this->subtitle_size = entry->num_value;
+
+  update_font_size (this_gen);
+}
+
+static void update_time_offset(void *this_gen, cfg_entry_t *entry)
+{
+  sputext_decoder_t *this = (sputext_decoder_t *)this_gen;
+
+  this->time_offset = entry->num_value;
+
+  printf("libsputext: time_offset = %d\n", this->time_offset );
+}
+
 static void spudec_dispose (spu_decoder_t *this_gen) {
   free (this_gen);
 }
@@ -1004,11 +1050,12 @@ static void spudec_dispose (spu_decoder_t *this_gen) {
 spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine) {
 
   sputext_decoder_t *this ;
+  static char *subtitle_size_strings[] = { "small", "normal", "large", NULL };
 
-  if (iface_version != 6) {
-    printf("libsputext: doesn't support plugin api version %d.\n"
-	   "libsputext: This means there is a version mismatch between xine and\n"
-	   "libsputext: this plugin.\n", iface_version);
+  if (iface_version != 9) {
+    printf(_("libsputext: doesn't support plugin api version %d.\n"
+	     "libsputext: This means there is a version mismatch between xine and\n"
+	     "libsputext: this plugin.\n"), iface_version);
     return NULL;
   }
 
@@ -1028,18 +1075,29 @@ spu_decoder_t *init_spu_decoder_plugin (int iface_version, xine_t *xine) {
   this->font                            = xine->config->register_string(xine->config, 
 									"codec.spu_font", 
 									"sans", 
-									"font for avi subtitles", 
+									_("font for avi subtitles"), 
 									NULL, update_osd_font, this);
+  this->subtitle_size                    = xine->config->register_enum(xine->config, 
+								       "codec.spu_subtitle_size", 
+								       1,
+								       subtitle_size_strings,
+								       _("subtitle size (relative window size)"), 
+								       NULL, update_subtitle_size, this);
   this->src_encoding                    = xine->config->register_string(xine->config, 
 									"codec.spu_src_encoding", 
 									"windows-1250", 
-									"source encoging of subtitles", 
-									NULL, update_osd_dst_encoding, this);
+									_("source encoding of subtitles"), 
+									NULL, update_osd_src_encoding, this);
   this->dst_encoding                    = xine->config->register_string(xine->config, 
 									"codec.spu_dst_encoding", 
 									"iso-8859-2", 
-									"target encoging for subtitles (have to match font encoding)", 
+									_("target encoding for subtitles (have to match font encoding)"), 
 									NULL, update_osd_dst_encoding, this);
+  this->time_offset                     = xine->config->register_num   (xine->config, 
+								        "codec.spu_time_offset", 
+								        0,
+								        _("subtitle time offset in 1/100 sec"), 
+								        NULL, update_time_offset, this);
 
   return (spu_decoder_t *) this;
 }
