@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.11 2002/04/23 15:45:25 esnel Exp $
+ * $Id: xine_decoder.c,v 1.14 2002/07/05 17:32:04 mroi Exp $
  *
  * (ogg/)vorbis audio decoder plugin (libvorbis wrapper) for xine
  */
@@ -243,11 +243,11 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
 
   vorbis_decoder_t *this ;
 
-  if (iface_version != 7) {
-    printf( "libvorbis: plugin doesn't support plugin API version %d.\n"
-	    "libvorbis: this means there's a version mismatch between xine and this "
-	    "libvorbis: decoder plugin.\nInstalling current plugins should help.\n",
-	    iface_version);
+  if (iface_version != 9) {
+    printf(_("libvorbis: plugin doesn't support plugin API version %d.\n"
+	     "libvorbis: this means there's a version mismatch between xine and this "
+	     "libvorbis: decoder plugin.\nInstalling current plugins should help.\n"),
+	   iface_version);
 
     return NULL;
   }
