@@ -74,9 +74,10 @@ static ct check[] = {
 		{12, fccIYUV, &MEDIASUBTYPE_IYUV, CAP_IYUV},
 		{16, fccUYVY, &MEDIASUBTYPE_UYVY, CAP_UYVY},
 		{12, fccYV12, &MEDIASUBTYPE_YV12, CAP_YV12},
-		{16, fccYV12, &MEDIASUBTYPE_YV12, CAP_YV12},
+		//{16, fccYV12, &MEDIASUBTYPE_YV12, CAP_YV12},
 		{16, fccYVYU, &MEDIASUBTYPE_YVYU, CAP_YVYU},
-		//{12, fccI420, &MEDIASUBTYPE_I420, CAP_I420},
+		{12, fccI420, &MEDIASUBTYPE_I420, CAP_I420},
+		{9,  fccYVU9, &MEDIASUBTYPE_YVU9, CAP_YVU9},
 		{0},
 	    };
 
@@ -106,10 +107,10 @@ DS_VideoDecoder * DS_VideoDecoder_Open(char* dllname, GUID* guid, BITMAPINFOHEAD
         
 	bihs = (format->biSize < (int) sizeof(BITMAPINFOHEADER)) ?
 	    sizeof(BITMAPINFOHEADER) : format->biSize;
-        bihs = sizeof(VIDEOINFOHEADER) - sizeof(BITMAPINFOHEADER) + bihs;
      
         this->iv.m_bh = (BITMAPINFOHEADER*)malloc(bihs);
         memcpy(this->iv.m_bh, format, bihs);
+
         this->iv.m_State = STOP;
         //this->iv.m_pFrame = 0;
         this->iv.m_Mode = DIRECT;
@@ -118,6 +119,7 @@ DS_VideoDecoder * DS_VideoDecoder_Open(char* dllname, GUID* guid, BITMAPINFOHEAD
         this->iv.m_fQuality = 0.0f;
         this->iv.m_bCapable16b = true;
                 
+        bihs += sizeof(VIDEOINFOHEADER) - sizeof(BITMAPINFOHEADER);
 	this->m_sVhdr = (VIDEOINFOHEADER*)malloc(bihs);
 	memset(this->m_sVhdr, 0, bihs);
 	memcpy(&this->m_sVhdr->bmiHeader, this->iv.m_bh, this->iv.m_bh->biSize);
@@ -193,6 +195,7 @@ DS_VideoDecoder * DS_VideoDecoder_Open(char* dllname, GUID* guid, BITMAPINFOHEAD
 
 	switch (this->iv.m_bh->biCompression)
 	{
+#if 0
 	case fccDIV3:
 	case fccDIV4:
 	case fccDIV5:
@@ -207,10 +210,12 @@ DS_VideoDecoder * DS_VideoDecoder_Open(char* dllname, GUID* guid, BITMAPINFOHEAD
 	    //m_Caps = CAP_I420;
 	    this->m_Caps = (CAP_YUY2 | CAP_UYVY);
 	    break;
+#endif
 	default:
               
 	    this->m_Caps = CAP_NONE;
 
+	    printf("Decoder supports the following YUV formats: ");
 	    for (c = check; c->bits; c++)
 	    {
 		this->m_sVhdr2->bmiHeader.biBitCount = c->bits;
@@ -218,12 +223,16 @@ DS_VideoDecoder * DS_VideoDecoder_Open(char* dllname, GUID* guid, BITMAPINFOHEAD
 		this->m_sDestType.subtype = *c->subtype;
 		result = this->m_pDS_Filter->m_pOutputPin->vt->QueryAccept(this->m_pDS_Filter->m_pOutputPin, &this->m_sDestType);
 		if (!result)
+		{
 		    this->m_Caps = (this->m_Caps | c->cap);
+		    printf("%.4s ", &c->fcc);
+		}
 	    }
+	    printf("\n");
 	}
 
 	if (this->m_Caps != CAP_NONE)
-	    printf("Decoder is capable of YUV output ( flags 0x%x)\n", (int)this->m_Caps);
+	    printf("Decoder is capable of YUV output (flags 0x%x)\n", (int)this->m_Caps);
 
 	this->m_sVhdr2->bmiHeader.biBitCount = 24;
 	this->m_sVhdr2->bmiHeader.biCompression = 0;
@@ -483,12 +492,17 @@ int DS_VideoDecoder_SetDestFmt(DS_VideoDecoder *this, int bits, unsigned int csp
 	case fccIYUV:
 	    this->m_sDestType.subtype = MEDIASUBTYPE_IYUV;
 	    break;
+	case fccI420:
+	    this->m_sDestType.subtype = MEDIASUBTYPE_I420;
+	    break;
 	case fccUYVY:
 	    this->m_sDestType.subtype = MEDIASUBTYPE_UYVY;
 	    break;
 	case fccYVYU:
 	    this->m_sDestType.subtype = MEDIASUBTYPE_YVYU;
 	    break;
+	case fccYVU9:
+	    this->m_sDestType.subtype = MEDIASUBTYPE_YVU9;
 	default:
 	    ok = false;
             break;
@@ -527,6 +541,10 @@ int DS_VideoDecoder_SetDestFmt(DS_VideoDecoder *this, int bits, unsigned int csp
 	if(!(this->m_Caps & CAP_IYUV))
 	    should_test=false;
 	break;
+    case fccI420:
+	if(!(this->m_Caps & CAP_I420))
+	    should_test=false;
+	break;
     case fccUYVY:
 	if(!(this->m_Caps & CAP_UYVY))
 	    should_test=false;
@@ -535,6 +553,10 @@ int DS_VideoDecoder_SetDestFmt(DS_VideoDecoder *this, int bits, unsigned int csp
 	if(!(this->m_Caps & CAP_YVYU))
 	    should_test=false;
 	break;
+    case fccYVU9:
+	if(!(this->m_Caps & CAP_YVU9))
+	    should_test=false;
+	break;
     }
     if(should_test)
 	result = this->m_pDS_Filter->m_pOutputPin->vt->QueryAccept(this->m_pDS_Filter->m_pOutputPin, &this->m_sDestType);
@@ -725,7 +747,7 @@ int DS_VideoDecoder_SetValue(DS_VideoDecoder *this, const char* name, int value)
 {
     if (this->m_bIsDivX4) {
 	IDivxFilterInterface* pIDivx=NULL;
-	printf("DS_SetValue for DIVX4, name=%s  value=%d\n",name,value);
+//	printf("DS_SetValue for DIVX4, name=%s  value=%d\n",name,value);
 	if (this->m_pDS_Filter->m_pFilter->vt->QueryInterface((IUnknown*)this->m_pDS_Filter->m_pFilter, &IID_IDivxFilterInterface, (void**)&pIDivx))
 	{
 	    printf("No such interface\n");
@@ -764,7 +786,7 @@ int DS_VideoDecoder_SetValue(DS_VideoDecoder *this, const char* name, int value)
 // get5=set4 19
 	// get6=set5 23
     	hidden = (IHidden*)((int)this->m_pDS_Filter->m_pFilter + 0xb8);
-	printf("DS_SetValue for DIVX, name=%s  value=%d\n",name,value);
+//	printf("DS_SetValue for DIVX, name=%s  value=%d\n",name,value);
 	if (strcmp(name, "Quality") == 0)
 	{
             this->m_iLastQuality = value;
@@ -826,7 +848,7 @@ int DS_VideoDecoder_SetValue(DS_VideoDecoder *this, const char* name, int value)
 	return result;
     }
 #endif
-    printf("DS_SetValue for ????, name=%s  value=%d\n",name,value);
+//    printf("DS_SetValue for ????, name=%s  value=%d\n",name,value);
     return 0;
 }
 /*
