@@ -17,13 +17,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: w32codec.c,v 1.76 2002/04/28 15:32:41 esnel Exp $
+ * $Id: w32codec.c,v 1.88 2002/07/15 21:42:33 esnel Exp $
  *
  * routines for using w32 codecs
  * DirectShow support by Miguel Freitas (Nov/2001)
  *
  */
-
+                   
 #include <stdlib.h>
 #include <stdio.h>
 #include <inttypes.h>
@@ -81,6 +81,18 @@ static GUID dvsd_clsid =
 	{0xAF, 0x79, 0x00, 0xAA, 0x00, 0xB6, 0x7A, 0x42}
 };
 
+static GUID msmpeg4_clsid =
+{
+	0x82CCd3E0, 0xF71A, 0x11D0,
+	{ 0x9f, 0xe5, 0x00, 0x60, 0x97, 0x78, 0xea, 0x66}
+};
+
+static GUID mss1_clsid =
+{
+	0x3301a7c4, 0x0a8d, 0x11d4,
+	{ 0x91, 0x4d, 0x00, 0xc0, 0x4f, 0x61, 0x0d, 0x24 }
+};
+
 
 /* some data is shared inside wine loader.
  * this mutex seems to avoid some segfaults
@@ -95,11 +107,13 @@ extern char*   win32_def_path;
 typedef struct w32v_decoder_s {
   video_decoder_t   video_decoder;
 
+  xine_t           *xine;
+  
   vo_instance_t    *video_out;
   int64_t           video_step;
   int               decoder_ok;
 
-  BITMAPINFOHEADER  bih, o_bih; 
+  BITMAPINFOHEADER  *bih, o_bih; 
   char              scratch1[16]; /* some codecs overflow o_bih */
   HIC               hic;
   int               yuv_supported ;
@@ -122,11 +136,13 @@ typedef struct w32v_decoder_s {
   int               stream_id;
   int               skipframes;  
   
-  LDT_FS *ldt_fs;
+  ldt_fs_t *ldt_fs;
 } w32v_decoder_t;
 
 typedef struct w32a_decoder_s {
   audio_decoder_t   audio_decoder;
+  
+  xine_t           *xine;
 
   ao_instance_t    *audio_out;
   int               output_open;
@@ -152,7 +168,7 @@ typedef struct w32a_decoder_s {
   GUID             *guid;
   DS_AudioDecoder  *ds_dec;
   
-  LDT_FS *ldt_fs;
+  ldt_fs_t *ldt_fs;
 } w32a_decoder_t;
 
 
@@ -162,7 +178,7 @@ typedef struct w32a_decoder_s {
  * the last of them to close both.
  * hopefuly some day we will fix the wine loader to remove that.
  */   
-#define SYNC_SHUTDOWN
+//#define SYNC_SHUTDOWN
 
 #ifdef SYNC_SHUTDOWN
 w32v_decoder_t *w32v_instance;
@@ -275,12 +291,20 @@ static char* get_vids_codec_name(w32v_decoder_t *this,
   buf_type &= 0xffff0000;
 
   switch (buf_type) {
-  case BUF_VIDEO_MSMPEG4_V12:
+  case BUF_VIDEO_MSMPEG4_V1:
+  case BUF_VIDEO_MSMPEG4_V2:
     /* Microsoft MPEG-4 v1/v2 */
+    /* old dll is disabled now due segfaults 
+     * (using directshow instead)
     this->yuv_supported=1;
     this->yuv_hack_needed=1;
     this->flipped=1;
     return "mpg4c32.dll";
+    */
+    this->yuv_supported=1;
+    this->ds_driver = 1;
+    this->guid=&msmpeg4_clsid;
+    return "mpg4ds32.ax";    
 
   case BUF_VIDEO_MSMPEG4_V3:
     /* Microsoft MPEG-4 v3 */
@@ -365,6 +389,11 @@ static char* get_vids_codec_name(w32v_decoder_t *this,
     this->flipped=1;
     return "vp31vfw.dll";    
 
+  case BUF_VIDEO_MSS1:
+    this->ds_driver = 1;
+    this->guid=&mss1_clsid;
+    return "msscds32.ax";    
+
   }
 
   printf ("w32codec: this didn't happen: unknown video buf type %08x\n",
@@ -386,9 +415,10 @@ static char* get_vids_codec_name(w32v_decoder_t *this,
 static int w32v_can_handle (video_decoder_t *this_gen, int buf_type) {
   buf_type &= 0xFFFF0000;
 
-  return ( buf_type == BUF_VIDEO_MSMPEG4_V12 ||
-	   buf_type == BUF_VIDEO_MSMPEG4_V3 ||
-	   buf_type == BUF_VIDEO_IV50 ||
+  return ( buf_type == BUF_VIDEO_MSMPEG4_V1 ||
+           buf_type == BUF_VIDEO_MSMPEG4_V2 ||
+           buf_type == BUF_VIDEO_MSMPEG4_V3 ||
+           buf_type == BUF_VIDEO_IV50 ||
            buf_type == BUF_VIDEO_IV41 ||
            buf_type == BUF_VIDEO_IV32 ||
            buf_type == BUF_VIDEO_IV31 ||
@@ -400,7 +430,8 @@ static int w32v_can_handle (video_decoder_t *this_gen, int buf_type) {
 	   buf_type == BUF_VIDEO_DV ||
            buf_type == BUF_VIDEO_WMV7 ||
            buf_type == BUF_VIDEO_WMV8 ||
-           buf_type == BUF_VIDEO_VP31 );
+           buf_type == BUF_VIDEO_VP31 ||
+           buf_type == BUF_VIDEO_MSS1 );
 }
 
 static void w32v_init (video_decoder_t *this_gen, vo_instance_t *video_out) {
@@ -435,23 +466,23 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
   }
 
   this->hic = ICOpen (mmioFOURCC('v','i','d','c'), 
-		      this->bih.biCompression, 
+		      this->bih->biCompression, 
 		      ICMODE_FASTDECOMPRESS);
 
   if(!this->hic){
-    printf ("w32codec: ICOpen failed! unknown codec %08lx / wrong parameters?\n",
-	    this->bih.biCompression);
+    xine_log (this->xine, XINE_LOG_MSG, 
+              "w32codec: ICOpen failed! unknown codec %08lx / wrong parameters?\n",
+              this->bih->biCompression);
     this->decoder_ok = 0;
     return;
   }
 
-  ret = ICDecompressGetFormat(this->hic, &this->bih, &this->o_bih);
+  ret = ICDecompressGetFormat(this->hic, this->bih, &this->o_bih);
   if(ret){
-    printf("w32codec: ICDecompressGetFormat (%.4s %08lx/%d) failed: Error %ld\n",
-	   (char*)&this->o_bih.biCompression, 
-	   this->bih.biCompression,
-	   this->bih.biBitCount,
-	   (long)ret);
+    xine_log (this->xine, XINE_LOG_MSG, 
+              "w32codec: ICDecompressGetFormat (%.4s %08lx/%d) failed: Error %ld\n",
+              (char*)&this->o_bih.biCompression, this->bih->biCompression, 
+              this->bih->biBitCount, (long)ret);                
     this->decoder_ok = 0;
     return;
   }
@@ -469,7 +500,7 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
       * this->o_bih.biBitCount / 8;
 
   if (this->flipped)
-    this->o_bih.biHeight=-this->bih.biHeight; 
+    this->o_bih.biHeight=-this->bih->biHeight; 
 
   if(outfmt==IMGFMT_YUY2 && !this->yuv_hack_needed)
     this->o_bih.biCompression = mmioFOURCC('Y','U','Y','2');
@@ -477,21 +508,23 @@ static void w32v_init_codec (w32v_decoder_t *this, int buf_type) {
     this->o_bih.biCompression = 0;
       
   ret = (!this->ex_functions) 
-        ?ICDecompressQuery(this->hic, &this->bih, &this->o_bih)
-        :ICDecompressQueryEx(this->hic, &this->bih, &this->o_bih);
+        ?ICDecompressQuery(this->hic, this->bih, &this->o_bih)
+        :ICDecompressQueryEx(this->hic, this->bih, &this->o_bih);
   
   if(ret){
-    printf("w32codec: ICDecompressQuery failed: Error %ld\n", (long)ret);
+    xine_log (this->xine, XINE_LOG_MSG,
+              "w32codec: ICDecompressQuery failed: Error %ld\n", (long)ret);
     this->decoder_ok = 0;
     return;
   }
   
   ret = (!this->ex_functions) 
-        ?ICDecompressBegin(this->hic, &this->bih, &this->o_bih)
-        :ICDecompressBeginEx(this->hic, &this->bih, &this->o_bih);
+        ?ICDecompressBegin(this->hic, this->bih, &this->o_bih)
+        :ICDecompressBeginEx(this->hic, this->bih, &this->o_bih);
   
   if(ret){
-    printf("w32codec: ICDecompressBegin failed: Error %ld\n", (long)ret);
+    xine_log (this->xine, XINE_LOG_MSG,
+              "w32codec: ICDecompressBegin failed: Error %ld\n", (long)ret);
     this->decoder_ok = 0;
     return;
   }
@@ -527,12 +560,17 @@ static void w32v_init_ds_codec (w32v_decoder_t *this, int buf_type) {
 
   this->ldt_fs = Setup_LDT_Keeper();
   
+  /* hack: dvsd is the only fourcc accepted by qdv.dll */
+  if( buf_type ==  BUF_VIDEO_DV )
+    this->bih->biCompression = mmioFOURCC('d','v','s','d');
+
   this->ds_dec = DS_VideoDecoder_Open(win32_codec_name, this->guid,
-                                        &this->bih, this->flipped, 0);
+                                        this->bih, this->flipped, 0);
   
   if(!this->ds_dec){
-    printf ("w32codec: DS_VideoDecoder failed! unknown codec %08lx / wrong parameters?\n",
-	    this->bih.biCompression);
+    xine_log (this->xine, XINE_LOG_MSG,
+              "w32codec: DS_VideoDecoder failed! unknown codec %08lx / wrong parameters?\n",
+              this->bih->biCompression);
     this->decoder_ok = 0;
     return;
   }
@@ -549,14 +587,14 @@ static void w32v_init_ds_codec (w32v_decoder_t *this, int buf_type) {
   else
     this->o_bih.biBitCount=outfmt&0xFF;
 
-  this->o_bih.biWidth = this->bih.biWidth;
-  this->o_bih.biHeight = this->bih.biHeight;
+  this->o_bih.biWidth = this->bih->biWidth;
+  this->o_bih.biHeight = this->bih->biHeight;
   
   this->o_bih.biSizeImage = this->o_bih.biWidth * this->o_bih.biHeight
       * this->o_bih.biBitCount / 8;
 
   if (this->flipped)
-    this->o_bih.biHeight=-this->bih.biHeight; 
+    this->o_bih.biHeight=-this->bih->biHeight; 
 
   if(outfmt==IMGFMT_YUY2 && !this->yuv_hack_needed)
     this->o_bih.biCompression = mmioFOURCC('Y','U','Y','2');
@@ -609,7 +647,10 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 #endif
 
     /* init package containing bih */
-    memcpy ( &this->bih, buf->content, sizeof (BITMAPINFOHEADER));
+    if( this->bih )
+      free( this->bih );
+    this->bih = malloc(buf->size);
+    memcpy ( this->bih, buf->content, buf->size );
     this->video_step = buf->decoder_info[1];
 #ifdef LOG
     printf ("w32codec: video_step is %lld\n", this->video_step);
@@ -622,7 +663,14 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       w32v_init_codec (this, buf->type);
     else
       w32v_init_ds_codec (this, buf->type);
-      
+    
+    if( !this->decoder_ok ) {
+      xine_log (this->xine, XINE_LOG_MSG,
+              "w32codec: decoder failed to start. Is '%s' installed?\n", 
+              win32_codec_name );
+      xine_report_codec( this->xine, XINE_CODEC_VIDEO, 0, buf->type, 0);
+    }
+                                         
 #ifdef SYNC_SHUTDOWN
     if( this->decoder_ok )
       w32v_instance = this;
@@ -670,11 +718,11 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
       /* decoder video frame */
 
-      this->bih.biSizeImage = this->size;
+      this->bih->biSizeImage = this->size;
      
       img = this->video_out->get_frame (this->video_out,
-					this->bih.biWidth, 
-					this->bih.biHeight, 
+					this->bih->biWidth, 
+					this->bih->biHeight, 
 					42, 
 					IMGFMT_YUY2,
 					VO_BOTH_FIELDS);
@@ -698,10 +746,10 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       if( !this->ds_driver )
         ret = (!this->ex_functions)
               ?ICDecompress(this->hic, flags,
-			    &this->bih, this->buf, &this->o_bih, 
+			    this->bih, this->buf, &this->o_bih, 
 			    (this->skipframes)?NULL:img_buffer)
               :ICDecompressEx(this->hic, flags,
-			    &this->bih, this->buf, &this->o_bih,
+			    this->bih, this->buf, &this->o_bih,
 			    (this->skipframes)?NULL:img_buffer); 
       else {
         ret = DS_VideoDecoder_DecodeInternal(this->ds_dec, this->buf, this->size,
@@ -726,12 +774,12 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   
 	  xine_profiler_start_count (this->prof_rgb2yuv);
   
-	  for (row=0; row<this->bih.biHeight; row++) {
+	  for (row=0; row<this->bih->biHeight; row++) {
   
 	    uint16_t *pixel, *out;
   
 	    pixel = (uint16_t *) ( (uint8_t *)this->img_buffer + 2 * row * this->o_bih.biWidth );
-	    out = (uint16_t *) (img->base[0] + 2 * row * this->o_bih.biWidth );
+	    out = (uint16_t *) (img->base[0] + row * img->pitches[0] );
   
 	    for (col=0; col<this->o_bih.biWidth; col++, pixel++, out++) {
   	    
@@ -790,14 +838,13 @@ static void w32v_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       
       if (img->copy && !this->skipframes) {
 	int height = abs(this->o_bih.biHeight);
-	int stride = this->o_bih.biWidth;
-	uint8_t* src[3];
-	  
+	uint8_t *src[3];
+
 	src[0] = img->base[0];
-	  
+
 	while ((height -= 16) >= 0) {
 	  img->copy(img, src);
-	  src[0] += 32 * stride;
+	  src[0] += 16 * img->pitches[0];
 	}
       }
 
@@ -833,9 +880,9 @@ static void w32v_reset (video_decoder_t *this_gen) {
     if( this->hic )
     {
       if (!this->ex_functions) 
-        ICDecompressBegin(this->hic, &this->bih, &this->o_bih);
+        ICDecompressBegin(this->hic, this->bih, &this->o_bih);
       else
-        ICDecompressBeginEx(this->hic, &this->bih, &this->o_bih);
+        ICDecompressBeginEx(this->hic, this->bih, &this->o_bih);
     }
   } else {
   }
@@ -862,7 +909,9 @@ static void w32v_close (video_decoder_t *this_gen) {
       if( this->ds_dec )
         DS_VideoDecoder_Destroy(this->ds_dec);
     }
+#ifdef SYNC_SHUTDOWN
     w32v_instance = NULL;
+#endif
     Restore_LDT_Keeper( this->ldt_fs );
   }
   pthread_mutex_unlock(&win32_codec_mutex);
@@ -882,6 +931,11 @@ static void w32v_close (video_decoder_t *this_gen) {
     this->buf = NULL;
   }
 
+  if( this->bih ) {
+    free( this->bih );
+    this->bih = NULL;
+  }
+
   if( this->decoder_ok )
   {  
     this->decoder_ok = 0;
@@ -907,7 +961,7 @@ static int w32a_can_handle (audio_decoder_t *this_gen, int buf_type) {
 
   return ( (codec == BUF_AUDIO_DIVXA) ||
 	   (codec == BUF_AUDIO_MSADPCM) ||
-	   (codec == BUF_AUDIO_IMAADPCM) ||
+	   (codec == BUF_AUDIO_MSIMAADPCM) ||
 	   (codec == BUF_AUDIO_MSGSM) ||
 	   (codec == BUF_AUDIO_IMC) ||
 	   (codec == BUF_AUDIO_LH) || 
@@ -926,7 +980,7 @@ static char* get_auds_codec_name(w32a_decoder_t *this, int buf_type) {
     return "divxa32.acm";
   case BUF_AUDIO_MSADPCM:
     return "msadp32.acm";
-  case BUF_AUDIO_IMAADPCM:
+  case BUF_AUDIO_MSIMAADPCM:
     return "imaadp32.acm";
   case BUF_AUDIO_MSGSM:
     return "msgsm32.acm";
@@ -1006,7 +1060,6 @@ static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
   audio_buffer->vpts       = 0;
   this->audio_out->put_buffer (this->audio_out, audio_buffer);
 
-  
   wf.nChannels       = in_fmt->nChannels;
   wf.nSamplesPerSec  = in_fmt->nSamplesPerSec;
   wf.nAvgBytesPerSec = 2*wf.nSamplesPerSec*wf.nChannels;
@@ -1015,7 +1068,6 @@ static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
   wf.wBitsPerSample  = 16;
   wf.cbSize          = 0;
   
-  pthread_mutex_lock(&win32_codec_mutex);
   this->ldt_fs = Setup_LDT_Keeper();
   win32_codec_name = get_auds_codec_name (this, buf->type);
   
@@ -1026,11 +1078,12 @@ static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
                       NULL,0,0,0);
     if(ret){
       if(ret==ACMERR_NOTPOSSIBLE)
-        printf("w32codec: (ACM_Decoder) Unappropriate audio format\n");
+        xine_log (this->xine, XINE_LOG_MSG,
+                  "w32codec: (ACM_Decoder) Unappropriate audio format\n");
       else
-        printf("w32codec: (ACM_Decoder) acmStreamOpen error %d", (int) ret);
+        xine_log (this->xine, XINE_LOG_MSG,
+                  "w32codec: (ACM_Decoder) acmStreamOpen error %d\n", (int) ret);
       this->srcstream = 0;
-      pthread_mutex_unlock(&win32_codec_mutex);
       return 0;
     }
 
@@ -1046,9 +1099,8 @@ static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
     ret2 = this->ds_dec=DS_AudioDecoder_Open(win32_codec_name,this->guid, in_fmt);
     
     if( ret2 == NULL ) {
-      printf("w32codec: Error initializing DirectShow Audio\n");
+      xine_log (this->xine, XINE_LOG_MSG, "w32codec: Error initializing DirectShow Audio\n");
       this->srcstream = 0;
-      pthread_mutex_unlock(&win32_codec_mutex);
       return 0;
     }
     
@@ -1061,7 +1113,6 @@ static int w32a_init_audio (w32a_decoder_t *this, buf_element_t *buf ) {
        impossible */
     this->rec_audio_src_size*=2; 
   }
-  pthread_mutex_unlock(&win32_codec_mutex);
   printf("w32codec: Recommended source buffer size: %d\n", this->rec_audio_src_size); 
 
   if( this->buf )
@@ -1240,14 +1291,22 @@ static void w32a_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
     return;
   
   if (buf->decoder_flags & BUF_FLAG_HEADER) {
-    /* init package containing bih */
 
 #ifdef LOG
     printf ("w32codec: got audio header\n");
 #endif
 
+    pthread_mutex_lock(&win32_codec_mutex);
     this->decoder_ok = w32a_init_audio (this, buf);
-
+    
+    if( !this->decoder_ok ) {
+      xine_log (this->xine, XINE_LOG_MSG,
+              "w32codec: decoder failed to start. Is '%s' installed?\n", 
+              win32_codec_name );
+      xine_report_codec( this->xine, XINE_CODEC_AUDIO, 0, buf->type, 0);
+    }
+    pthread_mutex_unlock(&win32_codec_mutex);
+ 
 #ifdef SYNC_SHUTDOWN
     if( this->decoder_ok )
       w32a_instance = this;
@@ -1277,14 +1336,19 @@ static void w32a_close (audio_decoder_t *this_gen) {
 #endif
   {
     if( !this->ds_driver ) {
-      if( this->srcstream )
+      if( this->srcstream ) {
         acmStreamClose(this->srcstream, 0);
+        this->srcstream = 0;
+      }
     } else {
       if( this->ds_dec )
         DS_AudioDecoder_Destroy(this->ds_dec);
+      this->ds_dec = NULL;
     }
 
+#ifdef SYNC_SHUTDOWN
     w32a_instance = NULL;
+#endif
     Restore_LDT_Keeper(this->ldt_fs);
   }
   pthread_mutex_unlock(&win32_codec_mutex);
@@ -1326,21 +1390,23 @@ video_decoder_t *init_video_decoder_plugin (int iface_version, xine_t *xine) {
   w32v_decoder_t *this ;
   config_values_t *cfg;
 
-  if (iface_version != 7) {
+  if (iface_version != 10) {
     printf( "w32codec: plugin doesn't support plugin API version %d.\n"
 	    "w32codec: this means there's a version mismatch between xine and this "
-	    "w32codec: decoder plugin.\nInstalling current input plugins should help.\n",
+	    "w32codec: decoder plugin.\nInstalling current decoder plugins should help.\n",
 	    iface_version);
     
     return NULL;
   }
   cfg = xine->config;
   win32_def_path = cfg->register_string (cfg, "codec.win32_path", "/usr/lib/win32",
-					 "path to win32 codec dlls",
+					 _("path to win32 codec dlls"),
 					 NULL, NULL, NULL);
 
   this = (w32v_decoder_t *) xine_xmalloc (sizeof (w32v_decoder_t));
 
+  this->xine = xine;
+  
   this->video_decoder.interface_version   = iface_version;
   this->video_decoder.can_handle          = w32v_can_handle;
   this->video_decoder.init                = w32v_init;
@@ -1372,22 +1438,24 @@ audio_decoder_t *init_audio_decoder_plugin (int iface_version, xine_t *xine) {
   w32a_decoder_t *this ;
   config_values_t *cfg;
   
-  if (iface_version != 7) {
-    printf( "w32codec: plugin doesn't support plugin API version %d.\n"
-	    "w32codec: this means there's a version mismatch between xine and this "
-	    "w32codec: decoder plugin.\nInstalling current input plugins should help.\n",
-	    iface_version);
+  if (iface_version != 9) {
+    printf(_("w32codec: plugin doesn't support plugin API version %d.\n"
+	     "w32codec: this means there's a version mismatch between xine and this "
+	     "w32codec: decoder plugin.\nInstalling current decoder plugins should help.\n"),
+	   iface_version);
 
     return NULL;
   }
 
   cfg = xine->config;
   win32_def_path = cfg->register_string (cfg, "codec.win32_path", "/usr/lib/win32",
-					 "path to win32 codec dlls",
+					 _("path to win32 codec dlls"),
 					 NULL, NULL, NULL);
 
   this = (w32a_decoder_t *) xine_xmalloc (sizeof (w32a_decoder_t));
-  
+
+  this->xine = xine;
+    
   this->audio_decoder.interface_version   = iface_version;
   this->audio_decoder.can_handle          = w32a_can_handle;
   this->audio_decoder.init                = w32a_init;
