@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_aa.c,v 1.18 2002/03/26 00:38:56 f1rmb Exp $
+ * $Id: video_out_aa.c,v 1.24 2002/07/20 21:07:01 f1rmb Exp $
  *
  * video_out_aa.c, ascii-art output plugin for xine
  *
@@ -42,6 +42,7 @@
 
 #include "video_out.h"
 #include "xine_internal.h"
+#include "xineutils.h"
 
 /*
  * global variables
@@ -80,18 +81,6 @@ static uint32_t aa_get_capabilities (vo_driver_t *this) {
   return VO_CAP_YV12 | VO_CAP_YUY2;
 }
 
-static void *malloc_aligned (size_t alignment, size_t size, void **mem) {
-  char *aligned;
-
-  aligned = malloc (size+alignment);
-  *mem = aligned;
-
-  while ((int) aligned % alignment)
-    aligned++;
-
-  return aligned;
-}
-
 static void aa_dispose_frame (vo_frame_t *vo_img) {
   aa_frame_t *frame = (aa_frame_t *)vo_img;
   
@@ -119,7 +108,8 @@ static vo_frame_t *aa_alloc_frame(vo_driver_t *this) {
   frame->vo_frame.copy = NULL;
   frame->vo_frame.field = aa_frame_field;
   frame->vo_frame.dispose = aa_dispose_frame;
-
+  frame->vo_frame.driver = this;
+  
   return (vo_frame_t*) frame;
 }
 
@@ -127,8 +117,6 @@ static void aa_update_frame_format (vo_driver_t *this, vo_frame_t *img,
 				    uint32_t width, uint32_t height, 
 				    int ratio_code, int format, int flags) {
 
-  int image_size;
-
   aa_frame_t *frame = (aa_frame_t *) img;
 
   /* printf ("aa_update_format...\n"); */
@@ -156,19 +144,21 @@ static void aa_update_frame_format (vo_driver_t *this, vo_frame_t *img,
 
 
     if (format == IMGFMT_YV12) {
-      image_size = width * height;
-      frame->vo_frame.base[0] = malloc_aligned(16,image_size, (void**) &frame->mem[0]);
-      frame->vo_frame.base[1] = malloc_aligned(16,image_size/4, (void**) &frame->mem[1]);
-      frame->vo_frame.base[2] = malloc_aligned(16,image_size/4, (void**) &frame->mem[2]);
+      frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+      frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+      frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[0] * height, (void**) &frame->mem[0]);
+      frame->vo_frame.base[1] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[1] * ((height+1)/2), (void**) &frame->mem[1]);
+      frame->vo_frame.base[2] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[2] * ((height+1)/2), (void**) &frame->mem[2]);
 
       /* printf ("allocated yuv memory for %d x %d image\n", width, height); */
 
     } else if (format == IMGFMT_YUY2) {
-      image_size = width * 2 * height;
-      frame->vo_frame.base[0] = malloc_aligned(16,image_size, (void**) &frame->mem[0]);
+      frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[0] * height, (void**) &frame->mem[0]);
     } else {
       printf ("alert! unsupported image format %04x\n", format);
-      exit (1);
+      abort();
     }
 
     frame->ratio_code = ratio_code;
@@ -285,6 +275,7 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
   this->vo_driver.alloc_frame          = aa_alloc_frame ;
   this->vo_driver.update_frame_format  = aa_update_frame_format;
   this->vo_driver.display_frame        = aa_display_frame;
+  this->vo_driver.overlay_blend        = NULL;
   this->vo_driver.get_property         = aa_get_property;
   this->vo_driver.set_property         = aa_set_property;
   this->vo_driver.get_property_min_max = aa_get_property_min_max;
@@ -298,11 +289,12 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 static vo_info_t vo_info_aa = {
   5,
   "aa",
-  "xine video output plugin using the ascii-art library",
+  NULL,
   VISUAL_TYPE_AA,
   10
 };
 
 vo_info_t *get_video_out_plugin_info() {
+  vo_info_aa.description = _("xine video output plugin using the ascii-art library");
   return &vo_info_aa;
 }
