@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_fb.c,v 1.9 2002/03/25 03:08:38 miguelfreitas Exp $
+ * $Id: video_out_fb.c,v 1.13 2002/07/15 21:42:34 esnel Exp $
  * 
  * video_out_fb.c, frame buffer xine driver by Miguel Freitas
  *
@@ -115,7 +115,6 @@ typedef struct fb_driver_s {
   int              mem_size;
   uint8_t*         video_mem;       /* mmapped video memory */
   
-  int              zoom_mpeg1;
   int		   scaling_disabled;
   int              depth, bpp, bytes_per_pixel;
   int              expecting_event;
@@ -256,6 +255,7 @@ static void fb_compute_ideal_size (fb_driver_t *this, fb_frame_t *frame) {
     case ASPECT_AUTO:
       switch (frame->ratio_code) {
       case XINE_ASPECT_RATIO_ANAMORPHIC:  /* anamorphic     */
+      case XINE_ASPECT_RATIO_PAN_SCAN:    /* we display pan&scan as widescreen */
 	desired_ratio = 16.0 /9.0;
 	break;
       case XINE_ASPECT_RATIO_211_1:       /* 2.11:1 */
@@ -409,16 +409,18 @@ static void fb_update_frame_format (vo_driver_t *this_gen,
                                 this->bytes_per_pixel );
     
     if (format == IMGFMT_YV12) {
-      int image_size = frame->width * frame->height;
-      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, image_size,
+      frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+      frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+      frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[0] * height,
                                                       (void **)&frame->chunk[0]);
-      frame->vo_frame.base[1] = xine_xmalloc_aligned (16, image_size/4,
+      frame->vo_frame.base[1] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[1] * ((height+1)/2),
                                                       (void **)&frame->chunk[1]);
-      frame->vo_frame.base[2] = xine_xmalloc_aligned (16, image_size/4, 
+      frame->vo_frame.base[2] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[2] * ((height+1)/2),
                                                       (void **)&frame->chunk[2]);
     } else {
-      int image_size = frame->width * frame->height;
-      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, image_size*2, 
+      frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[0] * height,
                                                       (void **)&frame->chunk[0]);
       frame->chunk[1] = NULL;
       frame->chunk[2] = NULL;
@@ -441,8 +443,8 @@ static void fb_update_frame_format (vo_driver_t *this_gen,
       frame->yuv2rgb->configure (frame->yuv2rgb,
 				 frame->width,
 				 16,
-				 frame->width*2,
-				 frame->width,
+				 2*frame->vo_frame.pitches[0],
+				 2*frame->vo_frame.pitches[1],
 				 frame->output_width,
 				 frame->stripe_height,
 				 frame->bytes_per_line*2);
@@ -452,8 +454,8 @@ static void fb_update_frame_format (vo_driver_t *this_gen,
       frame->yuv2rgb->configure (frame->yuv2rgb,
 				 frame->width,
 				 16,
-				 frame->width,
-				 frame->width/2,
+				 frame->vo_frame.pitches[0],
+				 frame->vo_frame.pitches[1],
 				 frame->output_width,
 				 frame->stripe_height,
 				 frame->bytes_per_line);
@@ -701,19 +703,11 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 
   this->config		    = config;
   this->output_scale_factor = 1.0;
-  this->zoom_mpeg1	    = config->register_bool (config, "video.zoom_mpeg1", 1,
-						     "Zoom small video formats to double size",
+  
+  this->scaling_disabled    = config->register_bool (config, "video.disable_scaling", 0,
+						     _("disable all video scaling (faster!)"),
 						     NULL, NULL, NULL);
-  /*
-   * FIXME: replace getenv() with config->lookup_int, merge with zoom_mpeg1?
-   *
-   * this->video_scale = config->lookup_int (config, "video_scale", 2);
-   *  0: disable all scaling (including aspect ratio switching, ...)
-   *  1: enable aspect ratio switch
-   *  2: like 1, double the size for small videos
-   */
-  this->scaling_disabled    = getenv("VIDEO_OUT_NOSCALE") != NULL;
-
+  
   this->vo_driver.get_capabilities     = fb_get_capabilities;
   this->vo_driver.alloc_frame          = fb_alloc_frame;
   this->vo_driver.update_frame_format  = fb_update_frame_format;
@@ -727,8 +721,8 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
   this->vo_driver.redraw_needed        = fb_redraw_needed;
 
   device_name = config->register_string (config, "video.fb_device", "/dev/fb0",
-					  "framebuffer device", NULL, NULL, NULL);
-
+					 _("framebuffer device"), NULL, NULL, NULL);
+  
   if( (this->fd = open(device_name, O_RDWR) ) < 0) {
     printf("video_out_fb: aborting. (unable to open device \"%s\")\n", device_name);
     free(this);
@@ -885,11 +879,12 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 static vo_info_t vo_info_fb = {
   5,
   "fb",
-  "xine video output plugin using linux framebuffer device",
+  NULL,
   VISUAL_TYPE_FB,
   5
 };
 
 vo_info_t *get_video_out_plugin_info() {
+  vo_info_fb.description = _("xine video output plugin using linux framebuffer device");
   return &vo_info_fb;
 }
