@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_opengl.c,v 1.9 2002/04/26 17:19:04 jkeil Exp $
+ * $Id: video_out_opengl.c,v 1.15 2002/07/16 19:33:37 esnel Exp $
  * 
  * video_out_glut.c, glut based OpenGL rendering interface for xine
  * Matthias Hopf <mat@mshopf.de>
@@ -199,25 +199,6 @@ typedef struct opengl_driver_s {
 enum { CONTEXT_BAD = 0, CONTEXT_SAME_DRAWABLE, CONTEXT_RELOAD, CONTEXT_SET };
 
 
-/*
- * first, some utility functions
- */
-
-static void *my_malloc_aligned (size_t alignment, size_t size, uint8_t **chunk) {
-
-    uint8_t *pMem;
-
-    pMem = xine_xmalloc (size+alignment);
-
-    *chunk = pMem;
-
-    while ((int) pMem % alignment)
-	pMem++;
-
-    return pMem;
-}
-
-
 /*
  * and now, the driver functions
  */
@@ -358,16 +339,20 @@ static void opengl_update_frame_format (vo_driver_t *this_gen,
 
 	switch (format) {
 	case IMGFMT_YV12:
-	    frame->vo_frame.base[0] = my_malloc_aligned(16,image_size,
-							&frame->chunk[0]);
-	    frame->vo_frame.base[1] = my_malloc_aligned(16,image_size/4,
-							&frame->chunk[1]);
-	    frame->vo_frame.base[2] = my_malloc_aligned(16,image_size/4,
-							&frame->chunk[2]);
+	    frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+	    frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+	    frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+	    frame->vo_frame.base[0] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[0] * height,
+							   &frame->chunk[0]);
+	    frame->vo_frame.base[1] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[1] * ((height+1)/2),
+							   &frame->chunk[1]);
+	    frame->vo_frame.base[2] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[2] * ((height+1)/2),
+							   &frame->chunk[2]);
 	    break;
 	case IMGFMT_YUY2:
-	    frame->vo_frame.base[0] = my_malloc_aligned(16,image_size*2,
-							&frame->chunk[0]);
+	    frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
+	    frame->vo_frame.base[0] = xine_xmalloc_aligned(16, frame->vo_frame.pitches[0] * height,
+							   &frame->chunk[0]);
 	    break;
 	default:
 	    fprintf (stderr, "video_out_opengl: image format %d not supported, update video driver!\n", format);
@@ -418,6 +403,7 @@ static void opengl_compute_ideal_size (opengl_driver_t *this,
     case ASPECT_AUTO:
 	switch (ratio_code) {
 	case XINE_ASPECT_RATIO_ANAMORPHIC:  /* anamorphic     */
+	case XINE_ASPECT_RATIO_PAN_SCAN:    /* we display pan&scan as widescreen */
 	    desired_ratio = 16.0 /9.0;
 	    break;
 	case XINE_ASPECT_RATIO_211_1:       /* 2.11:1 */
@@ -686,7 +672,7 @@ static void opengl_render_image (opengl_driver_t *this, opengl_frame_t *frame,
 	 */
 	if (! glXMakeCurrent (this->display, this->drawable, ctx)) {
 	    fprintf (stderr, "video_out_opengl: no OpenGL support available (glXMakeCurrent)\n    The drawable does not seem to be updated correctly.\n");
-	    exit (1);
+	    abort();
 	}
 	DEBUGF ((stderr, "set context done\n"));
 	if (ctx == this->context)
@@ -1103,7 +1089,7 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
     this->yuv2rgb_gamma = config->register_range (config,
 						  "video.opengl_gamma", 0,
 						  -100, 100,
-						  "gamma correction for OpenGL driver",
+						  _("gamma correction for OpenGL driver"),
 						  NULL, NULL, NULL);
     this->yuv2rgb_factory = yuv2rgb_factory_init (YUV_FORMAT, YUV_SWAP_MODE, 
 						  this->yuv2rgb_cmap);
@@ -1115,12 +1101,13 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 static vo_info_t vo_info_shm = {
     5,
     "OpenGL",
-    "xine video output plugin using OpenGL(tm)",
+    NULL,
     VISUAL_TYPE_X11,
-    8
+    3
 };
 
 vo_info_t *get_video_out_plugin_info() {
+    vo_info_shm.description = _("xine video output plugin using OpenGL(tm)");
     return &vo_info_shm;
 }
 
