@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_xshm.c,v 1.72 2002/04/14 00:58:23 esnel Exp $
+ * $Id: video_out_xshm.c,v 1.82 2002/07/30 00:26:45 miguelfreitas Exp $
  * 
  * video_out_xshm.c, X11 shared memory extension interface for xine
  *
@@ -74,7 +74,7 @@ typedef struct xshm_frame_s {
   int                format;
   int                flags;
 
-  int                user_ratio;
+  int                user_ratio; 
 
   /* 
    * "ideal" size of this frame :
@@ -136,9 +136,11 @@ typedef struct xshm_driver_s {
   uint8_t           *yuv2rgb_cmap;
   yuv2rgb_factory_t *yuv2rgb_factory;
   int                user_ratio;
+  
+  /* force update screen if gamma changes */
+  int                force_redraw;
 
   /* speed tradeoffs */
-  int                zoom_mpeg1;
   int		     scaling_disabled;
 
   int                expecting_event; /* completion event */
@@ -204,20 +206,6 @@ static void x11_DeInstallXErrorHandler (xshm_driver_t *this)
   XFlush (this->display);
 }
 
-static void *my_malloc_aligned (size_t alignment, size_t size, uint8_t **chunk) {
-
-  uint8_t *pMem;
-
-  pMem = xine_xmalloc (size+alignment);
-
-  *chunk = pMem;
-
-  while ((int) pMem % alignment)
-    pMem++;
-
-  return pMem;
-}
-
 /*
  * allocate an XImage, try XShm first but fall back to 
  * plain X11 if XShm should fail
@@ -477,6 +465,7 @@ static void xshm_compute_ideal_size (xshm_driver_t *this, xshm_frame_t *frame) {
     case ASPECT_AUTO:
       switch (frame->ratio_code) {
       case XINE_ASPECT_RATIO_ANAMORPHIC:  /* anamorphic     */
+      case XINE_ASPECT_RATIO_PAN_SCAN:    /* we display pan&scan as widescreen */
 	desired_ratio = 16.0 /9.0;
 	break;
       case XINE_ASPECT_RATIO_211_1:       /* 2.11:1 */
@@ -512,8 +501,10 @@ static void xshm_compute_ideal_size (xshm_driver_t *this, xshm_frame_t *frame) {
 
     frame->ratio_factor = this->display_ratio * desired_ratio;
 
-    corr_factor = frame->ratio_factor / image_ratio ;
-
+    /* compiler bug? using frame->ratio_factor here gave me
+       some wrong numbers. */
+    corr_factor = this->display_ratio * desired_ratio / image_ratio ;
+    
     if (fabs(corr_factor - 1.0) < 0.005) {
       frame->ideal_width  = frame->width;
       frame->ideal_height = frame->height;
@@ -521,13 +512,12 @@ static void xshm_compute_ideal_size (xshm_driver_t *this, xshm_frame_t *frame) {
     } else {
 
       if (corr_factor >= 1.0) {
-	frame->ideal_width  = frame->width * corr_factor + 0.5;
-	frame->ideal_height = frame->height;
+        frame->ideal_width  = frame->width * corr_factor + 0.5;
+        frame->ideal_height = frame->height;
       } else {
-	frame->ideal_width  = frame->width;
-	frame->ideal_height = frame->height / corr_factor + 0.5;
+        frame->ideal_width  = frame->width;
+        frame->ideal_height = frame->height / corr_factor + 0.5;
       }
-
     }
   }
 }
@@ -669,13 +659,15 @@ static void xshm_update_frame_format (vo_driver_t *this_gen,
     XUnlockDisplay (this->display); 
 
     if (format == IMGFMT_YV12) {
-      int image_size = frame->width * frame->height;
-      frame->vo_frame.base[0] = my_malloc_aligned (16, image_size,   &frame->chunk[0]);
-      frame->vo_frame.base[1] = my_malloc_aligned (16, image_size/4, &frame->chunk[1]);
-      frame->vo_frame.base[2] = my_malloc_aligned (16, image_size/4, &frame->chunk[2]);
+      frame->vo_frame.pitches[0] = 8*((width + 7) / 8);
+      frame->vo_frame.pitches[1] = 8*((width + 15) / 16);
+      frame->vo_frame.pitches[2] = 8*((width + 15) / 16);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[0] * height,   &frame->chunk[0]);
+      frame->vo_frame.base[1] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[1] * ((height+1)/2), &frame->chunk[1]);
+      frame->vo_frame.base[2] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[2] * ((height+1)/2), &frame->chunk[2]);
     } else {
-      int image_size = frame->width * frame->height;
-      frame->vo_frame.base[0] = my_malloc_aligned (16, image_size*2, &frame->chunk[0]);
+      frame->vo_frame.pitches[0] = 8*((width + 3) / 4);
+      frame->vo_frame.base[0] = xine_xmalloc_aligned (16, frame->vo_frame.pitches[0] * height, &frame->chunk[0]);
       frame->chunk[1] = NULL;
       frame->chunk[2] = NULL;
     }
@@ -694,8 +686,8 @@ static void xshm_update_frame_format (vo_driver_t *this_gen,
       frame->yuv2rgb->configure (frame->yuv2rgb,
 				 frame->width,
 				 16,
-				 frame->width*2,
-				 frame->width,
+				 2*frame->vo_frame.pitches[0],
+				 2*frame->vo_frame.pitches[1],
 				 frame->output_width,
 				 frame->stripe_height,
 				 frame->image->bytes_per_line*2);
@@ -705,8 +697,8 @@ static void xshm_update_frame_format (vo_driver_t *this_gen,
       frame->yuv2rgb->configure (frame->yuv2rgb,
 				 frame->width,
 				 16,
-				 frame->width,
-				 frame->width/2,
+				 frame->vo_frame.pitches[0],
+				 frame->vo_frame.pitches[1],
 				 frame->output_width,
 				 frame->stripe_height,
 				 frame->image->bytes_per_line);
@@ -820,10 +812,10 @@ static int xshm_redraw_needed (vo_driver_t *this_gen) {
   int gui_x, gui_y, gui_width, gui_height, gui_win_x, gui_win_y;
   int ret = 0;
 
-  if( this->cur_frame )
-  {
+  if( this->cur_frame ) {
+  
     this->frame_output_cb (this->user_data,
-			   this->cur_frame->output_width, this->cur_frame->output_height, 
+			   this->cur_frame->ideal_width, this->cur_frame->ideal_height, 
 			   &gui_x, &gui_y, &gui_width, &gui_height,
 			   &gui_win_x, &gui_win_y );
 
@@ -845,10 +837,11 @@ static int xshm_redraw_needed (vo_driver_t *this_gen) {
     }
   } 
   else
-  {
     ret = 1;
-  }
-  
+
+  if( this->force_redraw )
+    ret = 1;
+      
   return ret;
 }
 
@@ -885,7 +878,7 @@ static void xshm_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
      */
 
     this->frame_output_cb (this->user_data,
-			   frame->output_width, frame->output_height, 
+			   frame->ideal_width, frame->ideal_height, 
 			   &gui_x, &gui_y, &gui_width, &gui_height,
 			   &gui_win_x, &gui_win_y);
 
@@ -915,7 +908,8 @@ static void xshm_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
     }
 
     this->cur_frame = frame;
-
+    this->force_redraw = 0;
+    
     xoffset  = (this->gui_width - frame->output_width) / 2 + this->gui_x;
     yoffset  = (this->gui_height - frame->output_height) / 2 + this->gui_y;
 
@@ -1016,7 +1010,10 @@ static int xshm_set_property (vo_driver_t *this_gen,
     this->yuv2rgb_gamma = value;
     this->yuv2rgb_factory->set_gamma (this->yuv2rgb_factory, value);
 
+    this->force_redraw = 1;
+#ifdef LOG
     printf ("video_out_xshm: gamma changed to %d\n",value);
+#endif
   } else {
     printf ("video_out_xshm: tried to set unsupported property %d\n", property);
   }
@@ -1094,8 +1091,9 @@ static int xshm_gui_data_exchange (vo_driver_t *this_gen,
   case GUI_DATA_EX_EXPOSE_EVENT:
     
   /* FIXME : take care of completion events */
-
+#ifdef LOG
     printf ("video_out_xshm: expose event\n");
+#endif
 
     if (this->cur_frame) {
       
@@ -1125,6 +1123,26 @@ static int xshm_gui_data_exchange (vo_driver_t *this_gen,
 		      0, 0, xoffset, yoffset,
 		      this->cur_frame->output_width, this->cur_frame->output_height);
 	  }
+
+	XSetForeground (this->display, this->gc, this->black.pixel);
+
+	if (this->cur_frame->output_height != this->cur_frame->gui_height) {
+	  int y = yoffset + this->cur_frame->output_height;
+
+	  XFillRectangle(this->display, this->drawable, this->gc, 0, 0,
+			 this->cur_frame->gui_width, yoffset);
+	  XFillRectangle(this->display, this->drawable, this->gc, 0, y,
+			 this->cur_frame->gui_width, (this->cur_frame->gui_height - y));
+	}
+	if (this->cur_frame->output_width != this->cur_frame->gui_width) {
+	  int x = xoffset + this->cur_frame->output_width;
+
+	  XFillRectangle(this->display, this->drawable, this->gc, 0, yoffset,
+			 xoffset, this->cur_frame->output_height);
+	  XFillRectangle(this->display, this->drawable, this->gc, x, yoffset,
+			 (this->cur_frame->gui_width - x), this->cur_frame->output_height);
+	}
+
 	XFlush (this->display);
 	
 	XUnlockDisplay (this->display);
@@ -1277,19 +1295,9 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
   this->gui_width	    = 0;
   this->gui_height	    = 0;
   this->user_ratio          = ASPECT_AUTO;
-  this->zoom_mpeg1	    = config->register_bool (config, "video.zoom_mpeg1", 1,
-						     "Zoom small video formats to double size",
-						     NULL, NULL, NULL);
-  /*
-   * FIXME: replace getenv() with config->lookup_int, merge with zoom_mpeg1?
-   *
-   * this->video_scale = config->lookup_int (config, "video_scale", 2);
-   *  0: disable all scaling (including aspect ratio switching, ...)
-   *  1: enable aspect ratio switch
-   *  2: like 1, double the size for small videos
-   */
+  
   this->scaling_disabled    = config->register_bool (config, "video.disable_scaling", 0,
-						     "disable all video scaling",
+						     _("disable all video scaling (faster!)"),
 						     NULL, NULL, NULL);
   this->drawable	    = visual->d;
   this->expecting_event	    = 0;
@@ -1377,6 +1385,7 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
   case TrueColor:
     switch (this->depth) {
     case 24:
+    case 32:
       if (this->bpp == 32) {
 	if (this->visual->red_mask == 0x00ff0000)
 	  mode = MODE_32_RGB;
@@ -1431,7 +1440,7 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
   this->yuv2rgb_swap  = swapped;
   this->yuv2rgb_gamma = config->register_range (config, "video.xshm_gamma", 0,
 						-100, 100, 
-						"gamma correction for XShm driver",
+						_("gamma correction for XShm driver"),
 						NULL, NULL, NULL);
 
   this->yuv2rgb_factory = yuv2rgb_factory_init (mode, swapped, 
@@ -1444,12 +1453,13 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 static vo_info_t vo_info_shm = {
   5,                /* interface version */
   "XShm",           /* id                */
-  "xine video output plugin using the MIT X shared memory extension",
+  NULL,
   VISUAL_TYPE_X11,  /* visual_type       */
   6
 };
 
 vo_info_t *get_video_out_plugin_info() {
+  vo_info_shm.description = _("xine video output plugin using the MIT X shared memory extension");
   return &vo_info_shm;
 }
 
