@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_xv.c,v 1.113 2002/04/19 22:58:46 miguelfreitas Exp $
+ * $Id: video_out_xv.c,v 1.124 2002/07/20 21:46:06 esnel Exp $
  * 
  * video_out_xv.c, X11 video extension interface for xine
  *
@@ -134,6 +134,7 @@ struct xv_driver_s {
   int                delivered_width;   
   int                delivered_height;     
   int                delivered_ratio_code;
+  int                delivered_duration;
 
   /* 
    * displayed part of delivered images,
@@ -216,13 +217,21 @@ static void xv_frame_dispose (vo_frame_t *vo_img) {
   xv_driver_t *this = (xv_driver_t *) vo_img->driver;
 
   if (frame->image) {
-    XLockDisplay (this->display);
-    XShmDetach (this->display, &frame->shminfo);
-    XFree (frame->image);
-    XUnlockDisplay (this->display);
+    
+    if (this->use_shm) {
+      XLockDisplay (this->display);
+      XShmDetach (this->display, &frame->shminfo);
+      XFree (frame->image);
+      XUnlockDisplay (this->display);
 
-    shmdt (frame->shminfo.shmaddr);
-    shmctl (frame->shminfo.shmid, IPC_RMID,NULL);
+      shmdt (frame->shminfo.shmaddr);
+      shmctl (frame->shminfo.shmid, IPC_RMID, NULL);
+    }
+    else {
+      XLockDisplay (this->display);
+      XFree (frame->image);
+      XUnlockDisplay (this->display);
+    }
   }
 
   free (frame);
@@ -235,7 +244,7 @@ static vo_frame_t *xv_alloc_frame (vo_driver_t *this_gen) {
   frame = (xv_frame_t *) malloc (sizeof (xv_frame_t));
   memset (frame, 0, sizeof(xv_frame_t));
 
-  if (frame==NULL) {
+  if (frame == NULL) {
     printf ("xv_alloc_frame: out of memory\n");
   }
 
@@ -294,7 +303,7 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
     break;
   default:
     fprintf (stderr, "create_ximage: unknown format %08x\n",format);
-    exit (1);
+    abort();
   }
 
   if (this->use_shm) {
@@ -397,7 +406,7 @@ static XvImage *create_ximage (xv_driver_t *this, XShmSegmentInfo *shminfo,
       break;
     default:
       fprintf (stderr, "create_ximage: unknown format %08x\n",format);
-      exit (1);
+      abort();
     }
 
     image = XvCreateImage (this->display, this->xv_port,
@@ -454,9 +463,12 @@ static void xv_update_frame_format (vo_driver_t *this_gen,
 
     frame->image = create_ximage (this, &frame->shminfo, width, height, format);
 
-    frame->vo_frame.base[0] = frame->image->data;
-    frame->vo_frame.base[1] = frame->image->data + width * height * 5 / 4;
-    frame->vo_frame.base[2] = frame->image->data + width * height;
+    frame->vo_frame.pitches[0] = frame->image->pitches[0];
+    frame->vo_frame.pitches[1] = frame->image->pitches[2];
+    frame->vo_frame.pitches[2] = frame->image->pitches[1];
+    frame->vo_frame.base[0] = frame->image->data + frame->image->offsets[0];
+    frame->vo_frame.base[1] = frame->image->data + frame->image->offsets[2];
+    frame->vo_frame.base[2] = frame->image->data + frame->image->offsets[1];
 
     frame->width  = width;
     frame->height = height;
@@ -621,6 +633,7 @@ static void xv_compute_ideal_size (xv_driver_t *this) {
   case ASPECT_AUTO:
     switch (this->delivered_ratio_code) {
     case XINE_ASPECT_RATIO_ANAMORPHIC:  /* anamorphic     */
+    case XINE_ASPECT_RATIO_PAN_SCAN:    /* we display pan&scan as widescreen */
       desired_ratio = 16.0 /9.0;
       break;
     case XINE_ASPECT_RATIO_211_1:       /* 2.11:1 */
@@ -839,6 +852,7 @@ static void xv_display_frame (vo_driver_t *this_gen, vo_frame_t *frame_gen) {
       this->delivered_width      = frame->width;
       this->delivered_height     = frame->height;
       this->delivered_ratio_code = frame->ratio_code;
+      this->delivered_duration   = frame->vo_frame.duration;
 
       xv_compute_ideal_size (this);
       
@@ -1037,6 +1051,28 @@ static int xv_gui_data_exchange (vo_driver_t *this_gen,
 		   this->output_xoffset, this->output_yoffset,
 		   this->output_width, this->output_height);
       }
+
+      XSetForeground (this->display, this->gc, this->black.pixel);
+
+      if (this->output_height != this->gui_height) {
+	int y = this->output_yoffset + this->output_height;
+
+	XFillRectangle(this->display, this->drawable, this->gc, 0, 0,
+		       this->gui_width, this->output_yoffset);
+	XFillRectangle(this->display, this->drawable, this->gc, 0, y,
+		       this->gui_width, (this->gui_height - y));
+      }
+      if (this->output_width != this->gui_width) {
+	int x = this->output_xoffset + this->output_width;
+
+	XFillRectangle(this->display, this->drawable, this->gc,
+		       0, this->output_yoffset,
+		       this->output_xoffset, this->output_height);
+	XFillRectangle(this->display, this->drawable, this->gc,
+		       x, this->output_yoffset,
+		       this->gui_width - x, this->output_height);
+      }
+
       XFlush(this->display);
       
       XUnlockDisplay (this->display);
@@ -1147,12 +1183,14 @@ static void xv_check_capability (xv_driver_t *this,
   /* is this a boolean property ? */
   if ((attr.min_value == 0) && (attr.max_value == 1)) {
     this->config->register_bool (this->config, this->scratch, int_default,
-				 "Xv property", NULL, xv_property_callback, &this->props[property]);
+				 _("Xv property"),
+				 NULL, xv_property_callback, &this->props[property]);
   
   } else {
     this->config->register_range (this->config, this->scratch, int_default,
 				  this->props[property].min, this->props[property].max,   
-				  "Xv property", NULL, xv_property_callback, &this->props[property]);
+				  _("Xv property"), 
+				  NULL, xv_property_callback, &this->props[property]);
   }
   
   entry = this->config->lookup_entry (this->config, this->scratch);
@@ -1393,13 +1431,13 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 	  /* This setting is specific to Permedia 2/3 cards. */
 	  xv_filter = config->register_range (config, "video.XV_FILTER", 0,
 					      attr[k].min_value, attr[k].max_value,
-					      "bilinear scaling mode (permedia 2/3)",
+					      _("bilinear scaling mode (permedia 2/3)"),
 					      NULL, xv_update_XV_FILTER, this);
 	  config->update_num(config,"video.XV_FILTER",xv_filter);
 	} else if(!strcmp(attr[k].name, "XV_DOUBLE_BUFFER")) {
 	  int xv_double_buffer;
 	  xv_double_buffer = config->register_bool (config, "video.XV_DOUBLE_BUFFER", 1,
-						    "double buffer to sync video to the retrace",
+						    _("double buffer to sync video to the retrace"),
 						    NULL, xv_update_XV_DOUBLE_BUFFER, this);
 	  config->update_num(config,"video.XV_DOUBLE_BUFFER",xv_double_buffer);
 	}
@@ -1440,15 +1478,15 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 
   /*
    * try to create a shared image
-   * to find out if MIT shm really works
+   * to find out if MIT shm really works, using supported format
    */
-
-  myimage = create_ximage (this, &myshminfo, 100, 100, IMGFMT_YV12);
+  myimage = create_ximage (this, &myshminfo, 100, 100, 
+			   (this->xv_format_yv12 != 0) ? IMGFMT_YV12 : IMGFMT_YUY2);
   dispose_ximage (this, &myshminfo, myimage);
 
   this->deinterlace_method = config->register_enum (config, "video.deinterlace_method", 4,
 						    deinterlace_methods, 
-						    "Software deinterlace method (Key I toggles deinterlacer on/off)",
+						    _("Software deinterlace method (Key I toggles deinterlacer on/off)"),
 						    NULL, xv_update_deinterlace, this);
   this->deinterlace_enabled = 0;
 
@@ -1458,12 +1496,13 @@ vo_driver_t *init_video_out_plugin (config_values_t *config, void *visual_gen) {
 static vo_info_t vo_info_xv = {
   5,
   "Xv",
-  "xine video output plugin using the MIT X video extension",
+  NULL,
   VISUAL_TYPE_X11,
-  10
+  9
 };
 
 vo_info_t *get_video_out_plugin_info() {
+  vo_info_xv.description = _("xine video output plugin using the MIT X video extension");
   return &vo_info_xv;
 }
 
