@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_decoder.c,v 1.71 2002/04/14 00:24:45 guenter Exp $
+ * $Id: audio_decoder.c,v 1.82 2002/07/26 14:51:23 mroi Exp $
  *
  *
  * functions that implement audio decoding
@@ -47,6 +47,7 @@ void *audio_decoder_loop (void *this_gen) {
   int              running = 1;
   audio_decoder_t *decoder;
   static int	   prof_audio_decode = -1;
+  static uint32_t  buftype_unknown = 0;
 
   if (prof_audio_decode == -1)
     prof_audio_decode = xine_profiler_allocate_slot ("audio decoder/output");
@@ -67,9 +68,15 @@ void *audio_decoder_loop (void *this_gen) {
 
     if (buf->input_pos)
       this->cur_input_pos = buf->input_pos;
-    
-    if (buf->input_time)
+    if (buf->input_length)
+      this->cur_input_length = buf->input_length;
+    if (buf->input_time) {
       this->cur_input_time = buf->input_time;
+      pthread_mutex_lock (&this->osd_lock);
+      if( this->curtime_needed_for_osd && !(--this->curtime_needed_for_osd) )
+          xine_internal_osd (this, ">",90000);
+      pthread_mutex_unlock (&this->osd_lock);
+    }
       
     switch (buf->type) {
       
@@ -77,6 +84,7 @@ void *audio_decoder_loop (void *this_gen) {
       if (this->cur_audio_decoder_plugin) {
 	this->cur_audio_decoder_plugin->close (this->cur_audio_decoder_plugin);
 	this->cur_audio_decoder_plugin = NULL;
+	this->audio_track_map_entries = 0;
 	this->audio_type = 0;
       }
       
@@ -93,16 +101,22 @@ void *audio_decoder_loop (void *this_gen) {
       if (this->cur_audio_decoder_plugin) {
 	this->cur_audio_decoder_plugin->close (this->cur_audio_decoder_plugin);
 	this->cur_audio_decoder_plugin = NULL;
+	this->audio_track_map_entries = 0;
 	this->audio_type = 0;
       }
       
       pthread_mutex_lock (&this->finished_lock);
 
-      if (!this->audio_finished && (buf->decoder_flags & BUF_FLAG_END_STREAM)) {
+      if (!this->audio_finished ) {
         this->audio_finished = 1;
-
+        
         if (this->video_finished) {
-          xine_notify_stream_finished (this);
+          if( this->playing_logo )
+            buf->decoder_flags = 0;
+          this->playing_logo = 0;
+          
+          if( buf->decoder_flags & BUF_FLAG_END_STREAM )
+            xine_notify_stream_finished (this);
         }
       }
 
@@ -120,6 +134,7 @@ void *audio_decoder_loop (void *this_gen) {
       if (this->cur_audio_decoder_plugin) {
 	this->cur_audio_decoder_plugin->close (this->cur_audio_decoder_plugin);
 	this->cur_audio_decoder_plugin = NULL;
+	this->audio_track_map_entries = 0;
 	this->audio_type = 0;
       }
       running = 0;
@@ -131,6 +146,8 @@ void *audio_decoder_loop (void *this_gen) {
     case BUF_CONTROL_RESET_DECODER:
       if (this->cur_audio_decoder_plugin)
         this->cur_audio_decoder_plugin->reset (this->cur_audio_decoder_plugin);
+      if (this->audio_out)
+        this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
       break;
           
     case BUF_CONTROL_DISCONTINUITY:
@@ -138,7 +155,11 @@ void *audio_decoder_loop (void *this_gen) {
       break;
 
     case BUF_CONTROL_NEWPTS:
-      this->metronom->handle_audio_discontinuity (this->metronom, DISC_ABSOLUTE, buf->disc_off);
+      if (buf->decoder_flags && BUF_FLAG_SEEK) {
+        this->metronom->handle_audio_discontinuity (this->metronom, DISC_STREAMSEEK, buf->disc_off);
+      } else {
+        this->metronom->handle_audio_discontinuity (this->metronom, DISC_ABSOLUTE, buf->disc_off);
+      }
       break;
 
     case BUF_CONTROL_AUDIO_CHANNEL:
@@ -200,8 +221,12 @@ void *audio_decoder_loop (void *this_gen) {
 	    } else
 	      audio_type = this->audio_track_map[0];
 
-	  } else 
-	    audio_type = this->audio_track_map[this->audio_channel_user];
+	  } else {
+	    if (this->audio_channel_user <= this->audio_track_map_entries)
+	      audio_type = this->audio_track_map[this->audio_channel_user];
+	    else
+	      audio_type = -1;
+	  }
 
 	  /* now, decode this buffer if it's the right audio type */
 	  
@@ -225,6 +250,7 @@ void *audio_decoder_loop (void *this_gen) {
 		xine_log (this, XINE_LOG_FORMAT, 
 			  "audio_decoder: using audio decoder plugin '%s'\n",
 			  decoder->get_identifier());
+		xine_report_codec( this, XINE_CODEC_AUDIO, 0, buf->type, 1);
 
 		this->cur_audio_decoder_plugin = decoder;
 		this->cur_audio_decoder_plugin->init (this->cur_audio_decoder_plugin, this->audio_out);
@@ -239,11 +265,20 @@ void *audio_decoder_loop (void *this_gen) {
 	    /* finally - decode data */
 	    
 	    if (decoder) 
-	    decoder->decode_data (decoder, buf);
+	      decoder->decode_data (decoder, buf);
+	    else if( buf->type != buftype_unknown ) {
+	      xine_log (this, XINE_LOG_MSG, "audio_decoder: no plugin available to handle '%s'\n",
+		        buf_audio_name( buf->type ) );
+	      xine_report_codec( this, XINE_CODEC_AUDIO, 0, buf->type, 0);
+	      buftype_unknown = buf->type;
+	    }
 	  }
 	} 
-      } else
-	printf ("audio_loop: unknown buffer type: %08x\n", buf->type);
+      } else if( buf->type != buftype_unknown ) {
+	  xine_log (this, XINE_LOG_MSG, "audio_decoder: unknown buffer type: %08x\n",
+		    buf->type );
+	  buftype_unknown = buf->type;
+      }
 
       xine_profiler_stop_count (prof_audio_decode);
     }
@@ -266,9 +301,17 @@ void audio_decoder_init (xine_t *this) {
     return;
   }
   
-  this->audio_fifo = fifo_buffer_new (50, 8192);
+  /* The fifo size is based on dvd playback where buffers are filled
+   * with 2k of data. With 230 buffers and a typical audio data rate
+   * of 1.8 Mbit/s (four ac3 streams), the fifo can hold about 2 seconds
+   * of audio, wich should be enough to compensate for drive delays.
+   * We provide buffers of 8k size instead of 2k for demuxers sending
+   * larger chunks.
+   */
+  this->audio_fifo = fifo_buffer_new (230, 8192);
   this->audio_channel_user = -1;
   this->audio_channel_auto = 0;
+  this->audio_track_map_entries = 0;
   this->audio_type = 0;
 
   /* future magic - coming soon
@@ -285,7 +328,7 @@ void audio_decoder_init (xine_t *this) {
 			     &pth_attrs, audio_decoder_loop, this)) != 0) {
     fprintf (stderr, "audio_decoder: can't create new thread (%s)\n",
 	     strerror(err));
-    exit (1);
+    abort();
   }
 }
 
