@@ -17,7 +17,7 @@
  * along with self program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_out.c,v 1.52 2002/04/14 00:24:45 guenter Exp $
+ * $Id: audio_out.c,v 1.62 2002/07/28 21:37:35 heikos Exp $
  * 
  * 22-8-2001 James imported some useful AC3 sections from the previous alsa driver.
  *   (c) 2001 Andy Lo A Foe <andy@alsaplayer.org>
@@ -188,13 +188,6 @@ static audio_buffer_t *fifo_remove (audio_fifo_t *fifo) {
   return buf;
 }
 
-/* 
- * This routine is currently not used, but I do not want to loose it.
- * I think "(c) 2001 Andy Lo A Foe <andy@alsaplayer.org>" originally added it
- * to ./xine-lib/src/audio_out/audio_alsa_out.c before the architecture changes
- * So it has moved to here.
- */
-
 void write_pause_burst(ao_instance_t *this, uint32_t num_frames)
 { 
   int error = 0;
@@ -238,12 +231,11 @@ static void ao_fill_gap (ao_instance_t *this, int64_t pts_len) {
   printf ("audio_out: inserting %d 0-frames to fill a gap of %lld pts\n",
 	  num_frames, pts_len);
 
-  if ((this->mode == AO_CAP_MODE_A52) || (this->mode == AO_CAP_MODE_AC5)) {
+  if ((this->output.mode == AO_CAP_MODE_A52) || (this->output.mode == AO_CAP_MODE_AC5)) {
     write_pause_burst(this,num_frames);
     return; 
   }
 
-
   while (num_frames > 0) {
     if (num_frames > ZERO_BUF_SIZE) {
       this->driver->write(this->driver, this->zero_space, ZERO_BUF_SIZE);
@@ -255,26 +247,63 @@ static void ao_fill_gap (ao_instance_t *this, int64_t pts_len) {
   }
 }
 
+static void ensure_buffer_size (audio_buffer_t *buf, int bytes_per_frame,
+                                int frames)
+{
+  int size = bytes_per_frame * frames;
+  
+  if (buf->mem_size < size) {
+    buf->mem = realloc( buf->mem, size );
+    buf->mem_size = size;
+  }
+  buf->num_frames = frames;
+}
+
+static audio_buffer_t * swap_frame_buffers ( ao_instance_t *this ) {
+  audio_buffer_t *tmp;
+
+  tmp = this->frame_buf[1];
+  this->frame_buf[1] = this->frame_buf[0];
+  this->frame_buf[0] = tmp;
+  return this->frame_buf[0];
+}
+
+static int mode_channels( int mode ) {
+  switch( mode ) {
+      case AO_CAP_MODE_MONO:
+        return 1;
+      case AO_CAP_MODE_STEREO:
+        return 2;
+      case AO_CAP_MODE_4CHANNEL:
+        return 4;
+      case AO_CAP_MODE_5CHANNEL:
+        return 5;
+      case AO_CAP_MODE_5_1CHANNEL:
+        return 6;
+  }
+  return 0;
+} 
 
 static void *ao_loop (void *this_gen) {
 
   ao_instance_t  *this = (ao_instance_t *) this_gen;
   int64_t         hw_vpts;
-  audio_buffer_t *buf;
+  audio_buffer_t *buf, *in_buf;
   int64_t         gap;
-  int             delay;
+  int64_t         delay;
   int64_t         cur_time;
   int             num_output_frames ;
   int             paused_wait;
   int64_t         last_sync_time;
   int             bufs_since_sync;
+  double          acc_output_frames, output_frame_excess = 0;
 
   last_sync_time = bufs_since_sync = 0;
   
   while ((this->audio_loop_running) ||
 	 (!this->audio_loop_running && this->out_fifo->first)) {
 
-    buf = fifo_remove (this->out_fifo);
+    in_buf = buf = fifo_remove (this->out_fifo);
     bufs_since_sync++;
 
 #ifdef LOG
@@ -282,7 +311,9 @@ static void *ao_loop (void *this_gen) {
 #endif
 
     do {
+      pthread_mutex_lock( &this->driver_lock );
       delay = this->driver->delay(this->driver);
+      pthread_mutex_unlock( &this->driver_lock );
 
       /*
        * where, in the timeline is the "end" of the 
@@ -298,8 +329,8 @@ static void *ao_loop (void *this_gen) {
 #endif
 
       /* External A52 decoder delay correction */
-      if ((this->mode==AO_CAP_MODE_A52) || (this->mode==AO_CAP_MODE_AC5)) 
-        delay+=10; 
+      if ((this->output.mode==AO_CAP_MODE_A52) || (this->output.mode==AO_CAP_MODE_AC5)) 
+        delay += this->passthrough_offset;
   
       hw_vpts += delay * 1024 / this->frames_per_kpts;
   
@@ -316,8 +347,10 @@ static void *ao_loop (void *this_gen) {
       paused_wait = (this->audio_paused == 2) ||
         (this->audio_paused && gap > this->gap_tolerance);
       
-      if ( paused_wait )
+      if ( paused_wait ) {
+        this->metronom->allow_full_ao_fill_gap = 1;
         xine_usec_sleep (50000);
+      }
     } while ( paused_wait );
 
 #ifdef LOG
@@ -328,6 +361,7 @@ static void *ao_loop (void *this_gen) {
     /*
      * output audio data synced to master clock
      */
+    pthread_mutex_lock( &this->driver_lock );
     
     if (gap < (-1 * AO_MAX_GAP) || !buf->num_frames || 
         this->audio_paused ) {
@@ -356,60 +390,130 @@ static void *ao_loop (void *this_gen) {
       
       /* for big gaps output silence */
       if ( gap > AO_MAX_GAP ) {
-        ao_fill_gap (this, gap/2);
+        if (this->metronom->allow_full_ao_fill_gap) {
+          ao_fill_gap (this, gap);
+          this->metronom->allow_full_ao_fill_gap = 0;
+        } else {
+          ao_fill_gap (this, gap / 2);
+        }
       }
   
       /*
        * resample and output audio data
        */
 
-      num_output_frames = (double) buf->num_frames * this->frame_rate_factor;
+      /* calculate number of output frames (after resampling) */
+      acc_output_frames = (double) buf->num_frames * this->frame_rate_factor
+        + output_frame_excess;
+
+      /* Truncate to an integer */
+      num_output_frames = acc_output_frames;
+
+      /* Keep track of the amount truncated */
+      output_frame_excess = acc_output_frames - (double) num_output_frames;
       
 #ifdef LOG
       printf ("audio_out: outputting %d frames\n", num_output_frames);
 #endif
       
-      if ((!this->do_resample) 
-	  && (this->mode != AO_CAP_MODE_A52) 
-	  && (this->mode != AO_CAP_MODE_AC5)) {
+      /* convert 8 bit samples as needed */
+      if( this->input.bits == 8 &&
+          (this->do_resample || this->output.bits != 8 ||
+           this->input.mode != this->output.mode ) ) {
+        ensure_buffer_size(this->frame_buf[1], 2*mode_channels(this->input.mode),
+                           buf->num_frames );
+        audio_out_resample_8to16((int8_t *)buf->mem, this->frame_buf[1]->mem,
+                                 mode_channels(this->input.mode) * buf->num_frames );
+        buf = swap_frame_buffers(this);
+      }
 
-	this->driver->write (this->driver, buf->mem,
-			     buf->num_frames );
-      } else switch (this->mode) {
-      case AO_CAP_MODE_MONO:
-	audio_out_resample_mono (buf->mem, buf->num_frames,
-				 this->frame_buffer, num_output_frames);
-	this->driver->write(this->driver, this->frame_buffer, num_output_frames);
-	break;
-      case AO_CAP_MODE_STEREO:
-	audio_out_resample_stereo (buf->mem, buf->num_frames,
-				   this->frame_buffer, num_output_frames);
-	this->driver->write(this->driver, this->frame_buffer, num_output_frames);
-	break;
-      case AO_CAP_MODE_4CHANNEL:
-	audio_out_resample_4channel (buf->mem, buf->num_frames,
-				     this->frame_buffer, num_output_frames);
-	this->driver->write(this->driver, this->frame_buffer, num_output_frames);
-	break;
-      case AO_CAP_MODE_5CHANNEL:
-	audio_out_resample_5channel (buf->mem, buf->num_frames,
-				     this->frame_buffer, num_output_frames);
-	this->driver->write(this->driver, this->frame_buffer, num_output_frames);
-	break;
-      case AO_CAP_MODE_5_1CHANNEL:
-	audio_out_resample_6channel (buf->mem, buf->num_frames,
-				     this->frame_buffer, num_output_frames);
-	this->driver->write(this->driver, this->frame_buffer, num_output_frames);
-	break;
-      case AO_CAP_MODE_A52:
-      case AO_CAP_MODE_AC5:
-	this->driver->write(this->driver, buf->mem, buf->num_frames);
-	break;
+      /* check if resampling may be skipped */
+      if ( this->do_resample &&  
+           buf->num_frames != num_output_frames ) {
+        switch (this->input.mode) {
+          case AO_CAP_MODE_MONO:
+            ensure_buffer_size(this->frame_buf[1], 2, num_output_frames);
+            audio_out_resample_mono (buf->mem, buf->num_frames,
+                                     this->frame_buf[1]->mem, num_output_frames);
+            buf = swap_frame_buffers(this);
+            break;
+          case AO_CAP_MODE_STEREO:
+            ensure_buffer_size(this->frame_buf[1], 4, num_output_frames);
+            audio_out_resample_stereo (buf->mem, buf->num_frames,
+                                       this->frame_buf[1]->mem, num_output_frames);
+            buf = swap_frame_buffers(this);
+            break;
+          case AO_CAP_MODE_4CHANNEL:
+            ensure_buffer_size(this->frame_buf[1], 8, num_output_frames);
+            audio_out_resample_4channel (buf->mem, buf->num_frames,
+                                         this->frame_buf[1]->mem, num_output_frames);
+            buf = swap_frame_buffers(this);
+            break;
+          case AO_CAP_MODE_5CHANNEL:
+            ensure_buffer_size(this->frame_buf[1], 10, num_output_frames);
+            audio_out_resample_5channel (buf->mem, buf->num_frames,
+                                         this->frame_buf[1]->mem, num_output_frames);
+            buf = swap_frame_buffers(this);
+            break;
+          case AO_CAP_MODE_5_1CHANNEL:
+            ensure_buffer_size(this->frame_buf[1], 12, num_output_frames);
+            audio_out_resample_6channel (buf->mem, buf->num_frames,
+                                         this->frame_buf[1]->mem, num_output_frames);
+            buf = swap_frame_buffers(this);
+            break;
+          case AO_CAP_MODE_A52:
+          case AO_CAP_MODE_AC5:
+            /* pass-through modes: no resampling */
+            break;
+        }
       }
       
+      /* mode conversion */
+      if ( this->input.mode != this->output.mode ) {
+        switch (this->input.mode) {
+          case AO_CAP_MODE_MONO:
+            if( this->output.mode == AO_CAP_MODE_STEREO ) {
+              ensure_buffer_size(this->frame_buf[1], 4, buf->num_frames );
+              audio_out_resample_monotostereo(buf->mem, this->frame_buf[1]->mem,
+                                              buf->num_frames );
+              buf = swap_frame_buffers(this);
+            }
+            break;
+          case AO_CAP_MODE_STEREO:
+            if( this->output.mode == AO_CAP_MODE_MONO ) {
+              ensure_buffer_size(this->frame_buf[1], 2, buf->num_frames );
+              audio_out_resample_stereotomono(buf->mem, this->frame_buf[1]->mem,
+                                              buf->num_frames );
+              buf = swap_frame_buffers(this);
+            }
+            break;
+          case AO_CAP_MODE_4CHANNEL:
+            break;
+          case AO_CAP_MODE_5CHANNEL:
+            break;
+          case AO_CAP_MODE_5_1CHANNEL:
+            break;
+          case AO_CAP_MODE_A52:
+          case AO_CAP_MODE_AC5:
+            break;
+        }
+      }
+      
+      /* convert back to 8 bits after resampling */
+      if( this->output.bits == 8 && (this->do_resample || 
+          this->input.mode != this->output.mode) ) {
+        ensure_buffer_size(this->frame_buf[1], 1*mode_channels(this->output.mode),
+                           buf->num_frames );
+        audio_out_resample_16to8(buf->mem, (int8_t *)this->frame_buf[1]->mem,
+                                 mode_channels(this->output.mode) * buf->num_frames );
+        buf = swap_frame_buffers(this);
+      }
+      
+      this->driver->write (this->driver, buf->mem, buf->num_frames );
     }
+    pthread_mutex_unlock( &this->driver_lock );
 
-    fifo_append (this->free_fifo, buf);
+    fifo_append (this->free_fifo, in_buf);
 
   }
 
@@ -431,18 +535,44 @@ static int ao_open(ao_instance_t *this,
 	    "audio_out: stream audio format is %d kHz sampling rate, %d bits. mode is %d.\n",
 	    rate, bits, mode);
 
-  if ((output_sample_rate=this->driver->open(this->driver,bits,(this->force_rate ? this->force_rate : rate),mode)) == 0) {
+  this->input.mode            = mode;
+  this->input.rate            = rate;
+  this->input.bits            = bits;
+  
+  /* not all drivers/cards support 8 bits */
+  if( this->input.bits == 8 && 
+      !(this->driver->get_capabilities(this->driver) & AO_CAP_8BITS) ) {
+    bits = 16;      
+    printf("audio_out: 8 bits not supported by driver, converting to 16 bits.\n");
+  }
+    
+  /* provide mono->stereo and stereo->mono conversions */
+  if( this->input.mode == AO_CAP_MODE_MONO && 
+      !(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_MONO) ) {
+    mode = AO_CAP_MODE_STEREO;
+    printf("audio_out: mono not supported by driver, converting to stereo.\n");
+  }
+  if( this->input.mode == AO_CAP_MODE_STEREO && 
+      !(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_STEREO) ) {
+    mode = AO_CAP_MODE_MONO;
+    printf("audio_out: stereo not supported by driver, converting to mono.\n");
+  }
+ 
+  pthread_mutex_lock( &this->driver_lock );
+  output_sample_rate=this->driver->open(this->driver,bits,(this->force_rate ? this->force_rate : rate),mode);
+  pthread_mutex_unlock( &this->driver_lock );
+  
+  if ( output_sample_rate == 0) {
     printf("audio_out: open failed!\n");
     return 0;
   }; 
 
   printf("audio_out: output sample rate %d\n", output_sample_rate);
 
-  this->mode                  = mode;
-  this->input_frame_rate      = rate;
-  this->bits                  = bits;
   this->last_audio_vpts       = 0;
-  this->output_frame_rate     = output_sample_rate;
+  this->output.mode           = mode;
+  this->output.rate           = output_sample_rate;
+  this->output.bits           = bits;
 
   switch (this->resample_conf) {
   case 1: /* force off */
@@ -452,22 +582,17 @@ static int ao_open(ao_instance_t *this,
     this->do_resample = 1;
     break;
   default: /* AUTO */
-    this->do_resample = this->output_frame_rate != this->input_frame_rate;
+    this->do_resample = this->output.rate != this->input.rate;
   }
 
-  /* HACK: we do not have resample functions for 8-bit audio */
-  if (this->bits==8)
-    this->do_resample = 0;
-
   if (this->do_resample) 
     printf("audio_out: will resample audio from %d to %d\n",
-	   this->input_frame_rate, this->output_frame_rate);
+	   this->input.rate, this->output.rate);
 
-  this->num_channels      = this->driver->num_channels(this->driver); 
-
-  this->frame_rate_factor = (double) this->output_frame_rate / (double) this->input_frame_rate; 
-  this->audio_step        = (uint32_t) 90000 * (uint32_t) 32768 / this->input_frame_rate;
-  this->frames_per_kpts   = this->output_frame_rate * 1024 / 90000;
+  this->frame_rate_factor = ((double)(this->output.rate)) / ((double)(this->input.rate));
+  /* FIXME: If this->frames_per_kpts line goes after this->audio_step line, xine crashes with FPE, when compiled with gcc 3.0.1!!! Why? */ 
+  this->frames_per_kpts   = this->output.rate * 1024 / 90000;
+  this->audio_step        = ( (uint32_t)(90000) * (uint32_t)(32768) )/ this->input.rate;
 #ifdef LOG
   printf ("audio_out : audio_step %d pts per 32768 frames\n", this->audio_step);
 #endif
@@ -490,12 +615,12 @@ static int ao_open(ao_instance_t *this,
 
     printf ("audio_out: can't create thread (%s)\n", strerror(err));
     printf ("audio_out: sorry, this should not happen. please restart xine.\n");
-    exit(1);
+    abort();
 
   } else
     printf ("audio_out: thread created\n");
 
-  return this->output_frame_rate;
+  return this->output.rate;
 }
 
 static audio_buffer_t *ao_get_buffer (ao_instance_t *this) {
@@ -559,16 +684,37 @@ static void ao_close(ao_instance_t *this) {
 
   printf ("audio_out: thread stopped, closing driver\n");
 
+  pthread_mutex_lock( &this->driver_lock );
   this->driver->close(this->driver);  
+  pthread_mutex_unlock( &this->driver_lock );
 }
 
 static void ao_exit(ao_instance_t *this) {
-
+  int vol;
+  int prop = 0;
+  
   audio_buffer_t *buf, *next;
+  
+  pthread_mutex_lock( &this->driver_lock );
+  
+  if((this->driver->get_capabilities(this->driver)) & AO_CAP_MIXER_VOL)
+    prop = AO_PROP_MIXER_VOL;
+  else if((this->driver->get_capabilities(this->driver)) & AO_CAP_PCM_VOL)
+    prop = AO_PROP_PCM_VOL;
+  
+  vol = this->driver->get_property(this->driver, prop);
+  this->xine->config->update_num(this->xine->config, "audio.mixer_volume", vol);
+
+  /* Save config is needed, otherwise value change will be lost */
+  this->xine->config->save(this->xine->config);
 
   this->driver->exit(this->driver);
+  pthread_mutex_unlock( &this->driver_lock );
 
-  free (this->frame_buffer);
+  free (this->frame_buf[0]->mem);
+  free (this->frame_buf[0]);
+  free (this->frame_buf[1]->mem);
+  free (this->frame_buf[1]);
   free (this->zero_space);
 
   buf = this->free_fifo->first;
@@ -602,18 +748,32 @@ static void ao_exit(ao_instance_t *this) {
 
 static uint32_t ao_get_capabilities (ao_instance_t *this) {
   uint32_t result;
+  
+  pthread_mutex_lock( &this->driver_lock );
   result=this->driver->get_capabilities(this->driver);  
+  pthread_mutex_unlock( &this->driver_lock );
+  
   return result;
 }
 
 static int ao_get_property (ao_instance_t *this, int property) {
+  int ret;
 
-  return(this->driver->get_property(this->driver, property));
+  pthread_mutex_lock( &this->driver_lock );
+  ret = this->driver->get_property(this->driver, property);
+  pthread_mutex_unlock( &this->driver_lock );
+  
+  return ret;
 }
 
 static int ao_set_property (ao_instance_t *this, int property, int value) {
+  int ret;
 
-  return(this->driver->set_property(this->driver, property, value));
+  pthread_mutex_lock( &this->driver_lock );
+  ret =  this->driver->set_property(this->driver, property, value);
+  pthread_mutex_unlock( &this->driver_lock );
+  
+  return ret;
 }
 
 static int ao_control (ao_instance_t *this, int cmd, ...) {
@@ -624,7 +784,9 @@ static int ao_control (ao_instance_t *this, int cmd, ...) {
 
   va_start(args, cmd);
   arg = va_arg(args, void*);
+  pthread_mutex_lock( &this->driver_lock );
   rval = this->driver->control(this->driver, cmd, arg);
+  pthread_mutex_unlock( &this->driver_lock );
   va_end(args);
 
   return rval;
@@ -642,6 +804,7 @@ ao_instance_t *ao_new_instance (ao_driver_t *driver, xine_t *xine) {
   this->driver                = driver;
   this->metronom              = xine->metronom;
   this->xine                  = xine;
+  pthread_mutex_init( &this->driver_lock, NULL );
 
   this->open                  = ao_open;
   this->get_buffer            = ao_get_buffer;
@@ -654,17 +817,22 @@ ao_instance_t *ao_new_instance (ao_driver_t *driver, xine_t *xine) {
   this->control		      = ao_control;
   this->audio_loop_running    = 0;
   this->audio_paused          = 0;
-  /* FIXME: is 4* good enough for all resample cases?? */
-  this->frame_buffer          = xine_xmalloc (4 * AUDIO_BUF_SIZE);
   this->zero_space            = xine_xmalloc (ZERO_BUF_SIZE * 2 * 6);
   this->gap_tolerance         = driver->get_gap_tolerance (this->driver);
 
   this->resample_conf = config->register_enum (config, "audio.resample_mode", 0,
-					       resample_modes, "adjust whether resampling is done or not",
+					       resample_modes,
+					       _("adjust whether resampling is done or not"),
 					       NULL, NULL, NULL);
-  this->force_rate    = config->register_range (config, "audio.force_rate", 0,
-						0, 96000, "if !=0 always resample to given rate",
-						NULL, NULL, NULL);
+  this->force_rate    = config->register_num (config, "audio.force_rate", 0,
+					      _("if !=0 always resample to given rate"),
+					      NULL, NULL, NULL);
+
+  this->passthrough_offset = config->register_num (config,
+						   "audio.passthrough_offset",
+						   10000,
+						   _("adjust if audio is offsync"),
+						   NULL, NULL, NULL);
 
   /*
    * pre-allocate memory for samples
@@ -683,7 +851,43 @@ ao_instance_t *ao_new_instance (ao_driver_t *driver, xine_t *xine) {
 
     fifo_append (this->free_fifo, buf);
   }
-
   
+  /* buffers used for audio conversions */
+  for (i=0; i<2; i++) {
+
+    audio_buffer_t *buf;
+
+    buf = (audio_buffer_t *) malloc (sizeof (audio_buffer_t));
+    buf->mem = malloc (4*AUDIO_BUF_SIZE);
+    buf->mem_size = 4*AUDIO_BUF_SIZE;
+
+    this->frame_buf[i] = buf;
+  }
+
+  /*
+   * Set audio volume to latest used one ?
+   */
+  if(this->driver){
+    int vol;
+    
+    vol = config->register_range (config, "audio.mixer_volume", 
+				  50, 0, 100, _("Audio volume"), 
+				  NULL, NULL, NULL);
+    
+    if(config->register_bool (config, "audio.remember_volume", 0,
+			      _("restore volume level at startup"), 
+			      _("if this not set, xine will not touch any mixer settings at startup"),
+			      NULL, NULL)) {
+      int prop = 0;
+
+      if((ao_get_capabilities(this)) & AO_CAP_MIXER_VOL)
+	prop = AO_PROP_MIXER_VOL;
+      else if((ao_get_capabilities(this)) & AO_CAP_PCM_VOL)
+	prop = AO_PROP_PCM_VOL;
+      
+      ao_set_property(this, prop, vol);
+    }
+  }
+
   return this;
 }
