@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer.h,v 1.43 2002/04/27 23:31:33 cvogler Exp $
+ * $Id: buffer.h,v 1.61 2002/07/19 03:03:37 miguelfreitas Exp $
  *
  *
  * contents:
@@ -38,6 +38,10 @@
 extern "C" {
 #endif
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <pthread.h>
 #include <inttypes.h>
@@ -78,8 +82,8 @@ extern "C" {
 #define BUF_VIDEO_MPEG		0x02000000
 #define BUF_VIDEO_MPEG4		0x02010000
 #define BUF_VIDEO_CINEPAK	0x02020000
-#define BUF_VIDEO_SORENSON	0x02030000
-#define BUF_VIDEO_MSMPEG4_V12	0x02040000
+#define BUF_VIDEO_SORENSON_V1	0x02030000
+#define BUF_VIDEO_MSMPEG4_V2	0x02040000
 #define BUF_VIDEO_MSMPEG4_V3	0x02050000
 #define BUF_VIDEO_MJPEG		0x02060000
 #define BUF_VIDEO_IV50		0x02070000
@@ -104,6 +108,21 @@ extern "C" {
 #define BUF_VIDEO_CYUV          0x021B0000
 #define BUF_VIDEO_DIVX5         0x021C0000
 #define BUF_VIDEO_XVID          0x021D0000
+#define BUF_VIDEO_SMC		0x021E0000
+#define BUF_VIDEO_RPZA		0x021F0000
+#define BUF_VIDEO_QTRLE		0x02200000
+#define BUF_VIDEO_MSRLE		0x02210000
+#define BUF_VIDEO_DUCKTM1	0x02220000
+#define BUF_VIDEO_FLI		0x02230000
+#define BUF_VIDEO_ROQ		0x02240000
+#define BUF_VIDEO_SORENSON_V3	0x02250000
+#define BUF_VIDEO_MSMPEG4_V1	0x02260000
+#define BUF_VIDEO_MSS1		0x02270000
+#define BUF_VIDEO_IDCIN		0x02280000
+#define BUF_VIDEO_PGVV		0x02290000
+#define BUF_VIDEO_ZYGO		0x022A0000
+#define BUF_VIDEO_TSCC		0x022B0000
+#define BUF_VIDEO_YVU9		0x022C0000
 
 /* audio buffer types:  (please keep in sync with buffer_types.c) */
 
@@ -115,7 +134,7 @@ extern "C" {
 #define BUF_AUDIO_DIVXA		0x03040000
 #define BUF_AUDIO_DTS		0x03050000
 #define BUF_AUDIO_MSADPCM	0x03060000
-#define BUF_AUDIO_IMAADPCM	0x03070000
+#define BUF_AUDIO_MSIMAADPCM	0x03070000
 #define BUF_AUDIO_MSGSM		0x03080000 
 #define BUF_AUDIO_VORBIS        0x03090000
 #define BUF_AUDIO_IMC           0x030a0000
@@ -125,7 +144,15 @@ extern "C" {
 #define BUF_AUDIO_AAC           0x030e0000
 #define BUF_AUDIO_REAL    	0x030f0000
 #define BUF_AUDIO_VIVOG723      0x03100000
-
+#define BUF_AUDIO_DK3ADPCM	0x03110000
+#define BUF_AUDIO_DK4ADPCM	0x03120000
+#define BUF_AUDIO_ROQ		0x03130000
+#define BUF_AUDIO_QTIMAADPCM	0x03140000
+#define BUF_AUDIO_MAC3		0x03150000
+#define BUF_AUDIO_MAC6		0x03160000
+#define BUF_AUDIO_QDESIGN1	0x03170000
+#define BUF_AUDIO_QDESIGN2	0x03180000
+#define BUF_AUDIO_QCLP		0x03190000
 
 /* spu buffer types:    */
  
@@ -176,6 +203,84 @@ struct buf_element_s {
 #define BUF_FLAG_END_USER    0x0020
 #define BUF_FLAG_END_STREAM  0x0040
 #define BUF_FLAG_FRAMERATE   0x0080
+#define BUF_FLAG_SEEK        0x0100
+#define BUF_FLAG_SPECIAL     0x0200
+
+/* these are the types of special buffers */
+/* 
+ * In a BUF_SPECIAL_PALETTE buffer:
+ * decoder_info[1] = BUF_SPECIAL_PALETTE
+ * decoder_info[2] = number of entries in palette table
+ * decoder_info[3] = pointer to palette table
+ * A palette table is an array of palette_entry_t structures. A decoder
+ * should not count on this array to exist for the duration of the
+ * program's execution and should copy, manipulate, and store the palette
+ * data privately if it needs the palette information.
+ */
+#define BUF_SPECIAL_PALETTE  1
+
+/*
+ * In a BUF_SPECIAL_IDCIN_HUFFMAN_TABLE buffer:
+ * decoder_info[1] = BUF_SPECIAL_IDCIN_HUFFMAN_TABLE
+ * decoder_info[2] = pointer to a 65536-element byte array containing the
+ *  Huffman tables from an Id CIN file
+ * A decoder should not count on the byte array to exist for the duration
+ * of the program's execution and should copy the data into its own
+ * private structures.
+ */
+#define BUF_SPECIAL_IDCIN_HUFFMAN_TABLE  2
+
+/*
+ * In a BUF_SPECIAL_ASPECT buffer:
+ * decoder_info[1] = BUF_SPECIAL_ASPECT
+ * decoder_info[2] = aspect ratio code
+ * decoder_info[3] = stream scale prohibitions
+ * This buffer is used to force mpeg decoders to use a certain aspect.
+ * Currently xine-dvdnav uses this, because it has more accurate information
+ * about the aspect from the dvd ifo-data.
+ * The stream scale prohibitions are also delivered, with bit 0 meaning
+ * "deny letterboxing" and bit 1 meaning "deny pan&scan"
+ */
+#define BUF_SPECIAL_ASPECT  3
+
+/*
+ * In a BUF_SPECIAL_DECODER_CONFIG buffer:
+ * decoder_info[1] = BUF_SPECIAL_DECODER_CONFIG
+ * decoder_info[2] = data size
+ * decoder_info[3] = pointer to data
+ * This buffer is used to pass config information from  .mp4 files 
+ * (atom esds) to decoders. both mpeg4 and aac streams use that.
+ */
+#define BUF_SPECIAL_DECODER_CONFIG  4
+
+/*
+ * In a BUF_SPECIAL_SAMPLE_SIZE_TABLE buffer:
+ * decoder_info[1] = BUF_SPECIAL_SAMPLE_SIZE_TABLE
+ * decoder_info[2] = 
+ * decoder_info[3] = pointer to table with sample sizes 
+ *                   (within current frame)
+ * libfaad needs to decode data on sample-sized chunks. 
+ * unfortunately original sample sizes are not know at decoder stage. 
+ * this buffer is used to pass such information.
+ */
+#define BUF_SPECIAL_SAMPLE_SIZE_TABLE 5
+
+/*
+ * In a BUF_SPECIAL_LPCM_CONFIG buffer:
+ * decoder_info[1] = BUF_SPECIAL_LPCM_CONFIG
+ * decoder_info[2] = config data
+ * lpcm data encoded into mpeg2 streams have a format configuration
+ * byte in every frame. this is used to detect the sample rate,
+ * number of bits and channels.
+ */
+#define BUF_SPECIAL_LPCM_CONFIG 6
+
+
+typedef struct palette_entry_s palette_entry_t;
+struct palette_entry_s
+{
+  unsigned char r, g, b;
+} ;
 
 typedef struct fifo_buffer_s fifo_buffer_t;
 struct fifo_buffer_s
@@ -231,18 +336,60 @@ struct fifo_buffer_s
 fifo_buffer_t *fifo_buffer_new (int num_buffers, uint32_t buf_size);
 
 
-/* return BUF_VIDEO_xxx given the fourcc */
-uint32_t fourcc_to_buf_video( void * fourcc );
+/* return BUF_VIDEO_xxx given the fourcc
+ * fourcc_int must be read in machine endianness
+ * example: fourcc_int = *(uint32_t *)fourcc_char;
+ */
+uint32_t fourcc_to_buf_video( uint32_t fourcc_int );
 
 /* return codec name given BUF_VIDEO_xxx */
 char * buf_video_name( uint32_t buf_type );
 
 /* return BUF_VIDEO_xxx given the formattag */
-uint32_t formattag_to_buf_audio( uint16_t formattag );
+uint32_t formattag_to_buf_audio( uint32_t formattag );
 
 /* return codec name given BUF_VIDEO_xxx */
 char * buf_audio_name( uint32_t buf_type );
 
+
+/* this is xine version of BITMAPINFOHEADER 
+ * - should be safe to compile on 64bits machines 
+ * - will always use machine endian format, so demuxers reading
+ *   stuff from win32 formats must use the function below.
+ */
+typedef struct __attribute__((__packed__)) {
+    int32_t        biSize;
+    int32_t        biWidth;
+    int32_t        biHeight;
+    int16_t        biPlanes;
+    int16_t        biBitCount;
+    uint32_t       biCompression;
+    int32_t        biSizeImage;
+    int32_t        biXPelsPerMeter;
+    int32_t        biYPelsPerMeter;
+    int32_t        biClrUsed;
+    int32_t        biClrImportant;
+} xine_bmiheader;
+
+/* convert xine_bmiheader struct from little endian */
+void xine_bmiheader_le2me( xine_bmiheader *bih );
+
+/* this is xine version of WAVEFORMATEX 
+ * (the same comments from xine_bmiheader)
+ */
+typedef struct __attribute__((__packed__)) {
+  int16_t   wFormatTag;
+  int16_t   nChannels;
+  int32_t   nSamplesPerSec;
+  int32_t   nAvgBytesPerSec;
+  int16_t   nBlockAlign;
+  int16_t   wBitsPerSample;
+  int16_t   cbSize;
+} xine_waveformatex;
+
+/* convert xine_waveformatex struct from little endian */
+void xine_waveformatex_le2me( xine_waveformatex *wavex );
+
 #ifdef __cplusplus
 }
 #endif
