@@ -62,6 +62,22 @@ extern "C" {
 #define XINE_EVENT_CLOSED_CAPTION       24
 #define XINE_EVENT_INPUT_BUTTON_FORCE   25
 
+#define XINE_EVENT_INPUT_NUMBER_0       30
+#define XINE_EVENT_INPUT_NUMBER_1       31
+#define XINE_EVENT_INPUT_NUMBER_2       32
+#define XINE_EVENT_INPUT_NUMBER_3       33
+#define XINE_EVENT_INPUT_NUMBER_4       34
+#define XINE_EVENT_INPUT_NUMBER_5       35
+#define XINE_EVENT_INPUT_NUMBER_6       36
+#define XINE_EVENT_INPUT_NUMBER_7       37
+#define XINE_EVENT_INPUT_NUMBER_8       38
+#define XINE_EVENT_INPUT_NUMBER_9       39
+#define XINE_EVENT_INPUT_NUMBER_10_ADD  40
+
+#define XINE_EVENT_ASPECT_CHANGE	41 /* Generally should be viewed as a hint to the GUI */
+
+
+
 /*
  * generic event type.
  */
@@ -125,6 +141,7 @@ typedef struct {
   xine_event_t     event;
   int              width;
   int              height;
+  int              aspect;
 } xine_frame_change_event_t;
 
 /*
@@ -138,6 +155,15 @@ typedef struct {
   uint32_t         scr;
 } xine_closed_caption_event_t;
 
+/*
+ * Aspect ratio change (suggested)
+ */
+typedef struct {
+  xine_event_t     event;
+  int              ratio_code;
+  int              scale_permission;
+} xine_aspect_ratio_event_t;
+
 #ifdef __cplusplus
 }
 #endif
