@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: metronom.c,v 1.83 2002/04/27 16:33:24 miguelfreitas Exp $
+ * $Id: metronom.c,v 1.87 2002/06/29 14:32:36 tmattern Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -283,6 +283,10 @@ static void metronom_handle_video_discontinuity (metronom_t *this, int type,
 
   pthread_mutex_lock (&this->lock);
 
+#ifdef LOG
+  printf ("metronom: av_diff=%ld\n", this->video_vpts - this->audio_vpts);
+#endif
+
   this->video_discontinuity_count++;
   pthread_cond_signal (&this->video_discontinuity_reached);
   
@@ -315,16 +319,38 @@ static void metronom_handle_video_discontinuity (metronom_t *this, int type,
 
   switch (type) {
   case DISC_STREAMSTART:
+#ifdef LOG
+    printf ("metronom: DISC_STREAMSTART\n");
+#endif
     this->vpts_offset      = this->video_vpts;
     this->in_discontinuity = 0;
+    this->force_audio_jump = 0;
     break;
   case DISC_ABSOLUTE:
+#ifdef LOG
+    printf ("metronom: DISC_ABSOLUTE\n");
+#endif
     this->next_vpts_offset = this->video_vpts - disc_off;
     this->in_discontinuity = 30;
+    this->force_audio_jump = 0;
     break;
   case DISC_RELATIVE:
+#ifdef LOG
+    printf ("metronom: DISC_RELATIVE\n");
+#endif
     this->next_vpts_offset = this->vpts_offset - disc_off;
     this->in_discontinuity = 30;
+    this->force_audio_jump = 0;
+    break;
+  case DISC_STREAMSEEK:
+#ifdef LOG
+    printf ("metronom: DISC_STREAMSEEK\n");
+#endif
+    this->vpts_offset      = this->video_vpts - disc_off;
+    this->next_vpts_offset = this->video_vpts - disc_off;
+    this->in_discontinuity = 30;
+    this->force_audio_jump = 1;
+    this->allow_full_ao_fill_gap = 1;
     break;
   }
   
@@ -341,9 +367,10 @@ static void metronom_got_video_frame (metronom_t *this, vo_frame_t *img) {
   
   pthread_mutex_lock (&this->lock);
 
-/*
+#ifdef LOG
   printf("metronom: got_video_frame pts = %lld\n", pts );
-*/
+#endif
+
   if (this->in_discontinuity) {
     this->in_discontinuity--;
 
@@ -470,17 +497,17 @@ static int64_t metronom_got_audio_samples (metronom_t *this, int64_t pts,
 
   pthread_mutex_lock (&this->lock);
 
-  if (this->in_discontinuity) 
+  if (this->in_discontinuity && !this->force_audio_jump) 
     pts = 0; /* ignore pts during discontinuities */
-  
   if (pts) {
     vpts = pts + this->vpts_offset;
     diff = this->audio_vpts - vpts;
     
     /* compare predicted and given vpts */
-    if( abs(diff) > AUDIO_DRIFT_TOLERANCE ) {
+    if( (abs(diff) > AUDIO_DRIFT_TOLERANCE) || this->force_audio_jump ) {
       this->audio_vpts = vpts;
       this->audio_drift_step = 0;
+      this->force_audio_jump = 0;
       printf("metronom: audio jump\n");
     }
     else {
@@ -537,9 +564,9 @@ static void metronom_set_option (metronom_t *this, int option, int64_t value) {
     break;
   case METRONOM_ADJ_VPTS_OFFSET:
     this->vpts_offset += value;
-/*#ifdef LOG*/
+#ifdef LOG
     printf ("metronom: adjusting vpts_offset by %lld\n", value );
-/*#endif*/
+#endif
     break;
   default:
     printf ("metronom: unknown option in set_option: %d\n",
@@ -722,6 +749,7 @@ metronom_t * metronom_init (int have_audio, void *xine) {
   this->have_audio                = have_audio;
   this->audio_vpts                = PREBUFFER_PTS_OFFSET;
   this->audio_discontinuity_count = 0;
+  this->allow_full_ao_fill_gap    = 0;
   pthread_cond_init (&this->audio_discontinuity_reached, NULL);
     
 
