@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: metronom.h,v 1.30 2002/04/24 20:26:07 jcdutton Exp $
+ * $Id: metronom.h,v 1.34 2002/06/29 14:32:36 tmattern Exp $
  *
  * metronom: general pts => virtual calculation/assoc
  *                   
@@ -59,6 +59,7 @@ typedef struct scr_plugin_s scr_plugin_t;
 #define DISC_STREAMSTART 0 
 #define DISC_RELATIVE    1     
 #define DISC_ABSOLUTE    2
+#define DISC_STREAMSEEK  3
 
 struct metronom_s {
 
@@ -131,6 +132,11 @@ struct metronom_s {
    *                    a specified offset from the former ones soon
    * DISC_ABSOLUTE    : typically a new menu stream (nav packets)
    *                    pts will start from given value soon
+   * DISC_STREAMSEEK  : used by video and audio decoder loop,
+   *                    when a buffer with BUF_FLAG_SEEK set is encountered;
+   *                    applies the necessary vpts offset for the seek in
+   *                    metronom, but keeps the vpts difference between
+   *                    audio and video, so that metronom doesn't cough
    *
    * for DISC_RELATIVE and DISC_ABSOLUTE metronom will enter a
    * special discontinuity mode which means that it will ignore
@@ -237,6 +243,8 @@ struct metronom_s {
   pthread_cond_t  audio_discontinuity_reached;
   pthread_cond_t  cancel;
 
+  int             allow_full_ao_fill_gap;
+  int             force_audio_jump;
 };
 
 metronom_t *metronom_init (int have_audio, void *xine);
