@@ -757,6 +757,7 @@ static void osd_load_fonts (osd_renderer_t *this, char *path) {
 
       }
     }
+    closedir (dir);
   }
 }
 
@@ -860,7 +861,7 @@ osd_renderer_t *osd_renderer_init( video_overlay_instance_t *video_overlay, conf
 
   this->textpalette = config->register_enum (config, "misc.osd_text_palette", 0,
                                              textpalettes_str, 
-                                             "Palette (foreground-border-background) to use on subtitles",
+                                             _("Palette (foreground-border-background) to use on subtitles"),
                                              NULL, update_text_palette, this);
   
   /*
