@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_decoder.c,v 1.83 2002/04/16 12:27:20 jcdutton Exp $
+ * $Id: video_decoder.c,v 1.92 2002/07/14 20:55:17 miguelfreitas Exp $
  *
  */
 
@@ -64,7 +64,8 @@ void *video_decoder_loop (void *this_gen) {
   spu_decoder_t   *spu_decoder;
   static int	   prof_video_decode = -1;
   static int	   prof_spu_decode = -1;
-
+  static uint32_t  buftype_unknown = 0;
+  
   if (prof_video_decode == -1)
     prof_video_decode = xine_profiler_allocate_slot ("video decoder");
   if (prof_spu_decode == -1)
@@ -82,9 +83,14 @@ void *video_decoder_loop (void *this_gen) {
       this->cur_input_pos = buf->input_pos;
     if (buf->input_length)
       this->cur_input_length = buf->input_length;
-    if (buf->input_time)
+    if (buf->input_time) {
       this->cur_input_time = buf->input_time;
-
+      pthread_mutex_lock (&this->osd_lock);
+      if( this->curtime_needed_for_osd && !(--this->curtime_needed_for_osd) )
+          xine_internal_osd (this, ">",90000);
+      pthread_mutex_unlock (&this->osd_lock);
+    }
+    
 #ifdef VIDEO_DECODER_LOG
     printf ("video_decoder: got buffer 0x%08x\n", buf->type);      
 #endif
@@ -132,7 +138,12 @@ void *video_decoder_loop (void *this_gen) {
       {
 	xine_ui_event_t  ui_event;
 	
+	/* We use widescreen spu as the auto selection, because widescreen
+	 * display is common. SPU decoders can choose differently if it suits
+	 * them. */
 	this->spu_channel_auto = buf->decoder_info[0];
+	this->spu_channel_letterbox = buf->decoder_info[1];
+	this->spu_channel_pan_scan = buf->decoder_info[2];
 	if (this->spu_channel_user == -1)
 	  this->spu_channel = this->spu_channel_auto;
 	
@@ -157,12 +168,16 @@ void *video_decoder_loop (void *this_gen) {
       pthread_mutex_lock (&this->finished_lock);
       this->spu_finished = 1;
 
-      if (!this->video_finished && (buf->decoder_flags & BUF_FLAG_END_STREAM )) {
-
+      if (!this->video_finished ) {
         this->video_finished = 1;
-
+        
         if (this->audio_finished) {
-          xine_notify_stream_finished (this);
+          if( this->playing_logo )
+            buf->decoder_flags = 0;
+          this->playing_logo = 0;
+          
+          if( buf->decoder_flags & BUF_FLAG_END_STREAM )
+            xine_notify_stream_finished (this);
         }
       }
 
@@ -207,11 +222,15 @@ void *video_decoder_loop (void *this_gen) {
 
       this->video_in_discontinuity = 1;
 
-      this->metronom->handle_video_discontinuity (this->metronom, DISC_ABSOLUTE, buf->disc_off);
-      
+      if (buf->decoder_flags && BUF_FLAG_SEEK) {
+        this->metronom->handle_video_discontinuity (this->metronom, DISC_STREAMSEEK, buf->disc_off);
+      } else {
+        this->metronom->handle_video_discontinuity (this->metronom, DISC_ABSOLUTE, buf->disc_off);
+      }
+
       this->video_in_discontinuity = 0;
       break;
-    
+
     case BUF_CONTROL_AUDIO_CHANNEL:
       {
 	xine_ui_event_t  ui_event;
@@ -256,13 +275,24 @@ void *video_decoder_loop (void *this_gen) {
 
 	    xine_log (this, XINE_LOG_FORMAT, "using video decoder plugin '%s'\n",
 		      decoder->get_identifier());
+    
+	    xine_report_codec( this, XINE_CODEC_VIDEO, 0, buf->type, 1);
 	    
 	  }
 
 	  decoder->decode_data (this->cur_video_decoder_plugin, buf);  
-	}
-      } else
-	printf ("video_decoder: unknown buffer type: %08x\n", buf->type);
+	
+	} else if( buf->type != buftype_unknown ) {
+	    xine_log (this, XINE_LOG_MSG, "video_decoder: no plugin available to handle '%s'\n",
+		        buf_video_name( buf->type ) );
+	    xine_report_codec( this, XINE_CODEC_VIDEO, 0, buf->type, 0);
+	    buftype_unknown = buf->type;
+        }
+      } else if( buf->type != buftype_unknown ) {
+	  xine_log (this, XINE_LOG_MSG, "video_decoder: unknown buffer type: %08x\n",
+		    buf->type );
+	  buftype_unknown = buf->type;
+      }
 
       xine_profiler_stop_count (prof_video_decode);
 
@@ -282,6 +312,13 @@ void video_decoder_init (xine_t *this) {
   struct sched_param   pth_params;
   int		       err;
 
+  /* The fifo size is based on dvd playback where buffers are filled
+   * with 2k of data. With 500 buffers and a typical video data rate
+   * of 4 Mbit/s, the fifo can hold about 2 seconds of video, wich
+   * should be enough to compensate for drive delays.
+   * We provide buffers of 8k size instead of 2k for demuxers sending
+   * larger chunks.
+   */
   this->video_fifo = fifo_buffer_new (500, 8192);
 
   pthread_attr_init(&pth_attrs);
@@ -294,7 +331,7 @@ void video_decoder_init (xine_t *this) {
 			     &pth_attrs, video_decoder_loop, this)) != 0) {
     fprintf (stderr, "video_decoder: can't create new thread (%s)\n",
 	     strerror(err));
-    exit (1);
+    abort();
   }
 
   this->video_in_discontinuity = 0;
