@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out.c,v 1.93 2002/04/09 03:38:01 miguelfreitas Exp $
+ * $Id: video_out.c,v 1.102 2002/08/03 20:42:32 siggi Exp $
  *
  * frame allocation / queuing / scheduling / output functions
  */
@@ -60,7 +60,6 @@ typedef struct {
 
   vo_frame_t               *last_frame;
   vo_frame_t               *img_backup;
-  int                       backup_is_logo;
   int                       redraw_needed;
   
   int                       video_loop_running;
@@ -74,10 +73,6 @@ typedef struct {
   /* pts value when decoder delivered last video frame */
   int64_t                   last_delivery_pts; 
 
-  char                     *logo_pathname;
-  pthread_mutex_t           logo_mutex;
-  int                       logo_w, logo_h;
-  uint8_t                  *logo_yuy2;
 
   video_overlay_instance_t *overlay_source;
   int                       overlay_enabled;
@@ -92,70 +87,11 @@ struct img_buf_fifo_s {
   vo_frame_t        *last;
   int                num_buffers;
 
+  int                locked_for_read;
   pthread_mutex_t    mutex;
   pthread_cond_t     not_empty;
 } ;
 
-/*
- * Logo related functions + config callback.
- */
-static uint16_t gzread_i16(gzFile *fp) {
-  uint16_t ret;
-  ret = gzgetc(fp) << 8 ;
-  ret |= gzgetc(fp);
-  return ret;
-}
-
-static int _load_logo_file(vos_t *this) {
-  gzFile *fp;
-  int     retval = 0;
-
-  pthread_mutex_lock(&this->logo_mutex);
-
-  if((fp = gzopen (this->logo_pathname, "rb")) != NULL) {
-
-    if(this->logo_yuy2)
-      free (this->logo_yuy2);
-
-    this->logo_w = gzread_i16 (fp);
-    this->logo_h = gzread_i16 (fp);
-#ifdef LOG    
-    printf ("video_out: loading logo %d x %d pixels, yuy2\n",
-	    this->logo_w, this->logo_h);
-#endif
-
-    this->logo_yuy2 = malloc (this->logo_w * this->logo_h *2);
-
-    gzread (fp, this->logo_yuy2, this->logo_w * this->logo_h *2);
-    gzclose (fp);
-
-    this->backup_is_logo = 0;
-
-    retval = 1;
-  }
-
-  pthread_mutex_unlock(&this->logo_mutex);
-  return retval;
-}
-
-static void _logo_change_cb(void *data, cfg_entry_t *cfg) {
-  vos_t            *this = (vos_t *) data;
-  char              default_logo[2048];
-  
-  this->logo_pathname = cfg->str_value;
-
-  if(!_load_logo_file(this)) {
-#ifdef LOG
-    printf("_load_logo_file() failed, reload default\n");
-#endif
-    snprintf(default_logo, 2048, "%s/xine_logo.zyuy2", XINE_SKINDIR);
-    cfg->config->update_string(cfg->config, cfg->key, default_logo);
-  }
-}
-/*
- * End of logo.
- */
-
 static img_buf_fifo_t *vo_new_img_buf_queue () {
 
   img_buf_fifo_t *queue;
@@ -165,6 +101,7 @@ static img_buf_fifo_t *vo_new_img_buf_queue () {
     queue->first       = NULL;
     queue->last        = NULL;
     queue->num_buffers = 0;
+    queue->locked_for_read = 0;
     pthread_mutex_init (&queue->mutex, NULL);
     pthread_cond_init  (&queue->not_empty, NULL);
   }
@@ -202,7 +139,7 @@ static vo_frame_t *vo_remove_from_img_buf_queue (img_buf_fifo_t *queue) {
 
   pthread_mutex_lock (&queue->mutex);
 
-  while (!queue->first) {
+  while (!queue->first || queue->locked_for_read) {
     pthread_cond_wait (&queue->not_empty, &queue->mutex);
   }
 
@@ -214,10 +151,6 @@ static vo_frame_t *vo_remove_from_img_buf_queue (img_buf_fifo_t *queue) {
     if (!queue->first) {
       queue->last = NULL;
       queue->num_buffers = 0;
-      /* I think this is a (pretty long-standing) bug
-	 guenter 21/03/2002
-	 pthread_cond_init  (&queue->not_empty, NULL);
-      */
     }
     else {
       queue->num_buffers--;
@@ -353,8 +286,6 @@ static int vo_frame_draw (vo_frame_t *img) {
     printf ("video_out: frame is ok => appending to display buffer\n");
 #endif
 
-    this->last_frame = img;
-
     vo_frame_inc_lock( img );
     vo_append_to_img_buf_queue (this->display_img_buf_queue, img);
 
@@ -370,7 +301,8 @@ static int vo_frame_draw (vo_frame_t *img) {
    * performance measurement
    */
 
-  if (this->num_frames_delivered>199) {
+  if ((this->num_frames_delivered % 200) == 0
+  	 && (this->num_frames_skipped || this->num_frames_discarded)) {
     xine_log(this->xine, XINE_LOG_MSG,
 	     _("%d frames delivered, %d frames skipped, %d frames discarded\n"), 
 	     this->num_frames_delivered, 
@@ -390,6 +322,99 @@ static int vo_frame_draw (vo_frame_t *img) {
  *
  */
 
+/* duplicate_frame(): this function is used to keep playing frames 
+ * while video is still or player paused. 
+ * 
+ * frame allocation inside vo loop is dangerous:
+ * we must never wait for a free frame -> deadlock condition.
+ * to avoid deadlocks we don't use vo_remove_from_img_buf_queue()
+ * and reimplement a slightly modified version here.
+ * free_img_buf_queue->mutex must be grabbed prior entering it.
+ * (must assure that free frames won't be exhausted by decoder thread).
+ */
+static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
+
+  vo_frame_t *dupl;
+  int         image_size;
+
+  if( !this->free_img_buf_queue->first)
+    return NULL;
+
+  dupl = this->free_img_buf_queue->first;
+  this->free_img_buf_queue->first = dupl->next;
+  dupl->next = NULL;
+  if (!this->free_img_buf_queue->first) {
+    this->free_img_buf_queue->last = NULL;
+    this->free_img_buf_queue->num_buffers = 0;
+  }
+  else {
+    this->free_img_buf_queue->num_buffers--;
+  }
+      
+  pthread_mutex_lock (&dupl->mutex);
+  dupl->lock_counter   = 1;
+  dupl->width          = img->width;
+  dupl->height         = img->height;
+  dupl->ratio          = img->ratio;
+  dupl->format         = img->format;
+  
+  this->driver->update_frame_format (this->driver, dupl, dupl->width, dupl->height, 
+				     dupl->ratio, dupl->format, VO_BOTH_FIELDS);
+
+  pthread_mutex_unlock (&dupl->mutex);
+  
+  image_size = img->pitches[0] * img->height;
+
+  if (img->format == IMGFMT_YV12) {
+    if (img->base[0])
+      xine_fast_memcpy(dupl->base[0], img->base[0], image_size);
+    if (img->base[1])
+      xine_fast_memcpy(dupl->base[1], img->base[1], img->pitches[1] * ((img->height+1)/2));
+    if (img->base[2])
+      xine_fast_memcpy(dupl->base[2], img->base[2], img->pitches[2] * ((img->height+1)/2));
+  } else {
+    if (img->base[0])
+      xine_fast_memcpy(dupl->base[0], img->base[0], image_size);
+  }  
+  
+  dupl->bad_frame = 0;
+  dupl->pts       = 0;
+  dupl->vpts      = 0;
+  dupl->duration  = img->duration;
+
+  if (img->format == IMGFMT_YV12) {
+    if (img->copy) {
+      int height = img->height;
+      uint8_t* src[3];
+  
+      src[0] = dupl->base[0];
+      src[1] = dupl->base[1];
+      src[2] = dupl->base[2];
+      while ((height -= 16) >= 0) {
+        dupl->copy(dupl, src);
+        src[0] += 16 * img->pitches[0];
+        src[1] +=  8 * img->pitches[1];
+        src[2] +=  8 * img->pitches[2];
+      }
+    }
+  } else {
+    if (img->copy) {
+      int height = img->height;
+      uint8_t* src[3];
+      
+      src[0] = dupl->base[0];
+      
+      while ((height -= 16) >= 0) {
+        dupl->copy(dupl, src);
+        src[0] += 16 * img->pitches[0];
+      }
+    }
+  }
+  
+  return dupl;
+}
+
+
 static void expire_frames (vos_t *this, int64_t cur_vpts) {
 
   int64_t       pts;
@@ -409,9 +434,14 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
     diff = cur_vpts - pts;
       
     if (diff > img->duration) {
-      xine_log(this->xine, XINE_LOG_MSG,
-	       _("video_out: throwing away image with pts %lld because "
-		 "it's too old (diff : %lld).\n"), pts, diff);
+
+      /* do not print this message in stop/exit (scr is adjusted to force
+       * discarding audio and video frames)
+       */
+      if( diff < 20 * 90000 )
+        xine_log(this->xine, XINE_LOG_MSG,
+	         _("video_out: throwing away image with pts %lld because "
+		   "it's too old (diff : %lld).\n"), pts, diff);
 
       this->num_frames_discarded++;
 
@@ -433,7 +463,6 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
 	printf("video_out: possible still frame (old)\n");
 
 	this->img_backup = img;
-	this->backup_is_logo = 0;
 	
 	/* wait 4 frames before drawing this one. 
 	   this allow slower systems to recover. */
@@ -466,51 +495,20 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
     printf ("video_out: no frame\n");
 #endif
 
-    /*
-     * display logo ?
-     */
-    if (!this->video_opened && (!this->img_backup || !this->backup_is_logo)) {
-
-      if (this->img_backup) {
-#ifdef LOG
-	printf("video_out: overwriting frame backup\n");
-#endif
-	vo_frame_dec_lock( this->img_backup );
-      }
-
-      printf("video_out: copying logo image\n");
-	
-      pthread_mutex_lock(&this->logo_mutex);
-
-      this->img_backup = vo_get_frame (&this->vo, this->logo_w, this->logo_h,
-				       42, IMGFMT_YUY2, VO_BOTH_FIELDS);
-
-      this->img_backup->duration       = 3000;
-      
-      xine_fast_memcpy(this->img_backup->base[0], this->logo_yuy2,
-		       this->logo_w*this->logo_h*2);
-      
-      pthread_mutex_unlock(&this->logo_mutex);
-      this->backup_is_logo = 1;
-      this->redraw_needed = 1;
-    }
-
     if (this->img_backup && (this->redraw_needed==1)) {
 
 #ifdef LOG
       printf("video_out: generating still frame (cur_vpts = %lld) \n",
 	     cur_vpts);
 #endif
-	
-      /* keep playing still frames */
-      img = this->vo.duplicate_frame (&this->vo, this->img_backup );
-  
-      do {
-	/* always restore duration so drift correction shouldn't cause any trouble */
-	img->duration = this->img_backup->duration;
-	this->metronom->got_video_frame(this->metronom, img);
-      } while (img->vpts < (cur_vpts - img->duration/2) );
 
+      /* keep playing still frames */
+      pthread_mutex_lock( &this->free_img_buf_queue->mutex );
+      img = duplicate_frame (this, this->img_backup );
+      pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
+      if( img )
+        img->vpts = cur_vpts;
+        
       return img;
 
     } else {
@@ -550,6 +548,7 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
     /* 
      * last frame? make backup for possible still image 
      */
+    pthread_mutex_lock( &this->free_img_buf_queue->mutex );
     if (img && !img->next &&
 	(this->xine->video_fifo->size(this->xine->video_fifo) < 10 
 	 || this->xine->video_in_discontinuity) ) {
@@ -557,9 +556,9 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
       printf ("video_out: possible still frame (fifosize = %d)\n",
 	      this->xine->video_fifo->size(this->xine->video_fifo));
         
-      this->img_backup = this->vo.duplicate_frame (&this->vo, img);
-      this->backup_is_logo = 0;
+      this->img_backup = duplicate_frame (this, img);
     }
+    pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
 
     /*
      * remove frame from display queue and show it
@@ -572,7 +571,7 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
 }
 
 static void overlay_and_display_frame (vos_t *this, 
-				       vo_frame_t *img) {
+				       vo_frame_t *img, int64_t vpts) {
 
 #ifdef LOG
   printf ("video_out: displaying image with vpts = %lld\n", 
@@ -580,15 +579,19 @@ static void overlay_and_display_frame (vos_t *this,
 #endif
   
   if (this->overlay_source) {
-    /* This is the only way for the overlay manager to get pts values
-     * for flushing its buffers. So don't remove it! */
-    
     this->overlay_source->multiple_overlay_blend (this->overlay_source, 
-						  img->vpts, 
+						  vpts, 
 						  this->driver, img,
 						  this->video_loop_running && this->overlay_enabled);
   }
-  
+
+  /* hold current frame for snapshot feature */
+  if( this->last_frame ) {
+    vo_frame_dec_lock( this->last_frame );
+  }
+  vo_frame_inc_lock( img );
+  this->last_frame = img;
+
   this->driver->display_frame (this->driver, img);
   
   this->redraw_needed = 0; 
@@ -597,9 +600,6 @@ static void overlay_and_display_frame (vos_t *this,
 static void check_redraw_needed (vos_t *this, int64_t vpts) {
 
   if (this->overlay_source) {
-    /* This is the only way for the overlay manager to get pts values
-     * for flushing its buffers. So don't remove it! */
-    
     if( this->overlay_source->redraw_needed (this->overlay_source, vpts) )
       this->redraw_needed = 1; 
   }
@@ -608,6 +608,57 @@ static void check_redraw_needed (vos_t *this, int64_t vpts) {
     this->redraw_needed = 1;
 }
 
+/* special loop for paused mode
+ * needed to update screen due overlay changes, resize, window
+ * movement, brightness adjusting etc.
+ */                   
+static void paused_loop( vos_t *this, int64_t vpts )
+{
+  vo_frame_t   *img;
+  
+  pthread_mutex_lock( &this->free_img_buf_queue->mutex );
+  /* prevent decoder thread from allocating new frames */
+  this->free_img_buf_queue->locked_for_read = 1;
+  
+  while( this->xine->speed == SPEED_PAUSE ) {
+  
+    /* we need at least one free frame to keep going */
+    if( this->display_img_buf_queue->first &&
+       !this->free_img_buf_queue->first ) {
+    
+      img = vo_remove_from_img_buf_queue (this->display_img_buf_queue);
+      img->next = NULL;
+      this->free_img_buf_queue->first = img;
+      this->free_img_buf_queue->last  = img;
+      this->free_img_buf_queue->num_buffers = 1;
+    }
+    
+    /* set img_backup to play the same frame several times */
+    if( this->display_img_buf_queue->first && !this->img_backup ) {
+      this->img_backup = vo_remove_from_img_buf_queue (this->display_img_buf_queue);
+      this->redraw_needed = 1;
+    }
+    
+    check_redraw_needed( this, vpts );
+    
+    if( this->redraw_needed && this->img_backup ) {
+      img = duplicate_frame (this, this->img_backup );
+      if( img ) {
+        pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
+        overlay_and_display_frame (this, img, vpts);
+        pthread_mutex_lock( &this->free_img_buf_queue->mutex );
+      }  
+    }
+    
+    xine_usec_sleep (20000);
+  } 
+  
+  this->free_img_buf_queue->locked_for_read = 0;
+   
+  if( this->free_img_buf_queue->first )
+    pthread_cond_signal (&this->free_img_buf_queue->not_empty);
+  pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
+}
 
 static void *video_out_loop (void *this_gen) {
 
@@ -650,7 +701,7 @@ static void *video_out_loop (void *this_gen) {
 #ifdef LOG
       printf ("video_out: displaying frame (id=%d)\n", img->id);
 #endif
-      overlay_and_display_frame (this, img);
+      overlay_and_display_frame (this, img, vpts);
     }
     else
     {
@@ -695,6 +746,9 @@ static void *video_out_loop (void *this_gen) {
  
     do {
       vpts = this->metronom->get_current_time (this->metronom);
+  
+      if( this->xine->speed == SPEED_PAUSE )
+        paused_loop( this, vpts );
 
       usec_to_sleep = (next_frame_vpts - vpts) * 100 / 9;
 
@@ -731,7 +785,11 @@ static void *video_out_loop (void *this_gen) {
     vo_frame_dec_lock( this->img_backup );
     this->img_backup = NULL;
   }
- 
+  if (this->last_frame) {
+    vo_frame_dec_lock( this->last_frame );
+    this->last_frame = NULL;
+  }
+
   pthread_exit(NULL);
 }
 
@@ -740,72 +798,6 @@ static uint32_t vo_get_capabilities (vo_instance_t *this_gen) {
   return this->driver->get_capabilities (this->driver);
 }
 
-static vo_frame_t * vo_duplicate_frame( vo_instance_t *this_gen, vo_frame_t *img ) {
-
-  vo_frame_t *dupl;
-  /* vos_t      *this = (vos_t *) this_gen; */
-  int         image_size;
-    
-  dupl = vo_get_frame (this_gen, img->width, img->height, img->ratio,
-		       img->format, VO_BOTH_FIELDS );
-  
-  image_size = img->width * img->height;
-
-  if (img->format == IMGFMT_YV12) {
-    /* The dxr3 video out plugin does not allocate memory for the dxr3
-     * decoder, so we must check for NULL */
-    if (img->base[0])
-      xine_fast_memcpy(dupl->base[0], img->base[0], image_size);
-    if (img->base[1])
-      xine_fast_memcpy(dupl->base[1], img->base[1], image_size >> 2);
-    if (img->base[2])
-      xine_fast_memcpy(dupl->base[2], img->base[2], image_size >> 2);
-  } else {
-    if (img->base[0])
-      xine_fast_memcpy(dupl->base[0], img->base[0], image_size * 2);
-  }  
-  
-  dupl->bad_frame = 0;
-  dupl->pts       = 0;
-  dupl->vpts      = 0;
-  dupl->duration  = img->duration;
-
-  /* Support copy; Dangerous, since some decoders may use a source that's
-   * not dupl->base. It's up to the copy implementation to check for NULL */ 
-  if (img->format == IMGFMT_YV12) {
-    if (img->copy) {
-      int height = img->height;
-      int stride = img->width;
-      uint8_t* src[3];
-  
-      src[0] = dupl->base[0];
-      src[1] = dupl->base[1];
-      src[2] = dupl->base[2];
-      while ((height -= 16) >= 0) {
-        dupl->copy(dupl, src);
-        src[0] += 16 * stride;
-        src[1] +=  4 * stride;
-        src[2] +=  4 * stride;
-      }
-    }
-  } else {
-    if (img->copy) {
-      int height = img->height;
-      int stride = img->width;
-      uint8_t* src[3];
-      
-      src[0] = dupl->base[0];
-      
-      while ((height -= 16) >= 0) {
-        dupl->copy(dupl, src);
-        src[0] += 32 * stride;
-      }
-    }
-  }
-  
-  return dupl;
-}
-
 static void vo_open (vo_instance_t *this_gen) {
 
   vos_t      *this = (vos_t *) this_gen;
@@ -871,12 +863,6 @@ static void vo_exit (vo_instance_t *this_gen) {
   free (this->free_img_buf_queue);
   free (this->display_img_buf_queue);
 
-  pthread_mutex_lock(&this->logo_mutex);
-  if(this->logo_yuy2)
-    free(this->logo_yuy2);
-  pthread_mutex_unlock(&this->logo_mutex);
-  pthread_mutex_destroy(&this->logo_mutex);
-
   free (this);
 }
 
@@ -899,6 +885,7 @@ static void vo_enable_overlay (vo_instance_t *this_gen, int overlay_enabled) {
   this->overlay_enabled = overlay_enabled;
 }
 
+
 vo_instance_t *vo_new_instance (vo_driver_t *driver, xine_t *xine) {
 
   vos_t            *this;
@@ -916,7 +903,7 @@ vo_instance_t *vo_new_instance (vo_driver_t *driver, xine_t *xine) {
 
   this->vo.open                  = vo_open;
   this->vo.get_frame             = vo_get_frame;
-  this->vo.duplicate_frame       = vo_duplicate_frame;
+  this->vo.duplicate_frame       = NULL; /* deprecated */
   this->vo.get_last_frame        = vo_get_last_frame;
   this->vo.close                 = vo_close;
   this->vo.exit                  = vo_exit;
@@ -931,8 +918,8 @@ vo_instance_t *vo_new_instance (vo_driver_t *driver, xine_t *xine) {
   this->display_img_buf_queue = vo_new_img_buf_queue ();
   this->video_loop_running    = 0;
 
+  this->last_frame            = NULL;
   this->img_backup            = NULL;
-  this->backup_is_logo        = 0;
   
   this->overlay_source        = video_overlay_new_instance();
   this->overlay_source->init (this->overlay_source);
@@ -962,26 +949,6 @@ vo_instance_t *vo_new_instance (vo_driver_t *driver, xine_t *xine) {
   }
 
 
-  /* 
-   * load xine logo
-   */
-  {
-    config_values_t  *config;
-    char              default_logo[2048];
-    
-    config = xine->config;
-
-    pthread_mutex_init(&this->logo_mutex, NULL);
-
-    snprintf(default_logo, 2048, "%s/xine_logo.zyuy2", XINE_SKINDIR);
-    
-    this->logo_pathname = config->register_string(config, "video.logo_file", default_logo,
-						  "logo displayed in video output window", NULL,
-						  _logo_change_cb, (void *) this);
-    
-    _load_logo_file(this);
-  }
-
   /*
    * start video output thread
    *
@@ -1003,7 +970,7 @@ vo_instance_t *vo_new_instance (vo_driver_t *driver, xine_t *xine) {
 	    strerror(err));
     /* FIXME: how does this happen ? */
     printf (_("video_out: sorry, this should not happen. please restart xine.\n"));
-    exit(1);
+    abort();
   } else
     printf ("video_out: thread created\n");
 
