@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.121 2002/04/28 17:39:36 guenter Exp $
+ * $Id: xine.c,v 1.147 2002/08/01 22:30:55 miguelfreitas Exp $
  *
  * top-level xine functions
  *
@@ -56,25 +56,73 @@
 #include "xineutils.h"
 #include "compat.h"
 
+#define LOGO_DELAY 500000 /* usec */
+
+static void play_logo_internal(xine_t *this) {
+  pthread_mutex_lock (&this->logo_lock);
+  this->playing_logo = 1;
+  if( !xine_play_internal(this, this->logo_mrl, 0, 0) )
+    this->playing_logo = 0;
+  else
+    this->status = XINE_LOGO;
+  pthread_mutex_unlock (&this->logo_lock);
+}
+
+/* config callback for logo mrl changing */
+static void _logo_change_cb(void *data, cfg_entry_t *cfg) {
+  xine_t            *this = (xine_t *) data;
+  
+  pthread_mutex_lock (&this->logo_lock);
+  this->logo_mrl = cfg->str_value;
+  pthread_mutex_unlock (&this->logo_lock);
+  
+  /*
+   * Start playback of new mrl only if 
+   * current status is XINE_STOP or XINE_LOGO 
+   */
+  pthread_mutex_lock (&this->xine_lock);
+  if((this->status == XINE_LOGO) || (this->status == XINE_STOP)) {
+    xine_stop_internal(this);  
+    this->metronom->adjust_clock(this->metronom,
+				 this->metronom->get_current_time(this->metronom) + 30 * 90000 );
+    play_logo_internal(this);
+  }
+  pthread_mutex_unlock (&this->xine_lock);
+}
+
 void * xine_notify_stream_finished_thread (void * this_gen) {
   xine_t *this = this_gen;
   xine_event_t event;
 
+  pthread_mutex_lock (&this->xine_lock);
   xine_stop_internal (this);
+  pthread_mutex_unlock (&this->xine_lock);
 
   event.type = XINE_EVENT_PLAYBACK_FINISHED;
-
   xine_send_event (this, &event);
 
-  pthread_detach( pthread_self() );
+  xine_usec_sleep (LOGO_DELAY);
+  
+  pthread_mutex_lock (&this->xine_lock);
+  if (this->status == XINE_STOP) {
+    play_logo_internal(this);
+  }
+  pthread_mutex_unlock (&this->xine_lock);
 
   return NULL;
 }
 
 void xine_notify_stream_finished (xine_t *this) {
-  pthread_t finished_thread;
   int err;
 
+  if (this->status == XINE_QUIT)
+    return;
+
+  if (this->finished_thread_running)
+    pthread_join (this->finished_thread, NULL);
+
+  this->finished_thread_running = 1;
+
   /* This thread will just execute xine_stop and (possibly) xine_play then die.
      It might look useless but i need to detach this code from the current
      thread to make sure that video_decoder and audio_decoder are running and
@@ -84,47 +132,52 @@ void xine_notify_stream_finished (xine_t *this) {
      This is not a theorical situation: i was able to trigger it with simple
      user actions (play,seek,etc). [MF]
   */
-  if ((err = pthread_create (&finished_thread,
+  if ((err = pthread_create (&this->finished_thread,
 			     NULL, xine_notify_stream_finished_thread, this)) != 0) {
     printf (_("xine_notify_stream_finished: can't create new thread (%s)\n"),
 	    strerror(err));
-    exit (1);
+    abort();
   }
 }
 
-/* internal use only - called from demuxers on seek/stop
- * warning: after clearing decoders fifos an absolute discontinuity
- *          indication must be sent. relative discontinuities are likely
- *          to cause "jumps" on metronom.
- */
-void xine_flush_engine (xine_t *this) {
+void xine_report_codec( xine_t *this, int codec_type, uint32_t fourcc, uint32_t buf_type, int handled ) {
 
-  buf_element_t *buf;
+  if( this->report_codec_cb ) {
+    if( codec_type == XINE_CODEC_VIDEO ) {
+      if( !buf_type )
+        buf_type = fourcc_to_buf_video( fourcc );
 
-  this->video_fifo->clear(this->video_fifo);
-  if( this->audio_fifo )
-    this->audio_fifo->clear(this->audio_fifo);
-  
-  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-  buf->type            = BUF_CONTROL_RESET_DECODER;
-  this->video_fifo->put (this->video_fifo, buf);
-
-  if(this->audio_fifo) {
-    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-    buf->type            = BUF_CONTROL_RESET_DECODER;
-    this->audio_fifo->put (this->audio_fifo, buf);
+      this->report_codec_cb( this->report_codec_user_data,
+                             codec_type, fourcc,
+                             buf_video_name( buf_type ), handled );
+    } else {
+      if( !buf_type )
+        buf_type = formattag_to_buf_audio( fourcc );
+    
+      this->report_codec_cb( this->report_codec_user_data,
+                             codec_type, fourcc,
+                             buf_audio_name( buf_type ), handled );
+    }
   }
-  
-  this->metronom->adjust_clock(this->metronom,
-			       this->metronom->get_current_time(this->metronom) + 30 * 90000 );
 }
 
-static void xine_internal_osd (xine_t *this, char *str, 
-			       uint32_t start_time, uint32_t duration) {
+int xine_register_report_codec_cb(xine_t *this, xine_report_codec_t report_codec,
+				 void *user_data) {
+  
+  this->report_codec_cb = report_codec;
+  this->report_codec_user_data = user_data;
+  return 0;
+}
+
+void xine_internal_osd (xine_t *this, char *str, int duration) {
 
   uint32_t seconds;
   char tstr[256];
-
+  int64_t start_time;
+  
+  this->curtime_needed_for_osd = 0;
+  start_time = this->metronom->get_current_time (this->metronom);
+    
   if (this->osd_display) {
    
     this->osd_renderer->filled_rect (this->osd, 0, 0, 299, 99, 0);
@@ -137,7 +190,7 @@ static void xine_internal_osd (xine_t *this, char *str,
              (seconds % (60*60)) / 60,
              seconds % 60);
     
-    this->osd_renderer->render_text (this->osd, 70, 5, tstr, OSD_TEXT1);
+    this->osd_renderer->render_text (this->osd, 45, 5, tstr, OSD_TEXT1);
   
     this->osd_renderer->show (this->osd, start_time);
     this->osd_renderer->hide (this->osd, start_time+duration);
@@ -174,24 +227,21 @@ static void xine_set_speed_internal (xine_t *this, int speed) {
 
 void xine_stop_internal (xine_t *this) {
 
-  pthread_mutex_lock (&this->xine_lock);
-
   printf ("xine_stop\n");
 
-  /* xine_internal_osd (this, "}", this->metronom->get_current_time (this->metronom), 30000); never works */
-
   if (this->status == XINE_STOP) {
     printf ("xine_stop ignored\n");
-    pthread_mutex_unlock (&this->xine_lock);
     return;
   }
-
+  
   if (this->audio_out)
     this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
 
   xine_set_speed_internal(this, SPEED_NORMAL);
 
-  this->status = XINE_STOP;
+  /* Don't change status if we're quitting */
+  if(this->status != XINE_QUIT)
+    this->status = XINE_STOP;
     
   printf ("xine_stop: stopping demuxer\n");
   if(this->cur_demuxer_plugin) {
@@ -202,20 +252,16 @@ void xine_stop_internal (xine_t *this) {
 
   if(this->cur_input_plugin) {
     this->cur_input_plugin->close(this->cur_input_plugin);
-    /*
-     * If we set it to NULL, xine_eject() will not work after
-     * a xine_stop() call.
-     *
-     * this->cur_input_plugin = NULL;
-     */
+    if (strcmp(this->cur_mrl, this->logo_mrl) != 0)
+      /* remember the last input plugin for a possible eject */
+      this->last_input_plugin = this->cur_input_plugin;
   }
 
   printf ("xine_stop: done\n");
-
-  pthread_mutex_unlock (&this->xine_lock);
 }
 
 void xine_stop (xine_t *this) {
+  pthread_mutex_lock (&this->xine_lock);
   xine_stop_internal(this);
   
   /*
@@ -224,6 +270,12 @@ void xine_stop (xine_t *this) {
   */
   this->metronom->adjust_clock(this->metronom,
 			       this->metronom->get_current_time(this->metronom) + 30 * 90000 );
+  
+  if(this->status == XINE_STOP) {
+    play_logo_internal(this);
+  }
+
+  pthread_mutex_unlock (&this->xine_lock);
 }
 
 
@@ -297,42 +349,59 @@ static int find_demuxer(xine_t *this, const char *MRL) {
   return 0;
 }
 
-int xine_play (xine_t *this, char *mrl, 
+int xine_play_internal (xine_t *this, char *mrl, 
 	       int start_pos, int start_time) {
 
   double     share ;
   off_t      pos, len;
   int        i;
+  int        demux_status;
 
   printf ("xine_play: xine open %s, start pos = %d, start time = %d (sec)\n", 
-	  mrl, start_pos, start_time);
+          mrl, start_pos, start_time);
 
-  pthread_mutex_lock (&this->xine_lock);
-
-  xine_set_speed_internal (this, SPEED_NORMAL);
-  
+  if (this->speed != SPEED_NORMAL) {
+    xine_set_speed_internal (this, SPEED_NORMAL);
+  }
   /*
    * stop engine only for different mrl
    */
 
-  if (this->status == XINE_PLAY && strcmp (mrl, this->cur_mrl) ) {
+  if ((this->status == XINE_PLAY && strcmp (mrl, this->cur_mrl)) || (this->status == XINE_LOGO)) {
     
     if(this->cur_demuxer_plugin) {
+      this->playing_logo = 0;
       this->cur_demuxer_plugin->stop (this->cur_demuxer_plugin);
     }
     
     if(this->cur_input_plugin) {
 
       if (strcmp (mrl, this->cur_mrl)) 
-	this->cur_input_plugin->close(this->cur_input_plugin);
+        this->cur_input_plugin->close(this->cur_input_plugin);
       else
-	this->cur_input_plugin->stop(this->cur_input_plugin);
+        this->cur_input_plugin->stop(this->cur_input_plugin);
     }
 
     if (this->audio_out)
       this->audio_out->control(this->audio_out, AO_CTRL_FLUSH_BUFFERS);
 
     this->status = XINE_STOP;
+  } else {
+  }
+
+  if (strcmp (mrl, this->cur_mrl)) {
+    /* Is it an 'opt:' mrlstyle ? */ 
+    if(config_file_change_opt(this->config, mrl)) {
+      xine_event_t event;
+    
+      this->status = XINE_STOP;
+    
+      event.type = XINE_EVENT_PLAYBACK_FINISHED;
+      pthread_mutex_unlock (&this->xine_lock);
+      xine_send_event (this, &event);
+      pthread_mutex_lock (&this->xine_lock);
+      return 1;
+    }
   }
 
   if (this->status == XINE_STOP ) {
@@ -353,7 +422,6 @@ int xine_play (xine_t *this, char *mrl,
 	        _("xine: cannot find input plugin for this MRL\n"));
       this->cur_demuxer_plugin = NULL;
       this->err = XINE_ERROR_NO_INPUT_PLUGIN;
-      pthread_mutex_unlock (&this->xine_lock);
 
       return 0;
     }
@@ -375,7 +443,6 @@ int xine_play (xine_t *this, char *mrl,
 	        _("xine: couldn't find demuxer for >%s<\n"), mrl);
       this->cur_input_plugin->close(this->cur_input_plugin);
       this->err = XINE_ERROR_NO_DEMUXER_PLUGIN;
-      pthread_mutex_unlock (&this->xine_lock);
       return 0;
     }
 
@@ -387,7 +454,6 @@ int xine_play (xine_t *this, char *mrl,
   /*
    * start demuxer
    */
-
   if (start_pos) {
     /* FIXME: do we need to protect concurrent access to input plugin here? */
     len = this->cur_input_plugin->get_length (this->cur_input_plugin);
@@ -396,17 +462,17 @@ int xine_play (xine_t *this, char *mrl,
   } else
     pos = 0;
   
-  if( this->status == XINE_STOP )
-    this->cur_demuxer_plugin->start (this->cur_demuxer_plugin,
-				     this->video_fifo,
-				     this->audio_fifo, 
-				     pos, start_time);
-  else
-    this->cur_demuxer_plugin->seek (this->cur_demuxer_plugin,
-				     pos, start_time);
-  
-  if (this->cur_demuxer_plugin->get_status(this->cur_demuxer_plugin) != DEMUX_OK) {
-    
+  if( this->status == XINE_STOP ) {
+    demux_status = this->cur_demuxer_plugin->start (this->cur_demuxer_plugin,
+						    this->video_fifo,
+						    this->audio_fifo, 
+						    pos, start_time);
+  }
+  else {
+    demux_status = this->cur_demuxer_plugin->seek (this->cur_demuxer_plugin,
+						   pos, start_time);
+  }
+  if (demux_status != DEMUX_OK) {
     xine_log (this, XINE_LOG_MSG, 
 	      _("xine_play: demuxer failed to start\n"));
     
@@ -415,8 +481,6 @@ int xine_play (xine_t *this, char *mrl,
     if( this->status == XINE_STOP )      
       this->cur_input_plugin->close(this->cur_input_plugin);
   
-    pthread_mutex_unlock (&this->xine_lock);
-
     return 0;
     
   } else {
@@ -424,42 +488,60 @@ int xine_play (xine_t *this, char *mrl,
     this->status = XINE_PLAY;
     strncpy (this->cur_mrl, mrl, 1024);
     
-    /* osd */
-    xine_usec_sleep(100000); /* FIXME: how do we assure an updated cur_input_time? */
-    xine_internal_osd (this, ">", this->metronom->get_current_time (this->metronom), 300000);
-
+    /* osd will be updated as soon as we know cur_input_time */
+    if( !this->playing_logo )
+      this->curtime_needed_for_osd = 5;
   }
 
-  pthread_mutex_unlock (&this->xine_lock);
-
   return 1;
 }             
 
+int xine_play (xine_t *this, char *mrl, 
+	       int start_pos, int start_time) {
+int ret;
+
+  pthread_mutex_lock (&this->xine_lock);
+  ret = xine_play_internal (this, mrl, start_pos, start_time);
+  pthread_mutex_unlock (&this->xine_lock);
+  
+  return ret;
+}
+
 int xine_eject (xine_t *this) {
   
-  if(this->cur_input_plugin == NULL) 
+  int status;
+
+  if(this->last_input_plugin == NULL) 
     return 0;
   
   pthread_mutex_lock (&this->xine_lock);
 
-  if ((this->status == XINE_STOP)
-      && this->cur_input_plugin && this->cur_input_plugin->eject_media) {
+  status = 0;
+  if (((this->status == XINE_STOP) || (this->status == XINE_LOGO))
+      && this->last_input_plugin && this->last_input_plugin->eject_media) {
 
-    pthread_mutex_unlock (&this->xine_lock);
-
-    return this->cur_input_plugin->eject_media (this->cur_input_plugin);
+    status = this->last_input_plugin->eject_media (this->last_input_plugin);
   }
 
   pthread_mutex_unlock (&this->xine_lock);
-  return 0;
+  return status;
 }
 
 void xine_exit (xine_t *this) {
 
-  int i, j;
+  int i;
+
+  this->status = XINE_QUIT;
 
   xine_stop(this);
-    
+
+  pthread_mutex_lock (&this->finished_lock);
+
+  if (this->finished_thread_running)
+    pthread_join (this->finished_thread, NULL);
+
+  pthread_mutex_unlock (&this->finished_lock);
+
   printf ("xine_exit: shutdown audio\n");
 
   audio_decoder_shutdown (this);
@@ -487,35 +569,22 @@ void xine_exit (xine_t *this) {
   for (i = 0; i < this->num_input_plugins; i++)
     this->input_plugins[i]->dispose (this->input_plugins[i]);
 
-  for (i = 0; i < DECODER_PLUGIN_MAX; i++) {
-    if (this->audio_decoder_plugins[i]) {
-      this->audio_decoder_plugins[i]->dispose (this->audio_decoder_plugins[i]);
+  for (i = 0; i < this->num_audio_decoders_loaded; i++) 
+    this->audio_decoders_loaded[i]->dispose (this->audio_decoders_loaded[i]);
 
-      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
-	if (this->audio_decoder_plugins[j] == this->audio_decoder_plugins[i])
-	  this->audio_decoder_plugins[j] = NULL;
-      }
-    }
-    if (this->video_decoder_plugins[i]) {
-      this->video_decoder_plugins[i]->dispose (this->video_decoder_plugins[i]);
+  for (i = 0; i < this->num_video_decoders_loaded; i++) 
+    this->video_decoders_loaded[i]->dispose (this->video_decoders_loaded[i]);
 
-      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
-	if (this->video_decoder_plugins[j] == this->video_decoder_plugins[i])
-	  this->video_decoder_plugins[j] = NULL;
-      }
-    }
-    if (this->spu_decoder_plugins[i]) {
-      this->spu_decoder_plugins[i]->dispose (this->spu_decoder_plugins[i]);
-
-      for (j = i + 1; j < DECODER_PLUGIN_MAX; j++) {
-	if (this->spu_decoder_plugins[j] == this->spu_decoder_plugins[i])
-	  this->spu_decoder_plugins[j] = NULL;
-      }
-    }
-  }
+  for (i = 0; i < this->num_spu_decoders_loaded; i++) 
+    this->spu_decoders_loaded[i]->dispose (this->spu_decoders_loaded[i]);
 
   xine_profiler_print_results ();
 
+  pthread_mutex_destroy (&this->logo_lock);
+  pthread_mutex_destroy (&this->xine_lock);
+  pthread_mutex_destroy (&this->finished_lock);
+  pthread_mutex_destroy (&this->osd_lock);
+
   free (this);
 
 }
@@ -529,16 +598,29 @@ xine_t *xine_init (vo_driver_t *vo,
 				     "extension", NULL};
   int          i;
 
+  /* setting default logo mrl */
+  pthread_mutex_init (&this->logo_lock, NULL);
+
+  pthread_mutex_lock (&this->logo_lock);
+  this->logo_mrl = config->register_string(config, "misc.logo_mrl", XINE_LOGO_FILE,
+					   _("logo mrl, displayed in video output window"),
+					   NULL, _logo_change_cb, (void *) this);
+  pthread_mutex_unlock (&this->logo_lock);
+
   this->video_driver = vo;
 
+  /* initialize color conversion tables and functions */
+  init_yuv_conversion();
+
   /* init log buffers */
   for (i = 0; i < XINE_LOG_NUM; i++)
     this->log_buffers[i] = new_scratch_buffer (25);
   
 #ifdef ENABLE_NLS
   bindtextdomain("xine-lib", XINE_LOCALEDIR);
+  textdomain("xine-lib");
 #endif 
-
+  
   printf ("xine: xine_init entered\n");
   
   this->err     = XINE_ERROR_NONE;
@@ -554,6 +636,10 @@ xine_t *xine_init (vo_driver_t *vo,
   pthread_mutex_init (&this->xine_lock, NULL);
 
   pthread_mutex_init (&this->finished_lock, NULL);
+  
+  pthread_mutex_init (&this->osd_lock, NULL);
+
+  this->finished_thread_running = 0;
 
   /*
    * init event listeners
@@ -561,7 +647,8 @@ xine_t *xine_init (vo_driver_t *vo,
   this->num_event_listeners = 0; /* Initially there are none */
   this->cur_input_plugin = NULL; /* In case the input plugin event handlers
                                   * are called too early. */
-  this->cur_spu_decoder_plugin = NULL; 
+  this->cur_spu_decoder_plugin = NULL;
+  this->report_codec_cb = NULL; 
   
   /*
    * create a metronom
@@ -573,24 +660,27 @@ xine_t *xine_init (vo_driver_t *vo,
    * load input and demuxer plugins
    */
 
-  load_input_plugins (this, config, INPUT_PLUGIN_IFACE_VERSION);
+  load_input_plugins (this, config);
 
   this->demux_strategy  = config->register_enum (config, "misc.demux_strategy", 0,
 						 demux_strategies, "demuxer selection strategy",
 						 NULL, NULL, NULL);
 
-  load_demux_plugins(this, config, DEMUXER_PLUGIN_IFACE_VERSION);
+  load_demux_plugins(this, config);
 
-  this->spu_channel_auto   = -1;
-  this->spu_channel_user   = -1;
-  this->cur_input_pos      = 0;
-  this->cur_input_length   = 0;
+  this->spu_channel_auto      = -1;
+  this->spu_channel_letterbox = -1;
+  this->spu_channel_pan_scan  = -1;
+  this->spu_channel_user      = -1;
+  this->cur_input_pos         = 0;
+  this->cur_input_length      = 0;
+  this->last_input_plugin     = NULL;
 
   /*
    * init and start decoder threads
    */
 
-  load_decoder_plugins (this, config, DECODER_PLUGIN_IFACE_VERSION);
+  load_decoder_plugins (this, config);
 
   this->video_out = vo_new_instance (vo, this);
   video_decoder_init (this);
@@ -630,6 +720,7 @@ xine_t *xine_init (vo_driver_t *vo,
     this->osd_renderer->hide (this->osd, 300000);
   }
 
+  play_logo_internal(this);
 
   return this;
 }
@@ -686,8 +777,12 @@ int xine_get_current_position (xine_t *this) {
 }
 
 int xine_get_status(xine_t *this) {
+  int status;
 
-  return this->status;
+  status = this->status;
+  if( status == XINE_LOGO )
+    status = XINE_STOP;
+  return status;
 }
 
 /* ***
@@ -762,30 +857,29 @@ void xine_set_speed (xine_t *this, int speed) {
 
   /* osd */
 
+  pthread_mutex_lock (&this->osd_lock);
   switch (speed) {
   case SPEED_PAUSE:
-    xine_internal_osd (this, "<", this->metronom->get_current_time (this->metronom), 10000);
+    xine_internal_osd (this, "<", 90000);
     break;
   case SPEED_SLOW_4:
-    xine_internal_osd (this, "<>", this->metronom->get_current_time (this->metronom), 20000 * speed);
+    xine_internal_osd (this, "<>", 20000 * speed);
     break;
   case SPEED_SLOW_2:
-    xine_internal_osd (this, "@>", this->metronom->get_current_time (this->metronom), 20000 * speed);
+    xine_internal_osd (this, "@>", 20000 * speed);
     break;
   case SPEED_NORMAL:
-    xine_internal_osd (this, ">", this->metronom->get_current_time (this->metronom), 20000 * speed);
+    xine_internal_osd (this, ">", 20000 * speed);
     break;
   case SPEED_FAST_2:
-    xine_internal_osd (this, "$$", this->metronom->get_current_time (this->metronom), 20000 * speed);
+    xine_internal_osd (this, "$$", 20000 * speed);
     break;
   case SPEED_FAST_4:
-    xine_internal_osd (this, "$$$", this->metronom->get_current_time (this->metronom), 20000 * speed);
+    xine_internal_osd (this, "$$$", 20000 * speed);
     break;
   } 
-
-  /* make sure osd can be displayed */
-  xine_usec_sleep(100000);
-
+  pthread_mutex_unlock (&this->osd_lock);
+    
   printf ("xine: set_speed %d\n", speed);
   xine_set_speed_internal (this, speed);
 
