@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_internal.h,v 1.79 2002/04/23 15:45:26 esnel Exp $
+ * $Id: xine_internal.h,v 1.92 2002/07/14 20:55:17 miguelfreitas Exp $
  *
  */
 
@@ -55,7 +55,8 @@ extern "C" {
 #define INPUT_PLUGIN_MAX       50
 #define DEMUXER_PLUGIN_MAX     50
 #define DECODER_PLUGIN_MAX     256
-#define DECODER_PLUGIN_IFACE_VERSION      7
+#define VIDEO_DECODER_IFACE_VERSION      10
+#define AUDIO_DECODER_IFACE_VERSION      9
 #define AUDIO_OUT_PLUGIN_MAX   50
 #define VIDEO_OUT_PLUGIN_MAX   50
 #define XINE_MAX_EVENT_LISTENERS 50
@@ -65,7 +66,7 @@ extern "C" {
  *
  * for a dynamic plugin make sure you provide this function call:
  * video_decoder_t *init_video_decoder_plugin (int iface_version,  
- *                                             config_values_t *cfg);
+ *                                             xine_t *xine);
  */
 
 typedef struct video_decoder_s video_decoder_t;
@@ -92,8 +93,6 @@ struct video_decoder_s {
 
   int priority;
 
-  metronom_t *metronom;
-
 };
 
 /*
@@ -101,7 +100,7 @@ struct video_decoder_s {
  *
  * for a dynamic plugin make sure you provide this function call:
  * audio_decoder_t *init_audio_decoder_plugin (int iface_version,  
- *                                             config_values_t *cfg);
+ *                                             xine_t *xine);
  */
 
 typedef struct audio_decoder_s audio_decoder_t;
@@ -140,6 +139,7 @@ struct audio_decoder_s {
 #define XINE_STOP      0 
 #define XINE_PLAY      1 
 #define XINE_QUIT      2
+#define XINE_LOGO      3
 
 /*
  * log output
@@ -151,6 +151,12 @@ struct audio_decoder_s {
 
 typedef void (*xine_event_listener_t) (void *user_data, xine_event_t *);
 
+#define XINE_CODEC_AUDIO   0
+#define XINE_CODEC_VIDEO   1
+
+typedef void (*xine_report_codec_t) (void *user_data, int codec_type,
+                                     uint32_t fourcc, char *description, int handled);
+
 struct xine_s {
   
   /* private : */
@@ -159,9 +165,16 @@ struct xine_s {
   
   config_values_t           *config;
 
+  /* MRL of displayed logo */
+  char                      *logo_mrl;
+  /* Logo manipulation mutex */
+  pthread_mutex_t            logo_lock;
+
   input_plugin_t            *input_plugins[INPUT_PLUGIN_MAX];
   int                        num_input_plugins;
   input_plugin_t            *cur_input_plugin;
+  /* kept to do proper ejecting (otherwise we eject the logo) */
+  input_plugin_t            *last_input_plugin;
 
   demux_plugin_t            *demuxer_plugins[DEMUXER_PLUGIN_MAX];
   int                        num_demuxer_plugins;
@@ -178,9 +191,10 @@ struct xine_s {
   spu_functions_t           *spu_out;
   pthread_t                  spu_thread;
   spu_decoder_t             *spu_decoder_plugins[DECODER_PLUGIN_MAX];
-  int                        num_spu_decoder_plugins;
   spu_decoder_t             *cur_spu_decoder_plugin;
   int                        spu_finished;
+  spu_decoder_t             *spu_decoders_loaded[DECODER_PLUGIN_MAX];
+  int                        num_spu_decoders_loaded;
 
   /* *_user: -2 => off
              -1 => auto (use *_auto value)
@@ -191,6 +205,8 @@ struct xine_s {
   int                        audio_channel_auto;
   int                        spu_channel_user;
   int                        spu_channel_auto;
+  int                        spu_channel_letterbox;
+  int                        spu_channel_pan_scan;
   int                        spu_channel;
 
   vo_driver_t               *video_driver;
@@ -199,6 +215,8 @@ struct xine_s {
   pthread_t                  video_thread;
   video_decoder_t           *video_decoder_plugins[DECODER_PLUGIN_MAX];
   video_decoder_t           *cur_video_decoder_plugin;
+  video_decoder_t           *video_decoders_loaded[DECODER_PLUGIN_MAX];
+  int                        num_video_decoders_loaded;
   int                        video_finished;
   int                        video_in_discontinuity;
   
@@ -211,8 +229,9 @@ struct xine_s {
   lrb_t                     *audio_temp;
   pthread_t                  audio_thread;
   audio_decoder_t           *audio_decoder_plugins[DECODER_PLUGIN_MAX];
-  int                        num_audio_decoder_plugins;
   audio_decoder_t           *cur_audio_decoder_plugin;
+  audio_decoder_t           *audio_decoders_loaded[DECODER_PLUGIN_MAX];
+  int                        num_audio_decoders_loaded;
   uint32_t                   audio_track_map[50];
   int                        audio_track_map_entries;
   int                        audio_finished;
@@ -237,6 +256,15 @@ struct xine_s {
 
   int                        err;
 
+  pthread_t                  finished_thread;
+  int                        finished_thread_running;
+  
+  xine_report_codec_t        report_codec_cb;
+  void                      *report_codec_user_data;
+  
+  int                        playing_logo;
+  int                        curtime_needed_for_osd;
+  pthread_mutex_t            osd_lock;
 };
 
 /*
@@ -266,6 +294,7 @@ xine_t *xine_init (vo_driver_t *vo,
  * returns 1 on succ, 0 on failure
  */
 int xine_play (xine_t *this, char *MRL, int start_pos, int start_time);
+int xine_play_internal (xine_t *this, char *MRL, int start_pos, int start_time);
 
 
 /*
@@ -414,7 +443,8 @@ char **xine_get_autoplay_mrls (xine_t *this, char *plugin_id, int *num_mrls);
  */
 
 void xine_notify_stream_finished (xine_t *this);
-void xine_flush_engine (xine_t *this);
+void xine_report_codec( xine_t *this, int codec_type, uint32_t fourcc, uint32_t buf_type, int handled );
+void xine_internal_osd (xine_t *this, char *str, int duration);
 
 /*
  * video decoder stuff
@@ -475,26 +505,26 @@ void audio_decoder_shutdown (xine_t *this);
  * load all available demuxer plugins
  */
 void load_demux_plugins (xine_t *this, 
-			 config_values_t *config, int iface_version);
+			 config_values_t *config);
                          
 /*
  *  list (open and close) all available demuxer plugins
  */
 void xine_list_demux_plugins (config_values_t *config,
-                          char **identifiers, char **mimetypes);
+			      char **identifiers, char **mimetypes);
                           
 /*
  * load all available input plugins
  */
 
 void load_input_plugins (xine_t *this, 
-			 config_values_t *config, int iface_version);
+			 config_values_t *config);
 
 /*
  * load all available decoder plugins
  */
 void load_decoder_plugins (xine_t *this, 
-			   config_values_t *config, int iface_version);
+			   config_values_t *config);
 
 /*
  * output driver load support functions
@@ -574,6 +604,16 @@ int xine_remove_event_listener(xine_t *this, xine_event_listener_t listener);
 
 void xine_send_event(xine_t *this, xine_event_t *event);
 
+/*
+ * register an codec reporting callback.
+ * return 0 if ok
+ * obs: set to NULL to unregister
+ */
+
+int xine_register_report_codec_cb(xine_t *this, xine_report_codec_t report_codec,
+				 void *user_data);
+
+
 /*
  * snapshot function
  *
@@ -593,6 +633,7 @@ int xine_get_current_frame (xine_t *this, int *width, int *height,
 #define XINE_ASPECT_RATIO_4_3         2
 #define XINE_ASPECT_RATIO_ANAMORPHIC  3
 #define XINE_ASPECT_RATIO_211_1       4
+#define XINE_ASPECT_RATIO_PAN_SCAN   41
 #define XINE_ASPECT_RATIO_DONT_TOUCH 42
 
 osd_renderer_t *xine_get_osd_renderer (xine_t *this);
@@ -618,6 +659,19 @@ int    xine_get_log_section_count (xine_t *this);
 
 int xine_get_error (xine_t *this);
 
+
+/* 
+ * demuxer helper functions from demux.c 
+ */
+
+void xine_demux_flush_engine(xine_t *this);
+
+void xine_demux_control_newpts( xine_t *this, int64_t pts, uint32_t flags );
+
+void xine_demux_control_start( xine_t *this );
+
+void xine_demux_control_end( xine_t *this, uint32_t flags );
+
 #ifdef __cplusplus
 }
 #endif
