@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xineutils.h,v 1.13 2002/04/27 14:35:39 richwareham Exp $
+ * $Id: xineutils.h,v 1.20 2002/07/24 08:00:13 f1rmb Exp $
  *
  */
 #ifndef XINEUTILS_H
@@ -34,7 +34,7 @@ extern "C" {
 #include <pthread.h>
 #include "attributes.h"
 #include "compat.h"
-
+#include "xmlparser.h"
   /*
    * debugable mutexes
    */
@@ -566,13 +566,13 @@ void xine_probe_fast_memcpy(config_values_t *config);
 		      /* Debugging/Monitoring */
 
 #if (__GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95) || !defined(__GNUC__)) && !defined(__cplusplus)
-#define xlerror(...) do {                                                     \
+#define xlerror(...) do {                                                        \
         printf("XINE lib %s:%d:(%s) ", __FILE__, __LINE__, __XINE_FUNCTION__);   \
         printf(__VA_ARGS__);                                                     \
 	printf("\n");                                                            \
 } while (0)
 #else
-#define xlerror(args...) do {                                                 \
+#define xlerror(args...) do {                                                    \
         printf("XINE lib %s:%d:(%s) ", __FILE__, __LINE__, __XINE_FUNCTION__);   \
         printf(##args);                                                          \
         printf("\n");                                                            \
@@ -580,13 +580,11 @@ void xine_probe_fast_memcpy(config_values_t *config);
 #endif
 
 
-#ifdef DEBUG
 /*
  * Debug stuff
  */
-
 /*
- * profiling
+ * profiling (unworkable in non DEBUG isn't defined)
  */
 void xine_profiler_init (void);
 int xine_profiler_allocate_slot (char *label);
@@ -594,16 +592,6 @@ void xine_profiler_start_count (int id);
 void xine_profiler_stop_count (int id);
 void xine_profiler_print_results (void);
 
-#else /* no DEBUG, release version */
-
-#define xine_profiler_init()
-#define xine_profiler_allocate_slot(label)	(-1)
-#define xine_profiler_start_count(id)
-#define xine_profiler_stop_count(id)
-#define xine_profiler_print_results()
-
-#endif /* DEBUG*/
-
 /*
  * Allocate and clean memory size_t 'size', then return the pointer
  * to the allocated memory.
@@ -640,13 +628,13 @@ void xine_usec_sleep(unsigned usec);
 
 
 void xine_strdupa(char *dest, char *src);
-#define xine_strdupa(d, s) {                                        \
-  (d) = NULL;                                                       \
-  if((s) != NULL) {                                                 \
-    (d) = (char *) alloca(strlen((s)) + 1);                         \
-    strcpy((d), (s));                                               \
-  }                                                                 \
-}
+#define xine_strdupa(d, s) do {                                             \
+                                (d) = NULL;                                 \
+                                if((s) != NULL) {                           \
+                                  (d) = (char *) alloca(strlen((s)) + 1);   \
+                                  strcpy((d), (s));                         \
+                                }                                           \
+                              } while(0)
 
 /* Shamefully copied from glibc 2.2.3 */
 #ifdef HAVE_STRPBRK
@@ -727,6 +715,87 @@ static inline void _x_setenv(const char *name, const char *val, int _xx)
 #define	xine_setenv	_x_setenv
 #endif
 
+/*
+ * Color Conversion Utility Functions
+ * The following data structures and functions facilitate the conversion
+ * of RGB images to packed YUV (YUY2) images. All of the meaty details
+ * are written in color.c.
+ */
+
+typedef struct yuv_planes_s {
+
+  unsigned char *y;
+  unsigned char *u;
+  unsigned char *v;
+  unsigned int row_width;    /* frame width */
+  unsigned int row_stride;   /* frame width + 2 */
+  unsigned int row_count;    /* frame height */
+
+} yuv_planes_t;
+
+void init_yuv_conversion(void);
+void init_yuv_planes(yuv_planes_t *yuv_planes, int width, int height);
+void free_yuv_planes(yuv_planes_t *yuv_planes);
+
+extern void (*yuv444_to_yuy2)
+  (yuv_planes_t *yuv_planes, unsigned char *yuy2_map, int pitch);
+
+#define SCALEFACTOR 65536
+#define CENTERSAMPLE 128
+
+#define COMPUTE_Y(r, g, b) \
+  (unsigned char) \
+  ((y_r_table[r] + y_g_table[g] + y_b_table[b]) / SCALEFACTOR)
+#define COMPUTE_U(r, g, b) \
+  (unsigned char) \
+  ((u_r_table[r] + u_g_table[g] + u_b_table[b]) / SCALEFACTOR + CENTERSAMPLE)
+#define COMPUTE_V(r, g, b) \
+  (unsigned char) \
+  ((v_r_table[r] + v_g_table[g] + v_b_table[b]) / SCALEFACTOR + CENTERSAMPLE)
+
+#define FINISH_LINE(yuv_planes, row_ptr) \
+  yuv_planes.u[row_ptr + yuv_planes.row_width] = \
+    yuv_planes.u[row_ptr + yuv_planes.row_width - 1]; \
+  yuv_planes.u[row_ptr + yuv_planes.row_width + 1] = \
+    yuv_planes.u[row_ptr + yuv_planes.row_width - 2]; \
+ \
+  yuv_planes.v[row_ptr + yuv_planes.row_width] = \
+    yuv_planes.v[row_ptr + yuv_planes.row_width - 1]; \
+  yuv_planes.v[row_ptr + yuv_planes.row_width + 1] = \
+    yuv_planes.v[row_ptr + yuv_planes.row_width - 2];
+
+#define UNPACK_BGR15(packed_pixel, r, g, b) \
+  b = (packed_pixel & 0x7C00) >> 7; \
+  g = (packed_pixel & 0x03E0) >> 2; \
+  r = (packed_pixel & 0x001F) << 3;
+
+#define UNPACK_BGR16(packed_pixel, r, g, b) \
+  b = (packed_pixel & 0xF800) >> 8; \
+  g = (packed_pixel & 0x07E0) >> 3; \
+  r = (packed_pixel & 0x001F) << 3;
+
+#define UNPACK_RGB15(packed_pixel, r, g, b) \
+  r = (packed_pixel & 0x7C00) >> 7; \
+  g = (packed_pixel & 0x03E0) >> 2; \
+  b = (packed_pixel & 0x001F) << 3;
+
+#define UNPACK_RGB16(packed_pixel, r, g, b) \
+  r = (packed_pixel & 0xF800) >> 8; \
+  g = (packed_pixel & 0x07E0) >> 3; \
+  b = (packed_pixel & 0x001F) << 3;
+
+extern int y_r_table[256];
+extern int y_g_table[256];
+extern int y_b_table[256];
+
+extern int u_r_table[256];
+extern int u_g_table[256];
+extern int u_b_table[256];
+
+extern int v_r_table[256];
+extern int v_g_table[256];
+extern int v_b_table[256];
+
 #ifdef __cplusplus
 }
 #endif
